/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/* Includes *******************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICOSTARGET_SOURCE
#define MXF_RICOSTARGET_SOURCE
#endif

#ifdef Linux
#define RICOSMAINTASK_SLEEP
#include "RiCOS.h"
static int should_sleep = FALSE;
pthread_t 		timerThread;
pthread_cond_t 	waitCond = PTHREAD_COND_INITIALIZER;
pthread_mutex_t	waitMutex = PTHREAD_MUTEX_INITIALIZER;

void *timerFunc(void *vars)
{
	while(1)
	{
		usleep(1000);
		if(cachedTimer != NULL)
		{
			(*(cachedTimer->tickCbkFunc))(cachedTimer->param);
			pthread_cond_signal(&waitCond);
		}
		should_sleep = FALSE;
	}
}

RiCBoolean RiCOSMainTask_WaitStep(void)
{
	static RhpBoolean deviceInitialized = RiCFALSE;
	if (!(deviceInitialized))
	{
		pthread_create(&timerThread,NULL,timerFunc,NULL);
                deviceInitialized = 1;
	}
	if (should_sleep)
	{
		pthread_mutex_lock(&waitMutex);
		pthread_cond_wait(&waitCond,&waitMutex);
		pthread_mutex_unlock(&waitMutex);
	}
	should_sleep = TRUE;

	return RiCFALSE;
}


RiCOSResult RiCOSAnimMutex_lock(RiCOSMutex *me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult res = 0;
	RiCOSHandle current;
	if (me != NULL) { 
		current = RiCOSTask_getCurrentTaskHandle();
		if (me->owner == current)
		{
			me->count ++;
		}
		else
		{
			pthread_mutex_lock((me->hMutex));
			me->owner = current;
		}
		res = 1;
	}
	return res;
#else
	return 1;
#endif
}


RiCOSResult RiCOSAnimMutex_free(RiCOSMutex *me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult res = 0;

	RiCOSHandle current;
	if (me != NULL) 
	{ 
		current = RiCOSTask_getCurrentTaskHandle();
		if (me->owner == current)
		{
			if (me->count == 0)
			{
				me->owner = (RiCOSHandle)NULL;
				pthread_mutex_unlock((me->hMutex));
			}
			else
			{
				me->count--;
			}
		}
		res = 1;
	}
	return res;
#else
	return 1;
#endif
}

#endif
