/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
*/

/******************************************************************
 ---------------------------------------------------------------
|					RiCOSTargetOSServices.c                     |
 --------------------------------------------------------------- 
*******************************************************************/
/******************************************************************/
/** This file is used to implement functions to over-ride		 **/
/** functions implementes in the OS-adaptor with the			 **/
/** implementations defined in RiCOSTargetServices.c.			 **/
/******************************************************************/


#include <errno.h>    
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h> 
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/utsname.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "mxf_cfg.h"
#include "RiCOSMainloop.h"
#include "RiCOSWrap.h"

void RiCOSEventsPoolMutex_init(void)
{
}

void	RiCOSEventsPoolMutex_lock(void)
{
}

void RiCOSEventsPoolMutex_free(void)
{
}

/**************/	
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSMutex_lock(const RiCOSMutex *me)
{
	return RiCTRUE;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSMutex_free(const RiCOSMutex *me)
{
	return RiCTRUE;
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

void OM_NOTIFY_ERROR(const RhpString callStr, const RhpString funcStr)
{
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCOSEndApplication( RiC_INT32 errorCode )
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	exit(errorCode);	
#endif
}


/***************************************************/
/****				OS Event Flag				****/
/***************************************************/
RiCBoolean RiCOSEventFlag_init(RiCOSEventFlag * const me, RiCOSTask * const context)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		*me = (RiCOSEventFlag)RiCOSEventFlag_create();
		if (sem_init (*me, 0, 0) == -1)
			retval = RiCFALSE;
		else
			retval = RiCTRUE;
#endif
	}
	return retval;
}



void RiCOSEventFlag_cleanup(RiCOSEventFlag * const me)
{
#ifdef MICROC_OXF_CLEANUP
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

	RiC_free(*me);

#endif
#endif
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSEventFlag * RiCOSEventFlag_create(void)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSEventFlag me = (sem_t*)RiC_malloc((RhpPositive)sizeof(sem_t));
	return (RiCOSEventFlag*)me;
#else
	return (RiCOSEventFlag *) NULL;
#endif
}


/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCOSEventFlag_destroy(RiCOSEventFlag * const me)
{
#ifdef MICROC_OXF_CLEANUP
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (me != NULL) {
		int rv;
		if ((me == NULL) || ((*me) == NULL)) return ;
		rv = sem_destroy(*me); 
		if (rv != 0)
		{
			OM_NOTIFY_ERROR("sem_destroy","RiCOSEventFlag_cleanup");
		}
	}
#endif
#endif
}


RiCOSResult RiCOSEventFlag_wait(RiCOSEventFlag * const me, RhpInteger tminms )
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult res = RiCFALSE;
	if ((me != NULL) && ((*me) != NULL)) 
	{
		if ( -1 == tminms ){
			int rc = -1;
			while (rc == -1) {       
				rc = sem_wait(*me);
				break;
			}
		}
		else {
			struct timespec tms;
			if ( 0 != sem_trywait(*me)) {
				int rv;
				/***RiCOSTimer_evaluateSleepTime(&tms, tminms);***/
				rv = sem_timedwait(*me, &tms);
				if (rv != 0)
				{
					OM_NOTIFY_ERROR("WaitForSingleObject","RiCOSEventFlag_wait");
				}
			}
		}
		res = RiCTRUE;
	}
	return res;
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}


RiCOSResult RiCOSEventFlag_signal(RiCOSEventFlag * const me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult rc = RiCFALSE;
	if ((me != NULL) && ((*me) != NULL) ) {
		if (sem_post(*me) == 0) {
			rc = RiCTRUE;
		}
		else
		{
			OM_NOTIFY_ERROR("SetEvent","RiCOSEventFlag_signal");
		}
	}
	return rc;
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}


RiCOSResult RiCOSEventFlag_reset(RiCOSEventFlag * const me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult rc = RiCFALSE;
	if ((me != NULL) && ((*me) != NULL)) {
		if (sem_trywait(*me) == 0) {
			rc = RiCTRUE;
		}
		else
		{
			OM_NOTIFY_ERROR("ResetEvent","RiCOSEventFlag_reset");
		}
	}
	return rc;
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}


/***************************************************/
/****				OS Mutex					****/
/***************************************************/

RiCBoolean RiCOSMutex_init(RiCOSMutex * const me, OSMutexHandle mutexIdentifier) 
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	int rv = -1;
	if (me != NULL)
	{
		me->hMutex =(pthread_mutex_t*) malloc(sizeof(pthread_mutex_t));
		rv = pthread_mutex_init((me->hMutex), NULL);
		if (rv != 0)
		{
			OM_NOTIFY_ERROR("CreateMutex","RiCOSMutex_init");
		}
	}
	return (rv==0);
#else
	return RiCTRUE;
#endif
}


void RiCOSMutex_cleanup(const RiCOSMutex * const me)
{
	if (me != NULL)
		RiC_free(me->hMutex);
}


/***************************************************/
/****				OS Task						****/
/***************************************************/

RiCOSHandle RiCOSTask_getCurrentTaskHandle(void)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	return (RiCOSHandle)pthread_self();
#else
	RiCOSHandle retval = (RiCOSHandle) 0;
	return retval;
#endif
}

/***************************************************/
/****			OS ConnectionPort				****/
/***************************************************/

/** shared ConnectionPort info **/
struct {
	RhpString m_Buf;
	RhpInteger m_BufSize;
} RiCLinuxCPCommon;

void readFromSocket(RiCOSConnectionPort * me)
{
	RhpCharacter lenStr[MAX_LEN_STR + 1];
	RhpInteger len;
	RhpInteger rv;
	RiCBoolean goAhead = RiCTRUE;

	if (me != NULL)
	{
		rv = RiCOSSocket_receive(&(me->m_ntSocket), lenStr, MAX_LEN_STR);
		if (rv == -1) {		/* critical error */
			exit(0);
		}

		lenStr[MAX_LEN_STR] = '\0';
		if (sscanf(lenStr,"%d",&len) == 1) 
		{
			if (len > RiCLinuxCPCommon.m_BufSize ) {	/* need to increase the buffer */
				RiCLinuxCPCommon.m_BufSize = 1000 + len;
				if (RiCLinuxCPCommon.m_Buf) {
					RiC_free(RiCLinuxCPCommon.m_Buf);
				}
				RiCLinuxCPCommon.m_Buf = (RhpString) RiC_malloc((RhpPositive)RiCLinuxCPCommon.m_BufSize);
			}
			
			if (len == -2) { /* Acknoledge message */
				(void) RiCOSEventFlag_signal(&(me->m_AckEventFlag));
				goAhead = RiCFALSE;
			}
				
			if(goAhead){
				RiCLinuxCPCommon.m_Buf[len] = '\0';
				rv = RiCOSSocket_receive(&(me->m_ntSocket), RiCLinuxCPCommon.m_Buf,len);
				if (rv == -1) {		/* critical error */
					exit(0);
				}
				
				/*
				len is not used by RiCSData but we need to change
				the interface so we will use the length of the data
				and not assume that the string is null terminated
				
				  if socket thread is doing some work that may send
				  messages we do not want to Block the Send by wainting for
				  acknoldegment since the thread cannot handle the acknoledgment
				  ( since the thraed is self locked)
				*/
				me->m_ShouldWaitForAck = 0;
				(void) RiCOSEventFlag_signal(&(me->m_AckEventFlag));
				me->m_dispatchfunc(RiCSData_string2RiCSData(RiCLinuxCPCommon.m_Buf));
				me->m_ShouldWaitForAck = 1;
			}
		}
	}
}
/**************/

void MLCP_readFromSockLoop(RiCOSConnectionPort * me)
{
	if (me != NULL){
		
		while (1) 
		{	/*lint !e716 (while(1) ... ) */	
			readFromSocket(me);
		}
	}
}



/*
 Public function implementation
*/

RiCBoolean RiCOSConnectionPort_init(RiCOSConnectionPort * const me)
{
	RiCBoolean b = RiCFALSE;
	static RiCBoolean bufInitialized = RiCFALSE;
	
	if (me != NULL){		
		/** init common info **/
		if (!bufInitialized) {
			RiCLinuxCPCommon.m_Buf = NULL;
			RiCLinuxCPCommon.m_BufSize = 0;
			bufInitialized = RiCTRUE;
		}
		
		b = RiCOSMutex_init(&(me->m_SendMutex), (OSMutexHandle)0);
		b &= RiCOSSocket_init(&(me->m_ntSocket));
		b &= RiCOSEventFlag_init(&(me->m_AckEventFlag), NULL);
		(void) RiCOSEventFlag_reset(&(me->m_AckEventFlag));
		
		me->m_Connected = 0;
		me->m_dispatchfunc = NULL;
		me->hSocketHandleThread = NULL;
		me->m_ShouldWaitForAck = 1;
		me->m_NumberOfMessagesBetweenAck = 0;
	}
	
	return b;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSConnectionPort_cleanup(RiCOSConnectionPort * const me)
{
	if (me != NULL){
		
		
		RiCOSMutex_cleanup(&(me->m_SendMutex));
		RiCOSSocket_cleanup(&(me->m_ntSocket));
		RiCOSEventFlag_cleanup(&(me->m_AckEventFlag));
		
		
		/*
		I assume that we will have only one connection port
		so I can free the data for m_Buf but if it is not the case
		it does not matter ( the readFromSockLoop will allocate it )
		*/
		if (RiCLinuxCPCommon.m_Buf) {
			RiC_free(RiCLinuxCPCommon.m_Buf);
			RiCLinuxCPCommon.m_Buf = NULL;
		}
		RiCLinuxCPCommon.m_BufSize = 0;
		
		/* close the animation thread */
		/* if (me->hSocketHandleThread) TerminateThread(me->hSocketHandleThread,0); */
		
		me->m_dispatchfunc = NULL;
		me->hSocketHandleThread = (RiCOSHandle)NULL;
	}	
}
#else
void RiCOSConnectionPort_cleanup(RiCOSConnectionPort * const me)
{
}
#endif

/**************/
RiCOSConnectionPort * RiCOSConnectionPort_create(void)
{
	RiCOSConnectionPort * me = 	(RiCOSConnectionPort*)RiC_malloc((RhpPositive)sizeof(RiCOSConnectionPort));
	(void) RiCOSConnectionPort_init(me);
	return me;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSConnectionPort_destroy(RiCOSConnectionPort * const me)
{

}
#else
void RiCOSConnectionPort_destroy(RiCOSConnectionPort * const me)
{

}
#endif

pthread_t 		readSocketLoopThread;
extern void MLCP_readFromSockLoop(RiCOSConnectionPort * me);
RhpInteger RiCOSConnectionPort_Connect(RiCOSConnectionPort * const me, const RhpCharacter * const SocketAddress, RhpPositive nSocketPort)
{
	RhpInteger res = 0;
	if (me!=NULL)
	{
		if (NULL == me->m_dispatchfunc) {
			fprintf(stderr,"RiCOSConnectionPort_SetDispatcher should be called before RiCOSConnectionPort::Connect()\n");
			res = 0;
		}
		if ( 0  == me->m_Connected ) {
			(void)RiCOSSocket_init(&me->m_ntSocket);
			me->m_Connected = RiCOSSocket_createSocket(&me->m_ntSocket,SocketAddress,nSocketPort);
		}
		if (0 == me->m_Connected)
			res = 0;
		
		/* Connection established
		invoking thread to recieve messages from the socket */
		pthread_create(&readSocketLoopThread,NULL,(void*)MLCP_readFromSockLoop,me);
		res = me->m_Connected;
	}
	return res;
}

/**************/
RhpInteger RiCOSConnectionPort_Send(RiCOSConnectionPort * const me, RiCSData * m)
{
	RhpInteger rv = 0;
	
	if (me != NULL){
		
		RiCOSAnimMutex_lock(&(me->m_SendMutex));
		if (me->m_Connected) {
			RhpCharacter lenStr[MAX_LEN_STR+1];
			sprintf(lenStr,"%d",RiCSData_getLength(m));

			rv = RiCOSSocket_send(&(me->m_ntSocket),lenStr,MAX_LEN_STR);
			if (rv > 0) {
				rv = RiCOSSocket_send(&(me->m_ntSocket),RiCSData_getRawData(m),(RhpInteger) RiCSData_getLength(m));
			}
			
			/* need to check with Rhapsody */
			if (me->m_ShouldWaitForAck) {
				const RhpInteger maxNumOfMessagesBetweenAck = 127; /* MUST match the number in rhapsody */
				if (maxNumOfMessagesBetweenAck > 0) { /*lint !e774 !e506 () */
					me->m_NumberOfMessagesBetweenAck++;
					if (me->m_NumberOfMessagesBetweenAck >= maxNumOfMessagesBetweenAck) {
						me->m_NumberOfMessagesBetweenAck = 0;
						(void) RiCOSEventFlag_wait(&(me->m_AckEventFlag), -1);
						(void) RiCOSEventFlag_reset(&(me->m_AckEventFlag));
					}
				}
			}
			
		}
		RiCOSAnimMutex_free(&(me->m_SendMutex));
		
#ifdef MICROC_OXF_CLEANUP
		/* cleanup */
		RiCSData_cleanup(m);
#endif
	}
	return rv;
}
/*lint -restore (ignoring return value on mutex lock() and free()) */

/**************/
RiCBoolean RiCOSConnectionPort_SetDispatcher(RiCOSConnectionPort * const me, RiCOS_dispatchfunc dispfunc)
{
	RiCBoolean retval = RiCFALSE;
	
	if (me != NULL) {
		
		me->m_dispatchfunc = dispfunc;
		retval = RiCTRUE;
	}

	return retval;
}



/***************************************************/
/****				OS Socket					****/
/***************************************************/
RhpInteger initializeOSSocket(RiCOSConnectionPort * const me)
{
}

RiCBoolean RiCOSSocket_init(RiCOSSocket * const me)
{
	RiCBoolean retval = RiCFALSE;
	
	if (me != NULL){
		
		me->theSock = 0;
		retval = RiCTRUE;
	}

	return retval;
}

#ifdef MICROC_OXF_CLEANUP
void RiCOSSocket_cleanup(RiCOSSocket * const me)
{
	if (me != NULL){
		if (me->theSock != 0)
			(void)close(me->theSock);
	}
}
#endif

RiCOSSocket * RiCOSSocket_create(void)
{

	return NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSSocket_destroy(RiCOSSocket * const me)
{

}
#endif

/**************/
RhpInteger RiCOSSocket_createSocket(RiCOSSocket * const me, const RhpCharacter * SocketAddress, RhpPositive nSocketPort)
{
	static struct sockaddr_in	addr;
	struct protoent	*protocol;
	RhpInteger	proto;
	struct hostent	*host = NULL;
	RhpString proto_name = "tcp";
	RhpCharacter	hostName[128];
	RhpInteger rvStat;
	RhpInteger retval = 1;

	if ( nSocketPort == 0 ) nSocketPort = 6423;
	
	addr.sin_family = AF_INET;
	protocol = getprotobyname(proto_name);
	if (protocol == (struct protoent *) 0) {
		retval = 0;
	}
	if(retval){
		proto = protocol->p_proto;
		
		(void) gethostname(hostName, sizeof(hostName)-1);
		if (NULL != SocketAddress){
			if(strlen(SocketAddress) != 0) {
				if (!strcmp(hostName,SocketAddress)) SocketAddress = NULL;
				else {
					strcpy(hostName,SocketAddress);
					if ((host = gethostbyname(hostName)) == (struct hostent *) 0) {
						retval = 0;
					}
				}
			}
		}
	}

	if(retval){
#ifdef unix
#ifndef	CYGWIN
		endprotoent();
#endif
#endif
		
		addr.sin_port = htons((u_short)nSocketPort);
		
		if ((NULL == SocketAddress) || (strlen(SocketAddress) == 0)){
			addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
		}
		else if(host){
			addr.sin_addr.s_addr = *((RhpUnlimitedPositiveNatural *) host->h_addr);
		}		
		
		if (me != NULL)
		{
			if ((me->theSock = socket(AF_INET, SOCK_STREAM, proto)) == INVALID_SOCKET) {
				me->theSock = 0;
				retval = 0;
			}
		
			if(retval){
				/*lint -save -e740 (Unusual pointer cast) */
				while (((rvStat = connect(me->theSock, (struct sockaddr *) &addr, sizeof(addr))) == 
					SOCKET_ERROR) && (WSAGetLastError() == WSAEINTR))
					;
				/*lint -restore */
				
				if (SOCKET_ERROR == rvStat) {
					retval = 0;
				}
			}
		}
	}

	return retval;
}

/**************/
RhpInteger RiCOSSocket_send(RiCOSSocket * const me, const RhpCharacter * const buf, RhpInteger bufLen)
{
	RhpInteger bytes_writ = 0;
	RhpInteger n;
	RiCBoolean flag = RiCFALSE;
	
	if (me != NULL){
		
		while ((bytes_writ >= 0) && (bytes_writ < bufLen)) {
			flag = RiCTRUE;
			n = send(me->theSock, buf + bytes_writ, bufLen - bytes_writ,0);
			if (SOCKET_ERROR == n) {
				if (WSAGetLastError() == WSAEINTR){
					flag = RiCFALSE;
				}
				else{
					flag = RiCFALSE;
					bytes_writ = -1;
				}
			}
			
			if(flag == RiCTRUE){
				bytes_writ += n;
			}
		}
	}

	return bytes_writ;
}

/**************/
RhpInteger RiCOSSocket_receive(RiCOSSocket * const me, RhpString buf, RhpInteger bufLen)
{
	RhpInteger bytes_read = 0;
	RhpInteger n;
	RiCBoolean flag = RiCFALSE;
	
	if (me != NULL){
		
		while ( (bytes_read >= 0) && (bytes_read < bufLen) ) {
			flag = RiCTRUE;
			n = recv(me->theSock, buf + bytes_read, bufLen - bytes_read,0);
			if (SOCKET_ERROR == n) {
				if (WSAGetLastError() == WSAEINTR){
					flag = RiCFALSE;
				}
				else{
					flag = RiCFALSE;
					bytes_read = -1;
				}
			}
			
			if(flag == RiCTRUE){
				bytes_read += n;
			}
		}
	}
	return bytes_read;
}




RhpInteger WSAGetLastError()
{
	return errno;
}

#endif
