/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
#include "mxf_cfg.h"

#ifdef DEBUG
#include <stdio.h>
#define PDEBUG(X) printf(X)
#define PDEBUG2PARAMS(X,Y) printf(X,Y)
#define PDEBUG3PARAMS(X,Y,Z) printf(X,Y,Z)
#else
#define PDEBUG(X) 
#define PDEBUG2PARAMS(X,Y)
#define PDEBUG3PARAMS(X,Y,Z)
#endif


/*********** Use RS232 port on windows OS **************/

#ifdef _OM_TARGET_MON_WINDOWS_RS232

#include <stdio.h>

extern unsigned char OM_TGT_MON_MessageBuffer[];
extern unsigned short int  OM_TGT_MON_MessageLength;


static FILE *comFile=NULL;
FILE *portOpen()
{
	return comFile;
}
	 
void win_RS232_InitProtocol()
{
#ifndef _OM_DUMP_2_FILE
		comFile = fopen("com2:","w+");
#else
		comFile = fopen("c:\\serialdump.ser","w+");
#endif
	return;
}

void win_RS232_send()
{
	int written = fwrite(OM_TGT_MON_MessageBuffer,sizeof(char),OM_TGT_MON_MessageLength,portOpen());
	if (written != OM_TGT_MON_MessageLength)
		PDEBUG3PARAMS("Error writing: Written: %d Length: %d\n",written,OM_TGT_MON_MessageLength);
	else
		PDEBUG2PARAMS("Successfuly written %d\n",written);
	    fflush(portOpen());
	return;
}

#endif

/************** End RS232 on windows OS ***********/



/************** Definitions and functions needed for TCP protocol *****************/

#ifdef _OM_TARGET_MON_WINDOWS_TCP

#include <windows.h>

extern unsigned char OM_TGT_MON_MessageBuffer[];
extern unsigned short int  OM_TGT_MON_MessageLength;


static WSADATA        wsd;   
static SOCKET         s;   
static SOCKADDR_IN    recipient;
static int inited = FALSE;

void tcp_InitProtocol()
{
	if (WSAStartup(MAKEWORD(2, 2), &wsd) != 0)   
	{   
		PDEBUG("WSAStartup failed!\n");
	}
	else
	{
		PDEBUG("WSAStartup Success!\n");
	}

	s = socket(AF_INET, SOCK_STREAM, 0);   
	if (s == INVALID_SOCKET)   
	{   
	    PDEBUG2PARAMS("socket() failed; %d\n", WSAGetLastError());
	}
	else
	{
		PDEBUG("Socket Success \n");
	}
	recipient.sin_family = AF_INET;   
	recipient.sin_port = htons((short) 24816);
	if ((recipient.sin_addr.s_addr = inet_addr("127.0.0.1")) == INADDR_NONE)   
	{   
		struct hostent *host=NULL;   

	   host = gethostbyname("localhost");   
	   if (host)
	   {
		 memcpy(&recipient.sin_addr, host->h_addr_list[0],host->h_length);
		 PDEBUG("Hostname Success \n");
	   }
	   else  
		{   
			PDEBUG2PARAMS("gethostbyname() failed: %d\n", 0);
		}
	}
	if ( connect( s, (SOCKADDR*) &recipient, sizeof(recipient) ) == SOCKET_ERROR) 
	{
		PDEBUG( "Failed to connect.\n" );
	}

	return;
}

void tcp_send()
{
	int            ret;

	ret = send(s, OM_TGT_MON_MessageBuffer, OM_TGT_MON_MessageLength, 0);   
    if (ret == SOCKET_ERROR)   
    {   
       PDEBUG2PARAMS("sendto() failed; %d\n", WSAGetLastError());   
    }  
	else 
	{PDEBUG2PARAMS("Sent successfull %d\n",ret);}
	
	return;
}

#endif

/************ End Of Tcp rotocol **************/
