#ifndef FIXED_POINT_H
#define FIXED_POINT_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
*/

/******************* Fixed-Point types **************************/
/* Basic types to be used as the type of Fixed-Point variable's */
/****************************************************************/
typedef signed char FXP_8Bit_T;
typedef signed short FXP_16Bit_T;
typedef signed long int FXP_32Bit_T;

/********************* Fixed-Point conversion types ***************************/
/* Basic types used internally to perform Fixed-Point variable's  conversions */
/******************************************************************************/
#ifndef RIC_FXP_DOUBLE
#define RIC_FXP_DOUBLE
typedef double RiC_FXP_DOUBLE;
#endif
#ifndef RIC_FXP_INT32
#define RIC_FXP_INT32
typedef signed long int RiC_FXP_INT32;
#endif

/**************** Fixed-Point utility macros **************/
/* Utility macros used internally by the operation macros */
/**********************************************************/
#ifndef FXP_SHIFT
#define FXP_SHIFT(FXPVar) FXPShift_##FXPVar
#endif

#ifndef FXP_TYPE
#define FXP_TYPE(FXPVar) FXPType_##FXPVar
#endif

#ifndef FXP_WORD_SIZE
#define FXP_WORD_SIZE(FXPVar) FXPWordSize_##FXPVar
#endif

#ifndef _FXP_SHIFT_CLASS
#define _FXP_SHIFT_CLASS(CLASS, FXP) FXP_SHIFT(CLASS ##FXP)
#endif
#ifndef FXP_SHIFT_CLASS
#define FXP_SHIFT_CLASS(FXPVar, CLASS) _FXP_SHIFT_CLASS(CLASS##_, FXPVar)
#endif

#ifndef _FXP_TYPE_CLASS
#define _FXP_TYPE_CLASS(CLASS, FXP) FXP_TYPE(CLASS ##FXP)
#endif
#ifndef FXP_TYPE_CLASS
#define FXP_TYPE_CLASS(FXPVar, CLASS) _FXP_TYPE_CLASS(CLASS##_, FXPVar)
#endif

#ifndef _FXP_WORD_SIZE_CLASS
#define _FXP_WORD_SIZE_CLASS(CLASS, FXP) FXP_WORD_SIZE(CLASS ##FXP)
#endif
#ifndef FXP_WORD_SIZE_CLASS
#define FXP_WORD_SIZE_CLASS(FXPVar, CLASS) _FXP_WORD_SIZE_CLASS(CLASS##_, FXPVar)
#endif


/************************** Fixed-Point conversion macros *************************/
/* Macros for conversting Fixed-Point variabales to integer/double and vice versa */
/**********************************************************************************/
#ifndef FXP2INT
#define FXP2INT(FPvalue, FPshift) ((FPvalue) >> (FPshift))
#endif
#ifndef FXP2DOUBLE
#define FXP2DOUBLE(FPvalue, FPshift) (((RiC_FXP_DOUBLE)(FPvalue)) / (1 << (FPshift)))
#endif
#ifndef DOUBLE2FXP
#define DOUBLE2FXP(Dvalue, FPshift) ((RiC_FXP_INT32)((Dvalue) * (1 << (FPshift))))
#endif

/********************************* Fixed-Point Macros for Globals ******************************************/
/* This set of macros should be used when using global variables (not structured under a class/structure). */
/* In this case only the names of the variables are required.                                              */
/***********************************************************************************************************/

/************************************************************/
/* Performs the operation: FPDestination = FPSource         */
/************************************************************/
/* FPDestination - Destination Fixed-Point variable's data */
/* FPSource      - Source Fixed-Point variable's data       */
/************************************************************/
#ifndef FXP_ASSIGN
#define FXP_ASSIGN(FPDestination, FPSource) ((FPDestination) = \
	(((FXP_SHIFT(FPDestination)) > (FXP_SHIFT(FPSource))) ? \
	((((FXP_TYPE(FPDestination))(FPSource)) << ((FXP_SHIFT(FPDestination)) - (FXP_SHIFT(FPSource))))) : \
	((((FXP_TYPE(FPDestination))(FPSource)) >> ((FXP_SHIFT(FPSource)) - (FXP_SHIFT(FPDestination))))) \
	))
#endif


/************************************************/
/* Performs the operation: FPVar1 + FPVar2      */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_SUM
#define FXP_SUM(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) + (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) + ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) + ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) + (FPVar2))) \
	)
#endif


/************************************************************/
/* Performs the operation: FPDestination = FPVar1 + FPVar2  */
/************************************************************/
/* FPDestination - Destination Fixed-Point variable's data  */
/* FPVar1        - First Fixed-Point variable's data        */
/* FPVar2        - Second Fixed-Point variable's data       */
/************************************************************/
#ifndef FXP_ASSIGN_SUM
#define FXP_ASSIGN_SUM(FPDestination, FPVar1, FPVar2) \
	((FPDestination) = \
	(((FXP_SHIFT(FPDestination)) > ((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2)))) ? \
	((((FXP_TYPE(FPDestination))(FXP_SUM(FPVar1, FPVar2))) << ((FXP_SHIFT(FPDestination)) - ((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2)))))) : \
	((((FXP_TYPE(FPDestination))(FXP_SUM(FPVar1, FPVar2))) >> (((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2))) - (FXP_SHIFT(FPDestination))))) \
	))
#endif

/************************************************/
/* Performs the operation: FPVar1 - FPVar2      */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_SUB
#define FXP_SUB(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) - (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) - ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) - ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) - (FPVar2))) \
	)
#endif

/************************************************************/
/* Performs the operation: FPDestination = FPVar1 - FPVar2  */
/************************************************************/
/* FPDestination - Destination Fixed-Point variable's data  */
/* FPVar1        - First Fixed-Point variable's data        */
/* FPVar2        - Second Fixed-Point variable's data       */
/************************************************************/
#ifndef FXP_ASSIGN_SUB
#define FXP_ASSIGN_SUB(FPDestination, FPVar1, FPVar2) \
	((FPDestination) = \
	(((FXP_SHIFT(FPDestination)) > ((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2)))) ? \
	((((FXP_TYPE(FPDestination))(FXP_SUB(FPVar1, FPVar2))) << ((FXP_SHIFT(FPDestination)) - ((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2)))))) : \
	((((FXP_TYPE(FPDestination))(FXP_SUB(FPVar1, FPVar2))) >> (((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? (FXP_SHIFT(FPVar1)) : (FXP_SHIFT(FPVar2))) - (FXP_SHIFT(FPDestination))))) \
	))
#endif

/************************************************************/
/* Performs the operation: FPDestination = FPVar1 * FPVar2  */
/************************************************************/
/* FPVar1 - First Fixed-Point variable's data               */
/* FPVar2 - Second Fixed-Point variable's data              */
/************************************************************/
#ifndef FXP_MUL
#define FXP_MUL(FPVar1, FPVar2) ((FPVar1) * (FPVar2))
#endif

/************************************************************/
/* Performs the operation: FPDestination = FPVar1 * FPVar2  */
/************************************************************/
/* FPDestination - Destination Fixed-Point variable's data  */
/* FPVar1        - First Fixed-Point variable's data        */
/* FPVar2        - Second Fixed-Point variable's data       */
/************************************************************/
#ifndef FXP_ASSIGN_MUL
#define FXP_ASSIGN_MUL(FPDestination, FPVar1, FPVar2) \
	((FPDestination) = \
	(((FXP_SHIFT(FPDestination)) > (FXP_SHIFT(FPVar1) + FXP_SHIFT(FPVar2))) ? \
	((((FXP_TYPE(FPDestination))(FXP_MUL(FPVar1, FPVar2))) << ((FXP_SHIFT(FPDestination)) - (FXP_SHIFT(FPVar1) + FXP_SHIFT(FPVar2))))) : \
	((((FXP_TYPE(FPDestination))(FXP_MUL(FPVar1, FPVar2))) >> ((FXP_SHIFT(FPVar1) + FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPDestination))))) \
	))
#endif

/************************************************************/
/* Performs the operation: FPDestination = FPVar1 / FPVar2  */
/************************************************************/
/* FPVar1 - First Fixed-Point variable's data               */
/* FPVar2 - Second Fixed-Point variable's data              */
/************************************************************/
#ifndef FXP_DIV
#define FXP_DIV(FPVar1, FPVar2) ((FPVar1) / (FPVar2))
#endif

/************************************************************/
/* Performs the operation: FPDestination = FPVar1 / FPVar2  */
/************************************************************/
/* FPDestination - Destination Fixed-Point variable's data  */
/* FPVar1        - First Fixed-Point variable's data        */
/* FPVar2        - Second Fixed-Point variable's data       */
/************************************************************/
#ifndef FXP_ASSIGN_DIV
#define FXP_ASSIGN_DIV(FPDestination, FPVar1, FPVar2) \
	((FPDestination) = \
	(((FXP_SHIFT(FPDestination)) > (FXP_SHIFT(FPVar1) - FXP_SHIFT(FPVar2))) ? \
	((((FXP_TYPE(FPDestination))(FXP_DIV(FPVar1, FPVar2))) << ((FXP_SHIFT(FPDestination)) - (FXP_SHIFT(FPVar1) - FXP_SHIFT(FPVar2))))) : \
	((((FXP_TYPE(FPDestination))(FXP_DIV(FPVar1, FPVar2))) >> ((FXP_SHIFT(FPVar1) - FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPDestination))))) \
	))
#endif

/************************************************/
/* Performs the test: FPVar1 == FPVar2          */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_EQUAL
#define FXP_IS_EQUAL(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) == (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) == ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) == ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) == (FPVar2))) \
	)
#endif

/************************************************/
/* Performs the test: FPVar1 != FPVar2          */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_NOT_EQUAL
#define FXP_IS_NOT_EQUAL(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) != (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) != ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) != ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) != (FPVar2))) \
	)
#endif

/************************************************/
/* Performs the test: FPVar1 > FPVar2           */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_GR_THAN
#define FXP_IS_GR_THAN(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) > (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) > ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) > ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) > (FPVar2))) \
	)
#endif

/************************************************/
/* Performs the test: FPVar1 >= FPVar2          */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_GR_THAN_OR_EQ
#define FXP_IS_GR_THAN_OR_EQ(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) >= (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) >= ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) >= ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) >= (FPVar2))) \
	)
#endif

/************************************************/
/* Performs the test: FPVar1 < FPVar2           */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_LOW_THAN
#define FXP_IS_LOW_THAN(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) < (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) < ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) < ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) < (FPVar2))) \
	)
#endif

/************************************************/
/* Performs the test: FPVar1 <= FPVar2          */
/************************************************/
/* FPVar1 - First Fixed-Point variable's data   */
/* FPVar2 - Second Fixed-Point variable's data  */
/************************************************/
#ifndef FXP_IS_LOW_THAN_OR_EQ
#define FXP_IS_LOW_THAN_OR_EQ(FPVar1, FPVar2) ( \
	((FXP_WORD_SIZE(FPVar1)) > (FXP_WORD_SIZE(FPVar2))) ? \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			((FPVar1) <= (((FXP_TYPE(FPVar1))(FPVar2)) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			(((FPVar1) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) <= ((FXP_TYPE(FPVar1))(FPVar2)))) \
			: \
		((FXP_SHIFT(FPVar1) > FXP_SHIFT(FPVar2)) ? \
			(((FXP_TYPE(FPVar2))(FPVar1)) <= ((FPVar2) << (((FXP_SHIFT(FPVar1)) - (FXP_SHIFT(FPVar2)))))) : \
			((((FXP_TYPE(FPVar2))(FPVar1)) << (((FXP_SHIFT(FPVar2)) - (FXP_SHIFT(FPVar1))))) <= (FPVar2))) \
	)
#endif

/************************************** Fixed-Point EXTENDED Macros **************************************/
/* This set of macros should be used when specifying ALL the fixed-point information that is required to */
/* perform the operation/test. May be used with fixed-point attributes defined under a                   */
/* class/structure, or when user would like to specify all fixed-point variable information.             */
/*********************************************************************************************************/

/********************************************************/
/* Performs the operation: FPDestNm = FPSrcNm           */
/********************************************************/
/* FPDestNm  - Destination Fixed-Point variable's data  */
/* FPDestTyp - Destination Fixed-Point variable's type  */
/* FPDestSh  - Destination Fixed-Point variable's shift */
/* FPSrcNm   - Source Fixed-Point variable's data       */
/* FPSrcSh   - Source Fixed-Point variable's shift      */
/********************************************************/
#ifndef FXP_ASSIGN_EXT
#define FXP_ASSIGN_EXT(FPDestNm, FPDestTyp, FPDestSh, FPSrcNm, FPSrcSh) \
	((FPDestNm) = \
	(((FPDestSh) > (FPSrcSh)) ? \
	((((FPDestTyp)(FPSrcNm)) << ((FPDestSh) - (FPSrcSh)))) : \
	((((FPDestTyp)(FPSrcNm)) >> ((FPSrcSh) - (FPDestSh)))) \
	))
#endif

/*******************************************************/
/* Performs the operation: FPVar1Nm + FPVar2Nm         */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_SUM_EXT
#define FXP_SUM_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) + (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) + ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) + ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) + (FPVar2Nm))) \
	)
#endif

/**********************************************************/
/* Performs the operation: FPDestNm = FPVar1Nm + FPVar2Nm */
/**********************************************************/
/* FPDestNm  - Destination Fixed-Point variable's data    */
/* FPDestTyp - Destination Fixed-Point variable's type    */
/* FPDestSh  - Destination Fixed-Point variable's shift   */
/* FPVar1Nm  - First Fixed-Point variable's data          */
/* FPVar1Typ - First Fixed-Point variable's type          */
/* FPVar1Sh  - First Fixed-Point variable's shift         */
/* FPVar1Wz  - First Fixed-Point variable's word-size     */
/* FPVar2Nm  - Second Fixed-Point variable's data         */
/* FPVar2Typ - Second Fixed-Point variable's type         */
/* FPVar2Sh  - Second Fixed-Point variable's shift        */
/* FPVar2Wz  - Second Fixed-Point variable's word-size    */
/**********************************************************/
#ifndef FXP_ASSIGN_SUM_EXT
#define FXP_ASSIGN_SUM_EXT(FPDestNm, FPDestTyp, FPDestSh, FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) \
	((FPDestNm) = \
	(((FPDestSh) > ((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh))) ? \
	((((FPDestTyp)(FXP_SUM_EXT((FPVar1Nm), FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz))) << ((FPDestSh) - ((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh))))) : \
	((((FPDestTyp)(FXP_SUM_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz))) >> (((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh)) - (FPDestSh)))) \
	))
#endif

/*******************************************************/
/* Performs the operation: FPVar1Nm - FPVar2Nm         */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_SUB_EXT
#define FXP_SUB_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) - (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) - ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) - ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) - (FPVar2Nm))) \
	)
#endif

/**********************************************************/
/* Performs the operation: FPDestNm = FPVar1Nm - FPVar2Nm */
/**********************************************************/
/* FPDestNm  - Destination Fixed-Point variable's data    */
/* FPDestTyp - Destination Fixed-Point variable's type    */
/* FPDestSh  - Destination Fixed-Point variable's shift   */
/* FPVar1Nm  - First Fixed-Point variable's data          */
/* FPVar1Typ - First Fixed-Point variable's type          */
/* FPVar1Sh  - First Fixed-Point variable's shift         */
/* FPVar1Wz  - First Fixed-Point variable's word-size     */
/* FPVar2Nm  - Second Fixed-Point variable's data         */
/* FPVar2Typ - Second Fixed-Point variable's type         */
/* FPVar2Sh  - Second Fixed-Point variable's shift        */
/* FPVar2Wz  - Second Fixed-Point variable's word-size    */
/**********************************************************/
#ifndef FXP_ASSIGN_SUB_EXT
#define FXP_ASSIGN_SUB_EXT(FPDestNm, FPDestTyp, FPDestSh, FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) \
	((FPDestNm) = \
	(((FPDestSh) > ((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh))) ? \
	((((FPDestTyp)(FXP_SUB_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz))) << ((FPDestSh) - ((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh))))) : \
	((((FPDestTyp)(FXP_SUB_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz))) >> (((FPVar1Sh > FPVar2Sh) ? (FPVar1Sh) : (FPVar2Sh)) - (FPDestSh)))) \
	))
#endif

/**********************************************************/
/* Performs the operation: FPDestNm = FPVar1Nm * FPVar2Nm */
/**********************************************************/
/* FPDestNm  - Destination Fixed-Point variable's data    */
/* FPDestTyp - Destination Fixed-Point variable's type    */
/* FPDestSh  - Destination Fixed-Point variable's shift   */
/* FPVar1Nm  - First Fixed-Point variable's data          */
/* FPVar1Sh  - First Fixed-Point variable's shift         */
/* FPVar2Nm  - Second Fixed-Point variable's data         */
/* FPVar2Sh  - Second Fixed-Point variable's shift        */
/**********************************************************/
#ifndef FXP_ASSIGN_MUL_EXT
#define FXP_ASSIGN_MUL_EXT(FPDestNm, FPDestTyp, FPDestSh, FPVar1Nm, FPVar1Sh, FPVar2Nm, FPVar2Sh) \
	((FPDestNm) = \
	(((FPDestSh) > (FPVar1Sh + FPVar2Sh)) ? \
	((((FPDestTyp)(FXP_MUL(FPVar1Nm, FPVar2Nm))) << ((FPDestSh) - (FPVar1Sh + FPVar2Sh)))) : \
	((((FPDestTyp)(FXP_MUL(FPVar1Nm, FPVar2Nm))) >> ((FPVar1Sh + FPVar2Sh) - (FPDestSh)))) \
	))
#endif

/**********************************************************/
/* Performs the operation: FPDestNm = FPVar1Nm / FPVar2Nm */
/**********************************************************/
/* FPDestNm  - Destination Fixed-Point variable's data    */
/* FPDestTyp - Destination Fixed-Point variable's type    */
/* FPDestSh  - Destination Fixed-Point variable's shift   */
/* FPVar1Nm  - First Fixed-Point variable's data          */
/* FPVar1Sh  - First Fixed-Point variable's shift         */
/* FPVar2Nm  - Second Fixed-Point variable's data         */
/* FPVar2Sh  - Second Fixed-Point variable's shift        */
/**********************************************************/
#ifndef FXP_ASSIGN_DIV_EXT
#define FXP_ASSIGN_DIV_EXT(FPDestNm, FPDestTyp, FPDestSh, FPVar1Nm, FPVar1Sh, FPVar2Nm, FPVar2Sh) \
	((FPDestNm) = \
	(((FPDestSh) > (FPVar1Sh - FPVar2Sh)) ? \
	((((FPDestTyp)(FXP_DIV(FPVar1Nm, FPVar2Nm))) << ((FPDestSh) - (FPVar1Sh - FPVar2Sh)))) : \
	((((FPDestTyp)(FXP_DIV(FPVar1Nm, FPVar2Nm))) >> ((FPVar1Sh - FPVar2Sh) - (FPDestSh)))) \
	))
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm == FPVar2Nm             */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_EQUAL_EXT
#define FXP_IS_EQUAL_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) == (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) == ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) == ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) == (FPVar2Nm))) \
	)
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm != FPVar2Nm             */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_NOT_EQUAL_EXT
#define FXP_IS_NOT_EQUAL_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) != (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) != ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) != ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) != (FPVar2Nm))) \
	)
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm > FPVar2Nm              */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_GR_THAN_EXT
#define FXP_IS_GR_THAN_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) > (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) > ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) > ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) > (FPVar2Nm))) \
	)
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm >= FPVar2Nm             */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_GR_THAN_OR_EQ_EXT
#define FXP_IS_GR_THAN_OR_EQ_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) >= (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) >= ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) >= ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) >= (FPVar2Nm))) \
	)
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm < FPVar2Nm              */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_LOW_THAN_EXT
#define FXP_IS_LOW_THAN_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) < (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) < ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) < ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) < (FPVar2Nm))) \
	)
#endif

/*******************************************************/
/* Performs the test: FPVar1Nm <= FPVar2Nm             */
/*******************************************************/
/* FPVar1Nm  - First Fixed-Point variable's data       */
/* FPVar1Typ - First Fixed-Point variable's type       */
/* FPVar1Sh  - First Fixed-Point variable's shift      */
/* FPVar1Wz  - First Fixed-Point variable's word-size  */
/* FPVar2Nm  - Second Fixed-Point variable's data      */
/* FPVar2Typ - Second Fixed-Point variable's type      */
/* FPVar2Sh  - Second Fixed-Point variable's shift     */
/* FPVar2Wz  - Second Fixed-Point variable's word-size */
/*******************************************************/
#ifndef FXP_IS_LOW_THAN_OR_EQ_EXT
#define FXP_IS_LOW_THAN_OR_EQ_EXT(FPVar1Nm, FPVar1Typ, FPVar1Sh, FPVar1Wz, FPVar2Nm, FPVar2Typ, FPVar2Sh, FPVar2Wz) ( \
	((FPVar1Wz) > (FPVar2Wz)) ? \
		((FPVar1Sh > FPVar2Sh) ? \
			((FPVar1Nm) <= (((FPVar1Typ)(FPVar2Nm)) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			(((FPVar1Nm) << (((FPVar2Sh) - (FPVar1Sh)))) <= ((FPVar1Typ)(FPVar2Nm)))) \
			: \
		((FPVar1Sh > FPVar2Sh) ? \
			(((FPVar2Typ)(FPVar1Nm)) <= ((FPVar2Nm) << (((FPVar1Sh) - (FPVar2Sh))))) : \
			((((FPVar2Typ)(FPVar1Nm)) << (((FPVar2Sh) - (FPVar1Sh)))) <= (FPVar2Nm))) \
	)
#endif

/************************************** Fixed-Point CLASS Macros *******************************************/
/* This set of macros should be used when specifying the fixed-point data, name and class-name required to */
/* perform the operation/test. May be used with fixed-point attributes defined under a                     */
/* class/structure, in which case the user specifies the name of the class in which the fixed-point        */
/* attribute is defined.                                                                                   */
/***********************************************************************************************************/

/********************************************************/
/* Performs the operation: FPDst = FPSrcSh              */
/********************************************************/
/* FPDst   - Destination Fixed-Point attribute's data   */
/* FPDstnm - Destination Fixed-Point attribute's name   */
/* FPDstc  - Destination Fixed-Point attribute's class  */
/* FPS     - First Fixed-Point attribute's data         */
/* FPSnm   - First Fixed-Point attribute's name         */
/* FPSc    - First Fixed-Point attribute's class        */
/********************************************************/
#ifndef FXP_ASSIGN_CLASS
#define FXP_ASSIGN_CLASS(FPDst, FPDstnm, FPDstc, FPS, FPSnm, FPSc) ((FPDst) = \
	(((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) > (FXP_SHIFT_CLASS(FPSnm, FPSc))) ? \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FPS)) << ((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) - (FXP_SHIFT_CLASS(FPSnm, FPSc))))) : \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FPS)) >> ((FXP_SHIFT_CLASS(FPSnm, FPSc)) - (FXP_SHIFT_CLASS(FPDstnm, FPDstc))))) \
	))
#endif

/*************************************************/
/* Performs the operation: FPV1 + FPV2           */
/*************************************************/
/* FPV1   - First Fixed-Point attribute's data   */
/* FPV1nm - First Fixed-Point attribute's name   */
/* FPV1c  - First Fixed-Point attribute's class  */
/* FPV2   - Second Fixed-Point attribute's data  */
/* FPV2nm - Second Fixed-Point attribute's name  */
/* FPV2c  - Second Fixed-Point attribute's class */
/*************************************************/
#ifndef FXP_SUM_CLASS
#define FXP_SUM_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) + (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) + ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) + ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) + (FPV2))) \
	)
#endif

/********************************************************/
/* Performs the operation: FPDst = FPV1 + FPV2          */
/********************************************************/
/* FPDst   - Destination Fixed-Point attribute's data   */
/* FPDstnm - Destination Fixed-Point attribute's name   */
/* FPDstc  - Destination Fixed-Point attribute's class  */
/* FPV1    - First Fixed-Point attribute's data         */
/* FPV1nm  - First Fixed-Point attribute's name         */
/* FPV1c   - First Fixed-Point attribute's class        */
/* FPV2    - Second Fixed-Point attribute's data        */
/* FPV2nm  - Second Fixed-Point attribute's name        */
/* FPV2c   - Second Fixed-Point attribute's class       */
/********************************************************/
#ifndef FXP_ASSIGN_SUM_CLASS
#define FXP_ASSIGN_SUM_CLASS(FPDst, FPDstnm, FPDstc, FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) \
	((FPDst) = \
	(((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) > ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))) ? \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_SUM_CLASS((FPV1), FPV1nm, FPV1c, (FPV2), FPV2nm, FPV2c))) << ((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) - ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_SUM_CLASS((FPV1), FPV1nm, FPV1c, (FPV2), FPV2nm, FPV2c))) >> (((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c))) - (FXP_SHIFT_CLASS(FPDstnm, FPDstc))))) \
	))
#endif


/*************************************************/
/* Performs the operation: FPV1 - FPV2           */
/*************************************************/
/* FPV1   - First Fixed-Point attribute's data   */
/* FPV1nm - First Fixed-Point attribute's name   */
/* FPV1c  - First Fixed-Point attribute's class  */
/* FPV2   - Second Fixed-Point attribute's data  */
/* FPV2nm - Second Fixed-Point attribute's name  */
/* FPV2c  - Second Fixed-Point attribute's class */
/*************************************************/
#ifndef FXP_SUB_CLASS
#define FXP_SUB_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) - (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) - ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) - ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) - (FPV2))) \
	)
#endif

/********************************************************/
/* Performs the operation: FPDst = FPV1 - FPV2          */
/********************************************************/
/* FPDst   - Destination Fixed-Point attribute's data   */
/* FPDstnm - Destination Fixed-Point attribute's name   */
/* FPDstc  - Destination Fixed-Point attribute's class  */
/* FPV1    - First Fixed-Point attribute's data         */
/* FPV1nm  - First Fixed-Point attribute's name         */
/* FPV1c   - First Fixed-Point attribute's class        */
/* FPV2    - Second Fixed-Point attribute's data        */
/* FPV2nm  - Second Fixed-Point attribute's name        */
/* FPV2c   - Second Fixed-Point attribute's class       */
/********************************************************/
#ifndef FXP_ASSIGN_SUB_CLASS
#define FXP_ASSIGN_SUB_CLASS(FPDst, FPDstnm, FPDstc, FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) \
	((FPDst) = \
	(((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) > ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))) ? \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_SUB_CLASS((FPV1), FPV1nm, FPV1c, (FPV2), FPV2nm, FPV2c))) << ((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) - ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_SUB_CLASS((FPV1), FPV1nm, FPV1c, (FPV2), FPV2nm, FPV2c))) >> (((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? (FXP_SHIFT_CLASS(FPV1nm, FPV1c)) : (FXP_SHIFT_CLASS(FPV2nm, FPV2c))) - (FXP_SHIFT_CLASS(FPDstnm, FPDstc))))) \
	))
#endif

/********************************************************/
/* Performs the operation: FPDst = FPV1 * FPV2          */
/********************************************************/
/* FPDst   - Destination Fixed-Point attribute's data   */
/* FPDstnm - Destination Fixed-Point attribute's name   */
/* FPDstc  - Destination Fixed-Point attribute's class  */
/* FPV1    - First Fixed-Point attribute's data         */
/* FPV1nm  - First Fixed-Point attribute's name         */
/* FPV1c   - First Fixed-Point attribute's class        */
/* FPV2    - Second Fixed-Point attribute's data        */
/* FPV2nm  - Second Fixed-Point attribute's name        */
/* FPV2c   - Second Fixed-Point attribute's class       */
/********************************************************/
#ifndef FXP_ASSIGN_MUL_CLASS
#define FXP_ASSIGN_MUL_CLASS(FPDst, FPDstnm, FPDstc, FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) \
	((FPDst) = \
	(((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) > (FXP_SHIFT_CLASS(FPV1nm, FPV1c) + FXP_SHIFT_CLASS(FPV2nm, FPV2c))) ? \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_MUL(FPV1, FPV2))) << ((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c) + FXP_SHIFT_CLASS(FPV2nm, FPV2c))))) : \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_MUL(FPV1, FPV2))) >> ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) + FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPDstnm, FPDstc))))) \
	))
#endif

/********************************************************/
/* Performs the operation: FPDst = FPV1 / FPV2          */
/********************************************************/
/* FPDst   - Destination Fixed-Point attribute's data   */
/* FPDstnm - Destination Fixed-Point attribute's name   */
/* FPDstc  - Destination Fixed-Point attribute's class  */
/* FPV1    - First Fixed-Point attribute's data         */
/* FPV1nm  - First Fixed-Point attribute's name         */
/* FPV1c   - First Fixed-Point attribute's class        */
/* FPV2    - Second Fixed-Point attribute's data        */
/* FPV2nm  - Second Fixed-Point attribute's name        */
/* FPV2c   - Second Fixed-Point attribute's class       */
/********************************************************/
#ifndef FXP_ASSIGN_DIV_CLASS
#define FXP_ASSIGN_DIV_CLASS(FPDst, FPDstnm, FPDstc, FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) \
	((FPDst) = \
	(((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) > (FXP_SHIFT_CLASS(FPV1nm, FPV1c) - FXP_SHIFT_CLASS(FPV2nm, FPV2c))) ? \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_DIV(FPV1, FPV2))) << ((FXP_SHIFT_CLASS(FPDstnm, FPDstc)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c) - FXP_SHIFT_CLASS(FPV2nm, FPV2c))))) : \
	((((FXP_TYPE_CLASS(FPDstnm, FPDstc))(FXP_DIV(FPV1, FPV2))) >> ((FXP_SHIFT_CLASS(FPV1nm, FPV1c) - FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPDstnm, FPDstc))))) \
	))
#endif

/**************************************************/
/* Performs the test: FPV1 == FPV2                */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_EQUAL_CLASS
#define FXP_IS_EQUAL_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) == (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) == ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) == ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) == (FPV2))) \
	)
#endif

/**************************************************/
/* Performs the test: FPV1 != FPV2                */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_NOT_EQUAL_CLASS
#define FXP_IS_NOT_EQUAL_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) != (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) != ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) != ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) != (FPV2))) \
	)
#endif

/**************************************************/
/* Performs the test: FPV1 > FPV2                 */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_GR_THAN_CLASS
#define FXP_IS_GR_THAN_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) > (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) > ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) > ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) > (FPV2))) \
	)
#endif

/**************************************************/
/* Performs the test: FPV1 >= FPV2                */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_GR_THAN_OR_EQ_CLASS
#define FXP_IS_GR_THAN_OR_EQ_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) >= (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) >= ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) >= ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) >= (FPV2))) \
	)
#endif

/**************************************************/
/* Performs the test: FPV1 < FPV2                 */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_LOW_THAN_CLASS
#define FXP_IS_LOW_THAN_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) < (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) < ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) < ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) < (FPV2))) \
	)
#endif

/**************************************************/
/* Performs the test: FPV1 <= FPV2                */
/**************************************************/
/* FPV1    - First Fixed-Point attribute's data   */
/* FPV1nm  - First Fixed-Point attribute's name   */
/* FPV1c   - First Fixed-Point attribute's class  */
/* FPV2    - Second Fixed-Point attribute's data  */
/* FPV2nm  - Second Fixed-Point attribute's name  */
/* FPV2c   - Second Fixed-Point attribute's class */
/**************************************************/
#ifndef FXP_IS_LOW_THAN_OR_EQ_CLASS
#define FXP_IS_LOW_THAN_OR_EQ_CLASS(FPV1, FPV1nm, FPV1c, FPV2, FPV2nm, FPV2c) ( \
	((FXP_WORD_SIZE_CLASS(FPV1nm, FPV1c)) > (FXP_WORD_SIZE_CLASS(FPV2nm, FPV2c))) ? \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			((FPV1) <= (((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			(((FPV1) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) <= ((FXP_TYPE_CLASS(FPV1nm, FPV1c))(FPV2)))) \
			: \
		((FXP_SHIFT_CLASS(FPV1nm, FPV1c) > FXP_SHIFT_CLASS(FPV2nm, FPV2c)) ? \
			(((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) <= ((FPV2) << (((FXP_SHIFT_CLASS(FPV1nm, FPV1c)) - (FXP_SHIFT_CLASS(FPV2nm, FPV2c)))))) : \
			((((FXP_TYPE_CLASS(FPV2nm, FPV2c))(FPV1)) << (((FXP_SHIFT_CLASS(FPV2nm, FPV2c)) - (FXP_SHIFT_CLASS(FPV1nm, FPV1c))))) <= (FPV2))) \
	)
#endif

#endif

