#ifndef IRiCReactive_H
#define IRiCReactive_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2003, 2011. All Rights Reserved.

*/


/********************************************************************
 ---------------------------------------------------------------
|							  IRiCReactive.h                          |
 --------------------------------------------------------------- 
*********************************************************************/
#include "mxf_cfg.h"
#include "RiCEvent.h"


/*****************************************

  IRiCReactive: Interface for reactive classes.

  Known realizations: RiCReactive, 
  RiCDefaultReactiveInbound,
  RiCDefaultReactiveOutbound
			

*******************************************/


/*
	IRiCReactive_Vtbl (Virtual Table):
	Contains the offset and function pointers
	of the interface.
	This is initialized to const values by the realizing
	classes and these values remain unchanged.
*/
typedef struct IRiCReactive_Vtbl_t {
    size_t IRiCReactive_offset;
    RiCBoolean (*gen)(RhpAddress const me, RiCEvent* pEvent, RiCBoolean fromISR);
}IRiCReactive_Vtbl;

/*
	IRiCReactive: The interface itself.
	Since this is just an interface it contains
	a single pointer to its virtual table (no data
	memebers)
	This might be optimized so we would not need this
	structure at all, however the code is itself 
	would be less readable and we only save a single
	pointer.
*/
typedef struct IRiCReactive_t {
    const IRiCReactive_Vtbl* IRiCReactivevtbl;
}IRiCReactive;

/* Operations */
RiCBoolean IRiCReactive_gen(RhpAddress const me, RiCEvent* pEvent, RiCBoolean fromISR);

/* Constructors and destructors:*/

void IRiCReactive_Init(IRiCReactive* const me, const IRiCReactive_Vtbl* pVtbl);

#endif



