/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|		   RiCDefaultMulticastReactivePort.c                   |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICDEFAULTMULTICASTREACTIVEPORT_SOURCE
#define MXF_RICDEFAULTMULTICASTREACTIVEPORT_SOURCE
#endif

#include "RiCDefaultMulticastReactivePort.h"

#ifdef RIC_USE_DEFAULT_MULTI_CAST



/* Rapid port class */
void RiCMcRPort_Init(RiCMcRPort* const me)
{
	if (me != NULL)
	{
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				  */
		/* Cast to void* pointer is required to implement general purpose data structures */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		RiCMcRInbound_Init(&(me->inbound), (void*) me);
		RiCMcROutbound_Init(&(me->outbound));
	}
}

void RiCMcRPort_Cleanup(RiCMcRPort* const me)
{
	if (me != NULL)
	{
		RiCMcRInbound_Cleanup(&(me->inbound));
		RiCMcROutbound_Cleanup(&(me->outbound));
	}
}


/* Inbound */
void RiCMcRInbound_Init(RiCMcRInbound* const me, void* const portAddress)
{
	static RiCMcRInbound_Vtbl RiCMcRInboundVtbl = {
		0, RiCMcRInbound_gen
	};

	if (me != NULL)
	{
		me->m_vtbl = &(RiCMcRInboundVtbl);
		me->m_portAddress = portAddress;
		(void) RiCCollection_Init(&(me->itsIRiCReactive), 0U /* mxf Collection size is static */);
	}
}

void RiCMcRInbound_Cleanup(RiCMcRInbound* const me)
{
	if (me != NULL)
	{
		me->m_vtbl = NULL;
		me->m_portAddress = NULL;
#ifdef MICROC_OXF_CLEANUP
		(void) RiCCollection_Cleanup(&(me->itsIRiCReactive));
#endif
	}
}

RiCBoolean RiCMcRInbound_gen(void* const me, RiCEvent * event, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL) 
	{
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		RiCMcRInbound* _me = (RiCMcRInbound*) me;
		if (RiCCollection_getCount(&_me->itsIRiCReactive) > 0)
		{
			/* "stamping the event with the port address" */
			if (event != NULL)
			{
				event->port = _me->m_portAddress;
			}
			/* relaying the event */
			res = IRiCReactive_gen(RiCCollection_getAt(&_me->itsIRiCReactive, 0U), event, fromISR);
		}
	}

	return res;
}

RiCCollection* RiCMcRInbound_getItsIReactive(RiCMcRInbound* const me)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
	/* Cast from/to void* pointer is required to implement general purpose data collections */
	/*LDRA_INSPECTED 94 S */
	RiCCollection * iter = NULL;
	if (me != NULL)
	{
		RhpPositive position;
		iter = (RiCCollection *) &(me->itsIRiCReactive);
		RiCCollection_first(iter, &position);
	}
/*LDRA_INSPECTED 71 S */
	return iter;		
/*LDRA_INSPECTED 5 D */
}

void RiCMcRInbound_setItsIReactive(RiCMcRInbound* const me, void* const p_reactive)
{
	if (me != NULL) 
	{
		RiCCollection_setAt(&me->itsIRiCReactive , p_reactive, 0U);
	}
}

void RiCMcRInbound_addItsIReactive(RiCMcRInbound* const me, void* const p_reactive)
{
	if (me != NULL) 
	{
		RiCCollection_add(&me->itsIRiCReactive , p_reactive);
	}
}

/* Outbound */
void RiCMcROutbound_Init(RiCMcROutbound* const me)
{
	static RiCMcROutbound_Vtbl RiCMcROutboundVtbl = {
		0, RiCMcROutbound_gen
	};

	if (me != NULL)
	{
		me->m_vtbl = &(RiCMcROutboundVtbl);
		(void) RiCCollection_Init(&(me->itsIRiCReactive), 0U /* mxf Collection size is static */);
	}
}

void RiCMcROutbound_Cleanup(RiCMcROutbound* const me)
{

	if (me != NULL)
	{
		me->m_vtbl = NULL;
#ifdef MICROC_OXF_CLEANUP
		(void) RiCCollection_Cleanup(&(me->itsIRiCReactive));
#endif
	}
}

RiCBoolean RiCMcROutbound_gen(void* const me, RiCEvent * event, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL) 
	{
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		RiCMcROutbound* _me = (RiCMcROutbound*) me;
		if (RiCCollection_getCount(&_me->itsIRiCReactive) > 0)
		{
			res = IRiCReactive_gen(RiCCollection_getAt(&_me->itsIRiCReactive, 0U), event, fromISR);
		}
	}

	return res;
}

RiCCollection* RiCMcROutbound_getItsIReactive(RiCMcROutbound* const me)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
	/* Cast from/to void* pointer is required to implement general purpose data collections */
	/*LDRA_INSPECTED 94 S */
	RiCCollection * iter = NULL;
	if (me != NULL)
	{
		iter = (RiCCollection *) &(me->itsIRiCReactive);
		RhpPositive position;
		RiCCollection_first(iter, &position);
	}
	/*LDRA_INSPECTED 71 S */
	return iter;	
/*LDRA_INSPECTED 5 D */
}

void RiCMcROutbound_setItsIReactive(RiCMcROutbound* const me, void* const p_reactive)
{
	if (me != NULL) 
	{
		RiCCollection_setAt(&me->itsIRiCReactive , p_reactive, 0U);
	}
}

void RiCMcROutbound_addItsIReactive(RiCMcROutbound* const me, void* const p_reactive)
{
	if (me != NULL) 
	{
		RiCCollection_add(&me->itsIRiCReactive , p_reactive);
	}
}

#endif
