/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2000, 2011. All Rights Reserved.

*/

/********************************************************************
 ---------------------------------------------------------------
|			     	RiCDefaultReactivePort.c                    |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICDEFAULTREACTIVEPORT_SOURCE
#define MXF_RICDEFAULTREACTIVEPORT_SOURCE
#endif

#include "mxf_cfg.h"
#ifdef RIC_USE_RAPID_PORTS

#include "RiCDefaultReactivePort.h"

static RiCDefaultReactiveInbound_Vtbl RiCDefaultReactiveInboundVtbl = {
	0, RiCDefaultReactiveInbound_gen
};

static RiCDefaultReactiveOutbound_Vtbl RiCDefaultReactiveOutboundVtbl = {
	0, RiCDefaultReactiveOutbound_gen
};



/* Rapid port class */
void RiCDefaultReactivePort_Init(RiCDefaultReactivePort* const me)
{
	if (me != NULL)
	{
		RiCDefaultReactiveInbound_Init(&(me->inbound), me);
		RiCDefaultReactiveOutbound_Init(&(me->outbound));
	}
}



/* Inbound */

void RiCDefaultReactiveInbound_Init(RiCDefaultReactiveInbound* const me, RhpAddress pPortAddress)
{
	if (me != NULL)
	{
		me->vtbl = &(RiCDefaultReactiveInboundVtbl);
		me->portAddress = pPortAddress;
	}

}

RiCBoolean RiCDefaultReactiveInbound_gen(RhpAddress const me, RiCEvent * pEvent, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;

	if (me != NULL) 
	{
		RiCDefaultReactiveInbound* _me = (RiCDefaultReactiveInbound*)(me);
		if (_me->itsIRiCReactive != NULL)
		{
			/* "stamping the event with the port address" */
			if (pEvent != NULL)
			{
				pEvent->port = _me->portAddress;
			}
			/* relaying the event */
			res = IRiCReactive_gen(_me->itsIRiCReactive, pEvent, fromISR);
		}
	}

	return res;
}

IRiCReactive* RiCDefaultReactiveInbound_getItsIRiCReactive(RiCDefaultReactiveInbound* const me)
{
	IRiCReactive* res = NULL;

	if (me != NULL)
	{
		res = me->itsIRiCReactive;
	}
	return res;
		
}

void RiCDefaultReactiveInbound_setItsIRiCReactive(RiCDefaultReactiveInbound* const me, RhpAddress pReactive)
{
	if (me != NULL) 
	{
		me->itsIRiCReactive = (IRiCReactive*) pReactive;
	}
}

/* Outbound */

void RiCDefaultReactiveOutbound_Init(RiCDefaultReactiveOutbound* const me)
{

	if (me != NULL)
	{
		me->vtbl = &(RiCDefaultReactiveOutboundVtbl);
	}
}

RiCBoolean RiCDefaultReactiveOutbound_gen(RhpAddress const me, RiCEvent * pEvent, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;

	if (me != NULL) 
	{
		RiCDefaultReactiveOutbound* _me = (RiCDefaultReactiveOutbound*)(me);
		if (_me->itsIRiCReactive != NULL)
		{
			res = IRiCReactive_gen(_me->itsIRiCReactive, pEvent, fromISR);
		}
	}

	return res;
}

IRiCReactive* RiCDefaultReactiveOutbound_getItsIRiCReactive(RiCDefaultReactiveOutbound* const me)
{
	IRiCReactive* res = NULL;

	if (me != NULL)
	{
		res = me->itsIRiCReactive;
	}
	return res;
		
}

void RiCDefaultReactiveOutbound_setItsIRiCReactive(RiCDefaultReactiveOutbound* const me, RhpAddress pReactive)
{
	if (me != NULL) 
	{
		me->itsIRiCReactive = (IRiCReactive*) pReactive;
	}
}

#endif
