/*
	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/*********************************************************************
 ---------------------------------------------------------------
|							  RiCEvent.c                          |
  --------------------------------------------------------------- 
**********************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICEVENT_SOURCE
#define MXF_RICEVENT_SOURCE
#endif

#include <stdlib.h>
#include "RiCOSWrap.h"
#include "RiCReactive.h"
#include "RiCEvent.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "aominst.h" 
#include "aomeque.h" 
#include "aomstep.h" 
#include "aomthrd.h" 
#include "aommsg.h"
#include "aomevent.h"
#include "RiCString.h"
#else
#include "aom/aominst.h" 
#include "aom/aomeque.h" 
#include "aom/aomstep.h" 
#include "aom/aomthrd.h" 
#include "aom/aommsg.h"
#include "aom/aomevent.h"
#include "Utils/RiCString.h"
#endif
#endif 

/*****************************************************************************/
/*****************************************************************************/
/******								RiCEvent							******/
/*****************************************************************************/
/*****************************************************************************/

#ifdef _OMINSTRUMENT
const RiCEventId RiCTimeoutDelayId = -1;
#endif

/***************************************************/
/****          lock and free mutex		        ****/
/***************************************************/

void RiCEvent_lockEventsPoolMutex(void)
{
	RiCOSEventsPoolMutex_lock();
}

void RiCEvent_freeEventsPoolMutex(void)
{
	RiCOSEventsPoolMutex_free();
}

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

RiCBoolean RiCEvent_init( RiCEvent * const me, const RiCEventId eventId )
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        me->lId = eventId;
        me->eventDestroyOp = NULL;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
        me->frameworkEvent = RiCFALSE;
#endif
#ifdef RIC_USE_RAPID_PORTS
		me->port = NULL;
#endif
        
		result = RiCTRUE;
    }

    return result;
}

void RiCEvent_cleanup(RiCEvent * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if(me != NULL){
		me->lId	= RiCNull_id; 
		me->destination = NULL;
	}
#endif
}

void RiCEvent_destroy(RiCEvent * const me)
{
#ifdef MICROC_OXF_CLEANUP
	RiCEvent_cleanup(me);
#endif
}


/***************************************************/
/****				Event methods	            ****/
/***************************************************/

void RiCEvent_setDestination(RiCEvent * const me, struct RiCReactive_t * const dest)
{
	if (me != NULL){
		me->destination = dest;
	}
}

struct RiCReactive_t * RiCEvent_getDestination(const RiCEvent * const me)
{
	struct RiCReactive_t *retval = NULL;

	if (me != NULL){
		retval = me->destination;
	}
	/*LDRA_INSPECTED 71 S : Pointer is assigned to memory from wider scope */
	return retval;
}

void RiCEvent_setlId(RiCEvent * const me, RiCEventId evid)
{
	if (me != NULL){
		me->lId = evid;
	}
}

RiCEventId RiCEvent_getlId(const RiCEvent * const me)
{
	RiCEventId retval = (RiCEventId) 0;

	if (me != NULL){
		retval = me->lId;
	}
	return retval;
}

/*******************/
/*******************/
RiCBoolean RiCEvent_isTimeout(const RiCEvent * const me)
{ 
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		if (me->lId == RiCTimeout_id)	{ res = RiCTRUE;}
	}
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCEvent_isCancelledTimeout(const RiCEvent * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL) {
		if (me->lId == RiCCancelledEvent_id) { res = RiCTRUE;}
	}
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCEvent_isRealEvent(const RiCEvent * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL) {
		if (me->lId >= RiCTimeout_id) {res = RiCTRUE;}
	}
	return res;
}

RiCEvent* RiCEvent_getMemory(struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCEvent* res = NULL;
	RhpPositive ind = 0U;
	RhpBoolean flag = RiCFALSE;

	if((mxfGlblInst != NULL) && (mxfGlblInst->eventsPool != NULL)){
		for(; (ind<EVENTS_POOL_SIZE) && (flag == RiCFALSE); ind++){
			if(mxfGlblInst->eventsPool[ind].isAllocated == RiCFALSE){
				mxfGlblInst->eventsPool[ind].isAllocated = RiCTRUE;
				mxfGlblInst->eventsPool[ind].userData	= NULL;
				flag = RiCTRUE;
				res = &(mxfGlblInst->eventsPool[ind]);
			}
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif

	/*LDRA_INSPECTED 71 S : Pointer is assigned to memory from wider scope */
	return res;
}

void RiCEvent_returnMemory(RiCEvent * const me)
{
	if (me != NULL){
		me->isAllocated = RiCFALSE;
	}
}

void RiCEvent_setData(RiCEvent * const me, void * const data)
{
	if(me != NULL){
		me->userData = data;
	}
}


RiCBoolean RiCEvent_isTypeOf(const RiCEvent * const me, RiCEventId evid)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
		if(me->lId == evid){
			retval = RiCTRUE;
		}
	}

	return retval;
}

/*******************/
/*******************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCBoolean RiCEvent_isFrameworkEvent(const RiCEvent * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL) {
		res = me->frameworkEvent;
	}

	return res;
}

/*******************/
/*******************/
void RiCEvent_setFrameworkEvent(RiCEvent * const me, RiCBoolean isFrameworkEvent)
{
	if(me != NULL) {
		me->frameworkEvent = isFrameworkEvent;
	}
}
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

/*****************************************************************************/
/******								RiCTime								******/
/*****************************************************************************/

typedef struct RiCTime_t {
	timeUnit m_timeT;	/* In milli second */
} RiCTime;

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
static void RiCTime_init(RiCTime * const me, timeUnit t)
{
	if (me != NULL) {
		me->m_timeT = t;
	}
}

static void RiCTime_writeTime(const RiCTime * const me, RiCString * s)
{
	if(me != NULL) {
		RhpCharacter c[30];
		sprintf(c,"tm(%lu)",me->m_timeT);
		(void) RiCString_add(s, (RhpString)c);
	}
}
#endif

/*****************************************************************************/
/*****************************************************************************/
/******								RiCTimeout							******/
/*****************************************************************************/
/*****************************************************************************/

/** Returns a free memory section of a Timeout from the pool **/
static RiCTimeout* RiCTimeout_getMemory(struct RiCMxfGlobals_t *mxfGlblInst);

/** Return the Timeout's memory to the pool **/
static void RiCTimeout_returnMemory(RiCEvent * const me);

/** return the timeout data from an event **/
static RiCTimeout* RiCTimeout_getData(const RiCEvent * const me);

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCBoolean RiCTimeout_init(RiCEvent * const me, RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, RhpString state, struct RiCMxfGlobals_t * const mxfGlblInst)
#else
RiCBoolean RiCTimeout_init_low(RiCEvent * const me, RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t * const mxfGlblInst)
#endif
{
	if((me != NULL) && (mxfGlblInst != NULL)){
		RiCTimeout* tm = RiCTimeout_getData(me);
		RiCTimerManager sysTimer = mxfGlblInst->systemTimer;
		(void)RiCEvent_init(me, RiCTimeout_id);
		RiCEvent_setDestination(me, dest);
		me->eventDestroyOp = &RiCTimeout_Destroy;
		
		if(tm != NULL){
			tm->timeoutId = id;
			tm->dueTime = sysTimer.m_Time + delayTime;
		}
		
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		if(tm != NULL){
			tm->state = state;
			tm->delayTime = delayTime;

			/* create and register aom event */
			(void) ARCE_create(me, me, aomtimeoutEventClass);
		}
		
		{
			ARCEvent * aomE = ARCUserEventMap_Find(me);
			if (aomE) {
				ARCInstance * aomInstance = ARCI_reactiveObject2Instance((struct RiCReactive_t *)dest);
				ARCE_setSender(aomE,ARCAI(aomInstance)); /*lint !e740 () */
			}
		}
#endif
	}
	/*lint -save -e774 () */
	/*lint -restore */
	return RiCFALSE;
}
/*******************/
/*******************/
/* Do not inline this function, as it makes the generated application bigger */
void RiCTimeout_cleanup(RiCEvent * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL){
		RiCEvent_cleanup(me);
		{
			RiCTimeout * tm = RiCTimeout_getData(me);
			if(tm != NULL){
				tm->timeoutId = 0;
				tm->dueTime = 0;
				tm->nextTm = NULL;
				
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				{
					/* ARCUserEventMap_Remove(me); */
					ARCEvent * aomE = ARCUserEventMap_Find(me);
					if( aomE != NULL){ /* already destroyed */
						ARCE_destroy(aomE);
					}
				}
#endif
			}
		}
	}
#endif
}
/*******************/
/*******************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCEvent * RiCTimeout_create(RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t *mxfGlblInst, RhpString state)
#else
RiCEvent * RiCTimeout_create_low(RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t *mxfGlblInst)
#endif
{
	RiCEvent   *newTm  = NULL;
	RiCTimeout *tmData = NULL;
    RiCEvent_lockEventsPoolMutex();
	newTm = RiCEvent_getMemory(mxfGlblInst);
	tmData = RiCTimeout_getMemory(mxfGlblInst);
    RiCEvent_freeEventsPoolMutex();

	if((newTm != NULL) && (tmData != NULL) && (mxfGlblInst != NULL)){
		/* 'Type casting to or from pointers shall not be used.' */
		/* The following cast to void* is justified */
		/* and is for Rhapsody auto-generated code use only. */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		RiCEvent_setData(newTm, (void *) tmData);
		(void)RiCTimeout_init(newTm, id, dest, delayTime, state, mxfGlblInst);
		RiCTimeout_addToList(newTm, &(mxfGlblInst->systemTimer.timeouts));
	}
	/*LDRA_INSPECTED 71 S : Pointer is assigned to memory from wider scope */
	return newTm;
}


/*******************/
/*******************/
void RiCTimeout_Destroy(RiCEvent * const me)
{
	RiCEvent_returnMemory(me);
	RiCTimeout_returnMemory(me);
}

/***************************************************/
/****			RiCTimeout methods				****/
/***************************************************/

/***************************************************/
/****          Timeouts Pool Handling           ****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
static RiCTimeout* RiCTimeout_getMemory(struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCTimeout* res = NULL;
	RhpPositive ind = 0U;
	RhpBoolean flag = RiCFALSE;

	if((mxfGlblInst != NULL) && (mxfGlblInst->timeoutsPool != NULL)){
		for(; (ind<TIMEOUTS_POOL_SIZE) && (flag == RiCFALSE); ind++){
			if(mxfGlblInst->timeoutsPool[ind].isAllocated == RiCFALSE){
				mxfGlblInst->timeoutsPool[ind].isAllocated = RiCTRUE;
				flag = RiCTRUE;
				res = &(mxfGlblInst->timeoutsPool[ind]);
			}
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != NULL);
#endif

	/*LDRA_INSPECTED 71 S : Pointer is assigned to memory from wider scope */
	return res;
}

static RiCTimeout* RiCTimeout_getData(const RiCEvent * const me)
{
	/* 'Type casting to or from pointers shall not be used.' */
	/* The following cast from void* is justified */
	/* and is for Rhapsody auto-generated code use only. */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCTimeout* retval = NULL;
	if (me != NULL)
	{
		retval = (RiCTimeout*) me->userData;
	}
	return retval;
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
static void RiCTimeout_returnMemory(RiCEvent * const me)
{
	RiCTimeout * tm = RiCTimeout_getData(me);
	if (tm != NULL){
		tm->isAllocated = RiCFALSE;
	}
}

/*******************/
/*******************/
RiCEventId RiCTimeout_getTimeoutId(const RiCEvent * const me)
{
	RiCEventId res = (RiCEventId) 0;
	if (me != NULL){
		RiCTimeout * tm = RiCTimeout_getData(me);
		if(tm != NULL){
			res = tm->timeoutId;
		}
	}

	return res;
}


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
timeUnit RiCTimeout_getDelay( const RiCTimeout *const me )
{
    timeUnit result = 0;

    if( me != NULL )
    {
        result = me->delayTime;
    }

    return result;
}
#endif

/*******************/
/*******************/
timeUnit RiCTimeout_getDueTime(const RiCEvent * const me)
{
	timeUnit res = 0UL;

	if(me != NULL){
		RiCTimeout * tm = RiCTimeout_getData(me);
		if (tm != NULL){
			res = tm->dueTime;
		}
	}

	return res;
}

/*******************/
/*******************/
void RiCTimeout_setDueTime(RiCEvent * const me, const timeUnit dTime)
{

	if (me != NULL){
		RiCTimeout* tm = RiCTimeout_getData(me);
		if(tm != NULL){
			tm->dueTime = dTime;
		}
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTimeout_setTimeoutId(RiCTimeout * const me, RiCEventId id)
{
	if (me != NULL){
		me->timeoutId = id;
	}
}

/*******************/
/*******************/
#ifdef _OMINSTRUMENT
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCTimeout_isNotDelay( const RiCTimeout *const me )
{
    RiCBoolean result = RiCTRUE;

    if( me != NULL )
    {
        result = ( me->timeoutId != RiCTimeoutDelayId );
    }

    return result;
}

/*******************/
/*******************/
void RiCTimeout_setRelativeDueTime( RiCEvent *const me,
                                    const timeUnit dTime )
{
    if( me != NULL )
    {
		RiCTimeout *tm = RiCTimeout_getData(me);
		if(tm != NULL){
			tm->dueTime = dTime + tm->delayTime;
		}
    }
}
#endif

/*******************/
/*******************/
RhpUnlimitedNatural RiCTimeout_compare(const RiCEvent * const me, const RiCEvent * const other)
{
	RhpUnlimitedNatural res = 0L;
	RiCTimeout* meTm    = RiCTimeout_getData(me);
	RiCTimeout* otherTm = RiCTimeout_getData(other);

	if (meTm == NULL) {
		if (other == NULL){
			res = (RhpUnlimitedNatural) 0;
		}
		else {
			res = (RhpUnlimitedNatural) -1;
		}
	}
	else if (otherTm == NULL){
		res = (RhpUnlimitedNatural) 1;
	}
	else {
		res = (RhpUnlimitedNatural) (meTm->dueTime - otherTm->dueTime);
	}

	return res;
}

/*******************/
/*******************/
RiCBoolean RiCTimeout_equals(const RiCEvent * const me, const RiCEvent * const other)
{
	RiCBoolean res = RiCFALSE;
	RiCTimeout* meTm    = RiCTimeout_getData(me);
	RiCTimeout* otherTm = RiCTimeout_getData(other);

	if (me == NULL) {
		if (other == NULL){
			res = RiCTRUE;
		}
		else{
			res = RiCFALSE;
		}
	}
	else if (other == NULL){
		res = RiCFALSE;
	}
	else if(meTm == NULL){
		if (otherTm == NULL){
			res = RiCTRUE;
		}
		else{
			res = RiCFALSE;
		}
	}
	else if(otherTm == NULL){
		res = RiCFALSE;
	}
	else {
		RiCBoolean matchDest = RiCFALSE;
		RiCBoolean matchId = RiCFALSE;
		if (me->destination == other->destination) { matchDest = RiCTRUE;}
		if ((meTm->timeoutId == otherTm->timeoutId)	|| 
			(meTm->timeoutId == RiCAnyEvent_id)	|| 
			(RiCAnyEvent_id== otherTm->timeoutId)) {
			matchId = RiCTRUE;
		}
		if ((matchDest != RiCFALSE) && (matchId != RiCFALSE)) {
			res = RiCTRUE;
		}
	}

	return res;
}


/***************************************************/
/****		RiCTimeout animation methods		****/
/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

/*******************/
/*******************/
ARCEventClass * RiCTimeout_getClass(RiCEvent * const me)
{
	ARCEventClass * res = NULL;
	ARCEvent * aomE;
	if (me != NULL){
		aomE = ARCUserEventMap_Find(me);
		if (aomE){
			res = ARCE_getClass(aomE);
		}
	}

	return res;
}

/*******************/
/*******************/
/*lint -save -e715 (Symbol 'withParameters' not referenced) */
RiCSData * RiCTimeout_cserialize(RhpAddress me, RiCBoolean withParameters)
{
	RiCSData * res = NULL;
	RiCTimeout * tm = NULL;
	ARCSEvent * aomsevent;
	RiCString msg;
	RiCTime t;

	if (me != NULL) {
		RiCEvent *tmEv = (RiCEvent*) ((ARCEvent*)me)->myOMEvent; /*lint !e740 () */
		if (tmEv != NULL)
		{
			tm = tmEv->userData;
		}
		RiCString_Init(&msg, NULL);
		if (tm != NULL)
		{
			RiCTime_init(&t, tm->delayTime);
			/* Create the timeout's "signature" */
			if (RiCTimeout_isNotDelay(tm)) {		
				RiCTime_writeTime(&t, &msg);
				(void) RiCString_add(&msg, " at ");
				(void) RiCString_add(&msg, tm->state);
			} 
			else {
				(void) RiCString_add(&msg, "Thread Delay ");
				RiCTime_writeTime(&t, &msg);
			}
		}
		
		/* Create an ARCSEvent with this signature */
		aomsevent = ARCSE_create(RiCString_getBuffer(&msg), RiCSnoParameters);
		ARCSE_addEventId(aomsevent, (ARCEvent*)me);
		RiCString_Cleanup(&msg);
		res = ARCSD(aomsevent);
	}

	return res;
}
/*lint -restore */
#endif

/*******************/
/*******************/
RiCEvent *RiCTimeout_getNextTm(const RiCEvent * const me)
{
	RiCEvent *res = NULL;
	if(me != NULL){
		RiCTimeout * tm = RiCTimeout_getData(me);
		if (tm != NULL){
			res = tm->nextTm;
		}
	}
	return res;
}

void RiCTimeout_setNextTm(RiCEvent * const me, RiCEvent * const next)
{
	if(me != NULL){
		RiCTimeout * tm = RiCTimeout_getData(me);
		if (tm != NULL){
			tm->nextTm = next;
		}
	}
}

void RiCTimeout_addToList(RiCEvent * me, RiCEvent **theTmList)
{
	if((theTmList != NULL) && (me != NULL)){
		if(*theTmList == NULL){
			*theTmList = me;
		}
		else{
			RiCBoolean doBreak  = RiCFALSE;
			RiCEvent *curTm = *theTmList, *prevTm = NULL, *theNextTm  = NULL;
			while (doBreak == RiCFALSE)
			{
				if (curTm == NULL)
				{
					doBreak = RiCTRUE;
				}
				else
				{
					theNextTm = RiCTimeout_getNextTm(curTm);
					if(RiCTimeout_getDueTime(me) < RiCTimeout_getDueTime(curTm)){
						doBreak = RiCTRUE;
						RiCTimeout_setNextTm(me, curTm);
						if(curTm == *theTmList){ /* first item */
							*theTmList = me;
						}
						else{ /* middle item */
							RiCTimeout_setNextTm(prevTm, me);
						}
					}
					else if(theNextTm == NULL) { /* last item */
						RiCTimeout_setNextTm(curTm, me);
					}
					else { /* dummy else for MISRA compliance */ 
					}
					prevTm = curTm;
					curTm = theNextTm;
				}
			}
		}
	}
}

/*****************************************************************************/
/*****************************************************************************/
/******						RiCStartBehaviorEvent						******/
/*****************************************************************************/
/*****************************************************************************/

/***************************************************/
/****         initialization and cleanup        ****/
/***************************************************/

#ifdef MICROC_OXF_CLEANUP
static void RiCStartBehaviorEvent_Cleanup(RiCEvent* const me);
static void RiCStartBehaviorEvent_Cleanup(RiCEvent* const me)
{
    RiCEvent_cleanup(me);
}
#endif

static void RiC_Destroy_RiCStartBehaviorEvent(RiCEvent* const me);
static void RiC_Destroy_RiCStartBehaviorEvent(RiCEvent* const me)
{
#ifdef MICROC_OXF_CLEANUP
	if(me!=NULL)
	{
		RiCStartBehaviorEvent_Cleanup(me);
	}
#endif
	RiCEvent_returnMemory(me);
}

RiCEvent * RiCStartBehaviorEvent(struct RiCReactive_t * const dest, struct RiCMxfGlobals_t *mxfGlblInst)
{
	return RiC_Create_RiCStartBehaviorEvent(dest, mxfGlblInst);
}

void RiCStartBehaviorEvent_init(RiCEvent* const me, struct RiCReactive_t * const dest)
{
	if (me != NULL){
		(void)RiCEvent_init(me, RiCStartBehavior_id);
		me->eventDestroyOp = &RiC_Destroy_RiCStartBehaviorEvent;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* create and register aom event */
		(void) ARCE_create(me, me, aomStartBehaviorEventClass);
		
		{
			ARCEvent * aomE = ARCUserEventMap_Find(me);
			if (aomE) {
				ARCInstance * aomInstance = ARCI_reactiveObject2Instance((struct RiCReactive_t *)dest);
				ARCE_setSender(aomE,ARCAI(aomInstance)); /*lint !e740 () */
			}
		}
#endif
	}
}

/*#[ ignore */
RiCEvent * RiC_Create_RiCStartBehaviorEvent(struct RiCReactive_t * const dest, struct RiCMxfGlobals_t *mxfGlblInst)
{
    RiCEvent* me = NULL;
    RiCEvent_lockEventsPoolMutex();
    /*LDRA_INSPECTED 71 S : Pointer is assigned to memory from wider scope */
    me = RiCEvent_getMemory(mxfGlblInst);
    RiCEvent_freeEventsPoolMutex();
    if(me!=NULL)
        {
            RiCStartBehaviorEvent_init(me, dest);
        }
    return me;
}


/***************************************************/
/****  RiCStartBehaviorEvent animation methods	****/
/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
struct ARCEventClass;
struct RiCSData;
/** get the animation class **/
struct ARCEventClass * RiCStartBehaviorEvent_getClass(RiCEvent * const me)
{
	struct ARCEventClass * res = NULL;
	ARCEvent * aomE;
	if (me != NULL) {
		aomE = ARCUserEventMap_Find(me);
		if (aomE){
			res = ARCE_getClass(aomE);
		}
	}

	return res;
}

/** serilize the event **/
/*lint -save -e715 (Symbol 'withParameters' not referenced) */
struct RiCSData * RiCStartBehaviorEvent_cserialize(RhpAddress me, RiCBoolean withParameters)
{
	struct RiCSData * res = NULL;
	ARCSEvent * aomsevent;
	
	if (me != NULL){	
		/* Create an ARCSEvent with this signature */
		aomsevent = ARCSE_create("RiCStartBehaviorEvent", RiCSnoParameters);
		ARCSE_addEventId(aomsevent, (ARCEvent*)me);
		res = ARCSD(aomsevent);
	}

	return res;
}
#endif

/*lint -restore */

