#ifndef RiCEvent_H
#define RiCEvent_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/


/********************************************************************
 ---------------------------------------------------------------
|							  RiCEvent.h                          |
 --------------------------------------------------------------- 
*********************************************************************/
  

#include "mxf_cfg.h"

/***********************************************/
/**           Definition of event ids		  **/
/***********************************************/
/* null event id */
#define RiCNull_id ((RiCEventId) -1)
#ifndef Null_id
/*LDRA_INSPECTED 79 S */
#define Null_id RiCNull_id
#endif

/* The id of all timeout events */
#define RiCTimeout_id ((RiCEventId) -2)
#ifndef Timeout_id
/*LDRA_INSPECTED 79 S */
#define Timeout_id RiCTimeout_id
#endif


/*****************************************************************************/
/*****************************************************************************/
/******								RiCEvent							******/
/*****************************************************************************/
/*****************************************************************************/

/***********************************************/
/**           Predefined constants			  **/
/***********************************************/
#define RiCCancelledEvent_id ((RiCEventId) -3)	/* id for cancelled event */
#define RiCAnyEvent_id		 ((RiCEventId) -4)	/* wild card '*' matching any id */
#define RiCStartBehavior_id	 ((RiCEventId) -5)	/* id of the start behavior event */
#define RiCReactiveTerminationEvent_id ((RiCEventId) -6) /* id of reactive destruction event (memory deallocation) */



/***********************************************/
/**			Event Destroy operation			  **/
/***********************************************/
/*LDRA_INSPECTED 141 S : Type forward declaration */
struct RiCEvent_t;
typedef struct RiCEvent_t RiCEvent;
typedef void (*RiCEventDestroyOp)(RiCEvent* const me);

/***********************************************/
/**           RiCEvent definition			  **/
/***********************************************/
struct RiCEvent_t {
    struct RiCReactive_t * destination;
	RiCEventId lId;

	RiCEventDestroyOp eventDestroyOp;	/* event destroy operation (for support in static allocation) */
	RiCBoolean isAllocated;
	void* userData;

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCBoolean frameworkEvent;	/* an event needed by the Rhapsody framework */
#endif
#ifdef RIC_USE_RAPID_PORTS
	RhpAddress port;
#endif
};

/***************************************************/
/****          lock and free mutex		        ****/
/***************************************************/

void RiCEvent_lockEventsPoolMutex(void);
void RiCEvent_freeEventsPoolMutex(void);


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

RiCBoolean RiCEvent_init( RiCEvent * const me, const RiCEventId eventId );
void RiCEvent_cleanup(RiCEvent * const me);
void RiCEvent_destroy(RiCEvent * const me);


/***************************************************/
/****				Event methods	            ****/
/***************************************************/

/* set the event destenation */
void RiCEvent_setDestination(RiCEvent * const me, struct RiCReactive_t * const dest);

/* get the event destenation */
struct RiCReactive_t * RiCEvent_getDestination(const RiCEvent * const me);

/* set the event id */
void RiCEvent_setlId(RiCEvent * const me, RiCEventId evid);

/* get the event id */
RiCEventId RiCEvent_getlId(const RiCEvent * const me);

/* check if timeout event */
RiCBoolean RiCEvent_isTimeout(const RiCEvent * const me);

/* check if a timeout event which has been canceled */
RiCBoolean RiCEvent_isCancelledTimeout(const RiCEvent * const me);

/* check if the event is a user event */
RiCBoolean RiCEvent_isRealEvent(const RiCEvent * const me);

RiCEvent* RiCEvent_getMemory(struct RiCMxfGlobals_t *mxfGlblInst);
void RiCEvent_returnMemory(RiCEvent * const me);
void RiCEvent_setData(RiCEvent * const me, void * const data);

/* check if the event was generated by the framework */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCBoolean RiCEvent_isFrameworkEvent(const RiCEvent * const me);
#endif

/* set the FrameworkEvent flag */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCEvent_setFrameworkEvent(RiCEvent * const me, RiCBoolean isFrameworkEvent);
#endif

/* check is the event is from a given type (has a diven id)*/
RiCBoolean RiCEvent_isTypeOf(const RiCEvent * const me, RiCEventId evid);

/*****************************************************************************/
/*****************************************************************************/
/******								RiCTimeout							******/
/*****************************************************************************/
/*****************************************************************************/

/******************************************/
/**				definition				 **/
/******************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining objects that might be used	 */
/* Outside of the OXF itself by Generated code					 */
PUBLIC const RiCEventId RiCTimeoutDelayId;

/******************************************/
/**			RiCTimeout definition		 **/
/******************************************/
typedef struct RiCTimeout_t {
	timeUnit dueTime ;
	RiCEventId timeoutId;
	RiCBoolean isAllocated;
	RiCEvent *nextTm;
#ifdef _OMINSTRUMENT
	timeUnit delayTime;
	RiCHandle *state;
#endif
} RiCTimeout;


/******************************************/
/**		static memory allocation		 **/
/******************************************/

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCBoolean RiCTimeout_init(RiCEvent * const me, RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, RhpString state, struct RiCMxfGlobals_t * const mxfGlblInst);
#else
/*LDRA_INSPECTED 77 S */
#define RiCTimeout_init(me, id, dest, delayTime, state, mxfGlblInst) \
	RiCTimeout_init_low((me), (id), (dest), (delayTime), (mxfGlblInst))
RiCBoolean RiCTimeout_init_low(RiCEvent * const me, RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t * const mxfGlblInst);
#endif
                                    
void RiCTimeout_cleanup(RiCEvent * const me);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCEvent * RiCTimeout_create(RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t *mxfGlblInst, RhpString state);
#else
/*LDRA_INSPECTED 77 S */
#define RiCTimeout_create(id, dest, delayTime, mxfGlblInst, state) \
	RiCTimeout_create_low((id), (dest), (delayTime), (mxfGlblInst))
RiCEvent * RiCTimeout_create_low(RiCEventId id, struct RiCReactive_t * const dest, timeUnit delayTime, struct RiCMxfGlobals_t *mxfGlblInst);
#endif

/* event destroy format */
void RiCTimeout_Destroy(RiCEvent * const me);

/***************************************************/
/****			RiCTimeout methods				****/
/***************************************************/

/** return the timeout id **/
RiCEventId RiCTimeout_getTimeoutId(const RiCEvent * const me);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/** return the timeout delay time **/
timeUnit RiCTimeout_getDelay(const RiCTimeout * const me);
#endif

/** return the timeout due time **/
timeUnit RiCTimeout_getDueTime(const RiCEvent * const me);

/** set the timeout due time **/
void RiCTimeout_setDueTime(RiCEvent * const me, const timeUnit dTime);

/** set the timeout id **/
void RiCTimeout_setTimeoutId(RiCTimeout * const me, RiCEventId id);

#if (defined _OMINSTRUMENT)

void RiCTimeout_setRelativeDueTime( RiCEvent * const me, const timeUnit dTime );

/** check if the timeout is not a delay **/
RiCBoolean RiCTimeout_isNotDelay(const RiCTimeout * const me);
#endif

/** compare timeouts due time (me->dueTime - other->dueTime) **/
RhpUnlimitedNatural RiCTimeout_compare(const RiCEvent * const me, const RiCEvent * const other);

/** indicates whether some other timepot is "equal to" this one (timeoutId and destination) **/
RiCBoolean RiCTimeout_equals(const RiCEvent * const me, const RiCEvent * const other);

RiCEvent *RiCTimeout_getNextTm(const RiCEvent * const me);

void RiCTimeout_setNextTm(RiCEvent * const me, RiCEvent * const next);

void RiCTimeout_addToList(RiCEvent *me, RiCEvent **theTmList);

/***************************************************/
/****		RiCTimeout animation methods		****/
/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

struct ARCEventClass;
struct RiCSData;

/** return a handle to the timeout state **/
RiCHandle * RiCTimeout_getState(const RiCTimeout * const me);

/** set the state handle **/
void RiCTimeout_setState(RiCTimeout * const me, RiCHandle * state);

/** get the animation class **/
struct ARCEventClass * RiCTimeout_getClass(RiCEvent * const me);
/** serilize the timeout **/
struct RiCSData * RiCTimeout_cserialize(RhpAddress me, RiCBoolean withParameters);

#endif

/*****************************************************************************/
/*****************************************************************************/
/******						RiCStartBehaviorEvent						******/
/*****************************************************************************/
/*****************************************************************************/


/***************************************************/
/****         initialization and cleanup        ****/
/***************************************************/
RiCEvent * RiCStartBehaviorEvent(struct RiCReactive_t * const dest, struct RiCMxfGlobals_t *mxfGlblInst);
RiCEvent * RiC_Create_RiCStartBehaviorEvent(struct RiCReactive_t * const dest, struct RiCMxfGlobals_t *mxfGlblInst);

void RiCStartBehaviorEvent_init(RiCEvent * const me, struct RiCReactive_t * const dest);

#ifdef MICROC_OXF_CLEANUP
void RiCStartBehaviorEvent_cleanup(RiCEvent * const me);
#endif

/***************************************************/
/****  RiCStartBehaviorEvent animation methods	****/
/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

struct ARCEventClass;
struct RiCSData;
/** get the animation class **/
struct ARCEventClass * RiCStartBehaviorEvent_getClass(RiCEvent * const me);

/** serilize the timeout **/
struct RiCSData * RiCStartBehaviorEvent_cserialize(RhpAddress me, RiCBoolean withParameters);

#endif


#endif


