#ifndef RiCEventQueue_H
#define RiCEventQueue_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCEventQueue.h                          |
 --------------------------------------------------------------- 
********************************************************************/

#include "mxf_cfg.h"
#include "RiCEvent.h"

#ifdef RIC_USE_RICLIST
#ifdef MICROC_FLAT_INCLUDE
#include "RiCList.h"
#else
#include "Utils/RiCList.h"
#endif
#endif

/******************************************/
/** The EventQueue Definition			 **/
/******************************************/
typedef struct RiCEventQueue_t 
{
	RiCEvent* *the_queue;		/* the actual array */
	RhpPositive head;		/* head position */
	RhpPositive tail;		/* tail position */
	RhpPositive count;		/* current number of elements in queue */
	RhpPositive size;		/* number of elements in queue */
} RiCEventQueue;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

/** return RiCTrue on sucess **/
RiCBoolean RiCEventQueue_Init(RiCEventQueue * const me, RiCEvent* * const evQueue, RhpPositive queueSize);

#ifdef MICROC_OXF_CLEANUP
void RiCEventQueue_Cleanup(RiCEventQueue * const me);
#endif

/***************************************************/
/****            EventQueue methods             ****/
/***************************************************/

/** Push an element to the queue **/
RiCBoolean RiCEventQueue_push(RiCEventQueue * const me, RiCEvent* const pItem);

/** Pop an element from the queue **/
RiCEvent* RiCEventQueue_pop(RiCEventQueue * const me);

#ifdef RIC_USE_RICLIST
/** Return a link-list of the queue members, 
	the head of the linked list is the next element to be returned by RiCEventQueue_get() **/
void RiCEventQueue_getQueue(const RiCEventQueue * const me, RiCList * const pList);
#endif

/** Check if the queue is empty **/
RiCBoolean RiCEventQueue_isEmpty(const RiCEventQueue * const me);

/** Check if the queue is fuul **/
RiCBoolean RiCEventQueue_isFull(const RiCEventQueue * const me);


#endif

