/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/* Includes *******************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICOSAUTOSARRTE_SOURCE
#define MXF_RICOSAUTOSARRTE_SOURCE
#endif

#include "RiCOSAutosarRte.h"
#include "RiCOSWrap.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#include <stdio.h>
#include <stdlib.h>
#ifdef MICROC_FLAT_INCLUDE
#include "omsdata.h"
#else
#include "omcom/omsdata.h"
#endif
#endif
/***************************************************/
/****				Default values				****/
/***************************************************/
const RhpUnlimitedNatural RiCOSDefaultStackSize = 0L;
const RhpUnlimitedNatural RiCOSDefaultMessageQueueSize = 100L;
const RhpUnlimitedNatural RiCOSDefaultThreadPriority = 1L;

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
static void OM_NOTIFY_ERROR(const RhpString callStr, const RhpString funcStr);
static void OM_NOTIFY_ERROR(const RhpString callStr, const RhpString funcStr)
{
}
#endif


/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCOSEndApplication( RiC_INT32 errorCode )
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (TerminateProcess(GetCurrentProcess(),(RiC_UINT32)errorCode) == 0)
	{
		OM_NOTIFY_ERROR("TerminateProcess","RiCOSEndApplication");
	}	
#endif
}

void RiCOSOXFInitEpilog(void)
{
}

/***************************************************/
/****				OS Event Flag				****/
/***************************************************/
RiCBoolean RiCOSEventFlag_init(RiCOSEventFlag * const me, RiCOSTask * const context)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			*me = CreateEvent( NULL, FALSE /*autoreset*/, FALSE /* initially not signaled */,NULL );
			if (*me != NULL)
			{
				retval = RiCTRUE;
			}
			else
			{
				OM_NOTIFY_ERROR("CreateEvent","RiCOSEventFlag_init");
			}
#endif
	}
	
	return retval;
}

void RiCOSPendingEvent_init(RiCOSTask * const me, RiCOSTask * const context)
{
	if (me != NULL) {
		const RiCOSTask *osTaskStruct = context;
		me->pendingEvents = RiCFALSE;
		if(osTaskStruct != NULL){
			if(osTaskStruct->queueEvent != 0U){
				me->pendingEvents = RiCTRUE;
			}
		}
	}
}
/**************/
void RiCOSEventFlag_cleanup(RiCOSEventFlag * const me)
{
#ifdef MICROC_OXF_CLEANUP
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (CloseHandle((HANDLE)*me) == 0)
	{
		OM_NOTIFY_ERROR("CloseHandle","RiCNT_doCloseHandle");
	}
	*me = (RiCOSEventHandle) NULL;
#endif
#endif
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSEventFlag * RiCOSEventFlag_create(void)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSEventFlag * me = RiC_malloc(sizeof(RiCOSEventFlag));
	if (me != NULL)
	{
		(void) RiCOSEventFlag_init(me, NULL);
	}
	return me;
#else
	return (RiCOSEventFlag *) NULL;
#endif
}


void RiCOSPendingEvent_clear(RiCOSTask * const pOsTask)
{
	if(pOsTask != NULL){
		pOsTask->pendingEvents = RiCFALSE;	
	}
}

void RiCOSEventFlag_clear(RiCOSEventFlag *evFlag)
{
	/** Nothing **/
}


/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCOSEventFlag_destroy(RiCOSEventFlag * const me)
{
#ifdef MICROC_OXF_CLEANUP
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (me != NULL) {
		RiCOSEventFlag_cleanup(me);
		RiC_free(me);
	}
#endif
#endif
}

/**************/
RiCOSResult RiCOSEventFlag_wait(RiCOSEventFlag * const me, RhpInteger tminms )
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult retval = 1;
	if (me == NULL)
	{
		return (RiCOSResult)WAIT_FAILED;
	}

	if ( tminms == -1 )
	{
		retval = WaitForSingleObject(*me,INFINITE);
		if (retval == WAIT_FAILED)
		{
			OM_NOTIFY_ERROR("WaitForSingleObject","RiCOSEventFlag_wait");
		}
		return retval;
	}
	else
	{
		retval =  WaitForSingleObject(*me,(RiC_UINT32)tminms);
		if (retval == WAIT_FAILED)
		{
			OM_NOTIFY_ERROR("WaitForSingleObject","RiCOSEventFlag_wait");
		}
		return retval;		
	}
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}

/**************/
RiCOSResult RiCOSEventFlag_signal(RiCOSEventFlag * const me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult retval = 1;
	if (me != NULL)
	{
		retval =  (RiCOSResult) SetEvent(*me);
		if (retval == 0)
		{
			OM_NOTIFY_ERROR("SetEvent","RiCOSEventFlag_signal");
		}		
	}
	else
	{
		retval = 0;
	}
	return retval;
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}

RiCOSResult RiCOSPendingEvent_signal(RiCOSTask *pOsTask)
{
	RiCOSResult retval = (RiCOSResult) 0;
	if(pOsTask != NULL){
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		pOsTask->pendingEvents = (RiCBoolean) 1U;
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		retval = (RiCOSResult) 1U;
	}
	return retval;
}

/**************/
RiCOSResult RiCOSEventFlag_reset(RiCOSEventFlag * const me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCOSResult retval = 1;
	if (me != NULL)
	{
		retval =  (RiCOSResult) ResetEvent(*me);
		if (retval == 0)
		{
			OM_NOTIFY_ERROR("ResetEvent","RiCOSEventFlag_reset");
		}
	}
	else
	{
		retval = 0;
	}
	return retval;
#else
	/*Nothing! */
	return ((RiCOSResult)0);
#endif
}

RiCOSResult RiCOSPendingEvent_reset(RiCOSTask *pOsTask)
{
	/*Nothing! */
	return ((RiCOSResult)0);
}

RiCBoolean RiCOSEventFlag_hasPendingEvents(const RiCOSTask * const pOsTask)
{
	RiCBoolean retval = RiCFALSE;
	if (pOsTask != NULL)
	{
		if( (pOsTask->pendingEvents != RiCFALSE) ){
			retval = RiCTRUE;
		}
	}
	return retval;
}

/***************************************************/
/****				OS Mutex					****/
/***************************************************/

/**************/
RiCBoolean RiCOSMutex_init(RiCOSMutex * const me, OSMutexHandle mutexIdentifier) 
{
	RiCBoolean retval = RiCFALSE;
	if(me != NULL){
		me->hMutex = mutexIdentifier;
		retval = RiCTRUE;
	}
	return retval;
}  

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
void RiCOSMutex_cleanup(const RiCOSMutex * const me)
{
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSMutex * RiCOSMutex_create(void)
{
	return (RiCOSMutex *) NULL;
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
void RiCOSMutex_destroy(RiCOSMutex * const me)
{

}


/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef RIC_NEED_INIT_COMMUNICATION_LAYER
void	RiCOSTask_InitCommunicationLayer(void)
{
}
void	RiCOSTask_CleanupCommunicationLayer(void)
{
}
#endif

/**************/	
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSMutex_lock(const RiCOSMutex *me)
{
	RiCBoolean retval = RiCFALSE;
	if((me != NULL) && (me->hMutex != NULL)){
		(*(me->hMutex))(RiCTRUE);
		retval = RiCTRUE;
	}
	return retval;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSMutex_free(const RiCOSMutex *me)
{
	RiCBoolean retval = RiCFALSE;
	if((me != NULL) && (me->hMutex != NULL)){
		(*(me->hMutex))(RiCFALSE);
		retval = RiCTRUE;
	}
	return retval;
}

/**************/	
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSTimerManagerMutex_lock(const RiCOSMutex *me)
{
#ifndef RIC_USE_OS_CONTEXT
	Rte_Enter_TimerManager_EA();
#endif
	return RiCTRUE;
}

/**************/	
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSTimerManagerMutex_free(const RiCOSMutex *me)
{
#ifndef RIC_USE_OS_CONTEXT
	Rte_Exit_TimerManager_EA(); 
#endif
	return RiCTRUE;
}

void RiCOSEventsPoolMutex_init(void)
{
}

void	RiCOSEventsPoolMutex_lock(void)
{
#ifndef RIC_USE_OS_CONTEXT
	Rte_Enter_EventsPool_EA();
#endif
}

void RiCOSEventsPoolMutex_free(void)
{
#ifndef RIC_USE_OS_CONTEXT
	Rte_Exit_EventsPool_EA(); 
#endif
}

/***************************************************/
/****				OS Task						****/
/***************************************************/
RiCOSResult RiCOSTask_wrap(RiCOSTask * const me, RiCOSHandle threadOsHandle)
{
	RiCOSResult retval = (RiCOSResult) 0;
	
	if (me != NULL) {		
		me->osTask = threadOsHandle;
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		retval = (RiCOSResult) 1U;
	}

	return retval;
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCOSTask_cleanup(RiCOSTask * const me)
{
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSTask * RiCOSTask_create(const RiCOSTaskEndCallBack tfunc, const RhpAddress pParam, const RhpCharacter * const itsName, const RhpUnlimitedNatural itsStackSize)
{

	return (RiCOSTask *) NULL;
}


/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSTask_setPriority(RiCOSTask * const me, RhpInteger pr)
{
	/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
	return (RiCOSResult) 1U;
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSTask_suspend(const RiCOSTask * const me)
{
	return (RiCOSResult) 0;
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSTask_resume(const RiCOSTask * const me)
{
	return (RiCOSResult) 0;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSTask_start(const RiCOSTask * const me)
{
	return (RiCOSResult) 0;
}

/**************/
RiCOSHandle RiCOSTask_getCurrentTaskHandle(void)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	return (RiCOSHandle)GetCurrentThreadId();
#else
	return (RiCOSHandle) 0;
#endif
}

/**************/
RiCOSHandle RiCOSTask_getOSHandle(const RiCOSTask * const me)
{
	RiCOSHandle retval = ((RiCOSHandle)0);
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (me != NULL) {
		retval = me->osTask;
	}
#endif
	return retval;
}

#ifdef MICROC_OXF_CLEANUP
/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpInteger RiCOSTask_setEndOSTaskInCleanup(const RiCOSTask * const me, RiCBoolean val)
{
	return 0;
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCOSTask_exeOnMyTask(RiCOSTask * const me)
{
	RiCOSHandle executedOsHandle = (RiCOSHandle) 0;
	RiCOSHandle myOsHandle       = (RiCOSHandle) 0;
	RiCBoolean retval = RiCFALSE;
	
	if (me != NULL) {
		/* a handle to the thread that executes the delete */
		executedOsHandle = RiCOSTask_getCurrentTaskHandle();
		/* a handle to this' 'thread' */
		myOsHandle = RiCOSTask_getOSHandle(me);
		
		if(executedOsHandle == myOsHandle){
			retval = RiCTRUE;
		}
	}
	return retval;
}
#endif
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
void RiCOSTask_endMyTask(RiCOSTask * const me)
{
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCOSTask_endOtherTask(const RhpAddress handle)
{
	return RiCFALSE;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpInteger RiCOSTask_getTaskEndClbk(const RiCOSTask * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteTask)
{
	return 1;
}



void RiCOSMainTask_start(void)
{
}


/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpBoolean RiCOSTimerManager_registerTimedAction(RiCOSTimerHandle timerHandle, OSTimeUnit tDueTime, OSTimeUnit tPeriod)
{
	/* Return 1 to not have the TimerManager handle the timedAction registretion */
	
	return RiCTRUE;
}

RhpBoolean RiCOSTimerManager_getSystemTime(OSTimeUnit *sysTime)
{
	RhpBoolean retval = RiCFALSE;
	if(sysTime != NULL){
		Rte_Call_os_service_GetCounterValue(SwCounter, sysTime);
		retval = RiCTRUE;
	}
	return retval;
}


/***************************************************/
/****					OS Timer				****/
/***************************************************/
static RiCOSTimer * cachedTimer;

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void resetTimerCallbackFuction(void)
{
	cachedTimer = NULL;
}

static void setTimerCallbackFunction(RiCOSTimer * const me);
static void setTimerCallbackFunction(RiCOSTimer * const me)
{
	cachedTimer = me;
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void setTimerActivationCycle(void)
{

}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void runTimerCallbackFuction(void)
{
}

/**************/
RiCBoolean RiCOSTimer_init(RiCOSTimer * const me)
{
	return RiCTRUE;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
void RiCOSTimer_cleanup(RiCOSTimer * const me)
{
	/* Nothing! */
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSTimer * RiCOSTimer_create(timeUnit ptime, void (* const cllbkfunc)(RhpAddress), const RhpAddress params)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{

	return (RiCOSTimer *) NULL;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
void RiCOSTimer_destroy(RiCOSTimer * const me)
{

}
#endif


/***************************************************/
/****			OS Message Queue				****/
/***************************************************/
RiCBoolean RiCOSMessageQueue_init(RiCOSMessageQueue * const me, RiCEvent** const evQueue, RhpPositive eventQueueSize)
{
	RiCBoolean retval = RiCFALSE;
	
	if (me != NULL) {		
		(void) RiCEventQueue_Init(&(me->m_baseQueue), evQueue, eventQueueSize);
		
		retval = RiCTRUE;
	}
	
	return retval;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSMessageQueue_cleanup(RiCOSMessageQueue * const me)
{
	if (me != NULL){		
		RiCEventQueue_Cleanup(&(me->m_baseQueue));
	}
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSMessageQueue * RiCOSMessageQueue_create(RiCBoolean shouldGrow, RhpInteger initSize)
{

	return (RiCOSMessageQueue *) NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSMessageQueue_destroy(RiCOSMessageQueue * const me)
{

}
#endif

/*lint -save -e534 (Ignoring return value in the mutex lock() and free()) */
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSMessageQueue_put(RiCOSMessageQueue * const me, RhpAddress const message, RiCBoolean fromISR)
{
	RiCOSResult retval = (RiCOSResult) 0;
	
	if(me != NULL){
		retval = RiCEventQueue_push(&(me->m_baseQueue), message);
	}
	return retval;
}	/*lint !e715 (Symbol 'fromISR' not referenced) */

/**************/
RiCEvent* RiCOSMessageQueue_get(RiCOSMessageQueue * const me)
{
	RiCEvent* m = NULL;
	
	/* safe programming */
	if (me != NULL){		
		if(RiCOSMessageQueue_isEmpty(me) == RiCFALSE) {
			m = RiCEventQueue_pop(&(me->m_baseQueue));
		} 
		else {
			m = NULL;
		}
	}
	/*LDRA_INSPECTED 71 S */
	return m;
/*LDRA_INSPECTED 5 D */
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCOSMessageQueue_isFull(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCFALSE;
	if(me != NULL){
		retval = RiCEventQueue_isFull(&(me->m_baseQueue));
	}
	return retval;
}

RiCBoolean RiCOSMessageQueue_isEmpty(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCTRUE;
	if(me != NULL){
		retval = RiCEventQueue_isEmpty(&(me->m_baseQueue));
	}
	return retval;
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSResult RiCOSMessageQueue_pend(const RiCOSMessageQueue * const me)
{
	RiCOSResult retval = (RiCOSResult) 0;
	if (me != NULL){
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		retval = (RiCOSResult) 1U;
	}
	return retval;
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSResult RiCOSMessageQueue_getMessageList(const RiCOSMessageQueue * const me, RiCList * const pList)
{
	RiCOSResult retval = (RiCOSResult) 0;
	RiCList_removeAll(pList);
	
	if (me != NULL){
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		retval = (RiCOSResult) 1U;
		RiCEventQueue_getQueue(&(me->m_baseQueue), pList);
	}
	
	return retval;
}
#endif


/***************************************************/
/****			OS Semaphore					****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCOSSemaphore_init(RiCOSSemaphore * const me, RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{
	return (RiCTRUE);
} /*lint !e715 (Symbol 'semFlags' not referenced) */

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
void RiCOSSemaphore_cleanup(RiCOSSemaphore * const me)
{
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSSemaphore * RiCOSSemaphore_create(RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{

	return (RiCOSSemaphore *) NULL;
}

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
void RiCOSSemaphore_destroy(RiCOSSemaphore * const me)
{

}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSSemaphore_signal(RiCOSSemaphore * const me)
{
	return RiCTRUE;
}

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCOSResult RiCOSSemaphore_wait(RiCOSSemaphore *  const me, RhpUnlimitedNatural timeout)
{
	return (RiCOSResult) 1U;
}


void RiCOSOS_get_current_time(OSTimeUnit *currentTickP)
{

}

