/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/* Includes *******************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICOSMAINLOOP_SOURCE
#define MXF_RICOSMAINLOOP_SOURCE
#endif

#include "RiCOSMainloop.h"
#include "RiCOSWrap.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#include <stdio.h>
#include <stdlib.h>
#ifdef MICROC_FLAT_INCLUDE
#include "omsdata.h"
#else
#include "omcom/omsdata.h"
#endif
#endif

/***************************************************/
/****				Default values				****/
/***************************************************/
const RhpUnlimitedNatural RiCOSDefaultStackSize = 0L;
const RhpUnlimitedNatural RiCOSDefaultMessageQueueSize = 100;
const RhpUnlimitedNatural RiCOSDefaultThreadPriority = 1;


/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef RiCOSEndApplication_Defined
void RiCOSEndApplication( RiC_INT32 errorCode )
{

}
#endif

#ifndef RiCOSOXFInitEpilog_Defined
void RiCOSOXFInitEpilog(void)
{
}
#endif

/***************************************************/
/****				OS Event Flag				****/
/***************************************************/
#ifndef    RiCOSEventFlag_init_Defined
RiCBoolean RiCOSEventFlag_init(RiCOSEventFlag * const me, RiCOSTask * const context)
{
	return RiCFALSE;
}
#endif

#ifndef RiCOSPendingEvent_init_Defined
void RiCOSPendingEvent_init(RiCOSTask * const me, RiCOSTask * const context)
{
	if (me != NULL) {
		const RiCOSTask *osTaskStruct = context;
		me->pendingEvents = RiCFALSE;
		if(osTaskStruct != NULL){
			if(osTaskStruct->queueEvent != (RiCOSEventHandle)0){
				me->pendingEvents = RiCTRUE;
			}
		}
	}
}
#endif

/**************/
#ifndef RiCOSEventFlag_cleanup_Defined
void RiCOSEventFlag_cleanup(RiCOSEventFlag * const me)
{
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef          RiCOSEventFlag_create_Defined
RiCOSEventFlag * RiCOSEventFlag_create(void)
{
	return (RiCOSEventFlag *) NULL;
}
#endif


#ifndef RiCOSPendingEvent_clear_Defined
void RiCOSPendingEvent_clear(RiCOSTask * const pOsTask)
{
	if(pOsTask != NULL){
		pOsTask->pendingEvents = (RiCBoolean) 0;	
	}
}
#endif

#ifndef RiCOSPendingEvent_clear_Defined
void RiCOSEventFlag_clear(RiCOSEventFlag *evFlag)
{
	/** Nothing **/
}
#endif


/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef RiCOSEventFlag_destroy_Defined
void RiCOSEventFlag_destroy(RiCOSEventFlag * const me)
{
}
#endif

/**************/
#ifndef     RiCOSEventFlag_wait_Defined
RiCOSResult RiCOSEventFlag_wait(RiCOSEventFlag * const me, RhpInteger tminms )
{
	/*Nothing! */
	return ((RiCOSResult)0);
}
#endif

/**************/
#ifndef	    RiCOSEventFlag_signal_Defined
RiCOSResult RiCOSEventFlag_signal(RiCOSEventFlag * const me)
{
	/*Nothing! */
	return ((RiCOSResult)0);
}
#endif

#ifndef	    RiCOSPendingEvent_signal_Defined
RiCOSResult RiCOSPendingEvent_signal(RiCOSTask *pOsTask)
{
	RiCOSResult retval = (RiCOSResult) 0;
	if(pOsTask != NULL){
		pOsTask->pendingEvents = RiCTRUE;
		retval = (RiCOSResult) 1;
	}
	return retval;
}
#endif

/**************/
#ifndef     RiCOSEventFlag_reset_Defined
RiCOSResult RiCOSEventFlag_reset(RiCOSEventFlag * const me)
{
	/*Nothing! */
	return ((RiCOSResult)0);
}
#endif

#ifndef     RiCOSPendingEvent_reset_Defined
RiCOSResult RiCOSPendingEvent_reset(RiCOSTask *pOsTask)
{
	/*Nothing! */
	return ((RiCOSResult)0);
}
#endif

#ifndef    RiCOSEventFlag_hasPendingEvents_Defined
RiCBoolean RiCOSEventFlag_hasPendingEvents(const RiCOSTask * const pOsTask)
{
	RiCBoolean retval = RiCFALSE;
	if (pOsTask != NULL)
	{
		if( (pOsTask->pendingEvents != RiCFALSE) ){
			retval = RiCTRUE;
		}
	}
	return retval;
}
#endif

/***************************************************/
/****				OS Mutex					****/
/***************************************************/

/**************/
#ifndef    RiCOSMutex_init_Defined
RiCBoolean RiCOSMutex_init(RiCOSMutex * const me, OSMutexHandle mutexIdentifier) 
{
	return RiCTRUE;
}  
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef RiCOSMutex_cleanup_Defined
void RiCOSMutex_cleanup(const RiCOSMutex * const me)
{
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef      RiCOSMutex_create_Defined
RiCOSMutex * RiCOSMutex_create(void)
{
	return (RiCOSMutex *) NULL;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef RiCOSMutex_destroy_Defined
void RiCOSMutex_destroy(RiCOSMutex * const me)
{

}
#endif


#ifndef RIC_NEED_INIT_COMMUNICATION_LAYER
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void	RiCOSTask_InitCommunicationLayer(void)
{
}
void	RiCOSTask_CleanupCommunicationLayer(void)
{
}
#endif

/**************/	
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSMutex_lock_Defined
RhpBoolean RiCOSMutex_lock(const RiCOSMutex *me)
{
	return RiCTRUE;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef   RiCOSMutex_free_Defined
RhpBoolean RiCOSMutex_free(const RiCOSMutex *me)
{
	return RiCTRUE;
}
#endif

#ifndef    RiCOSTimerManagerMutex_lock_Defined
RhpBoolean RiCOSTimerManagerMutex_lock(const RiCOSMutex *me)
{
	return RiCOSMutex_lock(me);
}
#endif

#ifndef    RiCOSTimerManagerMutex_free_Defined
RhpBoolean RiCOSTimerManagerMutex_free(const RiCOSMutex *me)
{
	return RiCOSMutex_free(me);
}
#endif

#ifndef RiCOSEventsPoolMutex_init_Defined
void RiCOSEventsPoolMutex_init(void)
{
}
#endif

#ifndef RiCOSEventsPoolMutex_lock_Defined
void	RiCOSEventsPoolMutex_lock(void)
{
}
#endif

#ifndef RiCOSEventsPoolMutex_free_Defined
void RiCOSEventsPoolMutex_free(void)
{
}
#endif

/***************************************************/
/****				OS Task						****/
/***************************************************/
#ifndef     RiCOSTask_wrap_Defined
RiCOSResult RiCOSTask_wrap(RiCOSTask * const me, RiCOSHandle threadOsHandle)
{
	RiCOSResult retval = (RiCOSResult) 0;
	
	if (me != NULL) {		
		me->osTask = threadOsHandle;
		retval = (RiCOSResult) 1;
	}

	return retval;
}
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifndef RiCOSTask_cleanup_Defined
void RiCOSTask_cleanup(RiCOSTask * const me)
{
}
#endif
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSTask_create_Defined
RiCOSTask * RiCOSTask_create(const RiCOSTaskEndCallBack tfunc, const RhpAddress pParam, const RhpCharacter * const itsName, const RhpUnlimitedNatural itsStackSize)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{

	return (RiCOSTask *) NULL;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSTask_setPriority_Defined
RiCOSResult RiCOSTask_setPriority(RiCOSTask * const me, RhpInteger pr)
{
	return (RiCOSResult) 1;
}
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSTask_suspend_Defined
RiCOSResult RiCOSTask_suspend(const RiCOSTask * const me)
{
	RiCOSResult retval = (RiCOSResult) 0;
	return retval;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSTask_resume_Defined
RiCOSResult RiCOSTask_resume(const RiCOSTask * const me)
{
	RiCOSResult retval = (RiCOSResult) 0;
	return retval;
}
#endif
#endif
/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSTask_start_Defined
RiCOSResult RiCOSTask_start(const RiCOSTask * const me)
{
	RiCOSResult retval = (RiCOSResult) 0;
	return retval;
}
#endif

/**************/
#ifndef     RiCOSTask_getCurrentTaskHandle_Defined
RiCOSHandle RiCOSTask_getCurrentTaskHandle(void)
{
	RiCOSHandle retval = (RiCOSHandle) 0;
	return retval;
}
#endif

/**************/
#ifndef     RiCOSTask_getOSHandle_Defined
RiCOSHandle RiCOSTask_getOSHandle(const RiCOSTask * const me)
{
	RiCOSHandle retval = ((RiCOSHandle)0);
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (me != NULL) {
		retval = me->osTask;
	}
#endif
	return retval;
}
#endif

#ifdef MICROC_OXF_CLEANUP
/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSTask_setEndOSTaskInCleanup_Defined
RhpInteger RiCOSTask_setEndOSTaskInCleanup(const RiCOSTask * const me, RiCBoolean val)
{
	return 0;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef    RiCOSTask_exeOnMyTask_Defined
RiCBoolean RiCOSTask_exeOnMyTask(RiCOSTask * const me)
{
	RiCOSHandle executedOsHandle = (RiCOSHandle) 0;
	RiCOSHandle myOsHandle       = (RiCOSHandle) 0;
	RiCBoolean retval = RiCFALSE;
	
	if (me == NULL) {
		retval = RiCFALSE;
	}
	else{
		
		/* a handle to the thread that executes the delete */
		executedOsHandle = RiCOSTask_getCurrentTaskHandle();
		/* a handle to this' 'thread' */
		myOsHandle = RiCOSTask_getOSHandle(me);
		
		if(executedOsHandle == myOsHandle){
			retval = RiCTRUE;
		}
	}
	return retval;
}
#endif
#endif
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef RiCOSTask_endMyTask_Defined
void RiCOSTask_endMyTask(RiCOSTask * const me)
{
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSTask_endOtherTask_Defined
RiCBoolean RiCOSTask_endOtherTask(const RhpAddress handle)
{
	return RiCFALSE;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSTask_getTaskEndClbk_Defined
RhpInteger RiCOSTask_getTaskEndClbk(const RiCOSTask * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteTask)
{
	return 1;
}
#endif

#ifndef RiCOSMainTask_start_Defined
void    RiCOSMainTask_start(void)
{
	while(1)
	{

		if(RiCOSMainTask_WaitStep() == RiCFALSE){
			runTimerCallbackFuction();
		}
		
	}
}
#endif


/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSTimerManager_registerTimedAction_Defined
RhpBoolean RiCOSTimerManager_registerTimedAction(RiCOSTimerHandle timerHandle, OSTimeUnit tDueTime, OSTimeUnit tPeriod)
{
	/* Return False to have the TimerManager handle the timedAction registretion */
	
	return RiCFALSE;
}
#endif

/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSTimerManager_getSystemTime_Defined
RhpBoolean RiCOSTimerManager_getSystemTime(OSTimeUnit *sysTime)
{
	return RiCFALSE;
}
#endif


/***************************************************/
/****					OS Timer				****/
/***************************************************/
static RiCOSTimer * cachedTimer = NULL;

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef resetTimerCallbackFuction_Defined
void resetTimerCallbackFuction(void)
{
	cachedTimer = NULL;
}
#endif

static void setTimerCallbackFunction(RiCOSTimer * const me);
static void setTimerCallbackFunction(RiCOSTimer * const me)
{
	cachedTimer = me;
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef setTimerActivationCycle_Defined
void setTimerActivationCycle(void)
{

}
#endif

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef runTimerCallbackFuction_Defined
void runTimerCallbackFuction(void)
{
	if(cachedTimer != NULL){
		/*LDRA_INSPECTED 57 S : LDRA Analysis incorrect. Statement has side effect */
		(*(cachedTimer->cbkfunc))(cachedTimer->param);
	}
}
#endif


/**************/
#ifndef    RiCOSTimer_init_Defined
RiCBoolean RiCOSTimer_init(RiCOSTimer * const me)
{
	RiCBoolean retval = RiCFALSE;

	if (me != NULL) { 	

		OSTIMER_INIT();
		
		setTimerCallbackFunction(me);
		retval = RiCTRUE;
	}

	return retval;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSTimer_cleanup_Defined
void RiCOSTimer_cleanup(RiCOSTimer * const me)
{
	/* Nothing! */
}
#endif
#endif

#include "RiCOSTarget.c"

#ifndef RICOSMAINTASK_SLEEP
#define RICOSMAINTASK_SLEEP

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef    RiCOSMainTask_WaitStep_Defined
RiCBoolean RiCOSMainTask_WaitStep(void)
{	
	return RiCFALSE;
}
#endif

#endif


/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef      RiCOSTimer_create_Defined
RiCOSTimer * RiCOSTimer_create(timeUnit ptime, void (* const cllbkfunc)(RhpAddress), const RhpAddress params)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{

	return (RiCOSTimer *) NULL;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSTimer_destroy_Defined
void RiCOSTimer_destroy(RiCOSTimer * const me)
{

}
#endif
#endif


/***************************************************/
/****			OS Message Queue				****/
/***************************************************/
#ifndef    RiCOSMessageQueue_init_Defined
RiCBoolean RiCOSMessageQueue_init(RiCOSMessageQueue * const me, RiCEvent** const evQueue, RhpPositive eventQueueSize)
{
	RiCBoolean retval = RiCFALSE;
	
	if (me != NULL) {		
		(void) RiCEventQueue_Init(&(me->m_baseQueue), evQueue, eventQueueSize);
		
		retval = RiCTRUE;
	}
	
	return retval;
}
#endif

/**************/
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSMessageQueue_cleanup_Defined
void RiCOSMessageQueue_cleanup(RiCOSMessageQueue * const me)
{
	if (me != NULL){		
		RiCEventQueue_Cleanup(&(me->m_baseQueue));
	}
}
#endif
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef RiCOSMessageQueue_create_Defined
RiCOSMessageQueue * RiCOSMessageQueue_create(RiCBoolean shouldGrow, RhpInteger initSize)
{

	return (RiCOSMessageQueue *) NULL;
}
#endif

/**************/
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSMessageQueue_destroy_Defined
void RiCOSMessageQueue_destroy(RiCOSMessageQueue * const me)
{

}
#endif
#endif

/*lint -save -e534 (Ignoring return value in the mutex lock() and free()) */
/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSMessageQueue_put_Defined
RiCOSResult RiCOSMessageQueue_put(RiCOSMessageQueue * const me, RhpAddress const message, RiCBoolean fromISR)
{
	RiCOSResult retval = (RiCOSResult) 0;
	
	if(me != NULL){
		retval = RiCEventQueue_push(&(me->m_baseQueue), message);
	}
	return retval;
}	/*lint !e715 (Symbol 'fromISR' not referenced) */
#endif

/**************/
#ifndef   RiCOSMessageQueue_get_Defined
RiCEvent* RiCOSMessageQueue_get(RiCOSMessageQueue * const me)
{
	RiCEvent* m = NULL;
	
	/* safe programming */
	if (me != NULL){		
		if(RiCOSMessageQueue_isEmpty(me) == RiCFALSE) {
			m = RiCEventQueue_pop(&(me->m_baseQueue));
		} 
		else {
			m = NULL;
		}
	}
	/*LDRA_INSPECTED 71 S */
	return m;
/*LDRA_INSPECTED 5 D */
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef    RiCOSMessageQueue_isFull_Defined
RiCBoolean RiCOSMessageQueue_isFull(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCFALSE;
	if(me != NULL){
		retval = RiCEventQueue_isFull(&(me->m_baseQueue));
	}
	return retval;
}
#endif

#ifndef    RiCOSMessageQueue_isEmpty_Defined
RiCBoolean RiCOSMessageQueue_isEmpty(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCTRUE;
	if(me != NULL){
		retval = RiCEventQueue_isEmpty(&(me->m_baseQueue));
	}
	return retval;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifndef     RiCOSMessageQueue_pend_Defined
RiCOSResult RiCOSMessageQueue_pend(const RiCOSMessageQueue * const me)
{
	RiCOSResult retval = (RiCOSResult) 0;
	if (me != NULL){
		retval = (RiCOSResult) 1;
	}
	return retval;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifndef     RiCOSMessageQueue_getMessageList_Defined
RiCOSResult RiCOSMessageQueue_getMessageList(const RiCOSMessageQueue * const me, RiCList * const pList)
{
	RiCOSResult retval = (RiCOSResult) 0;
	RiCList_removeAll(pList);
	
	if (me != NULL){
		retval = (RiCOSResult) 1;
		RiCEventQueue_getQueue(&(me->m_baseQueue), pList);
	}
	
	return retval;
}
#endif
#endif


/***************************************************/
/****			OS Semaphore					****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef    RiCOSSemaphore_init_Defined
RiCBoolean RiCOSSemaphore_init(RiCOSSemaphore * const me, RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{
	return (RiCTRUE);
} /*lint !e715 (Symbol 'semFlags' not referenced) */
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSSemaphore_cleanup_Defined
void RiCOSSemaphore_cleanup(RiCOSSemaphore * const me)
{
}
#endif
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef          RiCOSSemaphore_create_Defined
RiCOSSemaphore * RiCOSSemaphore_create(RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{

	return (RiCOSSemaphore *) NULL;
}
#endif

/**************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
#ifndef RiCOSSemaphore_destroy_Defined
void RiCOSSemaphore_destroy(RiCOSSemaphore * const me)
{

}
#endif
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef RiCOSSemaphore_signal_Defined
RiCOSResult RiCOSSemaphore_signal(RiCOSSemaphore * const me)
{
	return RiCTRUE;
}
#endif

/**************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifndef     RiCOSSemaphore_wait_Defined
RiCOSResult RiCOSSemaphore_wait(RiCOSSemaphore *  const me, RhpUnlimitedNatural timeout)
{
	RiCOSResult retval = (RiCOSResult) 1;
	return retval;
}
#endif



/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#if ( (!(defined OSTIMER_INIT)) || (defined WIN32) )
#ifndef OSTIMER_INIT_Defined
void OSTIMER_INIT(void)
{
}
#endif
#endif

#ifndef RiCOSOS_get_current_time_Defined
void RiCOSOS_get_current_time(OSTimeUnit *currentTickP)
{

}
#endif

