#ifndef RiCOSMainloop_H
#define RiCOSMainloop_H 

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
*/

#include "mxf_cfg.h"
#include "RiCEventQueue.h"
/* Target Specific OS services and types */
#include <RiCOSTargetServices.h>
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "omsdata.h"
#else
#include "omcom/omsdata.h"
#endif
#endif


	/***************************************************/
	/****			OS specific Types				****/
	/***************************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	typedef void (*RiCOS_dispatchfunc)(RiCSData *);
#endif

#ifndef TargetOSTypes_Defined	
	#ifdef CONSTANT_AS_UNSIGNED_INT
		typedef RiC_UINT32 RiCOSResult;
	#else
		typedef RiC_UCHAR8 RiCOSResult;
	#endif
	typedef RiC_UCHAR8 RiCOSHandle;
	typedef RiC_UCHAR8 RiCOSEventHandle;
	typedef RiC_UCHAR8 RiCOSTimerHandle; 
	typedef RhpAddress OSMutexHandle;
	typedef RiC_UCHAR8 RiCOSEventFlag;
#endif
	
	typedef void (*RiCOSTaskEndCallBack)(RhpAddress pAddr);
	typedef timeUnit OSTimeUnit;

#ifndef RiCOSMutex_Defined
#define RiCOSMutex_Defined
	/***************************************************/
	/****				RiCOSMutex					****/
	/***************************************************/
	typedef struct RiCOSMutex_t {
		OSMutexHandle hMutex;
	} RiCOSMutex;
#endif
/* Justification: Violation of Misra98 Rule96 */
/* Macro body expands to braced initializer   */
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 79 S */
#define STATIC_INIT_RICOS_MUTEX() { (OSMutexHandle)0 }
	
	/***************************************************/
	/****				RiCOSTask					****/
	/***************************************************/
	typedef struct RiCOSTask_t {
		RiCOSHandle osTask;
		RiCOSEventHandle queueEvent;
		RiCBoolean			 pendingEvents;    /* the eventFlag to be used with a Asynchronized Active */
	} RiCOSTask;
	
	
	/***************************************************/
	/****				RiCOSTimer					****/
	/***************************************************/
	typedef struct RiCOSTimer_t {
		void  (* OXF_CONST cbkfunc)(RhpAddress);
		void  (* OXF_CONST tickCbkFunc)(RhpAddress);
		timeUnit m_Time ;
		RhpAddress param;
	} RiCOSTimer ;

	
	/***************************************************/
	/****			RiCOSMessageQueue				****/
	/***************************************************/
	typedef struct RiCOSMessageQueue_t {
		RiCEventQueue m_baseQueue;
	} RiCOSMessageQueue ;
	
	/***************************************************/
	/****				RiCOSSemaphore				****/
	/***************************************************/
	typedef struct RiCOSSemaphore_t {
		OSMutexHandle hSemaphore;
	} RiCOSSemaphore ;
	
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

	/***************************************************/
	/****			RiCOSConnectionPort				****/
	/***************************************************/

	typedef void (*RiCOSConnectionPortDispatchFunc)(RiCSData *);

	typedef struct RiCOSConnectionPort_t {
		RiCOSMutex m_SendMutex ;
		RhpInteger m_Connected;
		RiCOSConnectionPortDispatchFunc m_dispatchfunc;
		RiCOSHandle hSocketHandleThread;
		RiCOSSocket m_ntSocket;
		RhpInteger m_ShouldWaitForAck;
		RiCOSEventFlag m_AckEventFlag;
		RhpInteger m_NumberOfMessagesBetweenAck;
		RiCBoolean m_exitOnDisconnect;
	} RiCOSConnectionPort ;

void OM_NOTIFY_ERROR(const RhpString callStr, const RhpString funcStr);
#endif

#ifndef OSTIMER_INIT

#if ( (!(defined OSTIMER_INIT)) || (defined WIN32) )
void OSTIMER_INIT(void);
#endif

#endif

RiCBoolean RiCOSMainTask_WaitStep(void); 


#endif
