/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICOSOSEK_SOURCE
#define MXF_RICOSOSEK_SOURCE
#endif

#include "RiCOSOsek.h"
#include "RiCOSWrap.h"

/***************************************************/
/****				Default values				****/
/***************************************************/
const RhpUnlimitedNatural RiCOSDefaultMessageQueueSize = 100L;
const RhpUnlimitedNatural RiCOSDefaultThreadPriority = 1L;


void RiCOSOXFInitEpilog(void)
{
}
/***************************************************/
/****				OS Event Flag				****/
/***************************************************/

RiCBoolean RiCOSEventFlag_init(RiCOSEventFlag * const me, RiCOSTask * const context)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
		if(context != NULL){
			me->hEventFlag        = context->queueEvent;
			me->hEventFlagItsTask = context->osTask;
			
			retval                = RiCTRUE;
		}
	}
	
	return retval;
}

void RiCOSPendingEvent_init(RiCOSTask * const me, RiCOSTask * const context)
{
}

void RiCOSPendingEvent_clear(RiCOSTask * const pOsTask)
{
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSEventFlag_cleanup(RiCOSEventFlag * const me)
{
}
#endif

/**************/
RiCOSEventFlag * RiCOSEventFlag_create(void)
{
	return NULL;
}

/**************/
void RiCOSEventFlag_destroy(RiCOSEventFlag * const me)
{

}

/**************/
RiCOSResult RiCOSEventFlag_wait(RiCOSEventFlag * const me, RhpInteger tminms)
{
	RiCOSResult retval = ((RiCOSResult)0);
#ifdef OSEVENT
	StatusType status = E_OK;

	if (me != NULL) { 
		status = WaitEvent(me->hEventFlag);
		retval = (RiCOSResult)(status == E_OK);
	}
#endif
	
	return retval;
}

/**************/
RiCOSResult RiCOSEventFlag_signal(RiCOSEventFlag * const me)
{
	RiCOSResult retval = ((RiCOSResult)0);
#ifdef OSEVENT
	StatusType status = E_OK;

	if (me != NULL) { 
		status = SetEvent(me->hEventFlagItsTask, me->hEventFlag);
		retval = (RiCOSResult)(status == E_OK);
	}
	
#endif
	return retval;
}

RiCOSResult RiCOSPendingEvent_signal(RiCOSTask *pOsTask)
{
	return (RiCOSResult) 0;
}

/**************/
RiCOSResult RiCOSEventFlag_reset(RiCOSEventFlag * const me)
{
	RiCOSResult retval = ((RiCOSResult)0);
#ifdef OSEVENT
	StatusType status = E_OK;

	if (me != NULL) { 
		status = ClearEvent(me->hEventFlag);
		retval = (RiCOSResult)(status == E_OK);
	}
	
#endif
	return retval;
}

RiCOSResult RiCOSPendingEvent_reset(RiCOSTask *pOsTask)
{
	return (RiCOSResult) 0;
}

RiCBoolean RiCOSEventFlag_hasPendingEvents(const RiCOSTask * const pOsTask)
{
	return RiCTRUE;
}
/***************************************************/
/****				OS Mutex					****/
/***************************************************/

/**************/
RiCBoolean RiCOSMutex_init(RiCOSMutex * const me, OSMutexHandle mutexIdentifier) 
{
	RiCBoolean retval = RiCFALSE;
	if(me != NULL){
		me->hMutex = mutexIdentifier;
		retval = RiCTRUE;
	}
	return retval;
}  

/**************/
void RiCOSMutex_cleanup(const RiCOSMutex * const me)
{
}

/**************/
RiCOSMutex * RiCOSMutex_create(void)
{
	return NULL;
}

/**************/
void RiCOSMutex_destroy(RiCOSMutex * const me)
{

}


/**************/	
RhpBoolean RiCOSMutex_lock(const RiCOSMutex *me)
{
	RhpBoolean retval = RiCFALSE;

	if(me != NULL){
		retval = GetResource(me->hMutex);
	}
	return retval;
}

/**************/
RhpBoolean RiCOSMutex_free(const RiCOSMutex *me)
{
	RhpBoolean retval = RiCFALSE;

	if(me != NULL){
		retval = ReleaseResource(me->hMutex);
	}
	return retval;
}

/**************/	
RhpBoolean RiCOSTimerManagerMutex_lock(const RiCOSMutex *me)
{
	return GetResource(TIMER_TASK_RES);
}

/**************/
RhpBoolean RiCOSTimerManagerMutex_free(const RiCOSMutex *me)
{
	return ReleaseResource(TIMER_TASK_RES);
}

void RiCOSEventsPoolMutex_init(void)
{
}

void	RiCOSEventsPoolMutex_lock(void)
{
	GetResource(EVENTS_POOL_RES);
}

void RiCOSEventsPoolMutex_free(void)
{
	ReleaseResource(EVENTS_POOL_RES); 
}

/***************************************************/
/****				OS Task						****/
/***************************************************/
RiCOSResult RiCOSTask_wrap(RiCOSTask * const me, RiCOSHandle threadOsHandle)
{
	RiCOSResult retval = RiCFALSE;
	
	if (me != NULL) { 	
		me->osTask = RiCOSTask_getCurrentTaskHandle();
		
		retval = RiCTRUE;
	}

	return retval;
}

/**************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCOSTask_cleanup(RiCOSTask * const me)
{
}
#endif

/**************/
RiCOSTask * RiCOSTask_create(const RiCOSTaskEndCallBack tfunc, const RhpAddress pParam, const RhpCharacter * const itsName, const RhpUnlimitedNatural itsStackSize)
{

	return NULL;
}


/**************/
RiCOSResult RiCOSTask_setPriority(RiCOSTask * const me, RhpInteger pr)
{
	return ((RiCOSResult)1);
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/**************/
RiCOSResult RiCOSTask_suspend(const RiCOSTask * const me)
{
	RiCOSResult retval = ((RiCOSResult)0);
	StatusType status = E_OK;

	if (me != NULL) { 
		status = Schedule();
		retval = (RiCOSResult)(status == E_OK);
	}
	
	return retval;
}

/**************/
RiCOSResult RiCOSTask_resume(const RiCOSTask * const me)
{
	RiCOSResult retval = ((RiCOSResult)0);
	StatusType status = E_OK;
	
	if (me != NULL) { 
		status = ActivateTask(me->osTask); 
		retval = (RiCOSResult)(status == E_OK);
	}
	
	return retval;
}
#endif
/**************/
RiCOSResult RiCOSTask_start(const RiCOSTask * const me)
{
	RiCOSResult retval = ((RiCOSResult)0);
	StatusType status = E_OK;
	
	if (me != NULL) { 
		status = ActivateTask(me->osTask); 
		retval = (RiCOSResult)(status == E_OK);
	}
	
	return retval;
}

/**************/
RiCOSHandle RiCOSTask_getCurrentTaskHandle(void)
{
	RiCOSHandle retval = ((RiCOSHandle)0);

	(void) GetTaskID(&retval);

	return retval;
}

/**************/
RiCOSHandle RiCOSTask_getOSHandle(const RiCOSTask * const me)
{
	RiCOSHandle retval = ((RiCOSHandle)0);
	if (me != NULL) {
		retval = (me->osTask);
	}
	return retval;
}

#ifdef MICROC_OXF_CLEANUP
/**************/
RhpInteger RiCOSTask_setEndOSTaskInCleanup(const RiCOSTask * const me, RiCBoolean val)
{
	return 0;
}

/**************/
RiCBoolean RiCOSTask_exeOnMyTask(RiCOSTask * const me)
{
	RiCOSHandle executedOsHandle = 0;
	RiCOSHandle myOsHandle       = 0;
	RiCBoolean retval = RiCFALSE;
	
	if (me == NULL) {
		retval = RiCFALSE;
	}
	else{
		
		/* a handle to the thread that executes the delete */
		executedOsHandle = RiCOSTask_getCurrentTaskHandle();
		/* a handle to this' 'thread' */
		myOsHandle = RiCOSTask_getOSHandle(me);
		
		retval = ((executedOsHandle == myOsHandle) ? RiCTRUE : RiCFALSE);
	}
	return retval;
}
#endif

/**************/
void RiCOSTask_endMyTask(RiCOSTask * const me)
{
	(void) TerminateTask();
}

/**************/
RiCBoolean RiCOSTask_endOtherTask(const RhpAddress handle)
{
	return RiCFALSE;
}

/**************/
RhpInteger RiCOSTask_getTaskEndClbk(const RiCOSTask * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteTask)
{
	return 1;
}


void RiCOSMainTask_start(void)
{
	/* Nothing! */
}

/**************/
RhpBoolean RiCOSTimerManager_registerTimedAction(RiCOSTimerHandle timerHandle, OSTimeUnit tDueTime, OSTimeUnit tPeriod)
{
	/* Return False to have the TimerManager handle the timedAction registretion */
	RiCBoolean retval = RiCFALSE;
	
	if((tDueTime != 0) && (tPeriod != 0)){
		SetAbsAlarm(timerHandle, tDueTime, tPeriod);
		retval = RiCTRUE;
	}
	
	return retval;
}

RhpBoolean RiCOSTimerManager_getSystemTime(OSTimeUnit *sysTime)
{
	return RiCFALSE;
}

/***************************************************/
/****					OS Timer				****/
/***************************************************/
static RiCOSTimer * cachedTimer;

void resetTimerCallbackFuction(void)
{
	cachedTimer = NULL;
}

static void setTimerCallbackFunction(RiCOSTimer * const me);

static 
void setTimerCallbackFunction(RiCOSTimer * const me)
{
	cachedTimer = me;
}

void setTimerActivationCycle(void)
{
	if(cachedTimer != NULL){
        SetRelAlarm(TIMER_TASK_ALARM, (TickType)(cachedTimer->m_Time), (TickType)(cachedTimer->m_Time));
	}
}

void runTimerCallbackFuction(void)
{
	if(cachedTimer != NULL){
		(*(cachedTimer->tickCbkFunc))(cachedTimer->param);
		(*(cachedTimer->cbkfunc))(cachedTimer->param);
	}
}


/**************/
RiCBoolean RiCOSTimer_init(RiCOSTimer * const me)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
		setTimerCallbackFunction(me);
		retval = RiCTRUE;
	}

	return retval;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSTimer_cleanup(RiCOSTimer * const me)
{
	/* Nothing! */
}
#endif

/**************/
RiCOSTimer * RiCOSTimer_create(timeUnit ptime, void (* const cllbkfunc)(RhpAddress), const RhpAddress params)
{

	return NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSTimer_destroy(RiCOSTimer * const me)
{

}
#endif


/***************************************************/
/****			OS Message Queue				****/
/***************************************************/
RiCBoolean RiCOSMessageQueue_init(RiCOSMessageQueue * const me, RiCEvent** const evQueue, RhpPositive eventQueueSize)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) { 
		(void) RiCEventQueue_Init(&(me->m_baseQueue), evQueue, eventQueueSize);
		
		retval = RiCTRUE;
	}

	return retval;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSMessageQueue_cleanup(RiCOSMessageQueue * const me)
{
	if (me != NULL){
		RiCEventQueue_Cleanup(&(me->m_baseQueue));
	}
}
#endif

/**************/
RiCOSMessageQueue * RiCOSMessageQueue_create(RiCBoolean shouldGrow, RhpInteger initSize)
{
	return NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSMessageQueue_destroy(RiCOSMessageQueue * const me)
{

}
#endif

/*lint -save -e534 (Ignoring return value in the mutex lock() and free()) */
/**************/
RiCOSResult RiCOSMessageQueue_put(RiCOSMessageQueue * const me, RhpAddress const message, RiCBoolean fromISR)
{
	RiCOSResult retval = RiCFALSE;
	
	if(me != NULL){
		retval = RiCEventQueue_push(&(me->m_baseQueue), message);
	}
	return retval;
}	/*lint !e715 (Symbol 'fromISR' not referenced) */

/**************/
RiCEvent* RiCOSMessageQueue_get(RiCOSMessageQueue * const me)
{
	RiCEvent* m = NULL;
	
	/* safe programming */
	if (me != NULL){
		if(RiCOSMessageQueue_isEmpty(me) == RiCFALSE) {
			m = RiCEventQueue_pop(&(me->m_baseQueue));
		} 
		else {
			m = NULL;
		}
	}

	return m;
}

/**************/
RiCBoolean RiCOSMessageQueue_isFull(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCFALSE;
	if(me != NULL){
		retval = RiCEventQueue_isFull(&(me->m_baseQueue));
	}
	return retval;
}

/**************/
RiCBoolean RiCOSMessageQueue_isEmpty(const RiCOSMessageQueue * const me)
{
	RiCBoolean retval = RiCTRUE;
	if(me != NULL){
		retval = RiCEventQueue_isEmpty(&(me->m_baseQueue));
	}
	return retval;
}

/**************/
RiCOSResult RiCOSMessageQueue_pend(const RiCOSMessageQueue * const me)
{
	RiCOSResult retval = 0;
	if (me != NULL){
		retval = 1;
	}

	return retval;
}

/**************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCOSResult RiCOSMessageQueue_getMessageList(const RiCOSMessageQueue * const me, RiCList * const pList)
{
	RiCOSResult retval = 0;
	RiCList_removeAll(pList);
	
	if (me != NULL){
		RiCEventQueue_getQueue(&(me->m_baseQueue), pList);
		retval = 1;
	}

	return retval;
}
#endif


/***************************************************/
/****			OS Semaphore					****/
/***************************************************/
RiCBoolean RiCOSSemaphore_init(RiCOSSemaphore * const me, RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{
	return (RiCTRUE);
} /*lint !e715 (Symbol 'semFlags' not referenced) */

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSSemaphore_cleanup(RiCOSSemaphore * const me)
{
}
#endif

/**************/
RiCOSSemaphore * RiCOSSemaphore_create(RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name)
{
	return NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSSemaphore_destroy(RiCOSSemaphore * const me)
{

}
#endif

/**************/
RiCOSResult RiCOSSemaphore_signal(RiCOSSemaphore * const me)
{
	RiCOSResult retval = RiCFALSE;
	if (me != NULL)
	{
		ReleaseResource(me->hSemaphore);
		retval = RiCTRUE;
	}
	return retval;
}

/**************/
RiCOSResult RiCOSSemaphore_wait(RiCOSSemaphore *  const me, RhpUnlimitedNatural timeout)
{
	RiCOSResult retval = RiCFALSE;
	if (me != NULL) { 
		GetResource(me->hSemaphore);
		retval = ((RiCOSResult)1);
	}

	return retval;
}



#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#include <stdio.h>
#ifdef MICROC_FLAT_INCLUDE
#include "omsdata.h"
#else
#include "omcom/omsdata.h"
#endif
/***************************************************/
/****			OS ConnectionPort				****/
/***************************************************/

/** shared ConnectionPort info **/
struct {
	RhpString m_Buf;
	RhpInteger m_BufSize;
} RiCNTCPCommon;

/*
 Private functions
*/

RhpInteger NTCP_initializeWinSock(RiCOSConnectionPort * const me)
{
	RhpInteger retval = 0;
	static RhpInteger initialized = 0;
	
	if (me != NULL) {
		if ( 0 == initialized ) {
			WORD wVersionRequested;
			WSADATA wsaData;
			RhpInteger err;
			wVersionRequested = MAKEWORD(1, 1);
			
			err = WSAStartup(wVersionRequested, &wsaData);
			
			if (err != 0)  {
				/* Tell the user that we couldn't find a useable */
				/* winsock.dll.     */
				retval = 0;
			}
			else {
				retval = 1;
			}
			/* Confirm that the Windows Sockets DLL supports 1.1.*/
			/* Note that if the DLL supports versions greater */
		}
		retval =  1;
	}

	return retval;
}

void readFromSocket(RiCOSConnectionPort * me)
{
	RhpCharacter lenStr[MAX_LEN_STR + 1];
	RhpInteger len;
	RhpInteger rv;

	if (me != NULL)
	{
		rv = RiCOSSocket_receive(&(me->m_ntSocket), lenStr, MAX_LEN_STR);
		if (rv == -1) {		/* critical error */
			exit(0);
		}

		lenStr[MAX_LEN_STR] = '\0';
		if (sscanf(lenStr,"%d",&len) == 1) 
		{
			if (len > RiCNTCPCommon.m_BufSize ) {	/* need to increase the buffer */
				RiCNTCPCommon.m_BufSize = 1000 + len;
				if (RiCNTCPCommon.m_Buf) {
					RiC_free(RiCNTCPCommon.m_Buf);
				}
				RiCNTCPCommon.m_Buf = (RhpString) RiC_malloc((RhpPositive)RiCNTCPCommon.m_BufSize);
			}
			
			if (len == -2) { /* Acknoledge message */
				(void) RiCOSEventFlag_signal(&(me->m_AckEventFlag));
			}
			else {
				RiCNTCPCommon.m_Buf[len] = '\0';
				rv = RiCOSSocket_receive(&(me->m_ntSocket), RiCNTCPCommon.m_Buf,len);
				if (rv == -1) {		/* critical error */
					exit(0);
				}
					
				/*
				 len is not used by RiCSData but we need to change
				 the interface so we will use the length of the data
				 and not assume that the string is null terminated
				
				 if socket thread is doing some work that may send
				 messages we do not want to Block the Send by wainting for
				 acknoldegment since the thread cannot handle the acknoledgment
				 ( since the thraed is self locked)
				*/
				me->m_ShouldWaitForAck = 0;
				(void) RiCOSEventFlag_signal(&(me->m_AckEventFlag));
				me->m_dispatchfunc(RiCSData_string2RiCSData(RiCNTCPCommon.m_Buf));
				me->m_ShouldWaitForAck = 1;
			}
		}
	}
}
/**************/

void NTCP_readFromSockLoop(RiCOSConnectionPort * me)
{
	if (me != NULL){

		while (1) 
		{	/*lint !e716 (while(1) ... ) */	
			readFromSocket(me);
		}
	}
}



/*
 Public function implementation
*/

RiCBoolean RiCOSConnectionPort_init(RiCOSConnectionPort * const me)
{
	RiCBoolean retval;
	static RiCBoolean bufInitialized = RiCFALSE;
	
	if (me != NULL){
		/** init common info **/
		if (!bufInitialized) {
			RiCNTCPCommon.m_Buf = NULL;
			RiCNTCPCommon.m_BufSize = 0;
			bufInitialized = RiCTRUE;
		}
		
		retval = RiCOSMutex_init(&(me->m_SendMutex), (OSMutexHandle)0);
		retval &= RiCOSSocket_init(&(me->m_ntSocket));
		retval &= RiCOSEventFlag_init(&(me->m_AckEventFlag), NULL);
		(void) RiCOSEventFlag_reset(&(me->m_AckEventFlag));
		
		me->m_Connected = 0;
		me->m_dispatchfunc = NULL;
		me->hSocketHandleThread = NULL;
		me->m_ShouldWaitForAck = 1;
		me->m_NumberOfMessagesBetweenAck = 0;
	}

	return retval;
}

/**************/
void RiCOSConnectionPort_cleanup(RiCOSConnectionPort * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL){
		RiCOSMutex_cleanup(&(me->m_SendMutex));
		RiCOSSocket_cleanup(&(me->m_ntSocket));
		RiCOSEventFlag_cleanup(&(me->m_AckEventFlag));
		
		
		/*
		 I assume that we will have only one connection port
		 so I can free the data for m_Buf but if it is not the case
		 it does not matter ( the readFromSockLoop will allocate it )
		*/
		if (RiCNTCPCommon.m_Buf) {
			RiC_free(RiCNTCPCommon.m_Buf);
			RiCNTCPCommon.m_Buf = NULL;
		}
		RiCNTCPCommon.m_BufSize = 0;
		
		/* close the animation thread */
		/* if (me->hSocketHandleThread) TerminateThread(me->hSocketHandleThread,0); */
		
		me->m_dispatchfunc = NULL;
		me->hSocketHandleThread = (HANDLE)NULL;
	}
#endif
}


/**************/
RiCOSConnectionPort * RiCOSConnectionPort_create(void)
{
	return NULL;
}

/**************/
void RiCOSConnectionPort_destroy(RiCOSConnectionPort * const me)
{

}


/**************/
RhpInteger RiCOSConnectionPort_Connect(RiCOSConnectionPort * const me, const RhpCharacter * const SocketAddress,RhpPositive nSocketPort)
{
	RhpInteger retval = 0;
	RiCBoolean flag = RiCFALSE;

	if ((me != NULL) && (0  != NTCP_initializeWinSock(me)) && (NULL != me->m_dispatchfunc)){
		if ( 0  == me->m_Connected ) {
			me->m_Connected = RiCOSSocket_createSocket(&(me->m_ntSocket), SocketAddress,nSocketPort);
			if (0 == me->m_Connected){
				flag = RiCTRUE;
			}
		}
		/*
		 Connection established
		 invoking thread to recieve messages from the socket
		*/

		if(flag == RiCFALSE){
			RiCOSHandle  ThreadId; 	
			
#ifdef __USE_CREATE_THREAD__
				me->hSocketHandleThread = CreateThread( NULL,  0  /* default stack size  */, 
							(LPTHREAD_START_ROUTINE) NTCP_readFromSockLoop, /* thread function   */
							me, /* argument to thread  */    
							0,  /* use default creation flags    */
							&ThreadId);  /* returns the thread identifier */
#else
				me->hSocketHandleThread = (HANDLE) _beginthreadex( NULL,  0  /* default stack size  */, 
							(start_addr_t) NTCP_readFromSockLoop, /* thread function   */
							me, /* argument to thread  */
							0,  /* use default creation flags    */
							(UINT*)&ThreadId);  /* returns the thread identifier */
#endif
		}
		retval = me->m_Connected;
	}

	return retval;
}

/**************/
RhpInteger RiCOSConnectionPort_Send(RiCOSConnectionPort * const me, RiCSData * m)
{
	RhpInteger retval = 0;
	
	if (me != NULL){
		(void)RiCOSMutex_lock(&(me->m_SendMutex));
		if (me->m_Connected) {
			RhpCharacter lenStr[MAX_LEN_STR+1];
			
			retval = RiCOSSocket_send(&(me->m_ntSocket),lenStr,MAX_LEN_STR);
			if (retval > 0) {
				retval = RiCOSSocket_send(&(me->m_ntSocket),RiCSData_getRawData(m),(RhpInteger) RiCSData_getLength(m));
			}
			
			/* need to check with Rhapsody */
			if (me->m_ShouldWaitForAck) {
				const RhpInteger maxNumOfMessagesBetweenAck = 127; /* MUST match the number in rhapsody */
				if (maxNumOfMessagesBetweenAck > 0) { /*lint !e774 !e506 () */
					me->m_NumberOfMessagesBetweenAck++;
					if (me->m_NumberOfMessagesBetweenAck >= maxNumOfMessagesBetweenAck) {
						me->m_NumberOfMessagesBetweenAck = 0;
						(void) RiCOSEventFlag_wait(&(me->m_AckEventFlag), -1);
						(void) RiCOSEventFlag_reset(&(me->m_AckEventFlag));
					}
				}
			}
			
		}
		(void)RiCOSMutex_free(&(me->m_SendMutex));
		
		/* cleanup */
		RiCSData_cleanup(m);
	}
	
	return retval;
}
/*lint -restore (ignoring return value on mutex lock() and free()) */

/**************/
RiCBoolean RiCOSConnectionPort_SetDispatcher(RiCOSConnectionPort * const me, RiCOS_dispatchfunc dispfunc)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
		me->m_dispatchfunc = dispfunc;
		retval = RiCTRUE;
	}

	return retval;
}


/***************************************************/
/****					OS Socket				****/
/***************************************************/

/*
 Private definitions and functions
*/

#include <stdlib.h>
#include <stdio.h>

/*
 Public RiCOSSocket functions (implementation)
*/

RiCBoolean RiCOSSocket_init(RiCOSSocket * const me)
{
	RiCBoolean retval = RiCFALSE;
	if (me != NULL) {
		me->theSock = 0;
		retval = RiCTRUE;
	}

	return retval;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSSocket_cleanup(RiCOSSocket * const me)
{
	if (me != NULL){
		if (me->theSock != 0) (void) closesocket(me->theSock);
		(void) WSACleanup();
	}
}
#endif

/**************/
RiCOSSocket * RiCOSSocket_create(void)
{
	return NULL;
}

/**************/
#ifdef MICROC_OXF_CLEANUP
void RiCOSSocket_destroy(RiCOSSocket * const me)
{

}
#endif

/**************/
RhpInteger RiCOSSocket_createSocket(RiCOSSocket * const me, const RhpCharacter * const SocketAddress, RhpPositive nSocketPort)
{
	RhpInteger retval = 0;
	RiCBoolean flag = RiCFALSE;
	static struct sockaddr_in	addr;
	struct protoent	*protocol;
	RhpInteger	proto;
	struct hostent	*host = NULL;
	RhpString proto_name = "tcp";
	RhpCharacter	hostName[128];
	RhpInteger rvStat;
	if ( nSocketPort == 0 ) nSocketPort = 6423;
	
	addr.sin_family = AF_INET;
	protocol = getprotobyname(proto_name);
	if (protocol != (struct protoent *) 0) {
		proto = protocol->p_proto;
		
		(void) gethostname(hostName, sizeof(hostName)-1);
		if (NULL != SocketAddress && strlen(SocketAddress) != 0) {
			if (!strcmp(hostName,SocketAddress)) SocketAddress = NULL;
			else {
				strcpy(hostName,SocketAddress);
				if ((host = gethostbyname(hostName)) == (struct hostent *) 0) {
					flag = RiCTRUE;
				}
			}
		}
		
		if(flag == RiCFALSE) {
#ifdef unix
#ifndef	CYGWIN
			endprotoent();
#endif
#endif
			
			addr.sin_port = htons((u_short)nSocketPort);
			
			if (NULL == SocketAddress || strlen(SocketAddress) == 0) addr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
			else if(host) addr.sin_addr.s_addr = *((RhpUnlimitedPositiveNatural *) host->h_addr);
			
			
			if (me != NULL)
			{
				if ((me->theSock = socket(AF_INET, SOCK_STREAM, proto)) == INVALID_SOCKET) {
					me->theSock = 0;
				}
				else {
					/*lint -save -e740 (Unusual pointer cast) */
					while ((rvStat = connect(me->theSock, (struct sockaddr *) &addr, sizeof(addr))) == 
						SOCKET_ERROR && (WSAGetLastError() == WSAEINTR))
						;
					/*lint -restore */
				
					if (SOCKET_ERROR != rvStat) {
						retval = 1;
					}
				}
			}
		}
	}
	
	return retval;
}

/**************/
RhpInteger RiCOSSocket_send(RiCOSSocket * const me, const RhpCharacter * const buf, RhpInteger bufLen)
{
	RhpInteger bytes_writ = 0;
	RhpInteger n;
	RhpInteger errFlag = 0;
	RiCBoolean flag = RiCFALSE;
	
	if (me != NULL) {
		while ((bytes_writ < bufLen) && (flag == RiCFALSE)){
			errFlag = 1;
			n = send(me->theSock, buf + bytes_writ, bufLen - bytes_writ,0);
			if (SOCKET_ERROR == n) {
				if (WSAGetLastError() == WSAEINTR){
					errFlag = 0;
				}
				else{
					flag = RiCTRUE;
					bytes_writ = -1;
				}
			}

			if((errFlag == 1) && (flag == RiCFALSE)){
				bytes_writ += n;
			}
		}
	}
	
	return bytes_writ;
}

/**************/
RhpInteger RiCOSSocket_receive(RiCOSSocket * const me, RhpString buf, RhpInteger bufLen)
{
	RhpInteger bytes_read = 0;
	RhpInteger n;
	RiCBoolean flag = RiCFALSE;
	RhpInteger errFlag = 0;
	
	if (me != NULL) {
		while ((bytes_writ < bufLen) && (flag == RiCFALSE)){
			errFlag = 1;
			n = recv(me->theSock, buf + bytes_read, bufLen - bytes_read,0);
			if (SOCKET_ERROR == n) {
				if (WSAGetLastError() == WSAEINTR){
					errFlag = 0;
				}
				else{
					flag = RiCTRUE;
					bytes_writ = -1;
				}
			}

			if((errFlag == 1) && (flag == RiCFALSE)){
				bytes_read += n;
			}
		}
	}

	return bytes_read;
}



#endif


/*+*****************************************************************************
 * Input:     event flag pointer.
 *
 * Output:    None.
 *
 * 
 * Function:  Empty function to overcome MISRA violation
 *
 ****************************************************************************-*/
void RiCOSEventFlag_clear(RiCOSEventFlag *evFlag)
{
}

void RiCOSOS_get_current_time(OSTimeUnit *currentTickP)
{
	GetCounterValue((SYS_TIMER), (currentTickP));
}

void	RiCOSTask_InitCommunicationLayer(void)
{
}

void	RiCOSTask_CleanupCommunicationLayer(void)
{
}
