#ifndef RiCOSWrap_H
#define RiCOSWrap_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCOSWrap.h                        |
 --------------------------------------------------------------- 
*********************************************************************/
  


#include "RiCOS.h"
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "RiCList.h"
#else
#include "Utils/RiCList.h"
#endif
#endif
/*********************************/
/**		Forward Declerations	**/
/*********************************/
/*LDRA_INSPECTED 141 S : Type forward declaration */
struct RiCSData;

/*********************************/
/**	OS specific default values  **/
/*********************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining objects that might be used	 */
/* Outside of the OXF itself by Generated code					 */
PUBLIC const RhpUnlimitedNatural RiCOSDefaultMessageQueueSize;
PUBLIC const RhpUnlimitedNatural RiCOSDefaultThreadPriority;
PUBLIC const RhpUnlimitedNatural RiCOSDefaultStackSize;


/*********************************/
/**	OS specific initialization  **/
/*********************************/

void RiCOSEndApplication(RiC_INT32 errorCode);


void RiCOSOXFInitEpilog(void);

/*********************************/
/**		EventFlag interface		**/
/*********************************/
/* init and cleanup */
RiCBoolean RiCOSEventFlag_init(RiCOSEventFlag * const me, RiCOSTask * const context);
void RiCOSPendingEvent_init(RiCOSTask * const me, RiCOSTask * const context);

void RiCOSEventFlag_cleanup(RiCOSEventFlag * const me);


/* create and destroy */
RiCOSEventFlag * RiCOSEventFlag_create(void);

void RiCOSEventFlag_destroy(RiCOSEventFlag * const me);

void RiCOSEventFlag_clear(RiCOSEventFlag *evFlag);
void RiCOSPendingEvent_clear(RiCOSTask  * const pOsTask);

void RiCOSOS_get_current_time(OSTimeUnit *currentTickP); 

/* wait of an event of tminms miliseconds (-1 == forever) */
RiCOSResult RiCOSEventFlag_wait(RiCOSEventFlag * const me, RhpInteger tminms);

/* signal event arived */
RiCOSResult RiCOSEventFlag_signal(RiCOSEventFlag * const me);
RiCOSResult RiCOSPendingEvent_signal(RiCOSTask *pOsTask);

/* reset the event */
RiCOSResult RiCOSEventFlag_reset(RiCOSEventFlag * const me);
RiCOSResult RiCOSPendingEvent_reset(RiCOSTask *pOsTask);

/* check if the object has pending events */
RiCBoolean RiCOSEventFlag_hasPendingEvents(const RiCOSTask * const pOsTask);

/*********************************/
/**		Mutex interface			**/
/*********************************/
/* init and cleanup */
RiCBoolean RiCOSMutex_init(RiCOSMutex * const me, OSMutexHandle mutexIdentifier);
void	   RiCOSMutex_cleanup(const RiCOSMutex * const me);

/* create */
RiCOSMutex * RiCOSMutex_create(void);

void RiCOSMutex_destroy(RiCOSMutex * const me);


/*********************************/
/**		Task interface			**/
/*********************************/

/* Cleanup IP resources allocated for the thread (required by some adapters)*/
void RiCOSTask_CleanupCommunicationLayer(void);

/* Initialize IP resources allocated for the thread (required by some adapters).*/
void RiCOSTask_InitCommunicationLayer(void);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCOSTask_cleanup(RiCOSTask * const me);
#endif

void RiCOSMainTask_start(void);

/* create and destroy */
RiCOSTask * RiCOSTask_create(const RiCOSTaskEndCallBack tfunc, const RhpAddress pParam, const RhpCharacter * const itsName, const RhpUnlimitedNatural itsStackSize);

/* get the active task */
RiCOSHandle RiCOSTask_getCurrentTaskHandle(void);

/* set the task priority */
RiCOSResult RiCOSTask_setPriority(RiCOSTask * const me, RhpInteger pr);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* suspend the task */
RiCOSResult RiCOSTask_suspend(const RiCOSTask * const me);

/* resume suspendent task */
RiCOSResult RiCOSTask_resume(const RiCOSTask * const me);
#endif
/* start executing the task  */
RiCOSResult RiCOSTask_start(const RiCOSTask * const me);

/* return a handle to the os underlying task */
RiCOSHandle RiCOSTask_getOSHandle(const RiCOSTask * const me);

#ifdef MICROC_OXF_CLEANUP
/* if RiCTRUE the task will be ended at the RiCOSTask cleanup */
RhpInteger RiCOSTask_setEndOSTaskInCleanup(const RiCOSTask * const me, RiCBoolean val);

/* returns RiCTRUE if it is invoked from the the same os task as the on the object it represents */
RiCBoolean RiCOSTask_exeOnMyTask(RiCOSTask * const me);
#endif

/* end the task I'm working on */
void RiCOSTask_endMyTask(RiCOSTask * const me);

/* end a thread which is NOT my thread */
RiCBoolean RiCOSTask_endOtherTask(const RhpAddress handle);

/* asking for a callback to end my os thread 

 Parameters:
		clb_p  : a pointer to the callback function
		arg1_p : the argument for this function
		onExecuteThread :
			TRUE: I will kill my own thread
			FALSE: someone else will kill my thread
*/
RhpInteger RiCOSTask_getTaskEndClbk(const RiCOSTask * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteTask);


/*********************************/
/**		Timer interface			**/
/*********************************/

/* init and cleanup */
RiCBoolean RiCOSTimer_init(RiCOSTimer * const me);
#ifdef MICROC_OXF_CLEANUP
void RiCOSTimer_cleanup(RiCOSTimer * const me);
void RiCOSTimer_destroy(RiCOSTimer * const me);
#endif

/* create */
RiCOSTimer * RiCOSTimer_create(timeUnit ptime, void (* const cllbkfunc)(RhpAddress), const RhpAddress params);

/* timed actions */
RhpBoolean RiCOSTimerManager_registerTimedAction(RiCOSTimerHandle timerHandle, OSTimeUnit tDueTime, OSTimeUnit tPeriod);

RhpBoolean RiCOSTimerManager_getSystemTime(OSTimeUnit *sysTime);


/*********************************/
/**			Message Queue		**/
/*********************************/

/* init and cleanup */
RiCBoolean RiCOSMessageQueue_init(RiCOSMessageQueue * const me, RiCEvent** const evQueue, RhpPositive eventQueueSize);
#ifdef MICROC_OXF_CLEANUP
void RiCOSMessageQueue_cleanup(RiCOSMessageQueue * const me);
void RiCOSMessageQueue_destroy(RiCOSMessageQueue * const me);
#endif

/* create */
RiCOSMessageQueue * RiCOSMessageQueue_create(RiCBoolean shouldGrow, RhpInteger initSize);

/* put a message on the message queue */
RiCOSResult RiCOSMessageQueue_put(RiCOSMessageQueue * const me, RhpAddress const message, RiCBoolean fromISR);

/* get a message from the message queue */
RiCEvent* RiCOSMessageQueue_get(RiCOSMessageQueue * const me);

/**************/
/* check if the message queue is empty */
RiCBoolean RiCOSMessageQueue_isEmpty(const RiCOSMessageQueue * const me);

/* check if the message queue is full */
RiCBoolean RiCOSMessageQueue_isFull(const RiCOSMessageQueue * const me);

/* pend on the message queue for messages */
RiCOSResult RiCOSMessageQueue_pend(const RiCOSMessageQueue * const me);

/* get the messages frin the queue in a list */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCOSResult RiCOSMessageQueue_getMessageList(const RiCOSMessageQueue * const me, RiCList * const pList);
#endif



/*********************************/
/**			Semaphore			**/
/*********************************/

/* init and cleanup */
RiCBoolean RiCOSSemaphore_init(RiCOSSemaphore * const me, RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name);
#ifdef MICROC_OXF_CLEANUP
void RiCOSSemaphore_cleanup(RiCOSSemaphore * const me);
void RiCOSSemaphore_destroy(RiCOSSemaphore * const me);
#endif

/* create */
RiCOSSemaphore * RiCOSSemaphore_create(RhpUnlimitedPositiveNatural semFlags, RhpUnlimitedPositiveNatural initialCount, RhpUnlimitedPositiveNatural maxCount, const RhpCharacter * const name);

/* free semaphore token */
RiCOSResult RiCOSSemaphore_signal(RiCOSSemaphore * const me);

/* wait for a semaphore token */
RiCOSResult RiCOSSemaphore_wait(RiCOSSemaphore *  const me, RhpUnlimitedNatural timeout);


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/*********************************/
/**		ConnectionPort			**/
/*********************************/

#define MAX_LEN_STR 6

/* init and cleanup */
RiCBoolean RiCOSConnectionPort_init(RiCOSConnectionPort * const me);

void RiCOSConnectionPort_cleanup(RiCOSConnectionPort * const me);
void RiCOSConnectionPort_destroy(RiCOSConnectionPort * const me);

/* create */
RiCOSConnectionPort * RiCOSConnectionPort_create(void);

/* connect to a given socket */
RhpInteger RiCOSConnectionPort_Connect(RiCOSConnectionPort * const me, const RhpCharacter * const SocketAddress,RhpPositive nSocketPort);

/* send a message (should be thread protected)*/
RhpInteger RiCOSConnectionPort_Send(RiCOSConnectionPort * const me, struct RiCSData *m);

/* set the connection dispatcher */
RiCBoolean RiCOSConnectionPort_SetDispatcher(RiCOSConnectionPort * const me, RiCOS_dispatchfunc dispfunc);

/*********************************/
/**				Socket			**/
/*********************************/

/* init and cleanup */
RiCBoolean RiCOSSocket_init(RiCOSSocket * const me);
#ifdef MICROC_OXF_CLEANUP
void RiCOSSocket_cleanup(RiCOSSocket * const me);
void RiCOSSocket_destroy(RiCOSSocket * const me);
#endif

/* create */
RiCOSSocket * RiCOSSocket_create(void);

/* create the underlyng socket */
RhpInteger RiCOSSocket_createSocket(RiCOSSocket * const me, const RhpCharacter * SocketAddress, RhpPositive nSocketPort);

/* send date via the socket */
RhpInteger RiCOSSocket_send(RiCOSSocket * const me, const RhpCharacter * const buf, RhpInteger bufLen);

/* wait on the socket to receive data */
RhpInteger RiCOSSocket_receive(RiCOSSocket * const me, RhpString buf, RhpInteger bufLen);

#endif

void runTimerCallbackFuction(void);
void setTimerActivationCycle(void);
void resetTimerCallbackFuction(void);
RiCOSResult RiCOSTask_wrap(RiCOSTask * const me, RiCOSHandle threadOsHandle);
RhpBoolean RiCOSMutex_lock(const RiCOSMutex *me);
RhpBoolean RiCOSMutex_free(const RiCOSMutex *me);
RhpBoolean RiCOSTimerManagerMutex_lock(const RiCOSMutex *me);
RhpBoolean RiCOSTimerManagerMutex_free(const RiCOSMutex *me);
void RiCOSEventsPoolMutex_init(void);
void	RiCOSEventsPoolMutex_lock(void);
void RiCOSEventsPoolMutex_free(void);

#endif

