/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCOxf.c				             |
 --------------------------------------------------------------- 
 This file conatains the OXF initialization code
***********************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICOXF_SOURCE
#define MXF_RICOXF_SOURCE
#endif


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "aom.h"
#else
#include "aom/aom.h"
#endif
#endif

#include "RiCOxf.h"
#include "RiCOSWrap.h"
#include "RiCTaskEM.h"
#ifdef RIC_USE_MEMORY_MANAGER
#ifdef MICROC_FLAT_INCLUDE
#include "RiCMemoryManager.h"
#else
#include "Utils/RiCMemoryManager.h"
#endif
#endif
/*************************************************/
/**		Private functions used in RiCOXFInit	**/
/*************************************************/
static const RhpInteger BadOrMissingInitParams = 2;

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

static void rpUsage(RhpString programName)
{
	/* print an error msg if host not found*/
	fprintf(stderr,"Usage: %s [-h[ostname] hostname] [-p[ort] portNumber]\n",programName);
}

/*
ret val
 0 wrong usage
 1 the arg was proccessed
 2 no setting for arg in the program arguments
*/
static RhpInteger rpProcessHostArg(RhpInteger numProgArgs, RhpCharacter **progArgs, RhpCharacter **host)
{
	RhpInteger res = BadOrMissingInitParams;
	RhpInteger i;
	RhpBoolean flag = RiCFALSE;

	/* get name of host from parameters of main */
    *host = NULL;
	if (numProgArgs > 1) {
		for (i = 1; (i < numProgArgs) && (flag == RiCFALSE); i++) {
			if (!strncmp("-hostname",progArgs[i],strlen(progArgs[i]))) {
				if (i < numProgArgs-1) {
					*host = progArgs[i+1];
					res = 1;
					flag = RiCTRUE;
				}
				else {
					rpUsage(progArgs[0]);
					res = 0;
					flag = RiCTRUE;
				}
			}
		}
	}
	return res;
}
/*
ret val
 0 wrong usage
 1 the arg was proccessed
 2 no setting for arg in the program arguments
*/
static RhpInteger rpProcessPortArg(RhpInteger numProgArgs, RhpCharacter **progArgs, RiC_INT32 * port)
{
	RhpInteger res = BadOrMissingInitParams;
	RhpInteger i;
	RhpBoolean flag = RiCFALSE;

	/* get name of host from parameters of main */
    *port = 0;
	if (numProgArgs > 1) {
		for (i = 1; (i < numProgArgs) && (flag == RiCFALSE); i++) {
			if (!strncmp("-port",progArgs[i],strlen(progArgs[i]))) {
				if (i < numProgArgs-1) {
					*port = atoi(progArgs[i+1]);
					if (0 == port) {
						rpUsage(progArgs[0]);
						res = 0;
						flag = RiCTRUE;
					}
					else {
						res = 1;
						flag = RiCTRUE;
					}
				}
				else {
					rpUsage(progArgs[0]);
					res = 0;
					flag = RiCTRUE;
				}
			}
		}
	}

	return res;
}
#endif


/*************************************************/
/**					RiCOXFInit					**/
/*************************************************/
/* Forward declaretions */

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpInteger RiCOXFInit(RhpInteger numProgArgs, RhpCharacter **progArgs, RhpPositive defaultPort,
					  const RhpString defaultHost, RhpPositive ticktime, RhpPositive maxTM , 
					  RiCBoolean isRealTimeModel, struct RiCMxfGlobals_t * const mxfGlblInst)
{
	RhpInteger res = BadOrMissingInitParams;
	RhpBoolean flag = RiCFALSE;
	RiCTaskEM * mainTaskEM = NULL;
	
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	
	ARCThread * aomThread = NULL;
	ARCStepper * aomStepper = NULL;
	
	/*
	Initialize the aomDispatcher
	(and in trace mode the tomDispatcher
	*/
	RhpString hostName = NULL;
	RhpInteger portNumber = 0;
	
#ifdef RIC_USE_MEMORY_MANAGER
	RiCInitMemoryManager();
#endif

	switch (rpProcessHostArg(numProgArgs,progArgs,&hostName)) {
	case 1:
		break;
	case 2:
		hostName = defaultHost;
		break;
	case 0:
	default: 
		flag = RiCTRUE;
		res = 0;
		break;
	}
	if(flag == RiCFALSE) {
		switch (rpProcessPortArg(numProgArgs,progArgs,&portNumber)) {
		case 1:
			break;
		case 2:
			portNumber = (RhpInteger)defaultPort;
			break;
		case 0:
		default:
			flag = RiCTRUE;
			res = 0;
		}
		if((flag == RiCFALSE) && (mxfGlblInst != NULL)) {
			ARCSD_instance();	/*lint !e746 () */
			ARCSD_start(hostName, (RhpPositive)portNumber);
#else
		if((flag == RiCFALSE) && (mxfGlblInst != NULL)) {
#endif
#if ((!(defined _OMINSTRUMENT)) || (defined _OM_TARGET_MON))
#ifdef RIC_USE_MEMORY_MANAGER
	RiCInitMemoryManager(); /* it should be called in any way (animated or not ) */
#endif
#endif
			/* Currently we initialize the timer */
			/* enforcing initialization of the system timer */
			(void) RiCTimerManager_init(&(mxfGlblInst->systemTimer), (RhpUnlimitedPositiveNatural)ticktime, maxTM);
			
			/* get the default task */
			mainTaskEM = &(mxfGlblInst->mainTask.ric_task);
			/* Create the main task (actually its OXF wrapper) */
			/*LDRA_INSPECTED 45 D : Pointer is checked for null within function */
			/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
			{
				RiCEvent** mainQ = RiCMainTask_getMainTaskQueue(mxfGlblInst);
				RiCThread *mainThr = RiCMainTask_getMainTaskThread(mxfGlblInst);

				(void)RiCTaskEM_init(mainTaskEM, RiCTRUE, NULL, OSTimerTask, OSTimerQueueEvent, mainQ, 
					RiC_MAIN_TASK_QUEUE_SIZE, mainThr, RiCTRUE, NULL);
			}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* Create the breakpoint mananger */
			ARCBPM_init();	/*lint !e746 () */
			
			/* set name to Main task */
			if(mainTaskEM) {
				ARCT_setName_C(mainTaskEM->aomTask, "mainTask");
			}
			
#endif
			
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* Ensure no left over registration candidate in callstack from pre main */
			aomThread = RiCTaskEM_getARCTask(mainTaskEM);
			if (aomThread) ARCCS_restToMain(ARCT_getCallStack(aomThread));
			
			/* Ensure main task stops on first step (could be no if proxy not yet arrived) */
			aomStepper = RiCTaskEM_getStepper(mainTaskEM);
			if (ARCAI_getProxy(ARCAI(aomStepper)) == RiCInConstruction)	/*lint !e740 (Unusual pointer cast) */
				ARCAI_setInterestMask(ARCAI(aomStepper), OMStopWhenStep | OMStopWhenNext); /*lint !e740 (Unusual pointer cast) */
			
			/* Make "the first step" */
			ARCS_notifyStep(RiCTaskEM_getStepper(mainTaskEM));
#endif
			
			/* os specific actions to be taken after the environment is set */
			RiCOSOXFInitEpilog(); /*lint !e746 () */
			
			res = 1;
		}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	}
#endif
	
	RiCOSEventsPoolMutex_init();
	return res;
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCOXFStart(RhpInteger doFork) 
{
	/*LDRA_INSPECTED 94 S */
	RiCMainTask_start(); /* no "this" as this is a singleton */
}

/***********************************************************/
/* alternative memory allocation/deallocation/reallocation */
/***********************************************************/
#ifndef RIC_USE_MEMORY_MANAGER

#ifndef RiC_NO_RTOS_REALLOC

/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RhpAddress ricRealloc( RhpAddress pContents,
                  const size_t currentSize,
                  const size_t newSize )
{
#ifdef OXF_USE_DYNAMIC_ALLOC
    return realloc( pContents, newSize );
#else
	return (RhpAddress) NULL;
#endif
}

#endif

RhpAddress RiCGetMemory( const size_t requestedSize )
{
#ifdef OXF_USE_DYNAMIC_ALLOC
	return malloc(requestedSize);
#else
	return (RhpAddress) NULL;
#endif
}

/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
void RiCReturnMemory( const RhpAddress contents )
{
#ifdef OXF_USE_DYNAMIC_ALLOC
	free((RhpAddress)contents);
#endif
}

#endif

