#ifndef RiCPortMacros_H
#define RiCPortMacros_H

/*

	purpose: Specifies macros related to relay of operations
  via ports in C

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.

*/

/****************************************************/
/****  Utility Macros - For Misra Compatibility   ***/
/****************************************************/
/* Justification: Violation of Misra04 Rule19.13										*/
/* Usage of ## is required to provide better flexibility								*/
/* Justification: Violation of Misra98 Rule96											*/
/* Macro body cannot be included in parentheses									    	*/
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses	*/
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 125 S */
#define RIC_CONCAT_2(A, B) A ##B
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 125 S */
#define RIC_CONCAT_3(A, B ,C) RIC_CONCAT_2(A, B) ##C
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 125 S */
#define RIC_CONCAT_6(A, B, C, D, E, F) RIC_CONCAT_3(A, B ,C) ##RIC_CONCAT_3(D, E, F)

/* Justification: Violation of Misra98 Rule96										  */
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses */
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
#define OPORT(CLASS, PORT, INTERFACE) \
	RIC_CONCAT_6(CLASS,_,PORT,_C_,getIts_,INTERFACE)(RIC_CONCAT_3(CLASS,_get_,PORT)(me))

/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
#define OUT_PORT(CLASS, PORT, INTERFACE) OPORT(CLASS, PORT, INTERFACE)

/* flowport macros */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define FLOW_DATA_SEND(me, flowPort, flowPortVal, function, target, CLASS, serializeFunc) \
{ \
	char* fpValStr = serializeFunc(flowPortVal); \
	NOTIFY_FLOW_DATA_SEND((me), CLASS, flowPort, fpValStr); \
	function(target, flowPortVal, NULL); \
	NOTIFY_FLOW_DATA_SEND_END((me), CLASS) \
	free(fpValStr); \
}
#else
/* Justification: Violation of Misra98 Rule96										  */
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses */
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 79 S */
#define FLOW_DATA_SEND(me, flowPort, flowPortVal, function, target, CLASS, serializeFunc) \
	function(target, flowPortVal, NULL);
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define FLOW_DATA_RECEIVE(me, argName, argValue, CLASS, serializeFunc) \
{ \
	char* fpValStr = serializeFunc(argValue); \
	NOTIFY_FLOW_DATA_RECEIVE((me), CLASS, argName, fpValStr) \
	free(fpValStr); \
}
#else
/*LDRA_INSPECTED 340 S */
#define FLOW_DATA_RECEIVE(me, argName, argValue, CLASS, serializeFunc)
#endif

/* Direct flowport macros */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define DIRECT_FLOW_DATA_SEND(me, flowPort, flowPortVal, function, target, CLASS, serializeFunc) \
{ \
	char* fpValStr = serializeFunc(flowPortVal); \
	NOTIFY_FLOW_DATA_SEND((me), CLASS, flowPort, fpValStr) \
	function((target), (flowPortVal)); \
	NOTIFY_FLOW_DATA_SEND_END((me), CLASS) \
	free(fpValStr);	\
}
#else
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 79 S */
#define DIRECT_FLOW_DATA_SEND(me, flowPort, flowPortVal, function, target, CLASS, serializeFunc) \
	function((target), (flowPortVal));
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define DIRECT_FLOW_DATA_RECEIVE(me, argName, argValue, CLASS, serializeFunc) \
{ \
	char* fpValStr = serializeFunc(argValue); \
	NOTIFY_FLOW_DATA_RECEIVE((me), CLASS, argName, fpValStr) \
	free(fpValStr);	\
}
#else
/*LDRA_INSPECTED 340 S */
#define DIRECT_FLOW_DATA_RECEIVE(me, argName, argValue, CLASS, serializeFunc)
#endif

#ifdef _OMINSTRUMENT
#define FLOW_DATA_POP		\
{							\
	NOTIFY_FLOW_DATA_POP	\
}
#else
#define FLOW_DATA_POP
#endif

#endif
