/*
	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2015. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							RiCReactive.c                         |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICREACTIVE_SOURCE
#define MXF_RICREACTIVE_SOURCE
#endif

#include <stdlib.h>
#include "RiCOSWrap.h"
#include "RiCReactive.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "aommacro.h"
#include "aom.h"
#else
#include "aom/aommacro.h"
#include "aom/aom.h"
#endif
#endif

/******************************************/
/**				Private Methods			 **/
/******************************************/

static RhpUnlimitedNatural inNullConfig(const RiCReactive * const me);
static RhpUnlimitedPositiveNatural shouldCompleteRun(const RiCReactive * const me);
#ifdef RIC_USE_OS_CONTEXT
static RiCTakeEventStatus processEvent(RiCReactive * const me, RiCEvent * const event, RiCBoolean* const flag, RiCOSContextType osContext);
static RiCBoolean dispatchEvent(RiCReactive * const me, RiCEvent* const evReal, RiCEvent* const evNull, RiCBoolean* const flag, RiCBoolean* const res, RiCOSContextType osContext);
static RiCBoolean runToCompletion(RiCReactive * const me, RiCOSContextType osContext); 
#else
static RiCTakeEventStatus processEvent(RiCReactive * const me, RiCEvent * const event, RiCBoolean* const flag);
static RiCBoolean dispatchEvent(RiCReactive * const me, RiCEvent* const evReal, RiCEvent* const evNull, RiCBoolean* const flag, RiCBoolean* const res); 
static RiCBoolean runToCompletion(RiCReactive * const me); 
#endif
static RiCBoolean isBehaviorStarted(const RiCReactive * const me);
static RhpUnlimitedNatural shouldCompleteStartBehavior(const RiCReactive * const me);
static void setCompleteStartBehavior(RiCReactive * const me, RiCBoolean b);
static RiCBoolean RiC_gen(RiCReactive * const me, RiCEvent * const event, RiCBoolean fromISR);
static RhpUnlimitedNatural shouldTerminate(const RiCReactive * const me);
static void setBehaviorStarted(RiCReactive * const me, RiCBoolean b);


#ifdef RIC_USE_RAPID_PORTS

#define OFFSETOF_IRICREACTIVE 0

static RiCBoolean gen(RhpAddress const me, RiCEvent* event, RiCBoolean fromISR);
/* Values of IRiCReactive_Vtbl */
const IRiCReactive_Vtbl IRiCReactive_Vtbl_Values = 
{
	OFFSETOF_IRICREACTIVE,
	gen
};

static RiCBoolean gen(RhpAddress const me, RiCEvent* event, RiCBoolean fromISR)
{
	return RiCReactive_gen((RiCReactive*) me, event, fromISR);
}
#endif 

/** set the event destination and add it to the event queue **/
static RiCBoolean RiC_gen(RiCReactive * const me, RiCEvent * const event, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;
	if ((me != NULL) && (event != NULL)){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	/*
	if (!isValidOMReactive(this))
	notifyRunTimeError(this,"Event sent to invalid item");
		*/
#endif
		event->destination = me;
		res = RiCTaskEM_queueEvent(me->myTask, event, fromISR);
	}

	return res;
}

void RiCReactive_setEventGuard(RiCReactive * const me, RiCOSMutex * const evGuard)
{
	if (me != NULL){ 
		me->eventGuard = evGuard; 
	}
}

RiCOSMutex * RiCReactive_getEventGuard(const RiCReactive * const me)
{
	RiCOSMutex * retval = NULL;

	if(me != NULL){
		retval = me->eventGuard;
	}
	/*LDRA_INSPECTED 71 S */
	return retval;
}



/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* check if the reactive object is being destroyed */
RiCBoolean RiCReactive_isInCleanup(const RiCReactive * const me)
{
	RiCBoolean res = RiCFALSE;
#ifndef RHP_STATIC_APPLICATION
	if(me != NULL){
		if((me->omrStatus & RiCInCleanup) != 0UL){
			res = RiCTRUE;
		}
	}
#endif
	
	return res;
}

/*************************************************************************/
#ifdef RIC_USE_OS_CONTEXT
static RiCTakeEventStatus processEvent(RiCReactive * const me, RiCEvent* const event, RiCBoolean* const flag,  RiCOSContextType osContext) 
#else
static RiCTakeEventStatus processEvent(RiCReactive * const me, RiCEvent* const event, RiCBoolean* const flag) 
#endif					
{
	RiCTakeEventStatus res = eventNotConsumed;
	if (NULL != me) {
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
		if (RiCReactive_isBusy(me) == RiCFALSE) {
			/* protect from recursive Triggered Operation calls */
			RiCReactive_doBusy(me);
			/* set the current event */
			me->current_event = event;
			/* handle the event */
			if ((me->vtbl != NULL) && (me->vtbl->dispEventFunc != NULL)) {
				/*LDRA_INSPECTED 53 S : LDRA Analysis incorrect. Comma operator is not used */
	#ifdef RIC_USE_OS_CONTEXT
				res = (me->vtbl->dispEventFunc)(me->owner, event->lId, osContext);
	#else
				res = (me->vtbl->dispEventFunc)(me->owner, event->lId);
	#endif
			}
			/* check runToCompletion() */
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
			if (shouldCompleteRun(me) != 0UL) {
				RiCBoolean resRunToCompletion = RiCFALSE;
	#ifdef RIC_USE_OS_CONTEXT
				resRunToCompletion = runToCompletion(me, osContext);
	#else
				resRunToCompletion = runToCompletion(me);
	#endif
				if (resRunToCompletion == RiCFALSE) {
					/* infinit loop */
					/* notify the user and return */
					/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
	#ifdef RIC_USE_RICOUTPUT
					RiC_NOTIFY_TO_ERROR("ERROR: Infinite Loop of null transitions found\n");
	#endif
					res =  RiCTakeEventError;
					*flag = RiCTRUE;
				}
			}

			if(*flag == RiCFALSE){
				/* cleanup current_event */
				me->current_event = NULL;

				/* end protection from recursive Triggered Operation calls */
				RiCReactive_undoBusy(me);
			}
		}
	}
	return res;
}
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/** handle an event (mutual to takeEvent() and takeTrigger() **/
#ifdef RIC_USE_OS_CONTEXT
RiCTakeEventStatus RiCReactive_consumeEvent(RiCReactive * const me, RiCEvent * const event, RiCOSContextType osContext)
#else
RiCTakeEventStatus RiCReactive_consumeEvent(RiCReactive * const me, RiCEvent * const event)
#endif
{
	RiCTakeEventStatus res = eventNotConsumed;
	RiCBoolean flag = RiCFALSE;
	/* check legal me && event */
	if ((me == NULL) || (event == NULL)){
		res = RiCTakeEventError;
	}
	else if (RiCReactive_isInCleanup(me) != RiCFALSE){
		res = RiCTakeEventInCleanup;
	}
	else if ((me->vtbl != NULL) && (me->vtbl->dispEventFunc == NULL)) {/* check valid dispatch function */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		sendError("Event sent to Reactive with no statechart (no dispatch function) -- Event Ignored");
#endif
		/* If Non animated just ignore this event silently. */
		res = RiCTakeEventError;
	}
	else if (isBehaviorStarted(me) == RiCFALSE) {/* check that startBehavior was called */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		sendError("\nEvent ignored - Instance did not execute startBehavior()\n\n");
#endif
		res = eventNotConsumed;	/* ignore the event */
	}
	else {
		RhpUnlimitedNatural resShouldCompleteStartBhv = shouldCompleteStartBehavior(me);
		if (resShouldCompleteStartBhv != 0L) {
			RiCBoolean resRunToCompletion = RiCFALSE;
			/*
			 the first recieved event, need to preform runToCompletion(),
			 if in startBehavior() shouldCompleteStartBehavior() returned TRUE.
			 for any other event shouldCompleteStartBehavior() should return false.
			*/
			setCompleteStartBehavior(me, RiCFALSE);
			/* protect from recursive Triggered Operation calls */
			RiCReactive_doBusy(me);

			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
#ifdef RIC_USE_OS_CONTEXT
			resRunToCompletion = runToCompletion(me, osContext);
#else
			resRunToCompletion = runToCompletion(me);
#endif
			if (resRunToCompletion == RiCFALSE) {
				/* infinit loop */
				/* notify the user and return */
				/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
#ifdef RIC_USE_RICOUTPUT
				RiC_NOTIFY_TO_ERROR("ERROR: Infinite Loop of null transitions found\n");
#endif
				res = RiCTakeEventError;
				flag = RiCTRUE;
			}
			if(flag == RiCFALSE){
				/* end protection from recursive Triggered Operation calls */
				RiCReactive_undoBusy(me);
			}
		}

		if(flag == RiCFALSE){
			/* check that this is not the dummy RiCStartBehaviorEvent event */
			if (event->lId != RiCStartBehavior_id) {
#ifdef RIC_USE_OS_CONTEXT
				res = processEvent(me, event, &flag, osContext);
#else
				res = processEvent(me, event, &flag);
#endif
			}
		}
	}
	return res;
}

#ifdef RIC_USE_OS_CONTEXT
static RiCBoolean dispatchEvent(RiCReactive * const me, RiCEvent* const evReal, RiCEvent* const evNull, RiCBoolean* const flag, RiCBoolean* const res, RiCOSContextType osContext) 
#else
static RiCBoolean dispatchEvent(RiCReactive * const me, RiCEvent* const evReal, RiCEvent* const evNull, RiCBoolean* const flag, RiCBoolean* const res) 
#endif					
{
	RiCBoolean doBreak = RiCFALSE;
	RhpAddress pOwner = me->owner;
	if (NULL != me) {
		/* No more null transitions to take */
		if ((me->vtbl != NULL) && (me->vtbl->dispEventFunc == NULL)) {
			/* restore the real event */
			me->current_event = evReal;
	#ifdef MICROC_OXF_CLEANUP
			RiCEvent_cleanup(evNull);
	#endif						
			*res = RiCTRUE;
			*flag = RiCTRUE;
		}					
		if(*flag == RiCFALSE){
			/* perform the null transition */
			if (RiCReactive_isInCleanup(me) != RiCFALSE){
				doBreak = RiCTRUE;
			}
			/*LDRA_INSPECTED 53 S : LDRA Analysis incorrect. Comma operator is not used */
			else if(me->vtbl != NULL) {
				RiCTakeEventStatus resDispEventFuncCall = eventNotConsumed;
	#ifdef RIC_USE_OS_CONTEXT
				resDispEventFuncCall = (me->vtbl->dispEventFunc)(pOwner, RiCNull_id, osContext);
	#else
				resDispEventFuncCall = (me->vtbl->dispEventFunc)(pOwner, RiCNull_id);
	#endif
				if(resDispEventFuncCall == eventNotConsumed){
					doBreak = RiCTRUE;
				}
				else {
					doBreak = RiCFALSE;
				}
			}
			else{
				doBreak = RiCFALSE;
			}
		}
	}
	return doBreak;
}
/** run event to complition **/
#ifdef RIC_USE_OS_CONTEXT
static RiCBoolean runToCompletion(RiCReactive * const me, RiCOSContextType osContext) 
#else
static RiCBoolean runToCompletion(RiCReactive * const me) 
#endif
{
	RhpInteger nullStepCount = 0;
	RiCEvent* evReal = NULL;
	RiCEvent evNull;
	RiCBoolean doBreak = RiCFALSE;
	RiCBoolean flag = RiCFALSE;
	RiCBoolean res = RiCFALSE;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCCallStackMark aomLocalStackMark = {0, 0};
#endif
	if (me != NULL) {
		/* remember the real event, and replace it with a NULL event */
		/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
		(void)RiCEvent_init(&(evNull), RiCNull_id);
		RiCEvent_setDestination(&(evNull), me);
		evReal = me->current_event;
		me->current_event = &evNull;
		/* If untriggered transitions are "hot" take them */
		while ((flag == RiCFALSE) && (doBreak == RiCFALSE) ) {
			if(inNullConfig(me) != 0L){
				
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				/* pop up everything that had entered the call stack from this function */
				aomLocalStackMark = ARCGetCallStackMark((RhpAddress)&me, (RhpAddress)&me, &aomLocalStackMark);
				ARCCS_pop_to_mark(OMCurrentCallStack,(RhpAddress)&aomLocalStackMark);
#endif
				
				nullStepCount++;
				if (nullStepCount > OMDEFAULT_MAX_NULL_STEPS){
					doBreak = RiCTRUE; /* Found infinite loop */
				}
				
				if(doBreak == RiCFALSE){
#ifdef RIC_USE_OS_CONTEXT
					doBreak = dispatchEvent(me, evReal, &evNull, &flag, &res, osContext);
#else
					doBreak = dispatchEvent(me, evReal, &evNull, &flag, &res);
#endif
				}
			}
			else{
				doBreak = RiCTRUE;
			}
		}

		if(flag == RiCFALSE){
			/* ignore lint warning since me is checked */
			/* lint warning: Possible use of null pointer 'me' in left argument to operator '->' */
			/*lint -save -e613 */
			/* restore the real event */
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
			me->current_event = evReal;
#ifdef MICROC_OXF_CLEANUP
			RiCEvent_cleanup(&evNull);
#endif			
			/* Handle infinite loop */
			
			if (nullStepCount > OMDEFAULT_MAX_NULL_STEPS) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				ARCInstance* aomInstance = ARCI_reactiveObject2Instance(me); 
				if (aomInstance) {
					ARCAnimationItem * aomAI = ARCAI(aomInstance); /*lint !e740 () */
					OMNotifyRunTimeError(aomAI, "Infinite Loop of Null Transitions");
					ARCAI_destroy(aomAI);
				}
#endif
				/* reset the null config count */
				me->omrStatus = me->omrStatus & (~RiCNullConfigMask); /*lint -e613 */
				/* Throw it for someone else to handle */
				res = RiCFALSE;
				flag = RiCTRUE;
			}
			/*lint -restore */
		}
		if(flag == RiCFALSE){
			res = RiCTRUE;
		}
	}

	return res;
/*LDRA_INSPECTED 5 D */
}


/*
 Status check methods
*/


/*******************/
static RhpUnlimitedNatural shouldCompleteStartBehavior(const RiCReactive * const me)
{
	RhpUnlimitedNatural res = 0L;
	if (me != NULL){
		if ((me->omrStatus & RiCShouldCompleteStartBehavior) != 0UL) {
			res = 1;
		}
	}

	return res;
}

/*******************/
static void setCompleteStartBehavior(RiCReactive * const me, RiCBoolean b)
{
	if (me != NULL){
		if (b == RiCTRUE){
			me->omrStatus |= RiCShouldCompleteStartBehavior;
		}
		else{
			me->omrStatus &= (~RiCShouldCompleteStartBehavior);
		}
	}
}

/*******************/
static RhpUnlimitedNatural shouldTerminate(const RiCReactive * const me)
{
	RhpUnlimitedNatural res = 0L;
	if (me != NULL){
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		if ((me->omrStatus & RiCShouldTerminate) != 0UL) {
			res = 1;
		}
	}

	return res;
}

static RhpUnlimitedPositiveNatural shouldCompleteRun(const RiCReactive * const me)
{
	RhpUnlimitedPositiveNatural res = 0UL;
	if (me != NULL){
		res = me->omrStatus;
		res &= (~RiCShouldDelete);
		res &= (~RiCBehaviorStarted);
		res &= (RiCNullConfigMask);
	}

	return res;
}

/*******************/
static RhpUnlimitedNatural inNullConfig(const RiCReactive * const me)
{
	RhpUnlimitedNatural res = 0L;
	if (me != NULL){
		if ((me->omrStatus & RiCNullConfigMask) != 0UL) {
			res = 1;
		}
	}

	return res;
}

/*******************/
static void setBehaviorStarted(RiCReactive * const me, RiCBoolean b)
{
	if (me != NULL){
		if (b == RiCTRUE){
			me->omrStatus |= RiCBehaviorStarted;
		}
		else{
			me->omrStatus &= (~RiCBehaviorStarted);
		}
	}
}

/*******************/
static RiCBoolean isBehaviorStarted(const RiCReactive * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		if((me->omrStatus & RiCBehaviorStarted) != 0UL){
			res = RiCTRUE;
		}
	}

	return res;
}



/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef OXF_STATIC_INIT
RiCBoolean RiCReactive_init_static(RiCReactive * const me, RiCTaskEM * const pTask)
#else
RiCBoolean RiCReactive_init(RiCReactive * const me, RhpAddress const pOwner, RiCTaskEM * const pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst)
#endif
{
	if (me != NULL){
#ifndef OXF_STATIC_INIT
		RiCReactive_init_low(me, pOwner, pTask, pVtbl, mxfGlblInst);
#endif
#ifdef RIC_USE_RAPID_PORTS
		IRiCReactive_Init(&(me->iRiCReactive), &IRiCReactive_Vtbl_Values);
#endif
	}

	return RiCTRUE;
}

#if ((defined _OMINSTRUMENT) || (!(defined OXF_STATIC_INIT)))
void RiCReactive_init_low(RiCReactive * const me, RhpAddress const pOwner, RiCTaskEM * const pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst)
{
	if (me != NULL)
	{
		RiCReactive_setTask(me, pTask, RiCFALSE, mxfGlblInst);
		me->owner = pOwner;
		me->current_event = NULL;
		me->vtbl = (RiCReactive_Vtbl *) pVtbl;	/* const cast */
		me->eventGuard = NULL;
		me->omrStatus = RiCDefaultStatus;
		me->sm_busy = RiCFALSE;
		me->active = RiCFALSE;
		me->toGuardReactive = RiCFALSE;
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_cleanup(RiCReactive * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL){
		/* Mark that the object is under destruction. This is required
			In case where the object is destructed from another task */
		RiCReactive_setInCleanup(me);	
#ifdef RIC_USE_GLOBAL_TASK_LIST
		if (RiCTaskEMIsEndOfProcess() == RiCFALSE) { /*lint !e746 () */
			RiCReactive_cancelEvents(me, &mxfGlobals);	/* All events queued for this object need be "deleted". */
		}
#endif
		if (me->toGuardReactive == RiCTRUE) {
			RiCTaskEM *theTask = me->myTask;
			/* toGuard should be set to TRUE by the application class, if necessary */
			RiCTaskEM_free(theTask);
		}
	}
#endif
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
RiCReactive * RiCReactive_create(RhpAddress pOwner, RiCTaskEM * pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCReactive * me = (RiCReactive *)RiC_malloc((RhpPositive)sizeof(RiCReactive));
	if (me != NULL){
		(void) RiCReactive_init(me, pOwner, pTask, pVtbl, mxfGlblInst);
	}
	return me;
}
#endif

/*******************/
/*******************/
void RiCReactive_destroy(RiCReactive * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL) {
		RiCReactive_cleanup(me);
	}
#endif
}
/***************************************************/
/****				Reactive Methods			****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCReactive_startBehavior(RiCReactive * const me, struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCBoolean res = RiCTRUE; 
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCCallStackMark aomLocalStackMark = {0, 0};
#endif
	if ((me != NULL) && (me->vtbl != NULL)){
		/* start behavior only once */
		if (isBehaviorStarted(me) == RiCFALSE) {
			/* mark that startBehavior was called */
			setBehaviorStarted(me, RiCTRUE);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* animation */
			/*lint -save -e740 -e746 -e774 () */
			NOTIFY_START_BEHAVIOR(me);
			/*lint -restore */
#endif

			if((me->vtbl->rootState_entDefFunc) != NULL){
				OXF_CONST RiCReactive_Vtbl * OXF_CONST theVtbl = me->vtbl;
				(*(theVtbl->rootState_entDefFunc))(me->owner);
			}
			/*
			 generate a dummy event in case the instance doesn't recieve any external events
			 this cause the runToCompletion() after the default transition to be taken -
			 in the instance own thread (for active objects)
			*/
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
			if (shouldCompleteRun(me) != 0UL) {
				setCompleteStartBehavior(me, RiCTRUE);
				{
				#ifdef RIC_USE_GLOBAL_TASK_LIST
					RiCEvent *startBehEv = RiCStartBehaviorEvent(me, &mxfGlobals);
				#else
					RiCEvent *startBehEv = RiCStartBehaviorEvent(me, mxfGlblInst);
				#endif
					(void)RiCReactive_gen(me, startBehEv, RiCFALSE);
				}
			}
		}
		else {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			ARCInstance* aomInstance = ARCI_reactiveObject2Instance(me);
			ARCAnimationItem * aomAI = ARCAI(aomInstance); /*lint !e740 () */
			OMNotifyRunTimeError(aomAI, "Superfluous  startBehavior() ignored - instance is already started");
#endif
		}
		/* and deleting the item if it reached a terminate connector */
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null */
		if(shouldTerminate(me) != 0L){
			res = RiCFALSE;
		}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* roll back the callstack to its initial state when startBehavior had started */
		aomLocalStackMark = ARCGetCallStackMark((RhpAddress)&me, (RhpAddress)&me, &aomLocalStackMark);
		ARCCS_pop_to_mark(OMCurrentCallStack,(RhpAddress) &aomLocalStackMark);
#endif

	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCReactive_gen(RiCReactive * const me, RiCEvent * const event, RiCBoolean fromISR)
{
	RiCBoolean res = RiCFALSE;
	RiCBoolean flag = RiCFALSE;

	if ((me != NULL) && (event != NULL)){
		/* Destruction had begun, so no generation of new event for me */
		if (RiCReactive_isInCleanup(me) != RiCFALSE) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			if (!fromISR) sendError("Event sent to an under-destruction item is ignored.");
#endif
			res = RiCFALSE;
			flag = RiCTRUE;
		}
		
		if(flag == RiCFALSE){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			ARCEvent * aomE = ARCUserEventMap_Find(event);
			if ((aomE != NULL) && (!RiCEvent_isFrameworkEvent(event))) {
				if (fromISR) ARCE_setSender(aomE, RiCNotInteresting);
				else ARCE_setSender(aomE, ARCCS_top(OMCurrentCallStack));
			}
#endif
			res = RiC_gen(me, event, fromISR);
		}
	}
	
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCReactive_genBySender(RiCReactive * const me, RiCEvent * const event, RhpAddress sender)
{
	RiCBoolean res = RiCFALSE;
	RiCBoolean flag = RiCFALSE;

	if ((me != NULL) && (event != NULL)){
		/* Destruction had begun, so no generation of new event for me */
		if (RiCReactive_isInCleanup(me) != RiCFALSE) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			sendError("Event sent to an under-destruction item is ignored.");
#endif
			res = RiCFALSE; 
			flag = RiCTRUE;
		}

		if(flag == RiCFALSE){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			ARCEvent * aomE = ARCUserEventMap_Find(event);
			if ((aomE != NULL) && (!RiCEvent_isFrameworkEvent(event))) {
				ARCE_setSender(aomE, (ARCAnimationItem*)sender);
			}
#endif
		}
		res = RiC_gen(me, event, RiCFALSE);
	}
	
	return res;
}

/*******************/
/*******************/
#ifdef RIC_USE_OS_CONTEXT
RiCTakeEventStatus RiCReactive_takeEvent(RiCReactive * const me, RiCEvent* const event, RiCOSContextType osContext)
#else
RiCTakeEventStatus RiCReactive_takeEvent(RiCReactive * const me, RiCEvent* const event)
#endif
{
	RiCBoolean flag = RiCFALSE;
	RiCTakeEventStatus res = eventNotConsumed;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCCallStackMark aomLocalStackMark = {0, 0};
#endif

	if (me == NULL){
		res = RiCTakeEventError;
	}
	else if (RiCReactive_isInCleanup(me) != RiCFALSE) {
		/* already in termination process */
		res = RiCTakeEventInCleanup;
	}
	else if (shouldTerminate(me) != 0L){
		if(RiCReactive_shouldDelete(me) == RiCFALSE){
			res = RiCTakeEventCompleted;
			flag = RiCTRUE;
		}
	}
	else{
		/* empty else to comply with MISRA rules */
	}

	if(flag == RiCFALSE){
		/* lock guard to prevent race between event & TO consumption */
		if ((me != NULL) && (me->eventGuard != NULL)){
			(void)RiCOSMutex_lock(me->eventGuard);
		}
		
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* animation */
		NOTIFY_TAKE_EVENT(me, event);
#endif
#if (((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON))) || (defined RIC_OVERRIDE_CONSUME_EVENT))
		
		if( ( me->vtbl != NULL ) && ( me->vtbl->consumeEvent != NULL ) )
        {
            res = me->vtbl->consumeEvent( me->owner, event );
        }
        else
        {
            res = RiCReactive_consumeEvent( me, event );
        }
#else 
#ifdef RIC_USE_OS_CONTEXT
		res = RiCReactive_consumeEvent( me, event, osContext );
#else 
		res = RiCReactive_consumeEvent( me, event );
#endif
#endif
		
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* pop up everything that had entered the call stack from this function */
		aomLocalStackMark = ARCGetCallStackMark((RhpAddress)&me, (RhpAddress)&event, &aomLocalStackMark);
		ARCCS_pop_to_mark(OMCurrentCallStack,(RhpAddress) &aomLocalStackMark);
#endif
		
		/* free guard to prevent race between event & TO consumption */
		if ((me != NULL) && (me->eventGuard != NULL)){
			(void)RiCOSMutex_free(me->eventGuard);
		}
		
		/* result with a status which indicates that the item had reached a terminate connector */
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null within function */
		if(shouldTerminate(me) != 0L){
			res = RiCTakeEventReachTerminate;
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef RIC_USE_OS_CONTEXT
RiCTakeEventStatus RiCReactive_takeTrigger(RiCReactive * const me, RiCEvent* const event, RiCOSContextType osContext)
#else 
RiCTakeEventStatus RiCReactive_takeTrigger(RiCReactive * const me, RiCEvent* const event)
#endif
{
	RiCBoolean flag = RiCFALSE;
	RiCTakeEventStatus res = eventNotConsumed;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCCallStackMark aomLocalStackMark = {0, 0};												
#endif

	if (me == NULL){
		res = RiCTakeEventError;
	}
	else if (shouldTerminate(me) != 0L){
		if(RiCReactive_shouldDelete(me) == RiCFALSE){
			res = RiCTakeEventCompleted;
			flag = RiCTRUE;
		}
	}
	else{
		/* empty else to comply with MISRA rules */
	}
	
	if(flag == RiCFALSE){
		/* handle the event */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		if( ( me->vtbl != NULL ) && ( me->vtbl->consumeEvent != NULL ) )
        {
            res = me->vtbl->consumeEvent( me->owner, event );
        }
        else
        {
            res = RiCReactive_consumeEvent( me, event );
        }
#else
#ifdef RIC_USE_OS_CONTEXT
		res = RiCReactive_consumeEvent( me, event, osContext );
#else 
		res = RiCReactive_consumeEvent( me, event );
#endif
#endif
		
		
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* pop up everything that had entered the call stack from this function */
		aomLocalStackMark = ARCGetCallStackMark((RhpAddress)&me, (RhpAddress)&event, &aomLocalStackMark);
		ARCCS_pop_to_mark(OMCurrentCallStack,(RhpAddress) &aomLocalStackMark);
#endif
	}
	
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_cancelEvents(RiCReactive * const me, struct RiCMxfGlobals_t *mxfGlblInst)
{
	if(me != NULL){
		RiCTaskEM_cancelEvents(me->myTask, me, mxfGlblInst);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_discarnateTimeout(const RiCReactive * const me, RiCEvent * const tm)
{
	if (me != NULL){
		RiCTimeout_cleanup(tm);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_popNullConfig(RiCReactive * const me)
{
	if (me != NULL){
		(me->omrStatus)--;
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_pushNullConfig(RiCReactive * const me)
{
	if (me != NULL){
		(me->omrStatus)++;
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_terminate(RiCReactive * const me)
{
	if (me != NULL){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		NOTIFY_TERMINATION(me, "");
#endif
		me->omrStatus |= RiCShouldTerminate;
	}
}

/*******************/
/*******************/
RiCTaskEM * RiCReactive_getTask(const RiCReactive * const me)
{
	RiCTaskEM * res = NULL;
	if (me != NULL){
		res = me->myTask;
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#if ((!(defined OXF_STATIC_INIT)) || (defined _OMINSTRUMENT))
void RiCReactive_setTask(RiCReactive * const me, RiCTaskEM * const pTask, RiCBoolean isAct, struct RiCMxfGlobals_t *mxfGlblInst)
{
	if (me != NULL){
		if (me->myTask != pTask) {
			me->active = isAct;
			me->myTask = pTask; 
		}
		/* Make sure we have a thread
			The fallback is that the object is dispatched by the system thread. */
		if (me->myTask == NULL){
			if(mxfGlblInst != NULL){
				me->myTask = &(mxfGlblInst->mainTask.ric_task); /*lint !e746 () */
			}
#ifdef RIC_USE_GLOBAL_TASK_LIST
			else {
				me->myTask = &(mxfGlobals.mainTask.ric_task); /*lint !e746 () */
			}
#endif
		}
	}
}
#endif

/*******************/
/*******************/
RiCBoolean RiCReactive_isActive(const RiCReactive * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		res = me->active;
	}

	return res;
}



/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_setActive(RiCReactive * const me, RiCBoolean isActiveIn)
{
	if (me != NULL){
		me->active = isActiveIn;
	}
}


/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCReactive_isBusy(const RiCReactive * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		res = me->sm_busy;
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_doBusy(RiCReactive * const me)
{
	if (me != NULL){
		me->sm_busy = RiCTRUE;
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_undoBusy(RiCReactive * const me)
{
	if (me != NULL){
		me->sm_busy = RiCFALSE;
	}
}

/*******************/
/*******************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCBoolean RiCReactive_isFrameworkInstance( const RiCReactive *const me )
{
    RiCBoolean result = RiCFALSE;

    if( me != NULL )
    {
        result = me->frameworkInstance;
    }

    return result;
}

/*******************/
/*******************/
void RiCReactive_setFrameworkInstance( RiCReactive *const me,
                                       const RiCBoolean frameworkInstance )
{
    if( me != NULL )
    {
        me->frameworkInstance = frameworkInstance;
    }
}
#endif


/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_setToGuardReactive(RiCReactive * const me, RiCBoolean b)
{
	if (me != NULL){
		me->toGuardReactive |= b;
		RiCTaskEM_setToGuardTask(me->myTask, b);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void  RiCReactive_setInCleanup(RiCReactive * const me)
{
	if(me != NULL) {
		me->omrStatus |= RiCInCleanup; 
	}
}

/*******************/
/*******************/
RiCBoolean RiCReactive_shouldDelete(const RiCReactive * const me)
{
	RiCBoolean res = RiCFALSE;
	
	if (me != NULL){
		if((me->omrStatus & RiCShouldDelete) != 0UL){
			res = RiCTRUE;
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_setshouldDelete(RiCReactive * const me, RiCBoolean b)
{
	if (me != NULL){
		if (b == RiCTRUE){
			me->omrStatus |= RiCShouldDelete;
		}
		else{
			me->omrStatus &= (~RiCShouldDelete);
		}
	}
}

/*******************/
/*******************/
void RiCReactive_setShouldTerminate(RiCReactive * const me, RiCBoolean b)
{
	if (me != NULL){
		if (b == RiCTRUE){
			me->omrStatus |= RiCShouldTerminate;
		}
		else{
			me->omrStatus &= (~RiCShouldTerminate);
		}
	}
}

/*******************/
/*******************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCReactive_serializeStates(const RiCReactive * const me, ARCSState * aomsState)
{
	RiCBoolean isTerminated;
	
	if (me != NULL){
		isTerminated = (shouldTerminate(me) != 0L);
		ARCSS_addTerminate(aomsState, isTerminated);	/*lint !e713 !e534 () */
		if (me->vtbl != NULL) {
			if (me->vtbl->rootState_serializeStates != NULL) {
				me->vtbl->rootState_serializeStates((RhpAddress)me->owner, aomsState);
			}
		}
	}
}
#endif

/**************************************************/
/***		Serialization utility methods	*******/
/**************************************************/

/*****************************/
/* returns RiCReactive State */
/*****************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpUnlimitedPositiveNatural RiCReactive_getState(const RiCReactive * const me)
{
	RhpUnlimitedPositiveNatural res = RiCDefaultStatus;
	
	if (me != NULL){
		res = me->omrStatus;
	}

	return res;
}

/*****************************/
/* returns RiCReactive State */
/*****************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCReactive_setState(RiCReactive * const me, RhpUnlimitedPositiveNatural oxfState)
{
	if(me != NULL)
	{
		me->omrStatus = oxfState;
	}
}

#ifdef RIC_USE_RAPID_PORTS
/*+*****************************************************************************
 * Input:     base: Pointer to any application event with an underlying
 *                  RiCEvent as the first element in its structure.
 *
 * Output:    RiCReactive_getEvent():  Pointer to the given event's underlying
 *                                     RiCEvent
 *
 * InOut:     None
 *
 * Function:  Return the underlying RiCEvent for the given base event.  The
 *            base event can be pointing to any application event structure that
 *            contains an RiCEvent as its first data item.
 *
 *
 *			  Added for RapidPort Support
 *
 *
 ****************************************************************************-*/
RiCEvent *RiCReactive_getEvent(RiCEvent *base)
{
    return base;
}

#endif
