#ifndef RiCReactive_H
#define RiCReactive_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCReactive.h                      |
 --------------------------------------------------------------- 
********************************************************************/


#include "mxf_cfg.h"
#include "RiCEvent.h"
#include "RiCTimer.h"
#include "RiCTaskEM.h"
#ifdef MICROC_FLAT_INCLUDE
#ifdef RIC_USE_RAPID_PORTS
#include "IRiCReactive.h"
#endif
#ifdef RIC_USE_RICOUTPUT
#include "RiCOutput.h"
#endif
#else
#ifdef RIC_USE_RAPID_PORTS
#include "Auxiliary/IRiCReactive.h"
#endif
#ifdef RIC_USE_RICOUTPUT
#include "Utils/RiCOutput.h"
#endif
#endif

/******************************************/
/**				Constants				 **/
/******************************************/
#define  RiCDefaultStatus				0x00000000UL
#define  RiCNullConfig					0x00000001UL
#define  RiCNullConfigMask				0x0000FFFFUL
#define  RiCShouldTerminate				0x00010000UL
#define  RiCInCleanup					0x00020000UL
#define  RiCShouldDelete				0x00040000UL
#define  RiCBehaviorStarted				0x00080000UL
#define  RiCShouldCompleteStartBehavior	0x00100000UL

/******************************************/
/**			Forward declaretions		 **/
/******************************************/

/* actually dispatch an event in the user state machin - set by generated code */
#ifdef RIC_USE_OS_CONTEXT
typedef RiCTakeEventStatus (*RiCDispatchEvent)(RhpAddress const pAddr, RiCEventId eventId, RiCOSContextType osContext);
#else
typedef RiCTakeEventStatus (*RiCDispatchEvent)(RhpAddress const pAddr, RiCEventId eventId);
#endif
/* take the default transition of the user state machine - set by generated code */
typedef void (*RiC_rootState_entDef)(RhpAddress const pAddr);
/* destry a user object, when reaces a terminate connector - set by generated code */
typedef void (*RiCObjectDestroyMethod)(RhpAddress pAddr);
/* consume an event and triggered operation - user overridable method */
typedef RiCTakeEventStatus (*RiCConsumeEvent)(struct RiCReactive_t * const pReactive, struct RiCEvent_t* pEvent);
/* user entry - handle unconsumed evets */
typedef void (*RiCHandleEventNotConsumed)(struct RiCReactive_t * const pReactive, struct RiCEvent_t* pEvent);
/* user entry - handle unconsumed triggered operations */
typedef void (*RiCHandleTONotConsumed)(struct RiCReactive_t * const pReactive, struct RiCEvent_t* pEvent);

/* animation */
/*LDRA_INSPECTED 141 S : Type forward declaration */
struct ARCSList;
typedef struct ARCSList ARCSState;
typedef void  (*RiC_rootState_serializeStates)(const void * pAddr, ARCSState * pARCst);

/******************************************/
/**					Macros				 **/
/******************************************/

#ifdef RIC_USE_RICREACTIVE_STATE_MACROS
/* Justification: Violation of Misra04 Rule19.13										*/
/* Usage of ## is required when using events to provide better flexibility				*/
/* Justification: Violation of Misra98 Rule96											*/
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses	*/
/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
/*LDRA_INSPECTED 125 S */
#define IS_IN(me,state)		(state##_IN(me))
/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
/*LDRA_INSPECTED 125 S */
#define RiCIS_IN(me,state)	(state##_IN(me))
/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
/*LDRA_INSPECTED 125 S */
#define CIS_IN(me,state)	(RiCIS_IN((me),state))

/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
/*LDRA_INSPECTED 125 S */
#define IS_COMPLETED(me,state)		(state##_isCompleted(me))
/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
/*LDRA_INSPECTED 125 S */
#define RiCIS_COMPLETED(me,state)	(state##_isCompleted(me))
/*LDRA_INSPECTED 78 S  */
/*LDRA_INSPECTED 340 S */
#define CIS_COMPLETED(me,state)		(RiCIS_COMPLETED((me), state))

#endif

/*lint  -save -e773 (Expression-like macro ..) */
/*LDRA_INSPECTED 340 S */
#define RiCREPLY(retVal)	(params->ric_reply = (retVal))
/*lint -restore */

/* 
 notice dynamic allocated events
 counting on RiCEvent to be the first data so that downcast from RiCEvent is fine 
*/

/*************************
** Generation of events **
**************************/

#ifdef RIC_USE_RICREACTIVE_GEN_MACROS

/* Justification: Violation of Misra04 Rule19.13										*/
/* Usage of ## is required when using events to provide better flexibility				*/
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10						*/
/* Macro body cannot be included in parentheses											*/
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses	*/
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 79 S */
#define RiCGEN(INSTANCE,EVENT)											\
	((INSTANCE) != NULL) ? RiCReactive_gen(&((INSTANCE)->ric_reactive),	(EVENT), RiCFALSE): RiCFALSE 	

/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 340 S */
#define CGEN(INSTANCE,EVENT) (RiCGEN((INSTANCE),EVENT))

/*LDRA_INSPECTED 340 S */
#define RiCGEN_ISR(INSTANCE,EVENT) (RiCReactive_gen(&((INSTANCE)->ric_reactive), (RiCEvent*) (EVENT), RiCTRUE))
/*LDRA_INSPECTED 340 S */
#define CGEN_ISR(INSTANCE,EVENT) (RiCGEN_ISR((INSTANCE),(EVENT)))

/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 340 S */
#define RiC_Create_Event(EVENT) (EVENT)

#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 79 S */
/*LDRA_INSPECTED 125 S */
#define RiCGEN_BY_X(INSTANCE,EVENT,SENDER,theClass)									\
	((INSTANCE) != NULL) ? RiCReactive_genBySender(&((INSTANCE)->ric_reactive),		\
	(EVENT), aomX2Item((SENDER), aomc##theClass)) : RiCFALSE

/*LDRA_INSPECTED 340 S */
#define CGEN_BY_X(INSTANCE,EVENT,SENDER,theClass) (RiCGEN_BY_X((INSTANCE),(EVENT),(SENDER),(theClass)))

#if ((!(defined _OMINSTRUMENT)) || (defined _OM_TARGET_MON))
#ifndef RiCGui
#define RiCGui 0
#endif
#endif

/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 79 S */
#define RiCGEN_BY_GUI(INSTANCE,EVENT)							\
	((INSTANCE) != NULL) ? RiCReactive_genBySender(&((INSTANCE)->ric_reactive), \
	(EVENT), RiCGui):RiCFALSE

/*LDRA_INSPECTED 340 S */
#define CGEN_BY_GUI(INSTANCE,EVENT) (RiCGEN_BY_GUI((INSTANCE),(EVENT)))

/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 79 S */
#define DYNAMICLY_ALLOCATED(object) { RiCReactive_setshouldDelete(&(object)->ric_reactive, RiCTRUE); }
/* spelling fix - the old macro remain for backward competability reasons */
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
#define DYNAMICALLY_ALLOCATED(object) DYNAMICLY_ALLOCATED(object)

#endif

/******************************************/
/**		Virtual table of RiCReactive	**/
/******************************************/
typedef struct RiCReactive_Vtbl_t {
	RiCDispatchEvent OXF_CONST dispEventFunc;
	RiC_rootState_entDef OXF_CONST rootState_entDefFunc;
#if (defined _OMINSTRUMENT) 
	RiC_rootState_serializeStates rootState_serializeStates;
#endif
#if ((defined _OMINSTRUMENT) || (defined RIC_OVERRIDE_CONSUME_EVENT))
	RiCConsumeEvent consumeEvent;
#endif
} RiCReactive_Vtbl;


/******************************************/
/**	    	RapidPort Support       	 **/
/******************************************/
#ifdef RIC_USE_RAPID_PORTS

#ifdef RIC_USE_RICREACTIVE_GEN_MACROS

/* notice dynamic allocated events counting on RiCEvent to be the first data so
   that downcast from RiCEvent is fine */
/* Justification: Violation of Misra04 Rule19.13										*/
/* Usage of ## is required when using events to provide better flexibility				*/
/* Justification: Violation of Misra98 Rule96											*/
/* Macro body cannot be included in parentheses											*/
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses	*/
#define RiCGEN_PORT(PORT,EVENT) IRiCReactive_gen(&(PORT.outbound), (EVENT),RiCFALSE)

#define RiCGEN_PORT_I(CLASS,PORT,INTERFACE,EVENT)               \
	INTERFACE##_gen(OUT_PORT(CLASS,PORT,INTERFACE), (EVENT),RiCFALSE)

#define RiCGEN_PORT_ISR(PORT,EVENT)               \
 RiCReactive_gen(&(PORT.outbound), (EVENT), RiCTRUE)

/* Mixed reactive + operations explicit ports */
#define RiCGEN_PORT_ISR_I(CLASS,PORT,INTERFACE,EVENT)               \
	INTERFACE##_gen(OUT_PORT(CLASS,PORT,INTERFACE), (EVENT),RiCTRUE)	

#define RiCIS_PORT(INSTANCE, PORT) \
(((INSTANCE) != NULL) ?                      \
	RiCEvent_isPort(INSTANCE->ric_reactive.current_event, &(INSTANCE->PORT)) : 0) 

#define RiCIS_PORT(INSTANCE, PORT) \
(((INSTANCE) != NULL) ?                      \
	RiCEvent_isPort(INSTANCE->ric_reactive.current_event, &(INSTANCE->PORT)) : 0) 

/*Generating subMacro to avoid double ## usage: MISRA 72 */
#define RiCGEN_PORT_GET_EVENT(EVENT,RiCBOOL)	\
	RiCReactive_getEvent(EVENT), RiCBOOL)


#define RiC_MULTICAST_GEN_PORT(PORT,EVENT) \
{	\
	RiCCollection* collec = NULL; \
    RhpPositive ind = 0;  \
    IRiCReactive* pIRiCReactive = NULL; \
    collec = RiCMcROutbound_getItsIReactive(&(me->PORT.outbound)); \
    if(collec != NULL) \
	{ \
       	for(ind = 0 ; ind < RiCCollection_getCount(collec) ; ind++) \
        { \
			pIRiCReactive = (IRiCReactive*)RiCCollection_getAt(collec,ind); \
           	IRiCReactive_gen(pIRiCReactive, RiCGEN_PORT_GET_EVENT(EVENT,RiCFALSE); \
        } \
    }\
}

/* Generare an event via an interface */
#define RiCGEN_I(INSTANCE,EVENT)               \
(((INSTANCE) != NULL) ?                      \
 IRiCReactive_gen(&((INSTANCE)->iric_reactive), (EVENT), RiCFALSE) : 0 )

/* Justification: Violation of Misra04 Rule19.13										*/
/* Usage of ## is required when using events to provide better flexibility				*/
/* Justification: Violation of Misra98 Rule96											*/
/* Macro body cannot be included in parentheses											*/
/* Macro parameter is used as the operand of ## and cannot be enclosed in parentheses	*/
/*LDRA_INSPECTED 77 S */
/*LDRA_INSPECTED 78 S */
/*LDRA_INSPECTED 79 S */
/*LDRA_INSPECTED 125 S */
#define RiCGEN_EV_PORT(PORT,EVENT) (*PORT##ev_cb)(PORT##ev_trg, (EVENT))

#endif

RiCEvent *RiCReactive_getEvent(RiCEvent *base);
PUBLIC const IRiCReactive_Vtbl IRiCReactive_Vtbl_Values;

#endif


/******************************************/
/**				RiCReactive				**/
/******************************************/

typedef struct RiCReactive_t {

#ifdef RIC_USE_RAPID_PORTS
	/* IRiCReactive inheritance - assumed to be first member, 
		If this field is moved elsewhere - update OFFSETOF_IRICREACTIVE */
	IRiCReactive iRiCReactive;
#endif

	/* The actual object which own the reactive object */
	RhpAddress owner;
	/* The task, the reactive object run on */
	RiCTaskEM * myTask;
	/* The current event to handle */
	RiCEvent * current_event;
	/* Vtable of functions to process events */
#ifndef _OMINSTRUMENT
	OXF_CONST RiCReactive_Vtbl * OXF_CONST vtbl;
#else
	OXF_CONST RiCReactive_Vtbl * vtbl;
#endif
	/* Guard from race between event & triggered operations consumption -
	The guard is set by the generated code when the user class has a guarded triggereed operation */
	RiCOSMutex * eventGuard;
	/* Are we in a null coniguration or termination? */
	RhpUnlimitedPositiveNatural omrStatus;
	/* Flag to block self application of triggered operations */
	RiCBoolean sm_busy;
	/* When RiCTRUE the object has its own task */
	RiCBoolean active;
	/* free event consumption grad - check flag */
	RiCBoolean toGuardReactive;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	/* framework instance flag */
    RiCBoolean frameworkInstance;
#endif
} RiCReactive;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
#ifdef OXF_STATIC_INIT
/*LDRA_INSPECTED 340 S */
#define RiCReactive_init(me, pOwner, pTask, pVtbl, mxfGlblInst) (RiCReactive_init_static((me), (pTask)))
RiCBoolean RiCReactive_init_static(RiCReactive * const me, RiCTaskEM * const pTask);
#else
RiCBoolean RiCReactive_init(RiCReactive * const me, RhpAddress const pOwner, RiCTaskEM * const pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst);
#endif

void RiCReactive_init_low(RiCReactive * const me, RhpAddress const pOwner, RiCTaskEM * const pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst);

void RiCReactive_cleanup(RiCReactive * const me);
void RiCReactive_destroy(RiCReactive * const me);

RiCReactive * RiCReactive_create(RhpAddress pOwner, RiCTaskEM * pTask, const RiCReactive_Vtbl * pVtbl, struct RiCMxfGlobals_t *mxfGlblInst);

/***************************************************/
/****				Reactive Methods			****/
/***************************************************/

/* start the event dispatch mechanism */
RiCBoolean RiCReactive_startBehavior(RiCReactive * const me, struct RiCMxfGlobals_t *mxfGlblInst);

/* generate an event */
RiCBoolean RiCReactive_gen(RiCReactive * const me, RiCEvent * const event, RiCBoolean fromISR);

/* generate an event with a given sender */
RiCBoolean RiCReactive_genBySender(RiCReactive * const me, RiCEvent * const event, RhpAddress sender);

#ifdef RIC_USE_OS_CONTEXT
RiCTakeEventStatus RiCReactive_consumeEvent(RiCReactive * const me, RiCEvent * const event, RiCOSContextType osContext);
RiCTakeEventStatus RiCReactive_takeEvent(RiCReactive * const me, RiCEvent* const event, RiCOSContextType osContext);
RiCTakeEventStatus RiCReactive_takeTrigger(RiCReactive * const me, RiCEvent* const event, RiCOSContextType osContext);
#else
/** handle an event (mutual to takeEvent() and takeTrigger() **/
RiCTakeEventStatus RiCReactive_consumeEvent(RiCReactive * const me, RiCEvent * const event);
/* handle an event */
RiCTakeEventStatus RiCReactive_takeEvent(RiCReactive * const me, RiCEvent* const event);
/* handle triggered operation */
RiCTakeEventStatus RiCReactive_takeTrigger(RiCReactive * const me, RiCEvent* const event);
#endif

/* cancel pending events */
void RiCReactive_cancelEvents(RiCReactive * const me, struct RiCMxfGlobals_t *mxfGlblInst);

/* delete a timeout */
void RiCReactive_discarnateTimeout(const RiCReactive * const me, RiCEvent * const tm);

/* 
 PushNullConfig and PopNullConfig are reference counting method for null transitios.
 Any entrance to a state which its source is a null transition, increases the counter.
 Once the state is exited it decreases the counter.
 The counter is practically limited to consume several bits. It is assumed the the count should never exceed
 several dozens of enabled null transitions.
*/
void RiCReactive_popNullConfig(RiCReactive * const me);
void RiCReactive_pushNullConfig(RiCReactive * const me);

/* terminate on Null transitions error */
void RiCReactive_terminate(RiCReactive * const me);

/* return the object's task */
RiCTaskEM * RiCReactive_getTask(const RiCReactive * const me);

/* set the object's task */
#if ((!(defined OXF_STATIC_INIT)) || (defined _OMINSTRUMENT))
void RiCReactive_setTask(RiCReactive * const me, RiCTaskEM * const pTask, RiCBoolean isAct, struct RiCMxfGlobals_t *mxfGlblInst);
#else
#define RiCReactive_setTask(me, pTask, isAct, mxfGlblInst)
#endif


/* check if the object has its own task */
RiCBoolean RiCReactive_isActive(const RiCReactive * const me);

/* set the object's active flag */
void RiCReactive_setActive(RiCReactive * const me, RiCBoolean isActiveIn);

/* check the sm_busy flag */
RiCBoolean RiCReactive_isBusy(const RiCReactive * const me);

/* set the sm_busy flag */
void RiCReactive_doBusy(RiCReactive * const me);
void RiCReactive_undoBusy(RiCReactive * const me);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/* get the framework instance flag */
RiCBoolean RiCReactive_isFrameworkInstance( const RiCReactive *const me );
/* set the framework instance flag */
void RiCReactive_setFrameworkInstance( RiCReactive *const me,
                                       const RiCBoolean frameworkInstance );
#endif 
/* set the guard flag */
void RiCReactive_setToGuardReactive(RiCReactive * const me, RiCBoolean b);

/* set the event consumption guard */
void RiCReactive_setEventGuard(RiCReactive * const me, RiCOSMutex * const evGuard);

/* get the event consumption guard */
RiCOSMutex * RiCReactive_getEventGuard(const RiCReactive * const me);

/* check if the reactive object is being destroyed */
RiCBoolean RiCReactive_isInCleanup(const RiCReactive * const me);

/** set RiCReactive state to in cleanup **/
void  RiCReactive_setInCleanup(RiCReactive * const me);

/** check the should delete flag **/
RiCBoolean RiCReactive_shouldDelete(const RiCReactive * const me);

/** set the should delete flag **/
void RiCReactive_setshouldDelete(RiCReactive * const me, RiCBoolean b);

/** set the should Terminate flag **/
void RiCReactive_setShouldTerminate(RiCReactive * const me, RiCBoolean b);


/****************************************************/
/*********  Serialization utility methods  **********/
/****************************************************/

/* returns RiCReactive State */
RhpUnlimitedPositiveNatural RiCReactive_getState(const RiCReactive* const me);

/* sets RiCReactive State */
void RiCReactive_setState(RiCReactive* const me, RhpUnlimitedPositiveNatural oxfState);


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCReactive_serializeStates(const RiCReactive * const me, ARCSState * aricsState);
#endif

#endif


