/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/

/******************************************************************
 ---------------------------------------------------------------
|							RiCTaskEM.c                         |
 --------------------------------------------------------------- 
*******************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICTASKEM_SOURCE
#define MXF_RICTASKEM_SOURCE
#endif

#include "RiCOxf.h"
#include "RiCTaskEM.h"
#include "RiCReactive.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "aom.h"
#else
#include "aom/aom.h"
#endif
#endif

#ifdef RIC_USE_MXF_GLOBALS

RiCMxfGlobals mxfGlobals =
{
	{ /* systemTimer */
		{(&RiCTimerManager_cbkBridge), (&RiCTimerManager_tickCbkBridge), (RICTIMERMANAGER_TIME_CYCLE), (&(mxfGlobals.systemTimer))}, 
		STATIC_INIT_RICOS_MUTEX(), 
		(RiCEvent*)0, 
		(RiCTimedAction*)0, 
		RICTIMERMANAGER_DEFAULT_TICK_TIME, 
		0, 
		TRUE 
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		, RiCFALSE 
#endif
	},
	0U, /* timerCachedTicks */
#ifdef RIC_USE_GLOBAL_TASK_LIST
	{   /* taskGlobalInfo */
		{NULL, NULL}, 			
		STATIC_INIT_RICOS_MUTEX(),	
		RiCFALSE,
		RiCFALSE
	},
#endif
#ifdef OXF_STATIC_INIT
	{ /* mainTask */
		{ 
			&(mxfGlobals.mainTask.ric_thread), 
#ifdef RiCOSOsek_H
			{ RES_SCHEDULER },
#else
			{ (OSMutexHandle)0 },
#endif 
			{ (RiC_MAIN_TASK_QUEUE), 0, 0, 0, (RiC_MAIN_TASK_QUEUE_SIZE) } 
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			,((RhpAddress)0)
#endif
		},
		{ 
#ifdef RiCOSOsek_H
			{ (OSMainTask), (OSMainTaskQueueEvent) },
			{ (OSMainTaskQueueEvent), (OSMainTask) },
#else
			{ (0), (0), RiCFALSE }, 
			(0),
#endif
			(NULL), 
			(RiCFALSE) 
#ifdef MICROC_OXF_CLEANUP
			,0
#endif
		}
	},
#else
	{0},
#endif
	{{0}}, /* eventsPool */
	{{0}}, /* timeoutsPool */
	NULL,  /* osSpecificGlobals */
	NULL   /* osContext */
};

#endif

/******************************************/
/**				Private Methods			 **/
/******************************************/
/** set in cleanup flag **/
#ifdef MICROC_OXF_CLEANUP
static void RiCTaskEM_setEndOsTaskInCleanup(RiCTaskEM * const me, RiCBoolean inCleanup);
#endif

#ifdef RIC_USE_GLOBAL_TASK_LIST
static RiCTaskEM * getTaskFromHandle(RiCOSHandle handle);
/* get a task by its os handle */
static RiCTaskEM * getTaskFromHandle(RiCOSHandle handle)
{
	RiCTaskEM * res = NULL;
	gen_ptr pPos = NULL;
	RiCTaskEM * pTask = NULL;
	/* lock */
	RiCOSMutex * guard = (&(mxfGlobals.taskGlobalInfo.tasksListGuard));
	(void)RiCOSMutex_lock(guard);

	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
	/* Cast from/to void* pointer is required to implement general purpose data collections */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	/*LDRA_INSPECTED 45 D */
	/*LDRA_INSPECTED 45 D */
	pTask = (RiCTaskEM*) RiCList_first(&(mxfGlobals.taskGlobalInfo.tasksList), &(pPos));
	while((pTask != NULL) && (res == NULL)){
		if (pTask->myThread != NULL){
			if(RiCOSTask_getOSHandle(&(pTask->myThread->osTask)) == handle){
				res = pTask;
			}
		}
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		pTask = (RiCTaskEM*) RiCList_next(&(mxfGlobals.taskGlobalInfo.tasksList), &(pPos));
	}

	/* unlock */
	(void)RiCOSMutex_free(guard);

	/*LDRA_INSPECTED 71 S */
	return res;
}
#endif


#ifdef RIC_USE_GLOBAL_TASK_LIST
/** init task global info singleton **/
static void RiCTaskEM_Global_Info_init();
static void RiCTaskEM_Global_Info_init()
{
	mxfGlobals.taskGlobalInfo.endOfProcess = RiCFALSE;
	mxfGlobals.taskGlobalInfo.initialized = RiCTRUE;
	(void) RiCList_Init(&(mxfGlobals.taskGlobalInfo.tasksList));
	(void) RiCOSMutex_init(&(mxfGlobals.taskGlobalInfo.tasksListGuard), (OSMutexHandle)0);
}

/** register me in taskGlobalInfo.tasksList */
static void registerThread(RiCTaskEM * me);
static void registerThread(RiCTaskEM * me)
{
	RiCOSMutex * guard = &(mxfGlobals.taskGlobalInfo.tasksListGuard);
	/* init taskGlobalInfo if need to */
	if (mxfGlobals.taskGlobalInfo.initialized == RiCFALSE) {
		RiCTaskEM_Global_Info_init();
	}

	/* lock */
	(void)RiCOSMutex_lock(guard);

	if (!RiCList_find(&(mxfGlobals.taskGlobalInfo.tasksList), me)) {
		(void) RiCList_addTail(&(mxfGlobals.taskGlobalInfo.tasksList), me);
	}

	/* unlock */
	(void)RiCOSMutex_free(guard);
}

/** deregister me in mxfGlobals.taskGlobalInfo.tasksList */
static void deregisterThread(RiCTaskEM * me);
static void deregisterThread(RiCTaskEM * me)
{
	/* lock */
	RiCOSMutex * guard = &(mxfGlobals.taskGlobalInfo.tasksListGuard);
	(void)RiCOSMutex_lock(guard);

	RiCList_remove(&(mxfGlobals.taskGlobalInfo.tasksList), me);

	/* unlock */
	(void)RiCOSMutex_free(guard);
}

/*******************/
/*******************/
RiCBoolean RiCTaskEMIsEndOfProcess()
{
	return mxfGlobals.taskGlobalInfo.endOfProcess;
}

#endif

/** init function **/
#ifdef OXF_STATIC_INIT
static RiCBoolean init(RiCTaskEM * const me, RiCBoolean wrapThread, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier);
static RiCBoolean init(RiCTaskEM * const me, RiCBoolean wrapThread, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier)
#else
static RiCBoolean init(RiCTaskEM * const me, RiCBoolean wrapThread, const RiCOSTask* pOSTask, const RiCTaskEM_Vtbl * pVtbl, RiCEvent** const evQueue, 
					   RhpPositive evQueueSize, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier);
static RiCBoolean init(RiCTaskEM * const me, RiCBoolean wrapThread, const RiCOSTask* pOSTask, const RiCTaskEM_Vtbl * pVtbl, RiCEvent** const evQueue, 
					   RhpPositive evQueueSize, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier)
#endif
{
	RiCBoolean status = RiCFALSE;

	if (me != NULL)	{
#ifndef OXF_STATIC_INIT
		me->myThread = itsThread;
#endif
		if((itsThread != NULL) && (isThread == RiCTRUE)){
#ifndef OXF_STATIC_INIT
			itsThread->toGuardTask = RiCFALSE; /* the application class should set it to TRUE */
#endif
#ifdef MICROC_OXF_CLEANUP
			itsThread->deletionAllowed = RiCTRUE; /* FALSE means the task cannot be deleted */
#endif
			if (wrapThread == RiCTRUE) {
				/*
				When we wrap the thread WE ARE SURE that we are wrapping the
				current thread so we can ask about its identifier and then
				create the OSwrapper thread.
				*/
				RiCOSHandle currTaskHandle = RiCOSTask_getCurrentTaskHandle(); /*lint !e746 () */
				(void) RiCOSTask_wrap(&(itsThread->osTask), currTaskHandle);
			}
		}

		/* Create a "silent" event queue */
#ifndef OXF_STATIC_INIT
		status &= RiCOSMessageQueue_init(&(me->eventQueue), evQueue, evQueueSize);
		if((itsThread != NULL) && (isThread == RiCTRUE)){
			/*LDRA_INSPECTED 94 S */ /*LDRA_INSPECTED 203 S */
			(void) RiCOSEventFlag_init(&(itsThread->taskEventFlag), (RhpAddress)pOSTask);
			/*LDRA_INSPECTED 94 S */ /*LDRA_INSPECTED 203 S */
			/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
			RiCOSPendingEvent_init(&(itsThread->osTask),(RhpAddress)pOSTask);

			/* Set the v-table */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			itsThread->vtbl = (RiCTaskEM_Vtbl*) pVtbl;
		}
#endif
		/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
		(void) RiCOSMutex_init(&(me->taskMonitor), mutexIdentifier);


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/*
		Create an event queue which notifies on put and get
		Must be done this we so "theQueue" is properly cast to
		OMEventQueue for the OMThread and to 
		ARCEventQueue for the ARCThread
		*/
		me->aomTask = ARCT_create_T(me, &(me->eventQueue));
		if (ARCSD_isActive()) { /*lint !e746 () */
			ARCEventQueue * aomEQ = ARCT_getEventQueue(me->aomTask);
			ARCAI_waitForProxy(&aomEQ->aomaiBase);
		}
#endif
	}
	return status;
}



/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

#ifdef OXF_STATIC_INIT
RiCBoolean RiCTaskEM_init_static(RiCTaskEM * const me, RiCBoolean wrapTask, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier)
{
	return  init(me, wrapTask, itsThread, isThread, mutexIdentifier);
}
#else
RiCBoolean RiCTaskEM_init(RiCTaskEM * const me, RiCBoolean wrapTask, const RiCTaskEM_Vtbl * pVtbl, 
				RiCOSHandle OSTsk, RiCOSEventHandle OSTskQueueEvent, RiCEvent** const evQueue, 
				RhpPositive evQueueSize, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier)
{
	RiCBoolean res = RiCFALSE;
	RiCOSTask ricOsTask;

	if(isThread == RiCTRUE){
		ricOsTask.osTask   = OSTsk;
		ricOsTask.queueEvent = OSTskQueueEvent;
		if (me != NULL){
			if(itsThread != NULL){
				itsThread->osTask.osTask  = OSTsk;
			}
		}
	
	
	}
	res = init(me, wrapTask, &(ricOsTask), pVtbl, evQueue, evQueueSize, itsThread, isThread, mutexIdentifier);
	return res;
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCTaskEM_Init(RiCTaskEM * const me, const RiCOSTask * pOSTask, RhpUnlimitedNatural priority,
						 RhpUnlimitedNatural stackSize, RhpUnlimitedNatural messageQueueSize,
						 RiCBoolean dynamicMessageQueue, const RiCTaskEM_Vtbl * pVtbl, 
						 RiCEvent** const evQueue, RhpPositive evQueueSize, 
						 RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier)
{
	RiCBoolean initStatus = RiCFALSE;

	if (me != NULL) {
#ifdef OXF_STATIC_INIT
		initStatus = init(me, RiCFALSE, itsThread, isThread, mutexIdentifier);
#else
		initStatus = init(me, RiCFALSE, pOSTask, pVtbl, evQueue, evQueueSize, itsThread, isThread, mutexIdentifier);
#endif

		(void) RiCTaskEM_setPriority(me, (RhpInteger)priority);
	}

	return initStatus;
}

/*******************/
/*******************/
void RiCTaskEM_cleanup(RiCTaskEM * const me)
{
#ifdef MICROC_OXF_CLEANUP
	RiCBoolean exeOnMyTask;
	if ( (me != NULL) && (me->myThread != NULL) && (me->myThread->deletionAllowed == RiCTRUE) ) 
	{
		/* cleanup the event queue */
		RiCOSMessageQueue_cleanup ( & ( me -> eventQueue    ) );

		exeOnMyTask = RiCOSTask_exeOnMyTask(&(me->myThread->osTask));

#ifdef RIC_USE_GLOBAL_TASK_LIST
		deregisterThread(me);
#endif                      
		if (exeOnMyTask == RiCFALSE) {
			/*
			 The task running the cleanup is not my 'task'
			 so we can safely end it, thus aRhpVoid him from to interfer in the deletion agenda
			*/
			RiCTaskEM_setEndOsTaskInCleanup(me, RiCTRUE);
		}


#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* destroy the animated task */
		ARCT_destroy(me->aomTask);
		me->aomTask = NULL;
#endif

		/* cleanup the event queue */
		RiCOSMessageQueue_cleanup ( & ( me -> eventQueue    ) );
		RiCOSEventFlag_cleanup    ( & ( me -> myThread -> taskEventFlag ) );
		RiCOSMutex_cleanup        ( & ( me -> taskMonitor   ) );

		/* clear the v-table */
#ifndef OXF_STATIC_INIT
		me->myThread->vtbl = NULL;
#endif

		if ((me->myThread->toGuardTask) != RiCFALSE) {
			RiCTaskEM_free(me);
		}
	}
#endif
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTaskEM_flagWait(const RiCTaskEM * const me) 
{
	if((me != NULL) && (me->myThread != NULL)){
		(void)RiCOSEventFlag_wait(&(me->myThread->taskEventFlag), -1);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCTaskEM * RiCTaskEM_create(RiCBoolean wrapThread, const RiCTaskEM_Vtbl * pVtbl)
{
	return (RiCTaskEM * ) NULL;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTaskEM_beginMyTask(RiCTaskEM * const me) 
{
	if((me != NULL) && (me->myThread != NULL)){
		RiCOSPendingEvent_clear(&(me->myThread->osTask));
	}
}

void RiCTaskEM_endMyTask(RiCTaskEM * const me)
{
	if((me != NULL) && (me->myThread != NULL)){
		RiCOSTask_endMyTask(&(me->myThread->osTask));
	}
}

#ifndef INLINE_TASK_EXECUTION
/* execute the thread */
static RhpBoolean executeByVtbl(const RiCThread *myThrd, RiCTaskEM * const theTask);
static RhpBoolean executeByVtbl(const RiCThread *myThrd, RiCTaskEM * const theTask)
{
	RhpBoolean res = RiCFALSE;
	if((myThrd != NULL)  && (myThrd->vtbl != NULL) && (myThrd->vtbl->execute != NULL)){
		/*LDRA_INSPECTED 57 S : LDRA Analysis incorrect. Statement has side effect */
		(myThrd->vtbl->execute)(theTask);
		res = RiCTRUE;
	}

	return res;
}

/* execute the thread */
static RhpPositive doExecute(RiCTaskEM* const me);
static RhpPositive doExecute(RiCTaskEM* const me) 
{
	RhpPositive res = 0;
	if (me != NULL) {
		/* start the event loop */
		RhpBoolean retExecuteByVtbl = executeByVtbl(me->myThread, me);
		if (retExecuteByVtbl == RiCFALSE){
			RiCTaskEM_execute(me);
		}

		res = 1;
	}

	return res;
}

/***************************************************/
/****				Task methods				****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpPositive RiCTaskEM_doExecute(RiCTaskEM * const me)
{
	return doExecute(me);
}

static RiCBoolean RiCTaskEM_executeTerminate(RiCTaskEM * const me, RiCReactive * const dest, RiCBoolean toTerminate);
static RiCBoolean RiCTaskEM_executeTerminate(RiCTaskEM * const me, RiCReactive * const dest, RiCBoolean toTerminate)
{
	/* a handle to the thread that executes the delete */
	RiCOSHandle executedOsHandle = RiCOSTask_getCurrentTaskHandle();
	
	/* a handle to the dest's 'task' */
	RiCBoolean testForDelete = RiCFALSE;
	RiCOSTask * pOSTask = RiCTaskEM_getOSTask(me);
	RiCOSHandle destOsHandle = (RiCOSHandle) 0;
	if(pOSTask != NULL){
		destOsHandle = RiCOSTask_getOSHandle(pOSTask);
	}
	if (executedOsHandle != destOsHandle){
		testForDelete = RiCTRUE;
	}
	else if(RiCReactive_isActive(dest) == RiCFALSE) {
		testForDelete = RiCTRUE;
	}
	else {
		testForDelete = RiCFALSE;
	}
	
	if(testForDelete != RiCFALSE){
		/*
		dest does not runs on the task that execute the destructor
		OR
		it does runs on the task that execute the destructor 
		but dest is only a guest on this task i.e. it is not active and was created by an active object
		*/
		RiCBoolean shouldDelReactive = RiCReactive_shouldDelete(dest);
		if (shouldDelReactive == RiCFALSE) {
			RiCReactive_setShouldTerminate(dest, RiCTRUE);
		}
#ifdef MICROC_OXF_CLEANUP
		else {
			RiCReactive_destroy(dest);
		}
#endif
	}
	else {
		/* dest is actually me. Terminate gracefully. */
		toTerminate = RiCTRUE; /* the for loop will end */
	}
	return toTerminate;
}


static RiCTakeEventStatus RiCTaskEM_processEvent(RiCTaskEM * const me, RiCEvent * const ev, RiCReactive ** const dest);
static RiCTakeEventStatus RiCTaskEM_processEvent(RiCTaskEM * const me, RiCEvent * const ev, RiCReactive ** const dest)
{
	RiCTakeEventStatus result  = RiCTakeEventCompleted;
	RiCEventId evId = RiCEvent_getlId(ev);
	*dest = RiCEvent_getDestination(ev);
	
	if (evId != RiCCancelledEvent_id) {
		/*LDRA_INSPECTED 45 D : Pointer is checked for null within function */
		result = RiCReactive_takeEvent(*dest, ev);
	} 
	
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (!RiCEvent_isFrameworkEvent(ev)) { ARCE_going2DestryEvent(ev);}
#endif
	if (ev->eventDestroyOp != NULL) {
		/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
		/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
		ev->eventDestroyOp(ev);	/* the real event destroy op */
	}
	else { 
		RiCEvent_destroy(ev);
	}
	return result;
}

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework containing APIs that might be used	 */
/* Outside of the OXF itself by Generated code					 */
void RiCTaskEM_execute(RiCTaskEM * const me)
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	static RiC_INT8 first = RiCTRUE;
#endif
	RiCBoolean toTerminate = RiCFALSE; /* TRUE when the message loop should be endded */
	RiCBoolean continueFlag = RiCFALSE;

	if(me != NULL) {
		
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* force a cleanup of the call stack when the thread starts its event loop */
		if(first)
		{
			first = RiCFALSE;
			ARCCS_popall(OMCurrentCallStack); /*lint !e746 () */
		}
		
		ARCCS_notify(OMCurrentCallStack, NULL, omMethodMethod);
#endif
		
		while(toTerminate == RiCFALSE) {
			
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* Ensure we do not report again after "dummy" or cancelled events */
			RiCBoolean shouldNotify = RiCTRUE;
			/* Using function ARCEQ_isEmpty() for instrumentation */
			ARCEventQueue * aomEQ = ARCT_getEventQueue(me->aomTask);
			while (!ARCEQ_isEmpty(aomEQ)){
#else	/* _OMINSTRUMENT */
			RiCBoolean goAhead = RiCTRUE;
			while (goAhead != RiCFALSE) {
#endif
				RiCTakeEventStatus result = RiCTakeEventCompleted;
				RiCEvent * ev = NULL;
				RiCReactive * dest = NULL;
				continueFlag = RiCTRUE;
					
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				if (shouldNotify){
					if(!RiCTaskEMIsEndOfProcess()) {
						ARCS_notifyEvent(RiCTaskEM_getStepper(me));
						shouldNotify = RiCFALSE;
					}
				}
#endif
				
				/* Actually dispatch the event */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				if(aomEQ != NULL){
					ev = ARCEQ_get(aomEQ);
				}
#else
				/* Access the event queue */
				(void)RiCOSMutex_lock(&(me->taskMonitor));
				ev = RiCOSMessageQueue_get(&(me->eventQueue));
				{ /* the block is added to define isEmptyRes locally */
					RiCBoolean isEmptyRes = RiCOSMessageQueue_isEmpty(&(me->eventQueue));
					if(isEmptyRes != RiCFALSE) {
						if(me->myThread != NULL){
							(void) RiCOSEventFlag_reset(&(me->myThread->taskEventFlag));
							(void)RiCOSPendingEvent_reset(&(me->myThread->osTask));
						}
					}
				}
				(void)RiCOSMutex_free(&(me->taskMonitor));
				/* mutex is freed */
				
				if (ev == NULL) {
					goAhead = RiCFALSE;
					continueFlag = RiCFALSE;
				}
#endif
				
				if(continueFlag == RiCTRUE) {
					if(ev != NULL){
					#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
						if ((RiCEvent_getlId(ev)) != RiCCancelledEvent_id) { shouldNotify = RiCTRUE; }
					#endif
						result = RiCTaskEM_processEvent(me, ev, &dest);
						ev = NULL;					
					}
					if (result == RiCTakeEventReachTerminate) {
						toTerminate = RiCTaskEM_executeTerminate(me, dest, toTerminate);
						#if ((!(defined _OMINSTRUMENT)) || (defined _OM_TARGET_MON))
						if (toTerminate == RiCTRUE) {
							goAhead = RiCFALSE;
						}
						#endif
					}
				}
			}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* done with the events in this iteration so the call stack can be pop-ed entirly */
			ARCCS_popall(OMCurrentCallStack);
#endif
				
			if (toTerminate == RiCFALSE) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
				/*
				Entering idle state 
				If should stop on idle -- then stop
				else become idle 
				Note: if should stop on idle you never realy go idle
				*/
				(void)RiCTaskEM_hasPendingEvents(me);
#endif
				toTerminate = RiCTRUE;
			}
		}
	}
}
#endif
/* INLINE_TASK_EXECUTION */

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSResult RiCTaskEM_setPriority(RiCTaskEM * const me, RhpInteger pr)
{
	RiCOSResult res = (RiCOSResult) 0;
	if ((me != NULL) && (me->myThread != NULL)) { 
		res = RiCOSTask_setPriority(&(me->myThread->osTask), pr);
	}

	return res;
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/*******************/
/*******************/
RiCOSResult RiCTaskEM_suspend(RiCTaskEM * const me)
{
	RiCOSResult res = 0;
	if ((me != NULL) && (me->myThread != NULL)) { 
		res = RiCOSTask_suspend(&(me->myThread->osTask));
	}

	return res;
}

/*******************/
/*******************/
RiCOSResult RiCTaskEM_resume(RiCTaskEM * const me)
{
	RiCOSResult res = 0;
	if ((me != NULL) && (me->myThread != NULL)) { 
		res = RiCOSTask_resume(&(me->myThread->osTask));
	}

	return res;
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework containing APIs that might be used      */
/* Outside of the OXF itself by Generated code				     */
RiCOSResult RiCTaskEM_start(RiCTaskEM * const me)
{
	RiCOSResult res = (RiCOSResult) 0;
	if ((me != NULL) && (me->myThread != NULL)) { 
		res = RiCOSTask_start(&(me->myThread->osTask));
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSTask * RiCTaskEM_getOSTask(const RiCTaskEM * const me)
{
	RiCOSTask * res = NULL;
	if ((me != NULL) && (me->myThread != NULL)) { 
		res = (&(me->myThread->osTask));
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCOSHandle RiCTaskEM_getOSHandle(const RiCTaskEM * const me)
{
	RiCOSHandle res = (RiCOSHandle)0;
	if (me != NULL) { 
		res = RiCOSTask_getOSHandle(&(me->myThread->osTask));
	}

	return res;
}

/*******************/
/*******************/
RiCBoolean RiCTaskEM_queueEvent(RiCTaskEM * const me, RiCEvent * const ev, RiCBoolean fromISR)
{
	RiCOSResult result   = RiCFALSE;
	RiCBoolean retval	 = RiCFALSE;
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCBoolean res = RiCFALSE;
#endif
	RiCBoolean flag = RiCFALSE;
	
	if (me != NULL)
	{ 
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		if (!RiCEvent_isFrameworkEvent(ev)) {
			if (me->aomTask != NULL) 
			{
				ARCEventQueue * aomEQ = ARCT_getEventQueue(me->aomTask);
				if(aomEQ != NULL) {
					res = ARCEQ_put(aomEQ, ev, fromISR);
					
					if (me->myThread != NULL)
					{
						(void) RiCOSEventFlag_signal((&(me->myThread->taskEventFlag)));
						(void)RiCOSPendingEvent_signal(&(me->myThread->osTask));
					}
					
					flag = RiCTRUE;
				}
			}
			
			if(flag == RiCFALSE){
				res = RiCFALSE;
				flag = RiCTRUE;
			}
		}
#endif

		if(flag == RiCFALSE) { /* Access the event queue is guarded by a mutex */
			RiCBoolean wasEmpty = RiCFALSE;
			(void)RiCOSMutex_lock(&(me->taskMonitor));

			wasEmpty = RiCOSMessageQueue_isEmpty(&(me->eventQueue));
			
			/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
			result = RiCOSMessageQueue_put(&(me->eventQueue), (RhpAddress const)ev, fromISR);
			if ((wasEmpty == RiCTRUE) && (me->myThread != NULL)) 
			{
				/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
				(void)RiCOSEventFlag_signal((&(me->myThread->taskEventFlag)));
				(void)RiCOSPendingEvent_signal(&(me->myThread->osTask));
			}
			
			(void)RiCOSMutex_free(&(me->taskMonitor));
		}	/* mutex is freed */
	}
	
	if(result == RiCTRUE){
		retval = RiCTRUE;
	}
	return retval;
}
/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTaskEM_cancelEvent(const RiCTaskEM * const me, RiCEvent * const ev)
{

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	/* notify the user of this cancellation */
	if (me != NULL)	{
		ARCEventQueue * aomEQ = ARCT_getEventQueue(me->aomTask);
		ARCEvent * aomE = ARCUserEventMap_Find(ev);
		if (aomE) {
			ARCInstance* aomInstance = ARCI_reactiveObject2Instance(RiCEvent_getDestination(ev));
			ARCAnimationItem * aomAI = ARCAI(aomInstance);  /*lint !e740 () */
			if(aomEQ != NULL)
			{
				ARCEQ_notifyEventCancelled(aomEQ, aomE, aomAI);
			}
		}
	}
#endif
	/* cancel an event means marking it as cancelled */
	RiCEvent_setlId(ev, RiCCancelledEvent_id);
}

/*******************/
/*******************/
void RiCTaskEM_cancelEvents(RiCTaskEM * const me, struct RiCReactive_t * const dest, struct RiCMxfGlobals_t * const mxfGlblInst)
{
	if ((me != NULL) && (mxfGlblInst != NULL)){ 
		RhpPositive i = 0U;
		/* cancel any timeouts that are targeted to destination */
		/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
		(void) RiCTimerManager_unschedTm(&(mxfGlblInst->systemTimer), RiCAnyEvent_id ,dest);

		/*
		 cancel existing events that are targeted to destination
		 NOTE: There is not explicit care for making the following code guarded,
		 so base on the underline implementation of eventQueue,
		 messageList may not reflect the most updated content of the eventQueue
		*/
		for(i = 0U; i< me->eventQueue.m_baseQueue.count; i++){
			RiCEvent * tmEv = me->eventQueue.m_baseQueue.the_queue[i];
			if(tmEv != NULL){
				RiCReactive * tmDest = RiCEvent_getDestination(tmEv);
				if (tmDest == dest) {
					RiCTaskEM_cancelEvent(me, tmEv); 
				}
			}
		}
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCTaskEM_schedTm(const RiCTaskEM * const me, RhpUnlimitedPositiveNatural deltaTime, RiCEventId id, struct RiCReactive_t * reactive, const RhpString msg, struct RiCMxfGlobals_t * const mxfGlblInst)
{
	RiCEvent *timeout = NULL;
	if ((me != NULL) && (mxfGlblInst != NULL)){ 

		/* Create the timeout */
		(void)RiCOSTimerManagerMutex_lock(&(mxfGlblInst->systemTimer.CSMutex));
		timeout = RiCTimeout_create(id, reactive, deltaTime, mxfGlblInst, msg);
		(void)RiCOSTimerManagerMutex_free(&(mxfGlblInst->systemTimer.CSMutex)); 
		if(timeout != NULL)
		{
			ARCEvent* aomTimeout;
			RiCReactive* oxfDest;
			ARCInstance* aomDestI;
			ARCAnimationItem* aomDest;
			/* notify the event queue of this */
			aomTimeout = ARCUserEventMap_Find(timeout);
			oxfDest = RiCEvent_getDestination(timeout);
			aomDestI = ARCI_reactiveObject2Instance(oxfDest);
			aomDest = ARCAI(aomDestI);	/*lint !e740 (cast to base type) */
			ARCEQ_notifyTimeoutSet(RiCTaskEM_getEventQueue(me), aomTimeout, aomDest);
		}
	}

}
#else
void RiCTaskEM_schedTm_low(const RiCTaskEM * const me, RhpUnlimitedPositiveNatural deltaTime, RiCEventId id, struct RiCReactive_t * const reactive, struct RiCMxfGlobals_t * const mxfGlblInst)
{
	if ((me != NULL) && (mxfGlblInst != NULL)){ 

		/* Create the timeout */
		(void)RiCOSTimerManagerMutex_lock(&(mxfGlblInst->systemTimer.CSMutex));
		
		/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
		(void)RiCTimeout_create(id, reactive, deltaTime, mxfGlblInst, NULL);
		(void)RiCOSTimerManagerMutex_free(&(mxfGlblInst->systemTimer.CSMutex)); 
	}
}
#endif
/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTaskEM_unschedTm(RiCTaskEM * const me, RhpInteger id, struct RiCReactive_t * const reactive, struct RiCMxfGlobals_t * const mxfGlblInst)
{
	if ((me != NULL) && (mxfGlblInst != NULL)){
		/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */	/*LDRA_INSPECTED 458 S */
		RiCBoolean tmUnsched = RiCTimerManager_unschedTm(&(mxfGlblInst->systemTimer),(RiCEventId)id, reactive);
		if(tmUnsched == RiCFALSE){ /* timeout is `matured`, i.e. not found in TimerManager timeouts list - remove from event queue */
			RhpPositive ind = 0U;
			(void)RiCOSMutex_lock(&(me->taskMonitor));
			ind = me->eventQueue.m_baseQueue.head;
			while (ind != me->eventQueue.m_baseQueue.tail){
				RiCEvent * tmEv = me->eventQueue.m_baseQueue.the_queue[ind];
				RiCBoolean isTm = RiCEvent_isTimeout(tmEv);
				if(isTm == RiCTRUE){
					RiCEventId curId = RiCTimeout_getTimeoutId(tmEv);
					if (curId == id) {
						RiCReactive * const curDest = RiCEvent_getDestination(tmEv);
						if (curDest == reactive){
							RiCEvent_setlId(tmEv, RiCCancelledEvent_id);
						}
					}
				}
				ind++;
				if(ind == me->eventQueue.m_baseQueue.size){
					ind = 0U;
				}
			}
			(void)RiCOSMutex_free(&(me->taskMonitor));
		}
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCTaskEM_getTaskEndClbk(const RiCTaskEM * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteThread)
{
	RhpInteger res = 0;
	if ((me != NULL) && (me->myThread != NULL)) {
		res = RiCOSTask_getTaskEndClbk(&(me->myThread->osTask), clb_p, arg1_p, onExecuteThread);
	}

	return res;
}

/*******************/
/*******************/
void RiCTaskEM_setToGuardTask(RiCTaskEM * const me, RiCBoolean toGuard)
{
	if ((me != NULL) && (me->myThread != NULL)) { 
		me->myThread->toGuardTask |= toGuard;
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef MICROC_OXF_CLEANUP
static void RiCTaskEM_setEndOsTaskInCleanup(RiCTaskEM * const me, RiCBoolean inCleanup)
{
	if ((me != NULL) && (me->myThread != NULL)) {
		(void) RiCOSTask_setEndOSTaskInCleanup(&(me->myThread->osTask), inCleanup);
	}
}
#endif
/*******************/
/*******************/
void RiCTaskEM_free(const RiCTaskEM * const me) 
{
	if(me != NULL){
		(void)RiCOSMutex_free(&(me->taskMonitor));
	}
}

void RiCTaskEM_lock(const RiCTaskEM * const me)
{
	if (me != NULL) {
		(void)RiCOSMutex_lock(&(me->taskMonitor));
	}
}


/** Instrumentation methods **/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

ARCTask * RiCTaskEM_getARCTask(const RiCTaskEM * const me)
{
	ARCTask * res = NULL;
	if (me != NULL) { 
		res = me->aomTask;
	}

	return res;
}

/*******************/
/*******************/
ARCStepper * RiCTaskEM_getStepper(const RiCTaskEM * const me)
{
	ARCStepper * res = NULL;
	if ((me != NULL) || (me->aomTask != NULL)){
		res = ARCT_getStepper(me->aomTask);
	}

	return res;
}

/*******************/
/*******************/
ARCEventQueue * RiCTaskEM_getEventQueue(const RiCTaskEM * const me)
{
	ARCEventQueue * res = NULL;
	if ((me != NULL) && (me->aomTask != NULL)){ 
		res = ARCT_getEventQueue(me->aomTask); 
	}

	return res;
}

#endif


/***************************************************/
/****			Main task methods		        ****/
/***************************************************/

/*******************/
/*******************/
RiCEvent** RiCMainTask_getMainTaskQueue(struct RiCMxfGlobals_t * const mxfGlblInst)
{
	RiCEvent** retval = NULL;
#if ((defined RIC_MAIN_TASK_HAS_REACTIVES) || (defined _OMINSTRUMENT))
	if(mxfGlblInst != NULL){
		retval = (mxfGlblInst->mainTask.ric_eventQueue);
	}
#endif
	return retval;
}

/*******************/
/*******************/
RiCThread *RiCMainTask_getMainTaskThread(struct RiCMxfGlobals_t * const mxfGlblInst) 
{
	RiCThread *retval = NULL;
	if(mxfGlblInst != NULL){
		retval = &(mxfGlblInst->mainTask.ric_thread);
	}
	return retval;
}


/*******************/
/*******************/
RiCTaskEM * RiCMainTask(void)
{
	return &(mxfGlobals.mainTask.ric_task);
}

/*******************/
/*******************/
void RiCMainTask_start(void)
{
	RiCOSMainTask_start();
}
/*lint -restore */

/***************************************************/
/****			Global task methods		        ****/
/***************************************************/

#ifdef USE_DELAYS
void RiCOXFDelay(timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCDelay delay;
	(void) RiCDelay_init(&delay, t, mxfGlblInst);
}
#endif


RiCBoolean RiCTaskEM_hasPendingEvents(const RiCTaskEM * const ricTask)
{
	RiCBoolean retval = RiCFALSE;

	if (ricTask != NULL)
	{
		if (ricTask->myThread != NULL){
			retval = RiCOSEventFlag_hasPendingEvents(&(ricTask->myThread->osTask));
		}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		if (!(retval))
		{
			/*
			Entering idle state 
			If should stop on idle -- then stop
			else become idle 
			Note: if should stop on idle you never realy go idle
			*/
			ARCStepper * aomS = RiCTaskEM_getStepper(ricTask);
			ARCEventQueue * aomEQ = ARCT_getEventQueue(ricTask->aomTask);

			if ((aomS != NULL) && (ARCAI_isInteresting(&(aomS->aomaiBase), OMNotifyIdle))) { /*lint !e774 () */
				ARCS_notifyIdle(aomS);
			}
		}
#endif
	}
	return retval;
}


/*******************/
/*******************/
#ifdef RIC_USE_GLOBAL_TASK_LIST
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCTaskEM * RiCCurrentTask(void)
{
	return getTaskFromHandle(RiCOSTask_getCurrentTaskHandle());
}
#endif

void RiCTaskEM_destroy(RiCTaskEM * const me)
{
}

#ifdef MICROC_OXF_CLEANUP
void RiCTaskEM_cleanupAllTasks(void)
{
}
#endif

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCTaskEM_handleOpCallRequests(RiCTaskEM *me)
{
	if (me != NULL)	{
		ARCStepper * aomS = RiCTaskEM_getStepper(me);
		ARCEventQueue * aomEQ = ARCT_getEventQueue(me->aomTask);
		if (aomEQ != NULL) {
			(void) ARCEQ_pend(aomEQ);
			if (aomS != NULL) 		{
				ARCS_handleOpCallRequests(aomS);
			}
		}
	}
}
#endif
 
 
 
