#ifndef RiCTaskEM_H
#define RiCTaskEM_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/
/******************************************************************
 ---------------------------------------------------------------
|							RiCTaskEM.h                         |
 --------------------------------------------------------------- 
*******************************************************************/
  
#include "mxf_cfg.h"
#include "RiCOSWrap.h"
#include "RiCTimer.h"

#ifdef RIC_USE_GLOBAL_TASK_LIST
#ifdef MICROC_FLAT_INCLUDE
#include "RiCList.h"
#else
#include "Utils/RiCList.h"
#endif
#endif

/******************************************/
/**			Forward declarations		 **/
/******************************************/
typedef struct ARCThread ARCTask;

typedef enum RiCTakeEventStatus_t {
	eventNotConsumed = 0,
	RiCTakeEventError = 0,
	eventConsumed = 1,
	RiCTakeEventCompleted = 1,
	RiCTakeEventInCleanup = 2,
	RiCTakeEventReachTerminate = 3
} RiCTakeEventStatus;


/******************************************/
/**		Virtual table of RiCReactive	**/
/******************************************/
/*LDRA_INSPECTED 141 S : Type forward declaration */
struct RiCTaskEM_t;
typedef void (*RiCTaskEMExecute)(struct RiCTaskEM_t * const pTask);

typedef struct RiCTaskEM_Vtbl_t {
	RiCTaskEMExecute OXF_CONST execute;
} RiCTaskEM_Vtbl;

/******************************************/
/**			RiCTaskEM definition			 **/
/******************************************/
typedef struct RiCThread_t {
	RiCOSTask            osTask;			  /* the os specific task */
	RiCOSEventFlag       taskEventFlag;	/* the eventFlag to be used with a Asynchronized Active */
	RiCTaskEM_Vtbl * OXF_CONST vtbl;				/* the task v-table */
	RiCBoolean           toGuardTask;		/* guarded task flag */
#ifdef MICROC_OXF_CLEANUP
	RiCBoolean           deletionAllowed;	/* FALSE means the task cannot be deleted */
#endif
} RiCThread;


typedef struct RiCTaskEM_t {
	RiCThread * OXF_CONST myThread; /* The schedulable part */  
	RiCOSMutex  	  taskMonitor;	/* to be used when accessing protected code sections */
	RiCOSMessageQueue eventQueue;	/* the event queue */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCTask * aomTask;				/* the animation task */
#endif
} RiCTaskEM;

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
typedef struct RiCTaskEM_t RiCTask;
#endif


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
#ifdef OXF_STATIC_INIT
/*LDRA_INSPECTED 340 S */
#define RiCTaskEM_init(me, wrapTask, pVtbl, OSTsk, OSTskQueueEvent, evQueue, evQueueSize, itsThread, isThread, mutexIdentifier)\
	(RiCTaskEM_init_static((me), (wrapTask), (itsThread), (isThread), (mutexIdentifier)))
RiCBoolean RiCTaskEM_init_static(RiCTaskEM * const me, RiCBoolean wrapTask, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier);
#else
RiCBoolean RiCTaskEM_init(RiCTaskEM * const me, RiCBoolean wrapTask, const RiCTaskEM_Vtbl * pVtbl, 
				RiCOSHandle OSTsk, RiCOSEventHandle OSTskQueueEvent, RiCEvent** const evQueue, 
				RhpPositive evQueueSize, RiCThread * const itsThread, RiCBoolean isThread, OSMutexHandle mutexIdentifier);
#endif

/* init with task settings */
RiCBoolean RiCTaskEM_Init(RiCTaskEM * const me, const RiCOSTask * pOSTask, RhpUnlimitedNatural priority,
						 RhpUnlimitedNatural stackSize, RhpUnlimitedNatural messageQueueSize,
						 RiCBoolean dynamicMessageQueue, const RiCTaskEM_Vtbl * pVtbl, RiCEvent** const evQueue, 
						 RhpPositive evQueueSize, RiCThread * const itsThread, 
						 RiCBoolean isThread, OSMutexHandle mutexIdentifier);

/* cleanup the task */
void RiCTaskEM_cleanup(RiCTaskEM * const me);
void RiCTaskEM_destroy(RiCTaskEM * const me);


/* wait on the task flag */
void RiCTaskEM_flagWait(const RiCTaskEM * const me);

/* create with no task settings */
RiCTaskEM * RiCTaskEM_create(RiCBoolean wrapThread, const RiCTaskEM_Vtbl * pVtbl);

/* create with task settings */
#define RiCTaskEM_Create(name,priority, stackSize, messageQueueSize, dynamicMessageQueue, pVtbl) 

/* on task begin */
void RiCTaskEM_beginMyTask(RiCTaskEM * const me);

/* on task end */
void RiCTaskEM_endMyTask(RiCTaskEM * const me);

RiCBoolean RiCTaskEM_hasPendingEvents(const RiCTaskEM * const ricTask);

/***************************************************/
/****				Task methods				****/
/***************************************************/

/** the task main loop **/
#ifndef INLINE_TASK_EXECUTION
void RiCTaskEM_execute(RiCTaskEM * const me);

RhpPositive RiCTaskEM_doExecute(RiCTaskEM * const me);
#endif

/** set task priority **/
RiCOSResult RiCTaskEM_setPriority(RiCTaskEM * const me, RhpInteger pr);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/** suspend task **/
RiCOSResult RiCTaskEM_suspend(RiCTaskEM * const me);

/** resume suspended task **/
RiCOSResult RiCTaskEM_resume(RiCTaskEM * const me);
#endif

/** start task **/
RiCOSResult RiCTaskEM_start(RiCTaskEM * const me);

/** get the OSTask wrapper object **/
RiCOSTask * RiCTaskEM_getOSTask(const RiCTaskEM * const me);

/** get a handle to the underlying OS task **/
RiCOSHandle RiCTaskEM_getOSHandle(const RiCTaskEM * const me);

/** add an event to the event queue **/
RiCBoolean RiCTaskEM_queueEvent(RiCTaskEM * const me, RiCEvent * const ev, RiCBoolean fromISR);

/** remove an event from the event queue **/
void RiCTaskEM_cancelEvent(const RiCTaskEM * const me, RiCEvent * const ev);

/** remove all events of 'destination' from the event queue **/
void RiCTaskEM_cancelEvents(RiCTaskEM * const me, struct RiCReactive_t * const dest, struct RiCMxfGlobals_t * const mxfGlblInst);

/** set a timeout for 'reactive' **/
#if ((defined (_OMINSTRUMENT)) && (!(defined (_OM_TARGET_MON))))
void RiCTaskEM_schedTm(const RiCTaskEM * const me, RhpUnlimitedPositiveNatural deltaTime, RiCEventId id, struct RiCReactive_t * reactive, const RhpString msg, struct RiCMxfGlobals_t * const mxfGlblInst);
#else
/*LDRA_INSPECTED 340 S */
#define RiCTaskEM_schedTm(me, deltaTime, id, reactive, msg, mxfGlblInst) \
	( RiCTaskEM_schedTm_low((me), (deltaTime), (id), (reactive), (mxfGlblInst)) )
void RiCTaskEM_schedTm_low(const RiCTaskEM * const me, RhpUnlimitedPositiveNatural deltaTime, RiCEventId id, struct RiCReactive_t * const reactive, struct RiCMxfGlobals_t * const mxfGlblInst);
#endif

/** unset a timeout for 'reactive' **/
void RiCTaskEM_unschedTm(RiCTaskEM * const me, RhpInteger id, struct RiCReactive_t * const reactive, struct RiCMxfGlobals_t * const mxfGlblInst);

/** get a pointer to a callback function, used in execute()  **/
RhpInteger RiCTaskEM_getTaskEndClbk(const RiCTaskEM * const me, const RiCOSTaskEndCallBack * clb_p, const void ** arg1_p, RiCBoolean onExecuteThread);

/** set task as guarded (on/off) **/
void RiCTaskEM_setToGuardTask(RiCTaskEM * const me, RiCBoolean toGuard);

/** free task (guard) **/
void RiCTaskEM_free(const RiCTaskEM * const me); 

/** lock task (guard) **/
void RiCTaskEM_lock(const RiCTaskEM * const me);

/** Instrumentation methods **/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

struct ARCStepper;
struct ARCEventQueue;

	/** get the animation task **/
	ARCTask * RiCTaskEM_getARCTask(const  RiCTaskEM * const me);

	void RiCTaskEM_notifyIdle(const RiCTaskEM * const ricTask);

	/** get the animation task stepper **/
	struct ARCStepper * RiCTaskEM_getStepper(const  RiCTaskEM * const me);

	/** get the animation event queue **/
	struct ARCEventQueue * RiCTaskEM_getEventQueue(const  RiCTaskEM * const me);
#endif

/** close all active tasks **/
#ifdef MICROC_OXF_CLEANUP
void RiCTaskEM_cleanupAllTasks(void);
#endif


/**************************************/
/**		Main Task             		 **/
/**************************************/

/******************************************/
/**		Main Task Data            		 **/
/******************************************/
#if ((defined RIC_MAIN_TASK_HAS_REACTIVES) || (defined _OMINSTRUMENT))
#define RiC_MAIN_TASK_QUEUE_SIZE 10U
#else
#define RiC_MAIN_TASK_QUEUE_SIZE 0U
#endif

typedef struct RiCDefaultActive_t 
{
	RiCTaskEM ric_task;
	RiCThread ric_thread;

#if ((defined RIC_MAIN_TASK_HAS_REACTIVES) || (defined _OMINSTRUMENT))
	RiCEvent* ric_eventQueue[RiC_MAIN_TASK_QUEUE_SIZE];
#endif
} RiCDefaultActive;


#if ((defined RIC_MAIN_TASK_HAS_REACTIVES) || (defined _OMINSTRUMENT))
#define RiC_MAIN_TASK_QUEUE (mxfGlobals.mainTask.ric_eventQueue)
#else
#define RiC_MAIN_TASK_QUEUE NULL
#endif
  
/***************************************************/
/****			Main task methods		        ****/
/***************************************************/
RiCEvent** RiCMainTask_getMainTaskQueue(struct RiCMxfGlobals_t * const mxfGlblInst); 

RiCThread *RiCMainTask_getMainTaskThread(struct RiCMxfGlobals_t *const mxfGlblInst);

/** get the main task **/
RiCTaskEM * RiCMainTask(void);

/** start the main task **/
void RiCMainTask_start(void);

/***************************************************/
/****			Global task methods		        ****/
/***************************************************/

/** delay the task for t time units (miliseconds) **/
void RiCOXFDelay(timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst);

#ifdef RIC_USE_GLOBAL_TASK_LIST
/** get the current task **/
RiCTaskEM * RiCCurrentTask(void);
#endif

/*******************************************/
/**** Non-Extended OXF Mapping Macros	****/
/*******************************************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#define RiCTask_getARCTask(me) RiCTaskEM_getARCTask(me)
#define RiCTask_getOSHandle(me) RiCTaskEM_getOSHandle(me)
#define RiCTaskIsEndOfProcess() RiCTaskEMIsEndOfProcess()
#define RiCTask_getStepper(me) RiCTaskEM_getStepper(me)
void RiCTaskEM_handleOpCallRequests(RiCTaskEM *me);
/** check for close of process **/
RiCBoolean RiCTaskEMIsEndOfProcess();
#endif

#ifdef RIC_USE_GLOBAL_TASK_LIST
typedef struct RiCTaskGlobalInfo_t{
		RiCList tasksList;			/* list of all tasks */
		RiCOSMutex tasksListGuard;	/* list access guard */
		RiCBoolean endOfProcess;	/* end of process (termination) flag */
		RiCBoolean initialized;	
} RiCTaskGlobalInfo;
#endif

typedef struct RiCMxfGlobals_t{
	RiCTimerManager systemTimer;
	timeUnit timerCachedTicks;
#ifdef RIC_USE_GLOBAL_TASK_LIST
	RiCTaskGlobalInfo taskGlobalInfo;
#endif
	RiCDefaultActive mainTask;
	RiCEvent eventsPool[EVENTS_POOL_SIZE];
	RiCTimeout timeoutsPool[TIMEOUTS_POOL_SIZE];
	RhpAddress osSpecificGlobals;
	RhpAddress osContext;
} RiCMxfGlobals;

PUBLIC RiCMxfGlobals mxfGlobals;

#endif

