/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
*/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICTIMER_SOURCE
#define MXF_RICTIMER_SOURCE
#endif

#include <stdlib.h>
#include "RiCOxf.h"
#include "RiCOSWrap.h"
#include "RiCTimer.h"
#include "RiCTaskEM.h"
#include "RiCReactive.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "aominst.h" 
#include "aomeque.h" 
#include "aomstep.h" 
#include "aomthrd.h" 
#include "aommsg.h"
#include "aomevent.h"
#else
#include "aom/aominst.h" 
#include "aom/aomeque.h" 
#include "aom/aomstep.h" 
#include "aom/aomthrd.h" 
#include "aom/aommsg.h"
#include "aom/aomevent.h"
#endif
#include "aomtimerservice.h"
#endif


/*****************************************************************************/
/*****************************************************************************/
/******								RiCDelay							******/
/*****************************************************************************/
/*****************************************************************************/

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

static timeUnit RiCTimerManager_getCurrentTime(const RiCTimerManager * const me);
static timeUnit RiCTimerManager_getCurrentTime(const RiCTimerManager * const me)
{
	timeUnit retVal = (timeUnit)0;
	RhpBoolean gotSysTime = RiCOSTimerManager_getSystemTime(&retVal);
	if ((me != NULL) && (gotSysTime == RiCFALSE))
	{
		while ((me->Timer_available) == RiCFALSE){
			;
		}
		retVal = me->m_Time;
	}
	return retVal;
}

#ifdef USE_DELAYS
RiCBoolean RiCDelay_init(RiCDelay * const me, timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCBoolean res = RiCFALSE;
	RiCEvent *timeout = NULL;
	
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	ARCStepper * s;
#endif
	
	if ((me != NULL) && (mxfGlblInst != NULL)){
		/* Create the timeout event */
		timeout = RiCTimeout_create(RiCTimeoutDelayId, (RiCReactive*) me, t, NULL, NULL); /*lint !e740 () */
		
		/* Delegating the request to timer */
		if (timeout != NULL){
			RiCTimerManager_set(&(mxfGlblInst->systemTimer), timeout);
			/* stop and wait for this timeout */
			
			/* Create the stop signal */
			res = RiCOSEventFlag_init(&(me->stopSignal), NULL);
			(void) RiCOSEventFlag_reset(&(me->stopSignal));
			me->active = RiCTRUE;
			
			
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* notify my stepper that I'm stopping */
			s = OMGetCurrentStepper; /*lint !e746 () */
			ARCS_notifyLock(s, TRUE);
#endif
			
			/* Actually stop */
			(void) RiCOSEventFlag_wait(&(me->stopSignal), -1);
			
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
			/* notify my stepper that I'm continuing */
			ARCS_notifyLock(s, FALSE);
#endif
			
#ifdef MICROC_OXF_CLEANUP
			RiCOSEventFlag_cleanup(&(me->stopSignal));
#endif
			me->active = RiCFALSE;
		}
	}
	
	return res;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCDelay_cleanup(RiCDelay * const me)
{
	if (me != NULL) {
		RiCOSEventFlag_cleanup(&(me->stopSignal));
		me->active = RiCFALSE;
	}
}
#endif

/*******************/
/*******************/
RiCDelay * RiCDelay_create(timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst)
{
	RiCDelay * me = (RiCDelay * )RiC_malloc((RhpPositive)sizeof(RiCDelay));
	if (me != NULL) (void) RiCDelay_init(me, t, mxfGlblInst);

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	return me;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCDelay_destroy(RiCDelay * const me)
{
	if (me != NULL){
		RiCDelay_cleanup(me);
		RiC_free(me);
	}
}
#endif

/***************************************************/
/****				RiCDelay Methods			****/
/***************************************************/

void RiCDelay_wakeup(RiCDelay * const me)
{
	if (me != NULL){
		if (me->active) {
			(void) RiCOSEventFlag_signal(&(me->stopSignal));
		}
	}
}
#endif

/*****************************************************************************/
/*****************************************************************************/
/******							RiCTimerManager							******/
/*****************************************************************************/
/*****************************************************************************/

/***************************************************/
/****				Private methods				****/
/***************************************************/
static RiCBoolean RiCTimerManager_animConsumeEvent(RiCEvent *timeout);
static RiCBoolean RiCTimedAction_isPending(const RiCTimedAction* const timedAction);
static OSTimeUnit RiCTimedAction_getPeriod(const RiCTimedAction* const timedAction);
static OSTimeUnit RiCTimedAction_getDueTime(const RiCTimedAction* const timedAction);
static void RiCTimedAction_setDueTime(RiCTimedAction* const timedAction, OSTimeUnit p_time);
static void RiCTimedAction_advanceDueTime(RiCTimedAction* const timedAction);
static void RiCTimedAction_disable(RiCTimedAction* const timedAction);
static void RiCTimedAction_executeAction(RiCTimedAction* const timedAction);


static OSTimeUnit RiCTimedAction_getPeriod(const RiCTimedAction* const timedAction)
{
	OSTimeUnit retval = 0;
	if(timedAction != NULL){
		retval = timedAction->period;
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif

	return retval;

}

static RiCBoolean RiCTimedAction_isPending(const RiCTimedAction* const timedAction)
{
	RiCBoolean retval = RiCFALSE;
	if(timedAction != NULL){
		retval = timedAction->pending;
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif
	return retval;

}

static OSTimeUnit RiCTimedAction_getDueTime(const RiCTimedAction* const timedAction)
{
	OSTimeUnit retval = 0;
	if(timedAction != NULL){
		retval = timedAction->dueTime;
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif
	return retval;

}

static void RiCTimedAction_setDueTime(RiCTimedAction* const timedAction, OSTimeUnit p_time)
{
	if(timedAction != NULL){
		timedAction->dueTime = p_time;
	}
}

static void RiCTimedAction_advanceDueTime(RiCTimedAction* const timedAction)
{
	if(timedAction != NULL){
		(timedAction->dueTime) += (timedAction->period);
	}
}

static void RiCTimedAction_disable(RiCTimedAction* const timedAction)
{
	if(timedAction != NULL){
		timedAction->pending = RiCFALSE;
	}
}

static void RiCTimedAction_executeAction(RiCTimedAction* const timedAction)
{
	if((timedAction != NULL) && (timedAction->timedActionCB != NULL)){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/** check for opCalls from Tom **/
		if (timedAction->myTask != NULL)
			RiCTaskEM_handleOpCallRequests(timedAction->myTask);
#endif
		/*LDRA_INSPECTED 57 S : LDRA Analysis incorrect. Statement has side effect */
		(*(timedAction->timedActionCB))(timedAction->owner);
	}
	
}


static void RiCTimeout_notifyEventCancelled(RiCEvent * timeout, const RiCReactive * pReactive);
static void RiCTimeout_notifyEventCancelled(RiCEvent * timeout, const RiCReactive * pReactive)
{
	if((timeout != NULL) &&(pReactive != NULL)) {
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* notify the user of this cancellation */
		RiCTaskEM * task = RiCReactive_getTask(pReactive);
		if (task != NULL) {
			ARCEventQueue * eventQueue = ARCT_getEventQueue(task->aomTask);
			if (eventQueue != NULL) {
				ARCInstance * aomInstance = ARCI_reactiveObject2Instance(RiCEvent_getDestination(timeout)); /*lint !e740 () */
				ARCEQ_notifyEventCancelled(eventQueue, ARCUserEventMap_Find(timeout), ARCAI(aomInstance)); /*lint !e740 () */
			}
		}
#endif
	}
}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
/** get the next expired timeout */
static void goNext(RiCTimerManager * const me) 
{
	RiCEvent * tm;
	RiCTimedAction* timedAction;

	if (me != NULL){
		/* Advance to next relevent time */
		tm = me->timeouts;
		me->m_Time = RiCTimeout_getDueTime(tm);
		/* Go back to earlier relevent timed actions */
		timedAction = me->timedActions;
		while (timedAction != NULL) {
			if(RiCTimedAction_isPending(timedAction)) {
				if(RiCTimedAction_getDueTime(timedAction) < (OSTimeUnit) me->m_Time) {
					me->m_Time = RiCTimedAction_getDueTime(timedAction);
				}
			}
			timedAction = timedAction->next;
		}
	}
}
#endif

static void postExpiredTimeouts(RiCTimerManager * const me);
static void postExpiredTimeouts(RiCTimerManager * const me)
{
	RiCEvent *timeout = me->timeouts;
	RhpBoolean doBreak = RiCFALSE;
	while ((timeout != NULL) && (doBreak == RiCFALSE))
	{
		timeUnit dueTimeTmp = RiCTimeout_getDueTime(timeout);
		timeUnit curTime = RiCTimerManager_getCurrentTime(me);
		if(dueTimeTmp >  curTime) {
			doBreak = RiCTRUE;
		}
		else {
			RhpBoolean consumeEvRes = RiCFALSE;
			me->timeouts = RiCTimeout_getNextTm(timeout);
			RiCTimeout_setNextTm(timeout, NULL);
			consumeEvRes = RiCTimerManager_animConsumeEvent(timeout);
			if (consumeEvRes == RiCFALSE) {
				/* activate the timeout */
#ifdef RIC_USE_OS_CONTEXT
				RiCTimerManager_action(me, timeout, osContext);
#else
				RiCTimerManager_action(me, timeout);
#endif
			}
		}
		timeout = me->timeouts;
	}
}

static void postExpiredTimedActions(RiCTimerManager * const me);
static void postExpiredTimedActions(RiCTimerManager * const me)
{
	RiCTimedAction *timedAction = me->timedActions;
	while(timedAction != NULL) {
		/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
		if(RiCTimedAction_isPending(timedAction) != RiCFALSE){
			timeUnit curTime = RiCTimerManager_getCurrentTime(me);
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */ /*LDRA_INSPECTED 203 S */
			if(curTime >= RiCTimedAction_getDueTime(timedAction)){
				RiCTimedAction_executeAction(timedAction);
				if(RiCTimedAction_getPeriod(timedAction) != (OSTimeUnit) 0){
					RiCTimedAction_advanceDueTime(timedAction);
				}
				else {
					RiCTimedAction_disable(timedAction);
				}
			}
		}
		timedAction = timedAction->next;
	}
}

static void updateTimedActionsAndTimeouts(RiCTimerManager * const me, RiCBoolean shouldHandleTimedActions);
static void updateTimedActionsAndTimeouts(RiCTimerManager * const me, RiCBoolean shouldHandleTimedActions)
{
		RiCEvent *timeout = me->timeouts;
		/* 3.1 iterate over the timeouts and update the due time */
		while(timeout != NULL)	{
			timeUnit dueTimeTmp = RiCTimeout_getDueTime(timeout);
			RiCTimeout_setDueTime(timeout, (dueTimeTmp & (~RICTIMERMANAGER_OVERFLOW_MARK)));
			timeout = RiCTimeout_getNextTm(timeout);
		}

		/* 3.2 iterate over the timed-actions and update the triggered timeouts */
		if(shouldHandleTimedActions != RiCFALSE)
		{
			RiCTimedAction *timedAction = me->timedActions;
			while(timedAction != NULL) {
				OSTimeUnit currDueTimeTmAc = RiCTimedAction_getDueTime(timedAction);
				/*LDRA_INSPECTED 98 S : LDRA Analysis incorrect. Actual and formal parameters consistent */ /*LDRA_INSPECTED 458 S */
				/*LDRA_INSPECTED 97 S : Casting is required in Osek environment */
				RiCTimedAction_setDueTime(timedAction, currDueTimeTmAc & (~RICTIMERMANAGER_OVERFLOW_MARK));
				timedAction = timedAction->next;
			}
		}
		/* 3.3 update the timer counter */
		while ((me->Timer_available) == RiCFALSE){
			;
		}
		me->Timer_available = RiCFALSE;
		me->m_Time &= ~RICTIMERMANAGER_OVERFLOW_MARK;
		me->Timer_available = RiCTRUE;

}

/** post all expired timeouts **/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
static void post(RiCTimerManager * const me, RiCBoolean shouldHandleTimedActions, RiCBoolean fromAomThread);
static void post(RiCTimerManager * const me, RiCBoolean shouldHandleTimedActions, RiCBoolean fromAomThread)
{
#else
#ifdef RIC_USE_OS_CONTEXT
static void post(RiCTimerManager * const me, RiCOSContextType osContext);
static void post(RiCTimerManager * const me, RiCOSContextType osContext)
#else
static void post(RiCTimerManager * const me);
static void post(RiCTimerManager * const me)
#endif
{
	RiCBoolean shouldHandleTimedActions = RiCTRUE;
#endif
	if((me != NULL)){
		timeUnit curTime = 0UL;

		/* begin critical section */
		(void)RiCOSTimerManagerMutex_lock(&(me->CSMutex));

		/* 1. Execute the expired timeouts */
		postExpiredTimeouts(me);

		/* 2. Execute the expired timed actions */
		if(shouldHandleTimedActions != RiCFALSE)		{
			postExpiredTimedActions(me);
		}

		/* 3. check & reset time overflow */
		curTime = RiCTimerManager_getCurrentTime(me);
		/*LDRA_INSPECTED 139 S : Construct leads to feasible code */
		if (curTime >= RICTIMERMANAGER_OVERFLOW_MARK) {
			updateTimedActionsAndTimeouts(me, shouldHandleTimedActions);
		}

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* Try notify Idle for the main task */
		if ((!(fromAomThread)) && (me->timedActions == NULL)){
				RiCTaskEM_hasPendingEvents(RiCMainTask());
		}
#endif

		/* end critical section */
		(void)RiCOSTimerManagerMutex_free(&(me->CSMutex));
	}
}

#ifdef RIC_USE_MXF_GLOBALS

#ifdef RIC_USE_OS_CONTEXT
void RiCTimerManager_TimeTick(RiCOSContextType osContext)
#else
void RiCTimerManager_TimeTick(void)
#endif
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	/*
	This code ensures the timer will not advance during
	the "user's turn" in animation/trace
	This is done by not advancing the m_Time so the timers
	keeps ticking but never advances (until resume)
	For simulated time in effect this is a "busy wait".
	*/
	if (!(mxfGlobals.systemTimer.suspended)){
#endif
		if ((mxfGlobals.systemTimer.Timer_available) != RiCFALSE)
		{
			mxfGlobals.systemTimer.Timer_available = RiCFALSE;
			mxfGlobals.systemTimer.m_Time += mxfGlobals.timerCachedTicks;
			mxfGlobals.systemTimer.Timer_available = RiCTRUE;
			mxfGlobals.timerCachedTicks = RICTIMERMANAGER_DEFAULT_TICK_TIME;
		}
		else
		{
			mxfGlobals.timerCachedTicks += RICTIMERMANAGER_DEFAULT_TICK_TIME;
		}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	}
#endif

}

#ifdef RIC_USE_OS_CONTEXT
void RiCTimerManager_PostTimedElements(RiCOSContextType osContext)
#else
void RiCTimerManager_PostTimedElements(void)
#endif
{
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	if (!mxfGlobals.systemTimer.suspended){
		post(&(mxfGlobals.systemTimer), RiCTRUE, RiCFALSE);
	}
#else
#ifdef RIC_USE_OS_CONTEXT
	post(&(mxfGlobals.systemTimer), osContext);
#else
	post(&(mxfGlobals.systemTimer));
#endif
#endif
}

#endif

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef OXF_STATIC_INIT
RiCBoolean RiCTimedAction_init_static(RiCTimedAction * const me, struct RiCMxfGlobals_t *mxfGlblInst)
{
	if((me != NULL) && (mxfGlblInst != NULL)){
		RiCTimerManager_registerTimedAction(&(mxfGlblInst->systemTimer), me);
	}
	return RiCTRUE;
}
#else
#ifdef _OMINSTRUMENT
RiCBoolean RiCTimedAction_init(RiCTimedAction * const me, void (*clbck)(RhpAddress const), 
							   RhpAddress const tOwner, OSTimeUnit tDueTime,
							   OSTimeUnit tPeriod, RiCOSTimerHandle tTimer, 
							   RiCBoolean isAct, struct RiCTaskEM_t *pMyTask, struct RiCMxfGlobals_t *mxfGlblInst)
#else
RiCBoolean RiCTimedAction_init(RiCTimedAction * const me, void (*clbck)(RhpAddress const), 
							   RhpAddress const tOwner, OSTimeUnit tDueTime,
							   OSTimeUnit tPeriod, RiCOSTimerHandle tTimer, 
							   RiCBoolean isAct, struct RiCMxfGlobals_t *mxfGlblInst)
#endif
{
	if((me != NULL) && (mxfGlblInst != NULL)){
		me->timedActionCB = clbck;
		me->owner = tOwner;
		me->dueTime = tDueTime;
		me->period = tPeriod;
		me->pending = RiCTRUE;
		me->isActive = isAct;
		me->timer = tTimer;
		me->next = NULL;
#ifdef _OMINSTRUMENT
		me->myTask = pMyTask;
#endif
		
		RiCTimerManager_registerTimedAction(&(mxfGlblInst->systemTimer), me);
	}

	return RiCTRUE;
}
#endif

/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCTimerManager_init(RiCTimerManager * const me, timeUnit ticktime, RhpPositive maxTM)
{
	RiCBoolean retVal = RiCFALSE;
	if (me != NULL)
	{
		me->Timer_available = TRUE;
		retVal = RiCOSTimer_init(&(me->osTimer));
	}
	return retVal;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCTimerManager_cleanup(RiCTimerManager * const me)
{
	if (me != NULL) {
		/* internal timer need cleanup */
		RiCOSTimer_cleanup(&(me->osTimer));

		RiCOSMutex_cleanup(&(me->CSMutex));
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCTimerManager * RiCTimerManager_create(RhpInteger ticktime, RhpPositive maxTM)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCTimerManager * me = (RiCTimerManager *) RiC_malloc((RhpPositive)sizeof(RiCTimerManager));
	if (me != NULL){
		(void)RiCTimerManager_init(me, (RhpUnlimitedPositiveNatural) ticktime, maxTM);
	}
	

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif
	/*LDRA_INSPECTED 71 S */
	return me;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCTimerManager_destroy(RiCTimerManager * const me)
{
	if (me != NULL){
		RiCTimerManager_cleanup(me);
		RiC_free(me);
	}
}
#endif

/***************************************************/
/****		RiCTimerManager methods				****/
/***************************************************/

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCTimerManager *RiCTimerManager_getSystemTimer( void )
{
    return &mxfGlobals.systemTimer;

}
#endif

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef _OMINSTRUMENT
void RiCTimerManager_set(RiCTimerManager * const me, RiCEvent * timeout)
{
	if (me != NULL) {

		/* critical section for the timeouts structure */
		(void)RiCOSTimerManagerMutex_lock(&(me->CSMutex));
		{
			RiCTimeout_setRelativeDueTime(timeout,me->m_Time);
			RiCTimeout_addToList(timeout, &(me->timeouts));
		}
		(void)RiCOSTimerManagerMutex_free(&(me->CSMutex)); 
	}
}
#endif
/*******************
				RiCTimerManager_unschedTm.
Function logic:
1. first, look if the timeout is the current event
2. second, look if the timeout is in the timeouts list (for RiCAnyEvent_id, remove all)
3. third, look if the timeout is in the events queue (for RiCAnyEvent_id, remove all)
*******************/
RiCBoolean RiCTimerManager_unschedTm(RiCTimerManager * const me, RiCEventId id, struct RiCReactive_t * const pReactive)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{
	RiCBoolean doBreak = RiCFALSE;
	if(me != NULL){
		RiCEvent *curTm = NULL, *prevTm  = NULL;

		/* critical section	for the timeouts structure */
		(void)RiCOSTimerManagerMutex_lock(&(me->CSMutex)); 
		curTm = me->timeouts;
		while ( (curTm != NULL) && (doBreak == RiCFALSE))
		{
			RiCEventId curId = RiCTimeout_getTimeoutId(curTm);
			RiCReactive * const curDest = RiCEvent_getDestination(curTm);
			if(((id == curId) || (id == RiCAnyEvent_id)) && (pReactive == curDest)){
				if(id != RiCAnyEvent_id){
					doBreak = RiCTRUE;
				}
				/* remove tm from list */
				if(curTm == me->timeouts){ /* first item */
					me->timeouts = RiCTimeout_getNextTm(curTm);
				}
				else{
					RiCEvent *nextTmEv = RiCTimeout_getNextTm(curTm);
					RiCTimeout_setNextTm(prevTm, nextTmEv);
				}
				/* delete tm */
				RiCTimeout_notifyEventCancelled(curTm, pReactive);
				RiCTimeout_setNextTm(curTm, NULL);
				RiCTimeout_cleanup(curTm);
				RiCTimeout_Destroy(curTm);
			}
			prevTm = curTm;
			curTm = RiCTimeout_getNextTm(curTm);
		}

		/* critical section	for the timeouts structure */
		(void)RiCOSTimerManagerMutex_free(&(me->CSMutex));
	}
	return doBreak;
}



/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
timeUnit RiCTimerManager_getElapsedTime(const RiCTimerManager * const me)
{
	timeUnit res = 0UL;
	if (me != NULL){
		res = me->m_Time;
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif

	return res;
}

timeUnit RiCTimerManager_getSystemTimerElapsedTime(struct RiCMxfGlobals_t * const mxfGlblInst)
{
	timeUnit retval = 0UL;
	if(mxfGlblInst != NULL){
		retval = mxfGlblInst->systemTimer.m_Time;
	}
	return retval;
}

void RiCTimerManager_resetSystemTimerElapsedTime(struct RiCMxfGlobals_t * const mxfGlblInst)
{
	if(mxfGlblInst != NULL){
		mxfGlblInst->systemTimer.m_Time = 0UL;
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTimerManager_setElapsedTime(RiCTimerManager * const me, timeUnit newTime)
{
	if (me != NULL) {
		(void)RiCOSTimerManagerMutex_lock(&(me->CSMutex));
		me->m_Time = newTime & (~RICTIMERMANAGER_OVERFLOW_MARK);
		(void)RiCOSTimerManagerMutex_free(&(me->CSMutex));
	}
}

/*******************/
/*******************/
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
void RiCTimerManager_goNextAndPost(RiCTimerManager * const me)
{
    if( me != NULL )
    {
		/* Advance to the next timeout and post it */
		/* critical section for the timeouts structure */
		(void)RiCOSTimerManagerMutex_lock( &(me->CSMutex) );
		goNext(me);
		(void)RiCOSTimerManagerMutex_free( &(me->CSMutex) );
		post(me, RiCFALSE, RiCTRUE);	/* Handle that time event */
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
#ifdef RIC_USE_OS_CONTEXT
void RiCTimerManager_action(const RiCTimerManager * const me, RiCEvent * const timeout, RiCOSContextType osContext)
#else
void RiCTimerManager_action(const RiCTimerManager * const me, RiCEvent * const timeout)
#endif
{
	/* A timeout - look for the thread of the receiver */
	if(timeout != NULL){
		RiCReactive *dest = RiCEvent_getDestination(timeout); /*lint !e740 () */
		if (dest != NULL) {
#ifndef RIC_TIMEOUT_AS_TRIGGERED_OP
			RiCTaskEM *pTask = RiCReactive_getTask(dest);
			if (pTask != NULL) {
				(void) RiCTaskEM_queueEvent(pTask, timeout, RiCFALSE); /*lint !e740 () */
			}
#else
#ifdef RIC_USE_OS_CONTEXT
			(void) RiCReactive_takeTrigger(dest, timeout, osContext);
#else
			(void) RiCReactive_takeTrigger(dest, timeout);
#endif
			if (timeout->eventDestroyOp != NULL) {
				timeout->eventDestroyOp(timeout);	
			} /* No need to call RiCEvent_destroy otherwise, we assume eventDestroyOp exists */
#endif
		}
	}
}


/***************************************************/
/****	  RiCTimerManager animation methods		****/
/***************************************************/

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))

void RiCTimerManager_suspend(RiCTimerManager * const me)
{
	if (me != NULL){
		me->suspended = RiCTRUE;
	}
}

/*******************/
/*******************/
void RiCTimerManager_resume(RiCTimerManager * const me)
{
	if (me != NULL){
		me->suspended = RiCFALSE;
	}
}

/*******************/
/*******************/
void RiCTimerManager_RequestTimeNotification(struct RiCReactive_t * const timerServiceInstance, timeUnit deltaT, RiCEvent** timeout, timeUnit* baseTime)
{
	if ((timerServiceInstance != NULL) && (timeout != NULL))
	{
 		*timeout = RiCTimeout_create(0, timerServiceInstance, deltaT, NULL, NULL);
        if(*timeout != NULL)
        {
			RiCTimerManager_set(&(mxfGlobals.systemTimer), *timeout);
		}
	}
	if (baseTime != NULL)
	{
		*baseTime = RiCTimerManager_getElapsedTime(&(mxfGlobals.systemTimer));
	}
}

#endif

static RiCBoolean RiCTimerManager_animConsumeEvent(RiCEvent *timeout)
{
	RiCBoolean retval = RiCFALSE;

	if(timeout != NULL){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/* Special Timer event - Rhapsody timeout notification request
			see : RiCTimerManager_RequestTimeNotification */
		if (RiCTimeout_getTimeoutId(timeout) == 0) {
			/* animation */
			RiCReactive *dest = RiCEvent_getDestination(timeout);
			ARCTimerService_ConsumeEvent(dest, timeout);
			retval = RiCTRUE;
		}
#endif
	}

	return retval;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTimerManager_cbkBridge(const RhpAddress me)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{
	RiCMxfGlobals *mxfGlblInst = (RiCMxfGlobals *) me;
	if(mxfGlblInst != NULL){
		/* post all expiered timeouts */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		if (!mxfGlblInst->systemTimer.suspended){
			post(&(mxfGlblInst->systemTimer), RiCTRUE, RiCFALSE);
		}
#else
#ifdef RIC_USE_OS_CONTEXT
		post(&(mxfGlblInst->systemTimer), mxfGlblInst->osContext);
#else
		post(&(mxfGlblInst->systemTimer));
#endif
#endif
	}
} 

/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTimerManager_tickCbkBridge(const RhpAddress me)
/*LDRA_INSPECTED 103 S : LDRA Analysis incorrect. Function and prototype param consistent  */
{
	RiCMxfGlobals *mxfGlblInst = (RiCMxfGlobals *) me;
	if(mxfGlblInst != NULL){
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		/*
		This code ensures the timer will not advance during
		the "user's turn" in animation/trace
		This is done by not advancing the m_Time so the timers
		keeps ticking but never advances (until resume)
		For simulated time in effect this is a "busy wait".
		*/
		if (!(mxfGlblInst->systemTimer.suspended)){
#endif
			if ((mxfGlblInst->systemTimer.Timer_available) != RiCFALSE)
			{
				mxfGlblInst->systemTimer.Timer_available = RiCFALSE;
				mxfGlblInst->systemTimer.m_Time += mxfGlblInst->timerCachedTicks;
				mxfGlblInst->systemTimer.Timer_available = RiCTRUE;
				mxfGlblInst->timerCachedTicks = RICTIMERMANAGER_DEFAULT_TICK_TIME;
			}
			else
			{
				mxfGlblInst->timerCachedTicks += RICTIMERMANAGER_DEFAULT_TICK_TIME;
			}
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
		}
#endif
	}
} 

/***************************************************/
/****				RiTimedAction Methods		****/
/***************************************************/
/** Register a timed actions **/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCTimerManager_registerTimedAction(RiCTimerManager * me, RiCTimedAction* const timedAction)
{
	if(timedAction != NULL){
		/* Give the OS a chance to register the TimedAction using its own capabilities */
		RiCBoolean doRegister = RiCFALSE;
		if (timedAction->isActive == RiCFALSE) {
			doRegister = RiCTRUE;
		}
		if(doRegister == RiCFALSE){
			RhpBoolean registered = RiCOSTimerManager_registerTimedAction(timedAction->timer, timedAction->dueTime, timedAction->period);
			if (registered == RiCFALSE) {
				doRegister = RiCTRUE;
			}
		}
		if((doRegister != RiCFALSE) && (me != NULL))
		{
			/* If the OS did not register the action or it is not related to an Active, do the registration using the TimerManager */
			if(me->timedActions == NULL){
				me->timedActions = timedAction;
			}
			else {
				RiCTimedAction* tmpNode = me->timedActions;
				me->timedActions = timedAction;
				me->timedActions->next = tmpNode;
			}
		}
	}
}

