#ifndef RiCTimer_H
#define RiCTimer_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
*/
/******************************************************************
 ---------------------------------------------------------------
|							RiCTimer.h                         |
 --------------------------------------------------------------- 
*******************************************************************/
  

#include "mxf_cfg.h"
#include "RiCEvent.h"
#include "RiCOS.h"


/*****************************************************************************/
/*****************************************************************************/
/******								RiCDelay							******/
/*****************************************************************************/
/*****************************************************************************/

/******************************************/
/**			RiCDelay definition			 **/
/******************************************/
typedef struct RiCDelay_t {
	RiCOSEventFlag stopSignal;
	RiCBoolean active;
} RiCDelay;

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
RiCBoolean RiCDelay_init(RiCDelay * const me, timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst);

#ifdef MICROC_OXF_CLEANUP
void RiCDelay_cleanup(RiCDelay * const me);
#endif

RiCDelay * RiCDelay_create(timeUnit t, struct RiCMxfGlobals_t *mxfGlblInst);

#ifdef MICROC_OXF_CLEANUP
void RiCDelay_destroy(RiCDelay * const me);
#endif

/***************************************************/
/****				RiCDelay Methods			****/
/***************************************************/

/** activate a delay **/
void RiCDelay_wakeup(RiCDelay * const me);

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
struct RiCTaskEM_t;
#endif

/******************************************/
/**			RiCTimedAction definition			 **/
/******************************************/
typedef struct RiCTimedAction_t {
	void (*timedActionCB)(RhpAddress const); /* The call-back function to be called when time comes */
	RhpAddress owner;                   /* The owner of the action */
	OSTimeUnit dueTime;                 /* The pending time of the action */
	OSTimeUnit period;                  /* The period in which the action should be scheduled */
	RiCBoolean pending;                 /* A Flag that indicates if the ation is pending */
	RiCBoolean isActive;                /* A Flag that indicates if the ation is owned by an Active */
	
	RiCOSTimerHandle timer;             /* A handle to the timer to be used with the timed operation */

	struct RiCTimedAction_t* next;      /* The next timmed action */
#ifdef _OMINSTRUMENT
	struct RiCTaskEM_t *myTask;
#endif

} RiCTimedAction;

/*****************************************************************************/
/*****************************************************************************/
/******							RiCTimerManager							******/
/*****************************************************************************/
/*****************************************************************************/

/******************************************/
/**		RiCTimerManager definition		 **/
/******************************************/
typedef struct RiCTimerManager_t {
	RiCOSTimer osTimer;			  /* the os underlying timer */
	RiCOSMutex CSMutex;		 	  /* protecting timeouts list */
	RiCEvent* timeouts;			  /* The list of the registred timeouts */
	RiCTimedAction* timedActions; /* The list of the registred timed actions */
	timeUnit tick ; 			  /* timer resolution, updated every tick ms and counts time */
	timeUnit m_Time ; 			  /* current time */
	RiCBoolean Timer_available;	  /* protect timer update and read */
#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
	RiCBoolean suspended;		  /* Used by animation to suspend/resume */
#endif
} RiCTimerManager;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
#ifdef OXF_STATIC_INIT
RiCBoolean RiCTimedAction_init_static(RiCTimedAction * const me, struct RiCMxfGlobals_t *mxfGlblInst);
/*LDRA_INSPECTED 340 S */
#ifdef _OMINSTRUMENT
#define RiCTimedAction_init(me, clbck, tOwner, tDueTime,tPeriod, tTimer, isAct, pMyTask, mxfGlblInst) (RiCTimedAction_init_static((me), (mxfGlblInst)))
#else
#define RiCTimedAction_init(me, clbck, tOwner, tDueTime,tPeriod, tTimer, isAct, mxfGlblInst) (RiCTimedAction_init_static((me), (mxfGlblInst)))
#endif
#else
#ifdef _OMINSTRUMENT
RiCBoolean RiCTimedAction_init(RiCTimedAction * const me, void (*clbck)(RhpAddress const), 
							   RhpAddress const tOwner, OSTimeUnit tDueTime,
							   OSTimeUnit tPeriod, RiCOSTimerHandle tTimer, 
							   RiCBoolean isAct, struct RiCTaskEM_t *pMyTask, struct RiCMxfGlobals_t *mxfGlblInst);
#else
RiCBoolean RiCTimedAction_init(RiCTimedAction * const me, void (*clbck)(RhpAddress const), 
							   RhpAddress const tOwner, OSTimeUnit tDueTime,
							   OSTimeUnit tPeriod, RiCOSTimerHandle tTimer, 
							   RiCBoolean isAct, struct RiCMxfGlobals_t *mxfGlblInst);
#endif
#endif
RiCBoolean RiCTimerManager_init(RiCTimerManager * const me, timeUnit ticktime, RhpPositive maxTM);	

#ifdef MICROC_OXF_CLEANUP
void RiCTimerManager_cleanup(RiCTimerManager * const me);
#endif

RiCTimerManager * RiCTimerManager_create(RhpInteger ticktime, RhpPositive maxTM);

#ifdef MICROC_OXF_CLEANUP
void RiCTimerManager_destroy(RiCTimerManager * const me);
#endif

/***************************************************/
/****		RiCTimerManager methods				****/
/***************************************************/
/** add timeout to the timeouts requests list **/
void RiCTimerManager_set(RiCTimerManager * const me, RiCEvent * timeout);

/**
--------------------------------------------------------------------------------------------

	UnschedTm - cancelling a timeout for a single object or a single object
	
	This method is used (1) in case of exiting a state - timeout no longer relevant
						(2) In case where the object is destroyed. In that case all timers
						    associated with the object are destroyed. 

  Cancelling a timeout requires one of two actions:
		- Deleting the timeout from the timeouts list, or
		- Canceling it inside the event queue, if already dipatched. This in done bu iterating
		the "enuqued but not yet dispatched" list.
--------------------------------------------------------------------------------------------
**/
RiCBoolean RiCTimerManager_unschedTm(RiCTimerManager * const me, RiCEventId id, struct RiCReactive_t * const pReactive);

/** get the elapsed time as stored in the timer manager **/
timeUnit RiCTimerManager_getElapsedTime(const RiCTimerManager * const me);

/** set the elapsed time as stored in the timer manager **/
void RiCTimerManager_setElapsedTime(RiCTimerManager * const me, timeUnit newTime);

/** Activate the timeout **/
#ifdef RIC_USE_OS_CONTEXT
void RiCTimerManager_action(const RiCTimerManager * const me, RiCEvent * const timeout, RiCOSContextType osContext);
#else
void RiCTimerManager_action(const RiCTimerManager * const me, RiCEvent * const timeout);
#endif

/** get the elapsed time as stored in the System timer manager **/
timeUnit RiCTimerManager_getSystemTimerElapsedTime(struct RiCMxfGlobals_t * const mxfGlblInst);

void RiCTimerManager_resetSystemTimerElapsedTime(struct RiCMxfGlobals_t * const mxfGlblInst);

#ifdef RIC_USE_OS_CONTEXT
void RiCTimerManager_TimeTick(RiCOSContextType osContext);
void RiCTimerManager_PostTimedElements(RiCOSContextType osContext);
#else
void RiCTimerManager_TimeTick(void);
void RiCTimerManager_PostTimedElements(void);
#endif

#ifdef BACKWARD_COMPAT_752X
#ifndef PostTimedElements
#define PostTimedElements RiCTimerManager_PostTimedElements
#endif
#ifndef TimeTick
#define TimeTick RiCTimerManager_TimeTick
#endif
#endif


/***************************************************/
/****	  RiCTimerManager animation methods		****/
/***************************************************/

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
RiCTimerManager *RiCTimerManager_getSystemTimer( void );
/** Advance to the next timeout and post it **/
void RiCTimerManager_goNextAndPost(RiCTimerManager * const me);
/** Used by ARC to suspend/resume timer **/
void RiCTimerManager_suspend(RiCTimerManager * const me);
void RiCTimerManager_resume(RiCTimerManager * const me);
void RiCTimerManager_RequestTimeNotification(struct RiCReactive_t* const timerServiceInstance, timeUnit deltaT, RiCEvent** timeout, timeUnit* baseTime);
#endif

/***************************************************/
/****	  RiCTimerManager static methods		****/
/***************************************************/


/**
 This code ensures the timer will not advance during
 the "user's turn" in animation/trace
 This is done by not advancing the m_Time so the timers
 keeps ticking but never advances (until resume)
 For simulated time in effect this is a "busy wait".
**/
void RiCTimerManager_cbkBridge(const RhpAddress me);

void RiCTimerManager_tickCbkBridge(const RhpAddress me);

/** Register a timmed actions **/
void RiCTimerManager_registerTimedAction(RiCTimerManager * me, RiCTimedAction* const timedAction);

#endif

