#ifndef RiCTypes_H
#define RiCTypes_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2016. All Rights Reserved.
*/

/******************************************************************
 ----------	-----------------------------------------------------
|							RiCTypes.h                         |
 --------------------------------------------------------------- 
*******************************************************************/

#ifndef NO_STDDEF_H
#include <stddef.h>
#endif
#include "ricosconfig.h"
#include "mxf_cfg.h"

#ifndef STATIC
#define STATIC static
#endif

#ifndef PUBLIC
#define PUBLIC extern
#endif

typedef unsigned char RiC_UCHAR8;      /* character ASCII256 */
typedef unsigned char RiC_UINT8;       /* 8Bit unsigned integer */
typedef signed char RiC_INT8;          /* 8Bit signed integer */
typedef unsigned short RiC_UINT16;     /* 16Bit unsigned integer */
typedef signed short RiC_INT16;        /* 16Bit signed integer */
typedef unsigned int RiC_UINT32;       /* 32Bit unsigned integer */
typedef signed int RiC_INT32;          /* 32Bit signed integer */
#ifndef OM_NO_LONG_LONG_SUPPORT
typedef unsigned long long RiC_UINT64; /* 64Bit unsigned integer */
typedef signed long long RiC_INT64;	   /* 64Bit signed integer */
#else
typedef unsigned long RiC_UINT64; /* 64Bit unsigned integer */
typedef signed long RiC_INT64;	   /* 64Bit signed integer */
#endif
typedef float RiC_FLOAT32;             /* 32Bit floating point */
typedef double RiC_FLOAT64;            /* 64Bit floating point */
typedef signed char RiC_CHAR8;         /* character ASCII128 */

PUBLIC const RiC_INT32 RiC_INT32_MIN;
PUBLIC const RiC_INT32 RiC_INT32_MAX;


#ifdef CONSTANT_AS_UNSIGNED_INT
typedef RiC_UINT32 RiCBoolean;
#define RiCTRUE  (1)
#define RiCFALSE (0)
#else
typedef unsigned char RiCBoolean;
#define RiCTRUE  ((RiCBoolean)1U)
#define RiCFALSE ((RiCBoolean)0U)
#endif

#ifndef TRUE
#define TRUE RiCTRUE
#endif

#ifndef FALSE
#define FALSE RiCFALSE
#endif

#if defined(WIN32) && !defined(CYGWIN)
	typedef unsigned __int64 rhp_long64_t;
#else
	typedef unsigned long long rhp_long64_t;
#endif

typedef size_t Rhp_uintptr_t;

/* Time couter unit */
typedef unsigned long timeUnit;

typedef unsigned char RiCHandle;
typedef RiCHandle OMHandle;

/* Generic pointer (containers) */
typedef void* gen_ptr;

/* Constant generic pointer (containers) */
typedef const void *const_gen_ptr;

/******************************/
/* Language independent types */
/******************************/
typedef int				RhpInteger;
typedef long			RhpUnlimitedNatural;
typedef unsigned long	RhpUnlimitedPositiveNatural;
typedef unsigned int	RhpPositive;
typedef double			RhpReal;
typedef unsigned char	RhpCharacter;
#ifndef RHPSTRING_TYPEDEF_IN_MXF_CFG
typedef unsigned char*	RhpString;
#endif
typedef unsigned char*	RhpUnsignedString;
typedef signed char*	RhpSignedString;
typedef RiCBoolean		RhpBoolean;
typedef void*			RhpAddress;
typedef void			RhpVoid;
typedef short			RhpShort;
typedef unsigned short	RhpPositiveShort;
typedef int				Untyped;

/* -- support for C inheritance -- */
typedef unsigned long int	RiCUnLgInt;
typedef const void*			RiCVtbl;

/* Event ID */
typedef RhpShort RiCEventId;

/* Framework Globals Forward Declaration */
struct RiCMxfGlobals_t;

/******************************/
/* Language independent types */
/******************************/

/* reallocate memory of a buffer - used to support RTOS that doesn't support ANSI-C realloc()	*/
/* By default this method is implemented by realloc() in RiCOxf.c								*/
/* If your RTOS does not support reallc(), compile the framework with RiC_NO_RTOS_REALLOC and -	*/
/*	implement the method in the RTOS adaptor.													*/
RhpAddress ricRealloc( RhpAddress pContents, const size_t currentSize, const size_t newSize );
RhpAddress RiCGetMemory( const size_t requestedSize );
void RiCReturnMemory( const RhpAddress contents );

/* a dummy member to overcome objects without any members */
/*LDRA_INSPECTED 79 S */
#define RIC_EMPTY_STRUCT RiCBoolean dummy;

/* memory alignment for static memory support */
#define RiC_RAW_MEMORY_ALIGNMENT 16U

/* allow dynamic memory allocation */
#define USE_DYNAMIC_MEMORY_ALLOCATION

/* OMitoa */
#ifndef OMitoa
PUBLIC void OMitoa(RhpInteger val, RhpAddress str, RhpInteger base);
#endif

/***************************************************/
/****          Mock Functions                   ****/
/***************************************************/
void RiC_free(RhpAddress ptr);
RhpAddress RiC_malloc(RhpPositive memSize);

#endif

