/*

	purpose:	 	Static memory allocation (C version)

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 1999, 2012. All Rights Reserved.



 ---------------------------------------------------------------
|								MemAlloc.h	                      |
 --------------------------------------------------------------- 
*/

#ifndef MEMORY_ALLOCATOR_H
#define MEMORY_ALLOCATOR_H

/* Includes *******************************************************************/

#include "mxf_cfg.h"
#include "RiCProtected.h"

#if ((defined _OMINSTRUMENT) && (!(defined _OM_TARGET_MON)))
#ifdef MICROC_FLAT_INCLUDE
#include "om2str.h"
#else
#include "omcom/om2str.h"
#endif
#endif

#ifdef RIC_USE_MEMALLOC

/* Constants ******************************************************************/

/* Macros *********************************************************************/

/* Types **********************************************************************/

/* The static allocator. */
typedef struct RiCMemoryAllocator_t
{
    /* mutex for mutual exclusion */
    RiCOSMutex myMutex_;                              

    /* initial pool */
    RiC_UINT8 *initialBlock_;                         

    /* head of the free list */
    RhpAddress headOfFreeList_;                            

    /* callback function to override the memory allocation scheme */
    RhpAddress (*allocator_) ( RhpPositive numOfInstances );

    /* get the next empty memory location */
    RhpAddress (*getMemoryPoolNextChunk) ( RhpAddress const me );

    /* set the next empty memory location */
    void(*setMemoryPoolNextChunk) ( RhpAddress me, RhpAddress nextChunk );

    /* callback when the memory pool is empty */
    void(*MemoryPoolIsEmptyClbk) ( void );

    /* the size of a single object */
    size_t objectSize_;

    /* how much (in instance number) to increment if initial pool is exuasted */
    RhpPositive incrementNum_;

    /* the number of instances initialy allocated */
    RhpPositive initialNum_;

    /* activate the mutex */
    RiCBoolean isProtected_;

    /* when the flag is false, memoryPoolIsEmpty() will not be called */
    RiCBoolean memoryPoolIsEmptyFlag_;
} RiCMemoryAllocator;


/* Prototypes *****************************************************************/

void RiCMemoryAllocator_init( RiCMemoryAllocator * const me,
                              RiC_UINT8 * const memoryPool,
                              const size_t objectSize,
                              const RhpPositive initNum,
                              const RhpPositive incrementNum,
                              const RiCBoolean isProtected,
                              RhpAddress (*const getMemoryPoolNextChunkP)
                                    ( RhpAddress const me ),
                              void(*const setMemoryPoolNextChunkP)
                                  ( RhpAddress me, RhpAddress nextChunk ),
                              void(*const MemoryPoolIsEmptyClbkP) ( void ) );

void RiCMemoryAllocator_cleanup( const RiCMemoryAllocator *const me );

RhpAddress RiCMemoryAllocator_getMemory( RiCMemoryAllocator *const me,
                                    const size_t objSize );

void RiCMemoryAllocator_returnMemory( RiCMemoryAllocator * const me,
                                      RhpAddress  const deadObject,
                                      const size_t blockSize );

RhpAddress RiCMemoryAllocator_allocPool( const RiCMemoryAllocator *const me,
                                    const RhpPositive numOfInstances,
                                    size_t instSize );

RiC_INT32 RiCMemoryAllocator_initiatePool( RiCMemoryAllocator *const me,
                                           const RhpAddress newBlock,
                                           const RhpPositive numOfInstances );

void RiCMemoryAllocator_setIncrementNum( RiCMemoryAllocator *const me,
                                         const RhpPositive theValue );


void RiCMemoryAllocator_callMemoryPoolIsEmpty( RiCMemoryAllocator *const me,
                                               const RiCBoolean isEmptyFlag );

void RiCMemoryAllocator_setAllocator( RiCMemoryAllocator *const me,
                                      RhpAddress (*const newAllocator)(RhpPositive numOfInstances) );


void RiCMemoryAllocator_poolReallocationMsg(const RhpString className, RhpPositive incrementNumber);

#endif

#endif

