/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCCollection.c                   |
 --------------------------------------------------------------- 
*********************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICCOLLECTION_SOURCE
#define MXF_RICCOLLECTION_SOURCE
#endif

#include <stdlib.h>
#include "RiCCollection.h"
#ifdef RIC_USE_RICCOLLECTION

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCCollection_Init(RiCCollection * const me, RhpPositive cSize)
{
	if (me != NULL)
	{
		me->pos = 0U;
	}
	return RiCTRUE;
}

/**********************/
/**********************/
#ifdef MICROC_OXF_CLEANUP
void RiCCollection_Cleanup(RiCCollection * const me)
{
	if (me != NULL)
	{
		me->pos = 0;
	}
}
#endif

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCCollection * RiCCollection_Create(RhpPositive cSize)
{
/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCCollection * me = (RiCCollection *) RiC_malloc((RhpPositive)sizeof(RiCCollection));
	if (me != NULL){
		(void) RiCCollection_Init(me, cSize);
	}
#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return me;
}

/**********************/
/**********************/
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
#ifdef MICROC_OXF_CLEANUP
void RiCCollection_Destroy(RiCCollection * const me)
{
}
#endif

/***************************************************/
/****            Collection methods             ****/
/***************************************************/

RhpInteger RiCCollection_add(RiCCollection * const me, gen_ptr const pItem)
{
	RhpInteger res = -1;
	if (me != NULL)
	{
		if(me->pos < RiC_COLLECTION_STATIC_SIZE){
			me->collection[me->pos] = pItem;
			(me->pos)++;
			/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
			res = (RhpInteger)(me->pos - 1U);
		}
#ifdef USE_ASSERT
		else {
			RiC_ASSERT(NULL);
		}
#endif
	}

	return res;
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCCollection_remove(RiCCollection * const me, const_gen_ptr const pItem)
{
	RhpPositive i = 0U;
	if (me != NULL)
	{
		for (i = 0U; i < me->pos; i++) {
			if (me->collection[i] == pItem){
				--(me->pos);
				me->collection[i] = me->collection[me->pos];
			}
		}
	}
}

/**********************/
/**********************/
void RiCCollection_removeAt(RiCCollection * const me, RhpPositive ipos)
{
	if (me != NULL)
	{
		if (ipos < me->pos){
			--(me->pos);
			me->collection[ipos] = me->collection[me->pos];
		}
	}

}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCCollection_removeAll(RiCCollection * const me)
{
	if (me != NULL)
	{
		me->pos = 0U;
	}
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCCollection_find(const RiCCollection * const me, const_gen_ptr const pItem)
{
	RhpInteger res = -1;
	RhpBoolean flag = RiCFALSE;
	RhpPositive i;
	if (me != NULL)
	{
		for (i = 0U; (i < me->pos) && (flag == RiCFALSE); i++) {
			if (me->collection[i] == pItem){
				res = (RhpInteger) i;
				flag = RiCTRUE;
			}
		}
	}
	return res;
}

/**********************/
/**********************/
gen_ptr RiCCollection_getAt(const RiCCollection * const me, RhpPositive ipos)
{
	gen_ptr res = NULL;
	if (me != NULL)
	{
		if (ipos < me->pos){
			res = me->collection[ipos];
		}
#ifdef USE_ASSERT
		else
		{
			RiC_ASSERT(NULL);
		}
#endif
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCCollection_setAt(RiCCollection * const me, gen_ptr const pItem, RhpPositive ipos)
{
	RhpInteger res = 0;
	/* new element */
	if (me != NULL)
	{
		if (ipos >= me->pos){
			res = RiCCollection_add(me, pItem);
		}
		else {
			/* override existing */
			me->collection[ipos] = pItem;
			res = (RhpInteger)ipos;
		}
	}

	return res;
}

/**********************/
/**********************/
RhpPositive RiCCollection_getCount(const RiCCollection * const me)
{
	RhpPositive res = 0U;
	if (me != NULL){
		res = me->pos;
	}

	return res;
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
void RiCCollection_setFixedSize(const RiCCollection * const me, RiCBoolean b)
{
}


/***************************************************/
/****            Iteration methods              ****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCCollection_first(const RiCCollection * const me, RhpPositive * ipos)
{
	gen_ptr res = NULL;
	*ipos = 0U;
	if ((me != NULL) && (me->pos != 0U)){
		res = me->collection[0];
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif

	/*LDRA_INSPECTED 71 S */
	return res;
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCCollection_next(const RiCCollection * const me, RhpPositive * ipos)
{
	gen_ptr res = NULL;
	if ((me != NULL) && (ipos != NULL)){
		if ((me->pos) <= ((*ipos) + 1U)) {
			(*ipos)++;
			res = NULL;
		}
		else{
			++(*ipos);
			/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
			res = me->collection[(*ipos)];
		}
	}

	/*LDRA_INSPECTED 71 S */
	return res;

/*LDRA_INSPECTED 5 D */
}


/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCCollection_get(const RiCCollection * const me, RhpPositive ipos)
{
	gen_ptr res = NULL;
	if ((me != NULL) && (me->pos > ipos)){
		res = me->collection[ipos];
	}
#ifdef USE_ASSERT
	else
	{
		RiC_ASSERT(NULL);
	}
#endif

	/*LDRA_INSPECTED 71 S */
	return res;
}

/**********************/
/**********************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCCollection_isDone(const RiCCollection * const me, RhpPositive ipos)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL){
		res = (RiCBoolean)(ipos >= me->pos);
	}

	return res;
}

#endif
