#ifndef RiCCollection_H
#define RiCCollection_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/*********************************************************************
 ---------------------------------------------------------------
|							  RiCCollection.h                   |
 --------------------------------------------------------------- 
**********************************************************************/


#include "mxf_cfg.h"

/*************************/

/******************************************/
/** The Collection Definition			 **/
/******************************************/

typedef struct RiCCollection_t 
{
	gen_ptr collection[RiC_COLLECTION_STATIC_SIZE];	/* the actual array */
	RhpPositive pos;		/* corrent position */
} RiCCollection;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

/** return RiCTrue on sucess **/
RiCBoolean RiCCollection_Init(RiCCollection * const me, RhpPositive cSize);

#ifdef MICROC_OXF_CLEANUP
void RiCCollection_Cleanup(RiCCollection * const me);
#endif

RiCCollection * RiCCollection_Create(RhpPositive cSize);

#ifdef MICROC_OXF_CLEANUP
void RiCCollection_Destroy(RiCCollection * const me);
#endif

/***************************************************/
/****            Collection methods             ****/
/***************************************************/

/** Add an element to the collection,
	return the index of the added element or -1 on failer **/
RhpInteger RiCCollection_add(RiCCollection * const me, gen_ptr const pItem);

/** Remove an element from the collection **/
void RiCCollection_remove(RiCCollection * const me, const_gen_ptr const pItem);

/** Remove an element from the collection by its position **/
void RiCCollection_removeAt(RiCCollection * const me, RhpPositive ipos);

/** Remove all collection elements - without cleanup of the heap **/
void RiCCollection_removeAll(RiCCollection * const me);

/** Find an element in the collection,
	return the element index if found, or -1 otherwise **/
RhpInteger RiCCollection_find(const RiCCollection * const me, const_gen_ptr const pItem);

/** Return the element at a given index, or NULL, if there is no such index **/
gen_ptr RiCCollection_getAt(const RiCCollection * const me, RhpPositive ipos);

/** Set an element on a given index,
	if the index is not in scope add the element at the end of the collection.
	return the index of the element **/
RhpInteger RiCCollection_setAt(RiCCollection * const me, gen_ptr const pItem, RhpPositive ipos);

/** return the number of elements in the collection **/
RhpPositive RiCCollection_getCount(const RiCCollection * const me);

/** Set the fixed size flag **/
void RiCCollection_setFixedSize(const RiCCollection * const me, RiCBoolean b);

/***************************************************/
/****            Iteration methods              ****/
/***************************************************/
/********************************************************/

/** Set iteration to the first element in the collection **/
gen_ptr RiCCollection_first(const RiCCollection * const me, RhpPositive * ipos);

/** Set iteration to the next element in the collection **/
gen_ptr RiCCollection_next(const RiCCollection * const me, RhpPositive * ipos);

/** Get the current element in the iteration **/
gen_ptr RiCCollection_get(const RiCCollection * const me, RhpPositive ipos);

/** Check of last element, return RiCTRUE when the current element,
	is the last element in the collection **/
RiCBoolean RiCCollection_isDone(const RiCCollection * const me, RhpPositive ipos);


/*************************/

#endif
