/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCHeap.c                          |
 --------------------------------------------------------------- 
********************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICHEAP_SOURCE
#define MXF_RICHEAP_SOURCE
#endif

#include <stdlib.h>
#include "RiCHeap.h"

#ifdef RIC_USE_RICHEAP

/***************************************************/
/****			 Private operations		        ****/
/***************************************************/
static void takeUp(RiCHeap * const me, gen_ptr const pItem, RhpPositive emptyPos);
static void takeDown(RiCHeap * const me, gen_ptr const pItem, RhpPositive emptyPos);

/** Default isLarger operation, compare addresses **/
#ifndef OXF_STATIC_INIT

static RiCBoolean RiCHeap_isLarger(const gen_ptr me, const gen_ptr other);
static RiCBoolean RiCHeap_isLarger(const gen_ptr me, const gen_ptr other)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
	/* Cast is required to implement sorting of heap elements		 */
	/*LDRA_INSPECTED 439 S */ /*LDRA_INSPECTED 203 S */
	/*LDRA_INSPECTED 439 S */ /*LDRA_INSPECTED 203 S */
	return ((RiCBoolean) (((RiC_UINT64) me) > ((RiC_UINT64) other)));
}
#endif

/** add an item up the heap **/
static void takeUp(RiCHeap * const me, gen_ptr const pItem, RhpPositive emptyPos)
{
	/*
	 item is a node which needs to be placed in the heap.
	 Its position can be either 'emptyPos' which is currently empty
	 or some position higher than emptyPos (if elements currently there are bigger than item)
	*/
	RhpBoolean flag = RiCFALSE;
	RiCBoolean keepLooking = RiCTRUE;
	RhpPositive oneHigher;
	if (me != NULL)
	{
		while (flag == RiCFALSE) { /*lint !e716 (while(1) ... ) */
			keepLooking = RiCFALSE;
			oneHigher = emptyPos / 2U;
			if (oneHigher>0U){
				/*LDRA_INSPECTED 19 D : LDRA Analysis incorrect. Function Pointer call has no available function definition  */
				/*LDRA_INSPECTED 41 D : LDRA Analysis incorrect. Function Pointer call has no available function declaration */	/*LDRA_INSPECTED 496 S */
				if(me->isLarger(me->the_heap[oneHigher],pItem) != RiCFALSE) {
					keepLooking = RiCTRUE;
				} 
			}

			if(keepLooking == RiCTRUE){
				/* There is a higher position and the element there is bigger than item */
				/* 1. Put the bigger element in the empty position */
				me->the_heap[emptyPos] = me->the_heap[oneHigher];
				/* 2. Make its old position the empty one */
				emptyPos = oneHigher;
			}
			else {
				/* Put item in the empy position */
				me->the_heap[emptyPos] = pItem;
				flag = RiCTRUE;
			}
		}
	}
}

/** fill a hole down the heap **/
static void takeDown(RiCHeap * const me, gen_ptr const pItem, RhpPositive emptyPos)
{
	RiCBoolean doContinue = (RiCBoolean)(me != NULL);
	while(doContinue != RiCFALSE){
		/*
		item is a node which needs to be placed in the heap.
		Its position can be either 'emptyPos' which is currently empty
		or some position lower than emptyPos (if elements currently there are smaller than item)
		*/
		RhpPositive oneLower = emptyPos * 2U;
		doContinue = RiCFALSE;
		
		/* Case 1 - Currently there are no lower positions */
		if (oneLower > me->count) { /* Place e in empty position and quit */
			me->the_heap[emptyPos] = pItem;
		}
		else if (oneLower == me->count) { /* Case 2 - There is one lower position */
			/* The element there is smaller */
			if (me->isLarger(pItem, me->the_heap[oneLower]) != RiCFALSE) { 
				/* Put the smaller element in the empty position */
				me->the_heap[emptyPos] = me->the_heap[oneLower];
				/* Put item in the lower position (which we know does not have occupied positions below it) */
				me->the_heap[oneLower] = pItem;
			} 
			/* The element there is bigger, Put item in the empty position */
			else{
				me->the_heap[emptyPos] = pItem ;
			}
		}
		else {
			/* The general Case There are two lower positions */
			/* (oneLower and oneLower+1) */
			
			/*
			Make oneLower the position holding the smaller of the items
			Since me->the_heap[i] < me->the_heap[2*i] && me->the_heap[i] < me->the_heap[2*i+1]
			is our invariant
			*/
			if (me->isLarger(me->the_heap[oneLower], me->the_heap[oneLower+1U]) != RiCFALSE){
				oneLower++;
			}
			/* The "lower" element is smaller */
			if (me->isLarger(pItem, me->the_heap[oneLower]) != RiCFALSE) { 
				/* Put the smaller element in the empty position */
				me->the_heap[emptyPos] = me->the_heap[oneLower];
				emptyPos = oneLower;
				doContinue = RiCTRUE;
			} 
			else{ 
				me->the_heap[emptyPos] = pItem; /* Put e in the empty position */
			}
		}
	}
}



/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
/* Justification: Compilation warning Unreferenced Formal Parameter				  */
/* The OXF is a general purpose framework. This parameter is currently not in use */
RiCBoolean RiCHeap_Init(RiCHeap * const me, RhpPositive Hsize, RiC_isLarger isLargerIn)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		me->count = 0U;
#ifndef OXF_STATIC_INIT
		if(isLargerIn != NULL){
			me->isLarger = isLargerIn;
		}
		else{
			me->isLarger = (RiC_isLarger) &RiCHeap_isLarger; 
		}
#endif
		
		res = RiCTRUE;
	}

	return res;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCHeap_Cleanup(RiCHeap * const me)
{
	if (me != NULL)
	{
		me->count = 0;
#ifndef OXF_STATIC_INIT
		me->isLarger = NULL;
#endif
	}
}
#endif

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCHeap * RiCHeap_Create(RhpPositive Hsize, RiC_isLarger isLargerIn)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCHeap * me = (RiCHeap *) RiC_malloc((RhpPositive)sizeof(RiCHeap));
	if (me != NULL){
		(void) RiCHeap_Init(me, Hsize, isLargerIn);
	}
#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return me;
}

/*******************/
/*******************/
#ifdef MICROC_OXF_CLEANUP
void RiCHeap_Destroy(RiCHeap * const me)
{
	if (me != NULL) {
		RiCHeap_Cleanup(me);
		RiC_free(me);
	}
}
#endif

/***************************************************/
/****				Heap methods	            ****/
/***************************************************/

/*******************/
/*******************/
RiCBoolean RiCHeap_add(RiCHeap * const me, gen_ptr const pItem)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL)
	{
		if ((me->count + 1U) >= HEAP_CONST_SIZE) {
			res = RiCFALSE;
		}
		else {
			(me->count)++;
			takeUp(me, pItem, me->count);
			res = RiCTRUE;
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(res != RiCFALSE);
#endif

	return res;
}

/*******************/
/*******************/
gen_ptr RiCHeap_remove(RiCHeap * const me, gen_ptr const pItem)
{
	RhpInteger tmpPosition;
	gen_ptr retVal = NULL;
	gen_ptr last;
	if (me != NULL)
	{
		tmpPosition = RiCHeap_find(me, pItem);
		if (tmpPosition >= 0){
			RhpPositive position = (RhpPositive) tmpPosition;
			retVal = me->the_heap[position];
			/*
			 Remove theheap[position] by:
			 1. Decrementing count, 
			 2. Making position empty and
			 3. Trying to place in it the last heap element
			*/
			last = me->the_heap[me->count]; /* 2,3 */
			(me->count)--;
			/* Reorder the heap  (1) */
			if(last != NULL){
				if (position == 1U) {
					takeDown(me, last, position);
				}
				else if(me->isLarger(last, me->the_heap[position/2]) != RiCFALSE){
					takeDown(me, last, position);
				}
				else{
					takeUp(me, last, position);
				}
			}
		}
	}

#ifdef USE_ASSERT
	RiC_ASSERT(retVal != NULL);
#endif	

	/*LDRA_INSPECTED 71 S */
	return retVal;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCHeap_find(const RiCHeap * const me, gen_ptr const pItem)
{
	RhpInteger res = -1;
	RhpBoolean flag = RiCFALSE;
	RhpPositive i;
	if (me != NULL)
	{
		for (i = 1U; (i <= me->count) && (flag == RiCFALSE); i++) {
			if (me->the_heap[i] == pItem){
				res = (RhpInteger) i;
				flag = RiCTRUE;
			}
		}
	}
	return res;
}

/*******************/
/*******************/
gen_ptr RiCHeap_top(const RiCHeap * const me)
{
	if (me != NULL)
	{
		return me->the_heap[1];
	}
	else
	{
		return NULL;
	}
}

/*******************/
/*******************/
gen_ptr RiCHeap_trim(RiCHeap * const me)
{
	gen_ptr tmp = NULL;
	if (me != NULL)
	{
		RiCBoolean needTakeDown = RiCFALSE;
		tmp = RiCHeap_top(me);
		if(me->count > 1U){
			needTakeDown = RiCTRUE;
		}

		if (me->count > 0U) {
			--me->count;
			if (needTakeDown != RiCFALSE) {
				takeDown(me, me->the_heap[me->count + 1U], 1U);
			}
		}

#ifdef USE_ASSERT
		RiC_ASSERT(tmp != NULL);
#endif
	}

	/*LDRA_INSPECTED 71 S */
	return tmp;
}


/*******************/
/*******************/
RiCBoolean RiCHeap_isEmpty(const RiCHeap * const me)
{
	RiCBoolean retval = RiCTRUE;

	if(me != NULL){
		if(me->count != 0U){
			retval = RiCFALSE;
		}
	}

	return retval;
}

#endif
