#ifndef RiCHeap_H
#define RiCHeap_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/


/*******************************************************************
 ---------------------------------------------------------------
|							  RiCHeap.h                          |
 --------------------------------------------------------------- 
********************************************************************/


#include "mxf_cfg.h"

/******************************************/
/**		Compare function for the heap	 **/
/**										 **/
/** return true if me > other.			 **/
/******************************************/
typedef RiCBoolean (*RiC_isLarger)(gen_ptr me, gen_ptr other);


/******************************************/
/**			Defenition of RiCHeap		 **/
/******************************************/
typedef struct RiCHeap_t {
	gen_ptr the_heap[HEAP_CONST_SIZE];		/* the heap */
	RiC_isLarger OXF_CONST isLarger;	/* compare function */
	RhpPositive count;		/* number of elements currently on the heap */
	
} RiCHeap;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

RiCBoolean RiCHeap_Init(RiCHeap * const me, RhpPositive Hsize, RiC_isLarger isLargerIn);

#ifdef MICROC_OXF_CLEANUP
void RiCHeap_Cleanup(RiCHeap * const me);
#endif

RiCHeap * RiCHeap_Create(RhpPositive Hsize, RiC_isLarger isLargerIn);

#ifdef MICROC_OXF_CLEANUP
void RiCHeap_Destroy(RiCHeap * const me);
#endif

/***************************************************/
/****				Heap methods	            ****/
/***************************************************/

/** Add an element to the head **/
RiCBoolean RiCHeap_add(RiCHeap * const me, gen_ptr const pItem);

/** Remove an element from the heap **/
gen_ptr RiCHeap_remove(RiCHeap * const me, gen_ptr const pItem);

/** Find an element in the heap, return the element index or -1 if not found **/
RhpInteger RiCHeap_find(const RiCHeap * const me, gen_ptr const pItem);

/** Return the top of the heap **/
gen_ptr RiCHeap_top(const RiCHeap * const me);

/** Remove the top of the heap **/
gen_ptr RiCHeap_trim(RiCHeap * const me);

/** Check whether the heap is empty **/
RiCBoolean RiCHeap_isEmpty(const RiCHeap * const me);

#endif
