/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCList.c                          |
 --------------------------------------------------------------- 
*********************************************************************/

#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICLIST_SOURCE
#define MXF_RICLIST_SOURCE
#endif

#include <stdlib.h>
#include "RiCList.h"
#ifdef RIC_USE_RICLIST


/**********************************************/
/**			RiCListItem	Methods		    	 **/
/**********************************************/
static void RiCListItem_Init(RiCListItem * const me, gen_ptr const pItem);
#ifdef MICROC_OXF_CLEANUP
static void RiCListItem_Cleanup(RiCListItem * const me);
static void RiCListItem_Destroy(RiCListItem * const me);
#endif
static RiCListItem * RiCListItem_Create(gen_ptr const pItem);
static void RiCListItem_setNext(RiCListItem * const me, RiCListItem * const pNext);
static RiCListItem * RiCListItem_getNext(const RiCListItem * const me);
static gen_ptr RiCListItem_get(const RiCListItem * const me);

static void RiCListItem_Init(RiCListItem * const me, gen_ptr const pItem)
{
	if (me != NULL)
	{
		me->next = NULL;
		me->item = pItem;
	}
}

/*******************/
#ifdef MICROC_OXF_CLEANUP
static void RiCListItem_Cleanup(RiCListItem * const me) 
{
	if (me != NULL)
	{
		me->next = NULL;
		me->item = NULL;
	}
}
#endif
/*******************/
static RiCListItem * RiCListItem_Create(gen_ptr const pItem)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCListItem * me = (RiCListItem *)RiC_malloc((RhpPositive)sizeof(RiCListItem));
	if (me != NULL) {
		RiCListItem_Init(me, pItem);
	}

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif
	/*LDRA_INSPECTED 71 S */
	return me;
}

/*******************/
#ifdef MICROC_OXF_CLEANUP
static void RiCListItem_Destroy(RiCListItem * const me)
{
	if (me != NULL) {
		RiCListItem_Cleanup(me);
		RiC_free(me);
	}
}
#endif

/*******************/
static void RiCListItem_setNext(RiCListItem * const me, RiCListItem * const pNext)
{
	if (me != NULL){
		me->next = pNext;
	}
}

/*******************/
static RiCListItem * RiCListItem_getNext(const RiCListItem * const me)
{
	RiCListItem * res = NULL;
	if (me != NULL){
		res = me->next;
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
static gen_ptr RiCListItem_get(const RiCListItem * const me)
{
	gen_ptr res = NULL;
	if (me != NULL){
		res = me->item;
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}



/***********************************************************************/
/***********************************************************************/
/*							RiCList Methods							   */
/***********************************************************************/
/***********************************************************************/

/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

RiCBoolean RiCList_Init(RiCList * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL){
		me->head = NULL;
		me->tail = NULL;
		res = RiCTRUE;
	}

	return res;
}

/*******************/
/*******************/
void RiCList_Cleanup(RiCList * const me)
{
#ifdef MICROC_OXF_CLEANUP
	RiCList_removeAll(me);
#endif
}


/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCList * RiCList_Create(void)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCList * me = (RiCList *)RiC_malloc((RhpPositive)sizeof(RiCList));
	if (me != NULL) {
		(void) RiCList_Init(me);
	}

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	/*LDRA_INSPECTED 71 S */
	return me;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCList_Destroy(RiCList * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL) {
		RiCList_Cleanup(me);
		RiC_free(me);
	}
#endif
}


/***************************************************/
/****				List methods	            ****/
/***************************************************/

RiCBoolean RiCList_addHead(RiCList * const me, gen_ptr const pItem)
{
	RiCBoolean res = RiCFALSE;
	/* create a new list item */
	RiCListItem * listItem = RiCListItem_Create(pItem);
	if (me != NULL)
	{
		if (listItem != NULL){
			/* first item in the list */
			if (me->head == NULL) {
				me->head = listItem;
				me->tail = me->head;
			}
			else {
				/* list is not empty */
				RiCListItem_setNext(listItem, me->head);
				me->head = listItem;
			}
			res = RiCTRUE;
		}
	}

	return res;
}

/*******************/
/*******************/
RiCBoolean RiCList_addTail(RiCList * const me, gen_ptr const pItem)
{
	RiCBoolean res = RiCFALSE;
	/* create a new list item */
	RiCListItem * listItem = RiCListItem_Create(pItem);
	if (me != NULL)
	{
		if (listItem != NULL){
			/* first item in the list */
			if (me->tail == NULL) {
				me->head = listItem;
				me->tail = me->head;
			}
			else {
				/* list is not empty */
				RiCListItem_setNext(me->tail, listItem);
				me->tail = listItem;
			}
			res = RiCTRUE;
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCList_addAt(RiCList * const me, RhpInteger index, gen_ptr const pItem)
{
	RhpInteger res = -1;
	RiCListItem * newItem;
	RiCListItem * prev;
	RhpInteger tmp = index -1;

	/* add at the list head */
	if (index <= 0) {
		if (RiCList_addHead(me, pItem) != RiCFALSE){
			res = 0;
		}
		else {
			res = -1;
		}
	}
	else {
		/* add at the list tail */
		if (index > RiCList_getCount(me)) {
			if (RiCList_addTail(me, pItem) != RiCFALSE){
				res = RiCList_getCount(me);
			}
			else {
				res = -1;
			}
		}
		else {
			/*************************/
			/* add at the given index */
			/**************************/

			/* get the prev list item */
			if (me != NULL)
			{
				prev = me->head;
				for (; tmp > 0 ; tmp--){
					/*LDRA_INSPECTED 45 D : Pointer is checked for null within function */
					prev = RiCListItem_getNext(prev);
				}

				/* create new list item */
				newItem = RiCListItem_Create(pItem);
				if (newItem == NULL){
					res = -1;
				}
				else {
					/* add the new item after prev */
					RiCListItem_setNext(newItem, RiCListItem_getNext(prev));
					RiCListItem_setNext(prev, newItem);
					res = index;
				}
			}
		}
	}


#ifdef USE_ASSERT
	RiC_ASSERT(res != -1);
#endif

	return res;
}
	
/*******************/
/*******************/
gen_ptr RiCList_getHead(const RiCList * const me)
{
	if (me != NULL)
		return RiCListItem_get(me->head);
	else
		return NULL;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCList_getTail(const RiCList * const me)
{
	if (me != NULL)
		return RiCListItem_get(me->tail);
	else
		return NULL;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCList_getAt(const RiCList * const me, RhpInteger index)
{
	gen_ptr res = NULL;
	const RiCListItem * tmp;

	/* handle out of scope indexes */
	if ((index < 0)){
#ifdef USE_ASSERT
		RiC_ASSERT(NULL);
#endif
		res = NULL;
	}
	else if(index > RiCList_getCount(me)){
#ifdef USE_ASSERT
	RiC_ASSERT(NULL);
#endif
		res = NULL;
	}
	/* empty list */
	else if ((me == NULL) || (me->head == NULL)){
#ifdef USE_ASSERT
	RiC_ASSERT(NULL);
#endif
		res = NULL;
	}
	else {
		/* get the relevant list item */
		tmp = me->head;
		while (index > 0) {
			index--;
			tmp = RiCListItem_getNext(tmp);
			if (tmp == NULL){
				index = 0;
			}
		}

		/* return the element */
		res = RiCListItem_get(tmp);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
/*LDRA_INSPECTED 5 D */
}

/*******************/
/*******************/
RiCBoolean RiCList_removeHead(RiCList * const me)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL)
	{
#ifdef MICROC_OXF_CLEANUP
		RiCListItem * tmp = me->head;
#endif	
	/* empty list */
		if (RiCList_isEmpty(me) == RiCFALSE){
			/* single element in list */
			if (me->head == me->tail) {
#ifdef MICROC_OXF_CLEANUP
				RiCListItem_Destroy(me->head);
#endif
				me->head = NULL;
				me->tail = NULL;
			}
			else {
				/* more than one element in the list */
				me->head = RiCListItem_getNext(me->head);
#ifdef MICROC_OXF_CLEANUP
				RiCListItem_Destroy(tmp);
#endif
			}
			res = RiCTRUE;
		}
#ifdef USE_ASSERT
		else /* List is empty */
		{
			RiC_ASSERT(NULL);
		}
#endif
	}
	
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCList_removeTail(RiCList * const me)
{
	return RiCList_removeAt(me,  RiCList_getCount(me));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCList_removeAt(RiCList * const me, RhpInteger index)
{
	RiCBoolean res = RiCFALSE;
	RiCListItem * tmp;
	if (me != NULL)
	{
		RiCListItem * prev = me->head;

		/* boundery check */
		if (index < 0){
#ifdef USE_ASSERT
			RiC_ASSERT(NULL);
#endif
			res = RiCFALSE;
		}
		else if(index > RiCList_getCount(me)){
#ifdef USE_ASSERT
			RiC_ASSERT(NULL);
#endif
			res = RiCFALSE;
		}
		/* index == 0 */
		else if (index == 0){
			res = RiCList_removeHead(me);
		}
		else {
			RiCBoolean flag = RiCFALSE;
			/* get the list item */
			index--;
			while ((flag == RiCFALSE) && (index > 0)) {
				prev = RiCListItem_getNext(prev);
				if (prev == NULL){
					flag = RiCTRUE;
					res = RiCFALSE;	/* error */
				}
				index--;
			}
			
			if(flag == RiCFALSE){
				/* remove the item */
				tmp = RiCListItem_getNext(prev);
				if (tmp == me->tail){
					me->tail = prev;	/* last item - move tail */
				}
				RiCListItem_setNext(prev, RiCListItem_getNext(tmp));
#ifdef MICROC_OXF_CLEANUP
				RiCListItem_Destroy(tmp);
#endif
				res = RiCTRUE;
			}
		}
	}
	
	return res;
}

/*******************/
/*******************/
void RiCList_remove(RiCList * const me, const_gen_ptr pItem)
{
	if (me != NULL)
	{
		if (me->head != NULL){ /*/ Nothing to remove*/
			if (me->head->item == pItem) { /* Special case the object is first*/
				RiCListItem * oldhead = me->head;
				me->head = me->head->next;
				if (me->tail == oldhead) {
					me->tail = me->head;
				}
#ifdef MICROC_OXF_CLEANUP
				RiCListItem_Destroy(oldhead);
#endif
			}
			else {
				RiCBoolean flag = RiCFALSE;
				/* Search for the object*/
				RiCListItem * prev = me->head;
				RiCListItem * cur = me->head->next;
				
				while ((flag == RiCFALSE) && (cur != NULL)) {
					/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
					if ((cur->item == pItem) && (prev != NULL)) {
						/*LDRA_INSPECTED 45 D : LDRA Analysis incorrect. Pointer is checked for null before use */
						prev->next = cur->next;
						if (cur->next == NULL) {
							me->tail = prev;
						}
#ifdef MICROC_OXF_CLEANUP
						RiCListItem_Destroy(cur);
#endif
						flag = RiCTRUE;
					}

					if(flag == RiCFALSE){
						prev = cur;
						cur = cur->next;
					}
				}
#ifdef USE_ASSERT
				if (flag == RiCFALSE) /* element not found */
				{
					RiC_ASSERT(NULL);
				}
#endif
			}

		}
#ifdef USE_ASSERT
		else /* empty list */
		{
			RiC_ASSERT(NULL);
		}
#endif

	}
}

/*******************/
/*******************/
void RiCList_removeAll(RiCList * const me)
{
#ifdef MICROC_OXF_CLEANUP
	RiCListItem * tmp;
#endif
	if (me != NULL)
	{
		while (me->head != NULL) {
#ifdef MICROC_OXF_CLEANUP
			tmp = me->head;
#endif
			me->head = RiCListItem_getNext(me->head);
#ifdef MICROC_OXF_CLEANUP
			RiCListItem_Destroy(tmp);
#endif
		}
		me->head = NULL;
		me->tail = NULL;
	}
}

/*******************/
/*******************/
RiCBoolean RiCList_find(const RiCList * const me, const_gen_ptr pItem)
{
	RiCBoolean flag = RiCFALSE;
	RiCBoolean res = RiCFALSE;
	const RiCListItem * tmp;

	if (me != NULL)
	{
		tmp = me->head;
		while ((flag == RiCFALSE) && (tmp != NULL)) {
			if (RiCListItem_get(tmp) == pItem){
				flag = RiCTRUE;
				res = RiCTRUE;	/* found */
			}

			if(flag == RiCFALSE){
				tmp = RiCListItem_getNext(tmp);
			}
		}
	}
	/* item not found */
	return res;
}

/*******************/
/*******************/
RiCBoolean RiCList_isEmpty(const RiCList * const me)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL)
		res = ((RiCBoolean) (me->head == NULL));

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiC_INT32 RiCList_getCount(const RiCList * const me)
{
	const RiCListItem * tmp;
	RiC_INT32 retCount = 0;

	if (me != NULL)
	{
		tmp = me->head;
		while (tmp != NULL) {
			++retCount;
			tmp = RiCListItem_getNext(tmp);
		}
	}
	return retCount;
}


/***************************************************/
/****            Iteration methods              ****/
/***************************************************/

gen_ptr RiCList_first(const RiCList * const me, gen_ptr * pPos)
{
	gen_ptr res = NULL;
	*pPos = NULL;
	if (me != NULL){
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 93 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		*pPos = (gen_ptr) me->head;
		res = RiCListItem_get(me->head);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
gen_ptr RiCList_next(const RiCList * const me, gen_ptr * pPos)
{
	gen_ptr res = NULL;
	if (me != NULL){
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 93 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
		*pPos = (gen_ptr) RiCListItem_getNext((RiCListItem*) (*pPos));
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
		res = RiCListItem_get((RiCListItem*) (*pPos));
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
gen_ptr RiCList_get(const RiCList * const me, const_gen_ptr pPos)
{
	gen_ptr res = NULL;
	if (me != NULL){
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		res = RiCListItem_get((const RiCListItem*)pPos);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
RiCBoolean RiCList_isDone(const RiCList * const me, const_gen_ptr pPos)
{
	RiCBoolean res = RiCTRUE;
	if ((me != NULL) && (me->tail != NULL)){
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4						*/
		/* Cast from/to void* pointer is required to implement general purpose data collections */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		res = (RiCBoolean) (RiCListItem_get((const RiCListItem*)pPos) == NULL);
	}

	return res;
}

#endif
