#ifndef RiCList_H
#define RiCList_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/


/******************************************************************
 ---------------------------------------------------------------
|							  RiCList.h                          |
 --------------------------------------------------------------- 
*******************************************************************/


#include "mxf_cfg.h"

/******************************************/
/**    Forward declaretion of list item	 **/
/******************************************/

/**********************************************/
/**			Defenition of RiCListItem		 **/
/**********************************************/

typedef struct RiCListItem_t {
	struct RiCListItem_t * next;
	gen_ptr item;
}RiCListItem;


/******************************************/
/**			Defenition of RiCList		 **/
/******************************************/
typedef struct RiCList_t {
	RiCListItem * head;
	RiCListItem * tail;
} RiCList;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

RiCBoolean RiCList_Init(RiCList * const me);

void RiCList_Cleanup(RiCList * const me);


RiCList * RiCList_Create(void);

void RiCList_Destroy(RiCList * const me);

/***************************************************/
/****				List methods	            ****/
/***************************************************/

/** Add an element at the beginning of the list **/
RiCBoolean RiCList_addHead(RiCList * const me, gen_ptr const pItem);

/** Add an element at the end of the list **/
RiCBoolean RiCList_addTail(RiCList * const me, gen_ptr const pItem);

/** Add an element at a given index, 
	if the index is out of scope, add the element at the tail (or head of index < 0).
	return the index element, or -1 on failer **/
RhpInteger RiCList_addAt(RiCList * const me, RhpInteger index, gen_ptr const pItem);

/** Return the element at the list head **/
gen_ptr RiCList_getHead(const RiCList * const me);

/** Return the element at the list tail **/
gen_ptr RiCList_getTail(const RiCList * const me);

/** Return the element at a given index **/
gen_ptr RiCList_getAt(const RiCList * const me, RhpInteger index);

/** Remove the element at the list head **/
RiCBoolean RiCList_removeHead(RiCList * const me);

/** Remove the element at the tail of the list **/
RiCBoolean RiCList_removeTail(RiCList * const me);

/** Remove an element at a given index **/
RiCBoolean RiCList_removeAt(RiCList * const me, RhpInteger index);

/** Remove a given element **/
void RiCList_remove(RiCList * const me, const_gen_ptr pItem);

/** Remove all elements in the list **/
void RiCList_removeAll(RiCList * const me);

/** Find an item, return true if found, false otherwise **/
RiCBoolean RiCList_find(const RiCList * const me, const_gen_ptr pItem);

/** Check if the list is empty **/
RiCBoolean RiCList_isEmpty(const RiCList * const me);

/** Return the number of elements in the list **/
RiC_INT32 RiCList_getCount(const RiCList * const me);

/***************************************************/
/****            Iteration methods              ****/
/***************************************************/

/** Set iteration to the first element in the list **/
gen_ptr RiCList_first(const RiCList * const me, gen_ptr * pPos);

/** Set iteration to the next element in the list **/
gen_ptr RiCList_next(const RiCList * const me, gen_ptr * pPos);

/** Get the current element in the iteration **/
gen_ptr RiCList_get(const RiCList * const me, const_gen_ptr pPos);

/** Check of last element, return RiCTRUE when the current element,
	is the last element in the list **/
RiCBoolean RiCList_isDone(const RiCList * const me, const_gen_ptr pPos);


#endif

