#ifndef RiCProtected_H
#define RiCProtected_H
/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

#include "mxf_cfg.h"

/***************************************************/
/****				Guarding methods			****/
/***************************************************/

/* guard operation */
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define RIC_OPERATION_LOCK(object_monitor) { (void)RiCOSMutex_lock((RiCOSMutex*) (object_monitor)); }

/* free operation guard */
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define RIC_OPERATION_FREE(object_monitor) { (void)RiCOSMutex_free((RiCOSMutex*) (object_monitor)); }

/* reactive object cleanup */
#ifdef MICROC_OXF_CLEANUP
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define RIC_GUARD_REACTIVE_CLEANUP(reactive_me)							\
{																		\
	if ((reactive_me) != NULL){     									\
		if (!RiCReactive_isInCleanup((reactive_me))) {					\
			RiCReactive_setInCleanup((reactive_me));					\
			if (((reactive_me))->toGuardReactive)						\
				RiCTaskEM_lock(RiCReactive_getTask((reactive_me)));	\
		}																\
	}																	\
}

/* active object cleanup */
/* Justification: Violation of Misra98 Rule96 / Misra04 Rule19.10 */
/* Macro body expands to braced expression                        */
#define RIC_GUARD_ACTIVE_CLEANUP(reactive_me)							\
{																		\
	if ((reactive_me) != NULL) {    									\
		if (!RiCReactive_isInCleanup((reactive_me))) {					\
			RiCReactive_setInCleanup((reactive_me));					\
			if (RiCReactive_getTask((reactive_me))->toGuardTask)		\
				RiCTaskEM_lock(RiCReactive_getTask((reactive_me)));	\
		}																\
	}																	\
}
#endif




#endif

