#ifndef RiCQueue_H
#define RiCQueue_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCQueue.h                          |
 --------------------------------------------------------------- 
********************************************************************/
  


#include "mxf_cfg.h"
#include "RiCList.h"

/******************************************/
/**			Forward declaretions		 **/
/******************************************/


/******************************************/
/** The Queue Definition			 **/
/******************************************/
typedef struct RiCQueue_t 
{
	gen_ptr *the_queue;		/* the actual array */
	RhpPositive head;		/* head position */
	RhpPositive tail;		/* tail position */
	RhpPositive count;		/* current number of elements in queue */
	RhpPositive size;		/* number of elements in queue */
} RiCQueue;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

/** return RiCTrue on sucess **/
RiCBoolean RiCQueue_Init(RiCQueue * const me, gen_ptr * const evQueue, RhpPositive queueSize);

#ifdef MICROC_OXF_CLEANUP
void RiCQueue_Cleanup(RiCQueue * const me);
void RiCQueue_Destroy(RiCQueue * const me);
#endif

RiCQueue * RiCQueue_Create(void);


/***************************************************/
/****            Queue methods             ****/
/***************************************************/

/** Push an element to the queue **/
RiCBoolean RiCQueue_push(RiCQueue * const me, gen_ptr const pItem);

/** Pop an element from the queue **/
gen_ptr RiCQueue_pop(RiCQueue * const me);

#ifdef RIC_USE_RICLIST
/** Return a link-list of the queue members, 
	the head of the linked list is the next element to be returned by RiCQueue_get() **/
void RiCQueue_getQueue(const RiCQueue * const me, RiCList * const pList);
#endif

/** Return the queue max size **/
RhpPositive RiCQueue_getSize(const RiCQueue * const me);

/** Return the number of elements in the queue **/
RhpPositive RiCQueue_getCount(const RiCQueue * const me);

/** Check if the queue is empty **/
RiCBoolean RiCQueue_isEmpty(const RiCQueue * const me);

/** Check if the queue is fuul **/
RiCBoolean RiCQueue_isFull(const RiCQueue * const me);

/** Set queue to fixed size flag **/
void RiCQueue_setFixedSize(const RiCQueue * const me, RiCBoolean b);


/***************************************************/
/****            Iteration methods              ****/
/***************************************************/

/** Set iteration to the first element in the collection **/
gen_ptr RiCQueue_first(const RiCQueue * const me, RhpPositive * pPos);

/** Set iteration to the next element in the Queue **/
gen_ptr RiCQueue_next(const RiCQueue * const me, RhpPositive * pPos);

/** Get the current element in the iteration **/
gen_ptr RiCQueue_get(const RiCQueue * const me, RhpPositive pPos);

/** Check of last element, return RiCTRUE when the current element,
	is the last element in the Queue **/
RiCBoolean RiCQueue_isDone(const RiCQueue * const me, RhpPositive pPos);



#endif

