/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/*******************************************************************
 ---------------------------------------------------------------
|							  RiCStack.c                          |
 --------------------------------------------------------------- 
********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICSTACK_SOURCE
#define MXF_RICSTACK_SOURCE
#endif

#include <stdlib.h>
#include "RiCStack.h"
#ifdef RIC_USE_RICSTACK
/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCStack_Init(RiCStack * const me)
{
	if (me != NULL)
		(void) RiCList_Init(&(me->stack));
}

/*******************/
/*******************/
void RiCStack_Cleanup(RiCStack * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL)
		RiCList_Cleanup(&(me->stack));
#endif
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCStack * RiCStack_Create(void)
{
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
	/* Cast from void* pointer is required when using the memory allocation utility */
	/*LDRA_INSPECTED 94 S */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	RiCStack * me = (RiCStack *)RiC_malloc((RhpPositive)sizeof(RiCStack));
	if (me != NULL){
		RiCStack_Init(me);
	}
	/*LDRA_INSPECTED 71 S */

#ifdef USE_ASSERT
	RiC_ASSERT(me != NULL);
#endif

	return me;
}

/*******************/
/*******************/
void RiCStack_Destroy(RiCStack * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL) {
		RiCStack_Cleanup(me);
		RiC_free(me);
	}
#endif
}



/***************************************************/
/****				Stack methods	            ****/
/***************************************************/

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCStack_push(RiCStack * const me, gen_ptr const pItem)
{
	RiCBoolean retVal = RiCFALSE;
	if (me != NULL)
		retVal = RiCList_addHead(&(me->stack), pItem);
	return retVal;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCStack_pop(RiCStack * const me)
{
	gen_ptr tmp = NULL;

	if (me != NULL)
	{
		tmp = RiCStack_top(me);
		(void) RiCList_removeHead(&(me->stack));
	}
	/*LDRA_INSPECTED 71 S */
	return tmp;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCStack_top(const RiCStack * const me)
{
	gen_ptr tmp = NULL;
	if (me != NULL)
		tmp = RiCList_getHead(&(me->stack));
	return tmp;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCStack_isEmpty(const RiCStack * const me)
{
	RiCBoolean retVal = RiCTRUE;
	if (me != NULL)
		retVal = RiCList_isEmpty(&(me->stack));

	return retVal;
}

/***************************************************/
/****            Iteration methods              ****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCStack_first(const RiCStack * const me, gen_ptr * pPos)
{
	gen_ptr res = NULL;
	*pPos = NULL;

	if (me != NULL){
		res = RiCList_first(&(me->stack), pPos);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCStack_next(const RiCStack * const me, gen_ptr * const pPos)
{
	gen_ptr res = NULL;
	if (me != NULL){
		res = RiCList_next(&(me->stack), pPos);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
gen_ptr RiCStack_get(const RiCStack * const me, const gen_ptr pPos)
{
	gen_ptr res = NULL;
	if (me != NULL){
		res = RiCList_get(&(me->stack), pPos);
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCStack_isDone(const RiCStack * const me, const gen_ptr pPos)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL){
		res = RiCList_isDone(&(me->stack), pPos);
	}

	return res;
}

#endif
