/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/

/********************************************************************
 ---------------------------------------------------------------
|							  RiCString.c                          |
 --------------------------------------------------------------- 
*********************************************************************/
#ifndef MXF_SOURCE
#define MXF_SOURCE
#endif
#ifndef MXF_RICSTRING_SOURCE
#define MXF_RICSTRING_SOURCE
#endif

#include <stdlib.h>
#include <string.h>

#include "RiCString.h"

#ifdef RIC_OS_TOLOWER_DEFINED_IN_CTYPE
#include <ctype.h>
#endif

/***************************************************/
/****				Private methods				****/
/***************************************************/

static void reset(RiCString * const me);
static void reset(RiCString * const me)
{
	if (me != NULL)
	{
		me->count = 0U;
		me->size = 0U;
		if (me->string != NULL) {
			RiCReturnMemory(me->string);
			me->string = NULL;
		}
	}
}

/*****************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean grow(RiCString * const me, RhpPositive neededRoom)
{
	RiCBoolean res = RiCFALSE;
	RhpPositive oldSize;
	if (me != NULL)
	{
		if ((me->size - me->count) > (neededRoom + 1U)) { 
			res = RiCTRUE; 	/*no need to grow*/
		}
		else {
			res = RiCTRUE;
			/* compute needed space */
			neededRoom -= ((me->size + 1U) - me->count);
			oldSize = me->size;
			me->size += neededRoom + STRING_INCREMENT_SIZE;

			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
			/* Cast from void* pointer is required when using the memory allocation utility */
			/* reallocate string */
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
			me->string = (RhpString)ricRealloc(me->string, oldSize, me->size);
			if (me->string == NULL) {
				reset(me);
				res = RiCFALSE;
			}
			else {
				me->string[me->count] = END_OF_STRING;
			}
		}
	}
	return res;
}

/*****************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void mkLower(RhpString str)
{
	RhpInteger iPos = 0;
	while (str[iPos] != END_OF_STRING) {
		str[iPos] = (RhpCharacter) tolower((RhpInteger)str[iPos]);
		++iPos;
	}
}

/*****************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void mkUpper(RhpString str)
{
	RhpInteger iPos = 0;
	while (str[iPos] != END_OF_STRING) {
		str[iPos] = (RhpCharacter) toupper((RhpInteger)str[iPos]);
		++iPos;
	}
}

	
/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_Init(RiCString * const me, const RhpCharacter * const str)
{
	if (me != NULL)
	{
		me->count = 0U;
		me->size = 0U;
		me->string = NULL;
		(void) RiCString_set(me, str);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_Init_str(RiCString * const me, const RiCString * str)
{
	if (me != NULL)
	{
		me->count = 0U;
		me->size = 0U;
		me->string = NULL;
		(void) RiCString_set_str(me, str);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_Init_c(RiCString * const me, RhpCharacter c)
{
	if (me != NULL)
	{
		me->count = 0U;
		me->size = 0U;
		me->string = NULL;
		(void) RiCString_set_c(me, c);
	}
}

/*******************/
/*******************/
void RiCString_Cleanup(RiCString * const me)
{
#ifdef MICROC_OXF_CLEANUP
	reset(me);
#endif
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCString * RiCString_Create(const RhpCharacter * const str)
{
	RiCString * me = RiC_malloc((RhpPositive)sizeof(RiCString));
	if (me != NULL) {
		RiCString_Init(me, str);
	}
	/*LDRA_INSPECTED 71 S */
	return me;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCString * RiCString_Create_str(const RiCString * str)
{
	RiCString * me = RiC_malloc((RhpPositive)sizeof(RiCString));
	if (me != NULL){
		RiCString_Init_str(me, str);
	}
	/*LDRA_INSPECTED 71 S */
	return me;
}


/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCString * RiCString_Create_c(RhpCharacter c)
{
	RiCString * me = RiC_malloc((RhpPositive)sizeof(RiCString));
	if (me != NULL) {
		RiCString_Init_c(me, c);
	}
	/*LDRA_INSPECTED 71 S */
	return me;
}


/*******************/
/*******************/
void RiCString_Destroy(RiCString * const me)
{
#ifdef MICROC_OXF_CLEANUP
	if (me != NULL) {
		RiCString_Cleanup(me);
		RiC_free(me);
	}
#endif
}

/***************************************************/
/****				String methods	            ****/
/***************************************************/

/*
 String copy
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_set_str(RiCString * const me, const RiCString * const other)
{
	return RiCString_set(me, RiCString_getBuffer(other));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_set(RiCString * const me, const RhpCharacter * const str)
{
	RiCBoolean res = RiCFALSE;
	
	reset(me); 
	
	if ((str != NULL) && (me != NULL)){
		res = RiCFALSE;
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		if(strlen((RhpString)str) != 0U){
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			me->count = (RhpPositive) strlen((RhpString)str);
			me->size = me->count + 1U;
			/*LDRA_INSPECTED 94 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			me->string = (RhpString) RiC_malloc(me->size);
			
			if (me->string == NULL) {
				reset(me);
				res = RiCFALSE;
			}
			else {
				/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
				/* Cast is required when using Standard C library functions		 */
				/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
				/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
				(void)strcpy((RhpString)me->string, (RhpString)str);
				
				/* do not trust strcpy for the terminating NULL character */
				me->string[me->count] = END_OF_STRING;
			}
		}
	}
	
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_set_c(RiCString * const me, RhpCharacter c)
{
	RiCBoolean res = RiCFALSE;
	
	if (me != NULL)
	{
		res = RiCTRUE;
		reset(me);
		
		me->count = 1U;
		me->size = 2U;
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
		/* Cast from void* pointer is required when using the memory allocation utility */
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		me->string = (RhpString) RiC_malloc((RhpPositive)2);
		
		if (me->string == NULL) {
			reset(me);
			res = RiCFALSE;
		}
		else {
			me->string[0] = c;
			me->string[1] = END_OF_STRING;
		}
	}
	
	return res;
}

/*******************************************/

/*
 String concatination
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_add_str(RiCString * const me, const RiCString * const other)
{
	return RiCString_add(me, RiCString_getBuffer(other));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_add(RiCString * const me, const RhpCharacter * const str)
{
	RiCBoolean res = RiCFALSE;

	if ((str != NULL) && (me != NULL)){
		res = RiCTRUE;
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/* check for space - and grow if need to */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		if (grow(me, (RhpPositive)strlen((RhpString)str)) == RiCFALSE) {
			res = RiCFALSE;
		}
		else {
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
			/* Cast is required when using Standard C library functions		 */
			/* copy str to the tail */
			/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			(void)strcpy((RhpString)(&(me->string[me->count])), (RhpString)str);
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			me->count += (RhpPositive) strlen((RhpString)str);
			me->string[me->count] = END_OF_STRING;
		}
	}
	
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_add_c(RiCString * const me, RhpCharacter c)
{
	RiCBoolean res = RiCFALSE;

	if ((me != NULL) && (c != END_OF_STRING)) {
		res = RiCTRUE;
		/* check for space - and grow if need to */
		if (grow(me, (RhpPositive)1) == RiCFALSE) {
			res = RiCFALSE;
		}
		else{
			/* copy str to the tail */
			me->string[me->count] = c;
			(me->count)++;
			me->string[me->count] = END_OF_STRING;
		}
	}

	return res;
}

/*******************************************/

/*
 String general modifiers
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_setAt(RiCString * const me, RhpPositive i, RhpCharacter c)
{
	RiCBoolean res = RiCFALSE;
	if (me != NULL)
	{
		res = RiCTRUE;
		if (i > me->count) {
			res = RiCFALSE;
		}
		else if (i == me->count) {
			res = RiCString_add_c(me, c);
		}
		else {
			me->string[i] = c;
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_setBuffer(RiCString * const me, const RhpString str)
{
	if (me != NULL)
	{
		reset(me);
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		me->count = (RhpPositive) strlen((RhpString)str);
		if (str != NULL) {
			me->size = me->count + 1U;
			me->string = str;
		}
	}
}

/*******************************************/

/*
 String general accessors
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpString RiCString_getBuffer(const RiCString * const me)
{
	RhpString res = NULL;
	if (me != NULL) {
		res = me->string;
	}

	/*LDRA_INSPECTED 71 S */
	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpCharacter RiCString_getAt(const RiCString * const me, RhpPositive i)
{
	RhpCharacter res = END_OF_STRING;
	if (me != NULL) {
		if (me->count <= i) {
			res = END_OF_STRING;
		}
		else{
			res = me->string[i];
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpPositive RiCString_getLength(const RiCString * const me)
{
	RhpPositive res = 0U;
	if (me != NULL) {
		res = me->count;
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RiCBoolean RiCString_isEmpty(const RiCString * const me)
{
	RiCBoolean res = RiCTRUE;
	if (me != NULL) {
		if(me->count > 0U){
			res = RiCFALSE;
		}
	}

	return res;
}

/*******************************************/

/*
 Search methods
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_find(const RiCString * const me, const RhpCharacter * const str)
{
	RhpInteger res = -1;
	RhpInteger iPos, len;
	RiCBoolean flag = RiCFALSE;

	if ((me == NULL) || (str == NULL)) {
		res = -1;
	}
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
	/* Cast is required when using Standard C library functions		 */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	else if (strlen((RhpString)str) >= me->count) {
		res = -1;
	}
	else {
		/* search loop */
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		len = (RhpInteger) (me->count - ((RhpPositive) strlen((RhpString)str)));
		for (iPos = 0; (iPos < len) && (flag == RiCFALSE); iPos++) {
			/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
			/*LDRA_INSPECTED 93 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			if (strcmp((RhpString)(&(me->string[iPos])), (RhpString)str) == 0) {
				res = iPos;	/* found */
				flag = RiCTRUE;
			}
		}
	}

	return res;	/* not found */
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_find_str(const RiCString * const me, const RiCString * str)
{
	return RiCString_find(me, RiCString_getBuffer(str));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_find_c(const RiCString * const me, RhpCharacter c)
{
	RhpInteger res = -1;
	RiCBoolean flag = RiCFALSE;
	RhpPositive iPos;

	if ((me != NULL) && (c != END_OF_STRING) && (me->count >= 1U)) {
		/* search loop */
		for (iPos = 0U; (iPos < me->count) && (flag == RiCFALSE); iPos++) {
			if (me->string[iPos] == c) {
				res = (RhpInteger) iPos;	/* found */
				flag = RiCTRUE;
			}
		}
	}

	return res;	/* not found */
}

/*******************************************/
 
/*
 String compare methods
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compare(const RiCString * const me, const RhpCharacter* const str)
{
	RhpInteger res = 0;
	if((me == NULL) || (me->string == NULL)) {
		if(str == NULL){
			res = 0;
		}
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		else if(strlen((RhpString)str) == 0U) {
			res = 0;
		}
		else {
			res = -1;
		}
	}
	else{
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
		/* Cast is required when using Standard C library functions		 */
		/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		res = strcmp((RhpString)me->string, (RhpString)str);
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compare_str(const RiCString * const me, const RiCString * str)
{
	return RiCString_compare(me, RiCString_getBuffer(str));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compare_c(const RiCString * const me, RhpCharacter c)
{
	RhpInteger res = -1;
	RhpCharacter tmp[2] = {END_OF_STRING, END_OF_STRING};
	tmp[0] = c;
	/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
	/* Cast is required when using Standard C library functions		 */
	/*LDRA_INSPECTED 97 S : LDRA Analysis incorrect. Casting is required */
	/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
	if (me != NULL) {
		res = strcmp((RhpString)(&(me->string)), (RhpString) tmp);
	}

	return res;
	/*lint -restore */
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compareNoCase(const RiCString * const me, const RhpCharacter * const str)
{
	RhpString myStr = NULL;
	RhpString otherStr = NULL;
	RhpInteger res = 0x0fffffff;

	if (me != NULL) {
		/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4				*/
		/* Cast from void* pointer is required when using the memory allocation utility */
		/* allocate temporary strings */
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		myStr = (RhpString) RiC_malloc(me->count + 1U);
		/*LDRA_INSPECTED 94 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
		/*LDRA_INSPECTED 98 S */ /*LDRA_INSPECTED 458 S */
		otherStr = (RhpString) RiC_malloc(((RhpPositive) strlen((RhpString)str)) + 1U);

		if ((myStr != NULL) && (otherStr != NULL)) {	/*safe programming*/
			/* Justification: Violation of Misra98 Rule45 / Misra04 Rule11.4 */
			/* Cast is required when using Standard C library functions		 */
			/* copy data */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			(void)strcpy((RhpString)myStr, (RhpString)(me->string));
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			(void)strcpy((RhpString)otherStr, (RhpString)str);

			/* convert temporary strings to lowwer case */
			mkLower(myStr);
			mkLower(otherStr);

			/* compare */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			/*LDRA_INSPECTED 95 S */ /*LDRA_INSPECTED 203 S */
			res = strcmp((RhpString)myStr, (RhpString)otherStr);

			RiC_free(myStr);
			RiC_free(otherStr);
		}
	}

	return res;
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compareNoCase_str(const RiCString * const me, const RiCString * str)
{
	return RiCString_compareNoCase(me, RiCString_getBuffer(str));
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
RhpInteger RiCString_compareNoCase_c(const RiCString * const me, RhpCharacter c)
{
	RhpCharacter tmp[2] = {END_OF_STRING, END_OF_STRING};
	tmp[0] = c;
	return RiCString_compareNoCase(me, tmp); 
}

/*******************************************/

/*
 Case modifiers
*/

/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_makeLower(RiCString * const me)
{
	if (me != NULL) {
		mkLower(me->string);
	}
}

/*******************/
/*******************/
/* Justification: Violation of Misra98 Rule23 / Misra04 Rule8.10 */
/* The OXF is a framework conaining APIs that might be used		 */
/* Outside of the OXF itself by Generated code					 */
void RiCString_makeUpper(RiCString * const me)
{
	if (me != NULL) {
		mkUpper(me->string);
	}
}

