#ifndef RiCString_H
#define RiCString_H

/*

	Licensed Materials - Property of IBM
	(c) Copyright IBM Corporation 2006, 2011. All Rights Reserved.
*/
/*********************************************************************
 ---------------------------------------------------------------
|							  RiCString.h                          |
 --------------------------------------------------------------- 
**********************************************************************/

#include "mxf_cfg.h"

#define END_OF_STRING ((RhpCharacter) 0)

/******************************************/
/**			String growth rate			 **/
/******************************************/
#define STRING_INCREMENT_SIZE 20U

/******************************************/
/**			Defenition of RiCString		 **/
/******************************************/
typedef struct RiCString_t {
	RhpPositive size;		/* The current allocated size */
	RhpPositive count;		/* The number of characters in the string (without \0) */
	RhpString string;			/* the string */
} RiCString;


/***************************************************/
/****          Creation and Destruction         ****/
/***************************************************/

void RiCString_Init(RiCString * const me, const RhpCharacter * const str);
void RiCString_Init_str(RiCString * const me, const RiCString * str);
void RiCString_Init_c(RiCString * const me, RhpCharacter c);

void RiCString_Cleanup(RiCString * const me);
void RiCString_Destroy(RiCString * const me);


RiCString * RiCString_Create(const RhpCharacter * const str);
RiCString * RiCString_Create_str(const RiCString * str);
RiCString * RiCString_Create_c(RhpCharacter c);

void mkLower(RhpString str);
void mkUpper(RhpString str);
RiCBoolean grow(RiCString * const me, RhpPositive neededRoom);



/***************************************************/
/****				String methods	            ****/
/***************************************************/

/* copy a string */
RiCBoolean RiCString_set_str(RiCString * const me, const RiCString * const other);

/* copy raw string */
RiCBoolean RiCString_set(RiCString * const me, const RhpCharacter * const str);

/* copy a single character */
RiCBoolean RiCString_set_c(RiCString * const me, RhpCharacter c);

/*******************************************/

/* add a string (at the end of the string) */
RiCBoolean RiCString_add_str(RiCString * const me, const RiCString * const other);

/* add raw string */
RiCBoolean RiCString_add(RiCString * const me, const RhpCharacter * const str);

/* add a single character */
RiCBoolean RiCString_add_c(RiCString * const me, RhpCharacter c);

/*******************************************/

/* set a given character in the string */
RiCBoolean RiCString_setAt(RiCString * const me, RhpPositive i, RhpCharacter c);

/*******************************************/

/* set the string to a given buffer */
void RiCString_setBuffer(RiCString * const me, const RhpString str);

/*******************************************/

/* get the string buffer */
RhpString RiCString_getBuffer(const RiCString * const me);

/*******************************************/

/* get a charcter in the string at a given index, return \0 if out of scope */
RhpCharacter RiCString_getAt(const RiCString * const me, RhpPositive i);

/*******************************************/

/* return the string length */
RhpPositive RiCString_getLength(const RiCString * const me);

/* check if the string is empty */
RiCBoolean RiCString_isEmpty(const RiCString * const me);

/*******************************************/

/* find a raw string in the string, return the index or -1 if not found */
RhpInteger RiCString_find(const RiCString * const me, const RhpCharacter * const str);

/* find a string in the string, return the index or -1 if not found */
RhpInteger RiCString_find_str(const RiCString * const me, const RiCString * str);

/* find a char in the string, return the index or -1 if not found */
RhpInteger RiCString_find_c(const RiCString * const me, RhpCharacter c);

/*******************************************/

/* compare with raw string */
RhpInteger RiCString_compare(const RiCString * const me, const RhpCharacter* const str);

/* compare with other string */
RhpInteger RiCString_compare_str(const RiCString * const me, const RiCString * str);

/* compare with a char */
RhpInteger RiCString_compare_c(const RiCString * const me, RhpCharacter c);

/*******************************************/

/* compare no case with raw string */
RhpInteger RiCString_compareNoCase(const RiCString * const me, const RhpCharacter * const str);

/* compare no case with other string */
RhpInteger RiCString_compareNoCase_str(const RiCString * const me, const RiCString * str);

/* compare no case with a char */
RhpInteger RiCString_compareNoCase_c(const RiCString * const me, RhpCharacter c);

/*******************************************/

/* set the string to lowwer caps */
void RiCString_makeLower(RiCString * const me);

/* set the string to upper caps */
void RiCString_makeUpper(RiCString * const me);



#endif

