/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2016. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
var irpApplication;
var irpProject;
var properties;
var logger;
var oldProfile;
var newProfile;
var iRPInstance = Java.type('com.telelogic.rhapsody.core.IRPInstance');
var iRPDependency = Java.type('com.telelogic.rhapsody.core.IRPDependency');
var IRPComment = Java.type('com.telelogic.rhapsody.core.IRPComment');
var iRPGeneralization = Java.type('com.telelogic.rhapsody.core.IRPGeneralization');
var iRPHyperLink = Java.type('com.telelogic.rhapsody.core.IRPHyperLink');
var iRPObjectModelDiagram = Java.type('com.telelogic.rhapsody.core.IRPObjectModelDiagram');
var iRPPackage = Java.type('com.telelogic.rhapsody.core.IRPPackage');
var iRPCollection = Java.type('com.telelogic.rhapsody.core.IRPCollection');

setApplication = function (application){
	irpApplication = application;
}

setProject = function (project){
	irpProject = project;
}

setProperties = function (newProperties){
	properties = newProperties;
}

setOldProfile = function (profile){
	oldProfile = profile;
}

setNewProfile = function (profile){
	newProfile = profile;
}

setLogger = function (newLogger){
	logger = newLogger;
}

getProject = function (){
	return irpProject;
}

evaluateRule = function(mappingArray, stereotypeKey, element){
	logger.fine("evaluate Migration Rule "+stereotypeKey+" on element "+element.getFullPathName());
	mappingArray[stereotypeKey](element);
}

//Migration policy
handleRenameStereotype = function(element, oldStereotypeName, newStereotypeName){
	if(hasStereotypeApplied(element, oldStereotypeName)){
		if(unapplyStereotype(element, oldStereotypeName)){
			var element = applyStereotype(element, newStereotypeName);
			if(element!=null){
				//find Tags from old Profile and set value to Tag from new Profile
				var tags = element.getLocalTags();
				if(tags!=null && tags.getCount()>0){
					var newStereotype = findStereotype(newStereotypeName);
					if(newStereotype!=null){
						var stCount = tags.getCount();
						for (var i = 1; i <= stCount; i++) {
							var mytag = tags.getItem(i);
							var tagName = mytag.getName();
							var baseTag = mytag.getBase();
							if(baseTag!=null){
								logger.fine("Tag to handle after Stereotype change : "+tagName);
								var stereotypeOwner = baseTag.getOwner();
								if(stereotypeOwner!=null){
									var values = getTagValues(mytag);
									if(hasThisTagInStereotype(newStereotype, tagName)){
										deleteElement(mytag);
										setTagValue(element, baseTag.getName(), values)
									}
									else{
										transformTagToLocalTag(element, tagName);
									}
								}
								else{
									logger.fine("Tag not handled : "+mytag.getName()+" because there is no owner");
								}
							}
						}
					}
				}
				return element;
			}
		}
	}
	else{
		var stereotype = element.getStereotypes().getItem(1);
		if(stereotype!=null){
			var name = stereotype.getName();
			logger.warning("Stereotype "+oldStereotypeName+" is not applied on "+element.getName()+"<"+name+">");
		}
		else
			logger.warning("Stereotype "+oldStereotypeName+" is not applied on "+element.getName()+"<no stereotype>("+element.getStereotypes().getCount()+")");
	}
	return null;
}

handleMoveTagInNewElement = function(element, oldTagName, stereotypeNameNewElement, newTagName){
	var objectAsObjectType;
	if(iRPInstance.class.isInstance(element)){
		objectAsObjectType=element.getObjectAsObjectType();
	}
	var newElement;
	var oldTag;
	if(objectAsObjectType!=null){
		oldTag = getLocalTag(objectAsObjectType, oldTagName);
		if(oldTag!=null)
			newElement = getOrCreateElement(objectAsObjectType, stereotypeNameNewElement, null);
	}
	else{
		oldTag = getLocalTag(element, oldTagName);
		if(oldTag!=null)
			newElement = getOrCreateElement(element, stereotypeNameNewElement, null);
	}
	
	if(newElement!=null && oldTag!=null){
		var tagValues = getTagValues(oldTag);
		if(tagValues!=null){
			setTagValue(newElement, newTagName, tagValues);
			deleteElement(oldTag);
		}
	}
	return newElement;
}

handleMoveReferenceInNewElement = function(element, stereotypeNameFrom, stereotypeName, stereotypeNameTo){
	var dependencies = element.getDependencies();
	var count = dependencies.getCount();
	for (var i = 1; i <= count; i++) {
		var dependency = dependencies.getItem(i);
		if(hasStereotypeApplied(dependency, stereotypeNameFrom)){
			var newElement =  getOrCreateElement(element, stereotypeName, null);
			logger.fine("new element : "+newElement.getFullPathName());
			dependency.setOwner(newElement);
			logger.fine("new owner element : "+dependency.getOwner().getFullPathName());
			if(stereotypeNameTo!=null && !stereotypeNameFrom.equals(stereotypeNameTo)){
				logger.fine("on element "+dependency.getFullPathName()+" apply <"+stereotypeNameFrom+"> unapply <"+stereotypeNameTo+">");
				unapplyStereotype(dependency, stereotypeNameFrom);
				applyStereotype(dependency, stereotypeNameTo);
			}
		}
	}
}

handleMoveCommentInNewElement = function(element, stereotypeNameFrom, stereotypeName, stereotypeNameTo){
	var annotations = element.getAnnotations();
	var count = annotations.getCount();
	for (var i = 1; i <= count; i++) {
		var annotation = annotations.getItem(i);
		if(hasStereotypeAppliedAndSubtypes(annotation, stereotypeNameFrom)){
			var newElement = getOrCreateElement(element, stereotypeName, null);
			logger.fine("new element : "+newElement.getFullPathName());
			annotation.setOwner(newElement);
			logger.fine("new owner element : "+annotation.getOwner().getFullPathName());
			if(stereotypeNameTo!=null && !stereotypeNameFrom.equals(stereotypeNameTo)){
				unapplyStereotype(annotation, stereotypeNameFrom);
				applyStereotype(annotation, stereotypeNameTo);
			}
		}
	}
}

handleMoveInNewChild = function(elementToMove, stereotypeNameToCreate){
	var newElement = getOrCreateElement(elementToMove.getOwner(), stereotypeNameToCreate, null);
	logger.fine("new element : "+newElement.getFullPathName());
	elementToMove.setOwner(newElement);
	logger.fine("new owner element : "+elementToMove.getOwner().getFullPathName());
}

handleMoveInNewElement = function(element, stereotypeNameFrom, stereotypeName, stereotypeNameTo){
	var listSt = [];
	if(stereotypeNameFrom.contains("|")){
		listSt[0] = stereotypeNameFrom.split("|")[0];
		listSt[1] = stereotypeNameFrom.split("|")[1];
	}
	var stereotypeF = findStereotype(stereotypeNameFrom);
	if(stereotypeF!=null){
		var rhpType = stereotypeF.getOfMetaClass();
		logger.fine("rhpType of Stereotype <"+stereotypeF.getName()+"> : "+rhpType);
		if(rhpType!=null && !"".equals(rhpType)){
			var children;
			var count;
			if(iRPPackage.class.isInstance(element)){
				children = element.getAllNestedElements();
			}
			else if(iRPInstance.class.isInstance(element)){
				children = element.getObjectAsObjectType().getNestedElements();
			}
			else{
				children = element.getNestedElements();
			}
			
			if(children!=null){
				count = children.getCount();
				for (var i = 1; i <= count; i++) {
					var child = children.getItem(i);
					var metaClass = child.getMetaClass();
					logger.fine(child.getFullPathName()+" => metaclass : "+metaClass+" - rhpType to find : "+rhpType);
					if(metaClass.equals(rhpType)){
						if(hasStereotypeAppliedAndSubtypes(child, stereotypeNameFrom)){
							var newElement = getOrCreateElement(element, stereotypeName, null);
							logger.fine("new element : "+newElement.getFullPathName());
							child.setOwner(newElement);
							logger.fine("new owner element : "+child.getOwner().getFullPathName());
							if(stereotypeNameTo!=null && !stereotypeNameFrom.equals(stereotypeNameTo)){
								unapplyStereotype(child, stereotypeNameFrom);
								applyStereotype(child, stereotypeNameTo);
							}
						}
					}
				}
			}
		}
	}
}

handleMoveCommentInNewElementFromValidOwner = function(element, stereotypeNameOwner, stereotypeName){
	var owner = element.getOwner();
	if(owner!=null && hasStereotypeApplied(owner, stereotypeNameOwner)){
		var newElement = getOrCreateElement(owner, stereotypeName, null);
		element.setOwner(newElement);
	}
}
	
handleRenameTag = function(element, oldTag, newTag){
	logger.fine("try to rename Tag "+oldTag+" to "+newTag+" in "+element.getFullPathName());
	var objectAsObjectType;
	if(iRPInstance.class.isInstance(element)){
        objectAsObjectType=(element.getObjectAsObjectType());
    }
	var tag;
	if(objectAsObjectType!=null){
		tag = getTag(objectAsObjectType, oldTag);
	}
	else{
		tag = getTag(element, oldTag);
	}
	if(tag!=null){
		var value = getTagValues(tag);
		logger.fine("move Tag value '"+value+"' on Tag "+newTag+" in "+element.getFullPathName());
		deleteElement(tag);
		setTagValue(element, newTag, value);
	}
	else{
		logger.fine("Tag not found "+oldTag+" in "+element.getFullPathName());
	}
}

handleOldIref = function(element, stereotypeName){
	logger.warning("The stereotype '"+stereotypeName+"' is an old InstanceRef, please use new way with ContextTag...");
	unapplyStereotype(element, stereotypeName);
}

handleDeleteStereotype = function(element, stereotypeName){
	unapplyStereotypeAndTags(element, stereotypeName);
}

handleDeleteTag = function(element, tagName){
	transformTagToLocalTag(element, tagName);
}

transformTagToLocalTag = function(element, tagName){
	var localTag = getLocalTag(element, tagName);
	if(localTag!=null){
		var values = getTagValues(localTag);
		deleteElement(localTag);
		var newTag = element.addNewAggr("Tag","_"+tagName);
		newTag.setName(tagName);
		newTag.addStringDefaultValue(values);
		logger.fine("Tag transformed to local Tag : "+newTag.getFullPathName()+"="+newTag.getValue());
	}
	else{
		logger.fine("Tag not found : "+element.getFullPathName()+"."+tagName);
	}
}

handleDeleteStereotypeFromAggregates = function(element, stereotypeName){
	if(stereotypeName.startsWith("l_")){
		unapplyStereotypeFromDependencyChild(element, stereotypeName);
	}
	else{
		unapplyStereotypeFromChild(element, stereotypeName);
	}
	transformTagsFromOldStereotypeToLocalTag(element, stereotypeName);
}

handleChangeGeneralizationTarget = function(element){
	//Transform old Tag from old Stereotype superType to Localtag
	//unload Stereotypes not in Aggregates of Stereotype or Parent
}

handleChangeTargetPropertyOnStereotype = function(element, stereotypeName){
	unapplyStereotype(element, stereotypeName); 
}

handleDeleteStereotypeFromAggregatesFromChild = function(element, stereotypeName){
	var elements = null;
	if(iRPPackage.class.isInstance(element)){
		elements = element.getAllNestedElements();
	}
	else if(iRPInstance.class.isInstance(element)){
		elements = element.getObjectAsObjectType().getNestedElements();
	}
	else{
		elements = element.getNestedElements();
	}
	if(elements!=null && elements.getCount()>0){
		var stCount = elements.getCount();
		logger.fine(stCount+" children found in "+element.getName());
		for (var i = 1; i <= stCount; i++) {
			var child = elements.getItem(i);
			logger.fine("child found : "+child.getName());
			unapplyStereotypeChildren(child, stereotypeName);
		}
	}
}

handleRenameStereotypeFromDependencyChild = function(element, stereotypeNameFrom, stereotypeNameTo){
	var child = getDependencyIn(element, stereotypeNameFrom);
	if(child!=null){
		if(unapplyStereotype(child, stereotypeNameFrom)){
			applyStereotype(child, stereotypeNameTo);
		}
	}
}

handleRenameStereotypeFromAnnotationChild = function(element, stereotypeNameFrom, stereotypeNameTo){
	var child = getAnnotationIn(element, stereotypeNameFrom);
	if(child!=null){
		if(unapplyStereotype(child, stereotypeNameFrom)){
			applyStereotype(child, stereotypeNameTo);
		}
	}
}



handleTagToContainmentRef = function(element, tagName, stereotypeName, metaType, tagNameTarget){
	var objectAsObjectType;
	if(iRPInstance.class.isInstance(element)){
        objectAsObjectType=(element.getObjectAsObjectType());
    }
	var tag;
	if(objectAsObjectType!=null){
		tag = getLocalTag(objectAsObjectType, tagName);
	}
	else{
		tag = getLocalTag(element, tagName);
	}
	if(tag!=null){
		var tagValues = getTagValues(tag);
		if(tagValues!=null){
			var newElement = createElementOn(element, stereotypeName, null);
			if(newElement!=null){
				setTagValue(newElement, tagNameTarget, tagValues);
				tag.deleteFromProject();
			}
			else{
				logger.fine("element "+metaType+"<"+stereotypeName+"> not created" );
			}
		}
	}
	else{
		logger.fine("tag named "+tagName+" is not found" );
	}
}
//////////////////////////////////

getTagValues = function(element){
	var values = element.getValueSpecifications();
    var stCount = values.getCount();
	var allValues;
	
    for (var i = 1; i <= stCount; i++) {
		var value = values.getItem(i);
		if(allValues==null){
			var valueofValue = value.getValue();
			if(iRPCollection.class.isInstance(valueofValue)){
				var stCountC = valueofValue.getCount();
				for (var j = 1; j <= stCountC; j++) {
					if(allValues==null)
						allValues = valueofValue.getItem(j);
					else
						allValues = allValues + valueofValue.getItem(j);
				}
			}
			else
				allValues = value.getValue();
		}
		else
			allValues = allValues+", "+value.getValue();
	}
	logger.fine("values of tag '"+element.getName()+"' : "+allValues+" ("+stCount+")");
	return allValues;
}

getElementFromNewProfile = function(name, metaClass){
	var element = newProfile.findNestedElementRecursive(name, metaClass);
	return element;
}

getPropertyValue = function(propertyKey){
	var result = properties.getProperty(propertyKey, null);
	if(result==null){
		throw "The property " + propertyKey + " does not exist";
	}
	return result; 
}

getPropertyValueOfElement = function(element, ruleName){
	var guid = element.getGUID();
	var guidKey = guid.substring(5, guid.length());
	var key = ruleName+"|"+guidKey;
	var newStereotypeName = getPropertyValue(key);
	return newStereotypeName;
}

unapplyStereotype = function(element, stereotypeName){
	logger.fine("try to unapply Stereotype "+stereotypeName+" on "+element.getFullPathName());
	var stereotypes = element.getStereotypes();
    var stCount = stereotypes.getCount();
    for (var i = 1; i <= stCount; i++) {
		var stereotypeF = stereotypes.getItem(i);
        var name = stereotypeF.getName();
		if(name.equals(stereotypeName)){
			element.removeStereotype(stereotypeF);
			logger.fine("unapply Stereotype "+stereotypeName+" success on "+element.getFullPathName());
			return true;
		}
	}	
	return false;
}

unapplyStereotypeAndTags = function(element, stereotypeName){
	logger.fine("try to unapply Stereotype "+stereotypeName+" on "+element.getFullPathName());
	var stereotypes = element.getStereotypes();
    var stCount = stereotypes.getCount();
    for (var i = 1; i <= stCount; i++) {
		var stereotypeF = stereotypes.getItem(i);
        var name = stereotypeF.getName();
		if(name.equals(stereotypeName)){
			element.removeStereotype(stereotypeF);
			logger.fine("unapply Stereotype "+stereotypeName+" success on "+element.getFullPathName());
			transformTagsFromOldStereotypeToLocalTag(element, stereotypeName);
			return true;
		}
	}	
	return false;
}

transformTagsFromOldStereotypeToLocalTag = function(element, stereotypeName){
	logger.fine("transform deprecated Tags to local Tags on "+element.getFullPathName());
	var tags = element.getLocalTags();
	var stCount = tags.getCount();
	if(stCount!=0){
		for (var i = 1; i <= stCount; i++) {
			var mytag = tags.getItem(i);
			var tagName = mytag.getName();
			var baseTag = mytag.getBase();
			if(baseTag!=null && baseTag.getFullPathName().startsWith(oldProfile.getName())){
				logger.fine("Tag to handle : "+tagName);
				transformTagToLocalTag(element, tagName);
			}
			else{
				logger.fine("Tag not handled : "+mytag.getName()+" because there is no baseTag");
			}
		}
	}
	else{
		logger.fine("no Tags to handle for element "+element.getFullPathName());
	}
}

unapplyStereotypeChildren = function(element, stereotypeName){
	if(unapplyStereotype(element, stereotypeName)){
		return;
	}
	else{
		logger.fine("try to unapply Stereotypes children for "+stereotypeName);
		var sts = getStereotypesSubtypes(stereotypeName);
		if(sts!=null && sts.length>0){
			var arraysize = sts.length;
			for (var j = 0; j < arraysize; j++) {
				var st = sts[j];
				unapplyStereotype(element, st);
			}
		}
		else{
			logger.fine("Stereotype "+stereotypeName+" has no subtypes");
		}
	}
}

findStereotype = function(stereotypeName){
	var stereotypes = getProject().getAllStereotypes();
	var stCount = stereotypes.getCount();
	for (var i = 1; i <= stCount; i++) {
		var stereotype = stereotypes.getItem(i);
		var name = stereotype.getName();
		if(name.equals(stereotypeName)){
			return stereotype;
		}
	}
	logger.fine("Stereotype "+stereotypeName+" not found");
	return null;
}

applyStereotype = function(element, stereotypeName){
	logger.fine("try to apply Stereotype "+stereotypeName+" on "+element.getFullPathName());
	var stereotypeF = findStereotype(stereotypeName);
	try	{
		if(stereotypeF!=null){
			var metaclass = stereotypeF.getOfMetaClass();
			if(element.getMetaClass().equals(metaclass)){
				element.addSpecificStereotype(stereotypeF);
				logger.fine("apply Stereotype "+stereotypeName+" success on "+element.getFullPathName());
				return element;
			}
			else{
				logger.fine("Metaclass "+metaclass+" of "+stereotypeName+" is not equal to element type : "+element.getMetaClass());
				if(metaclass!=null && !"".equals(metaclass)){
					var elementStereotyped = changeMetaClassAndStereotype(element, stereotypeName);
					if(elementStereotyped!=null)
						return elementStereotyped;
				}
			} 
		}
		else{
			logger.severe("apply Stereotype "+stereotypeName+" failed on "+element.getFullPathName()+" because Stereotype not found");
		}
	}
	catch(e){
		logger.severe("apply Stereotype "+stereotypeName+" failed on "+element.getFullPathName()+" error : "+e);
	}
	return null;
}

getTagFromStereotype = function(element, tagName){
	if(element.getStereotypes().getCount()==1){
		var stereotype = element.getStereotypes().getItem(1);
		var tag = stereotype.getTag(tagName);
		if(tag!=null){
			return tag;
		}
		else{
			logger.fine("Tag "+tagName+" not found in "+element.getFullPathName()+"<"+stereotype.getName()+">");
		}
	}
	else{
		logger.fine("Tag "+tagName+" not found in "+element.getFullPathName());
	}
}

getTag = function(element, tagName){
	var tag = element.getTag(tagName);
	if (tag!=null && !tag.getOwner().equals(element)) {
		try	{
			tag = element.setTagValue(tag, null);
		}
		catch(e){
			logger.severe("error, impossible to set null on tag "+e);
		}
    }
	else if(tag==null){
		logger.fine("Tag "+tagName+" not found in "+element.getFullPathName());
	}
	return tag;
}

getLocalTag = function(element, tagName){
	var tags = element.getLocalTags();
    var stCount = tags.getCount();
	var tag = null;
    for (var i = 1; i <= stCount; i++) {
		var mytag = tags.getItem(i);
		if (mytag.getName().equals(tagName)) {
			tag = mytag;
		}
	}
	if (tag === undefined || tag === null) {
		//logger.fine("Tag "+tagName+" not found in "+element.getFullPathName());
	}
	else{
		logger.fine("local tag "+tagName+" found in "+element.getFullPathName()+" [tag owner : "+tag.getOwner().getFullPathName()+"]");
	}
	return tag;
}

setTagValue = function(element, tagName, tagValue){
	logger.fine("Try to set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName());
	var tag = getTag(element, tagName);
	if ((tag != null) && (tagValue != null)) {
		logger.fine("set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName());
		try{
			tag.addStringDefaultValue(tagValue.toString());
		}
		catch(e){
			logger.fine("error, impossible to set value '"+tagValue+"' to tag "+tagName+" in element "+element.getFullPathName()+" "+e);
			return null;
		}
	}
	else{
		logger.fine("Tag '"+tagName+"' not found in element "+element.getFullPathName());
		return null;
	}
	return tag;
}

getElementsWithStereotype = function(element, stereotypeName){
	var elements = null;
	var elementsFound = new Set();
	if(iRPPackage.class.isInstance(element)){
		elements = element.getAllNestedElements();
	}
	else{
		elements = element.getNestedElements();
	}
	var stCount = elements.getCount();
    for (var i = 1; i <= stCount; i++) {
        var child = elements.getItem(i);
		if(hasStereotypeApplied(child, stereotypeName)){
			elementsFound.add(child);
		}
	}
	return elementsFound;
}



hasStereotypeApplied = function(element, stereotypeName){
	logger.fine("try to find Stereotype <"+stereotypeName+"> on "+element.getFullPathName());
	try{
		var stereotypes = element.getStereotypes();
		var stCount = stereotypes.getCount();
		for (var i = 1; i <= stCount; i++) {
			var stereotypeF = stereotypes.getItem(i);
			var name = stereotypeF.getName();
			logger.fine("Stereotype found "+name+" on "+element.getFullPathName());
			if(name.equals(stereotypeName)){
				return true;
			}
		}
	}
	catch(e){
		logger.severe("error when search Stereotype applied "+e);
	}
	return false;
}

hasStereotypeAppliedAndSubtypes = function(element, stereotypeName){
	if(hasStereotypeApplied(element, stereotypeName)){
		return true;
	}
	else{
		var sts = getStereotypesSubtypes(stereotypeName);
		if(sts!=null && sts.length>0){
			var arraysize = sts.length;
			for (var j = 0; j < arraysize; j++) {
				var st = sts[j];
				if(hasStereotypeApplied(element, st)){
					return true;
				}
			}
		}
	}
	return false;
}

getStereotypesSubtypes = function(stereotypeName){
	var stereotypeF = findStereotype(stereotypeName);
	var list = [];
	var i=0;
	if(stereotypeF!=null){
		var refs = stereotypeF.getReferences();
		if(refs!=null){
			var refCount = refs.getCount();
			logger.fine("Stereotype "+stereotypeName+" has "+refCount+" references");
			for (var j = 1; j <= refCount; j++) {
				var ref = refs.getItem(j);
				if(ref!=null && iRPGeneralization.class.isInstance(ref)){
					var nameToStereotype = ref.getBaseClass().getName();
					var nameFromStereotype = ref.getDerivedClass().getName();
					logger.fine("For Stereotype <"+stereotypeName+"> : generalization from "+nameFromStereotype+" to "+nameToStereotype);
					if(nameToStereotype.equals(stereotypeName)){
						list[i]=nameFromStereotype;
						i++;
					}
				}
			}
		}
		else{
			logger.fine("Stereotype "+stereotypeName+" has 0 references");
		}
	}
	return list;
}

createElementOn = function(owner, stereotypeName, name){
	var stereotypeF = findStereotype(stereotypeName);
	if(stereotypeF!=null){
		var metaType = stereotypeF.getOfMetaClass();
		logger.fine("create "+metaType+"<"+stereotypeName+"> in "+owner.getName());
		var newElement = owner.addNewAggr(metaType, name);
		applyStereotype(newElement, stereotypeName);
		logger.fine("new element created : "+newElement.getFullPathName()+" <"+stereotypeName+">");
		return newElement;
	}
}

getAnnotationIn = function(owner, stereotypeName){
	var annotations = owner.getAnnotations();
    var stCount = annotations.getCount();
    for (var i = 1; i <= stCount; i++) {
		var annotation = annotations.getItem(i);
		if(hasStereotypeApplied(annotation, stereotypeName)){
			return annotation;
		}
	}
	return null;
}

getAnnotationsInRecursive = function(owner, stereotypeName){
	var elements = [];
	var annotations = owner.getNestedElementsByMetaClass("Comment", 1);
    var stCount = annotations.getCount();
	var j = 0;
    for (var i = 1; i <= stCount; i++) {
		var annotation = annotations.getItem(i);
		if(hasStereotypeApplied(annotation, stereotypeName)){
			elements[j]=annotation;
			j++
		}
	}
	return elements;
}

getAnnotationsIn = function(owner, stereotypeName){
	var elements = [];
	var annotations = owner.getAnnotations();
    var stCount = annotations.getCount();
	var j=0;
    for (var i = 1; i <= stCount; i++) {
		var annotation = annotations.getItem(i);
		if(stereotypeName==null || hasStereotypeApplied(annotation, stereotypeName)){
			elements[j] = annotation;
			j++;
		}
	}
	return elements;
}

getOperationsIn = function(owner, stereotypeName){
	var elements = [];
	var operations = owner.getNestedElementsByMetaClass("Operation", 0);
    var stCount = operations.getCount();
	var j=0;
    for (var i = 1; i <= stCount; i++) {
		var operation = operations.getItem(i);
		if(stereotypeName==null || hasStereotypeApplied(operation, stereotypeName)){
			elements[j] = operation;
			j++;
		}
	}
	return elements;
}

getAggregatesIn = function(owner, stereotypeName){
	var elements = [];
	var stereotype = findStereotype(stereotypeName);
	if(stereotype!=null){
		var metaClass = stereotype.getOfMetaClass();
		if(metaClass!=null){
			var aggregates = owner.getNestedElementsByMetaClass(metaClass, 0);
			var stCount = aggregates.getCount();
			var j=0;
			for (var i = 1; i <= stCount; i++) {
				var aggregate = aggregates.getItem(i);
				if(stereotypeName==null || hasStereotypeApplied(aggregate, stereotypeName)){
					elements[j] = aggregate;
					j++;
				}
			}
		}
		else{
			logger.fine("no metaClass for Stereotype "+stereotypeName);
		}
	}
	else{
		logger.fine("Stereotype not found : "+stereotypeName);
	}
	return elements;
}

getInstanceIn = function(owner, stereotypeName){
    var stCount = annotations.getCount();
	var ports = owner.getPorts();
    for (var i = 1; i <= stCount; i++) {
		var port = ports.getItem(i);
		if(hasStereotypeApplied(port, stereotypeName)){
			return port;
		}
	}
	return null;
}

getDependencyIn = function(owner, stereotypeName){
	var dependencies = owner.getDependencies();
    var stCount = dependencies.getCount();
    for (var i = 1; i <= stCount; i++) {
		var dependency = dependencies.getItem(i);
		if(hasStereotypeApplied(dependency, stereotypeName)){
			return dependency;
		}
	}
	return null;
}

getDependenciesIn = function(owner, stereotypeName){
	var elements = [];
	var dependencies = owner.getDependencies();
    var stCount = dependencies.getCount();
	var j=0;
    for (var i = 1; i <= stCount; i++) {
		var dependency = dependencies.getItem(i);
		if(hasStereotypeApplied(dependency, stereotypeName)){
			elements[j] = dependency;
			j++;
		}
	}
	return elements;
}

removeTag = function(element, tagName){
	var tag = getTag(element, tagName);
	if(tag!=null){
		logger.fine("delete Tag "+tagName+" in "+element.getFullPathName());
		deleteElement(tag);
	}
}

removeCommentIn = function(element, stereotypeName){
	var child = getAnnotationIn(element, stereotypeName)
	if(child!=null){
		deleteElement(child);
	}
}

unapplyStereotypeFromCommentChild = function(element, stereotypeName){
	var children = getAnnotationsIn(element, stereotypeName)
	logger.fine(element.getFullPathName()+" has "+children.length+" "+stereotypeName);
	if(children.length>0){
		var stCount = children.length;
		for (var i = 0; i < stCount; i++) {
			unapplyStereotypeAndTags(children[i], stereotypeName);
		}
		return true;
	}
}

unapplyStereotypeFromOperationChild = function(element, stereotypeName){
	var children = getOperationsIn(element, stereotypeName)
	logger.fine(element.getFullPathName()+" has "+children.length+" "+stereotypeName);
	if(children.length>0){
		var stCount = children.length;
		for (var i = 0; i < stCount; i++) {
			unapplyStereotypeAndTags(children[i], stereotypeName);
		}
		return true;
	}
}

unapplyStereotypeFromChild = function(element, stereotypeName){
	var children = getAggregatesIn(element, stereotypeName)
	logger.fine(element.getFullPathName()+" has "+children.length+" "+stereotypeName);
	if(children.length>0){
		var stCount = children.length;
		for (var i = 0; i < stCount; i++) {
			unapplyStereotypeAndTags(children[i], stereotypeName);
		}
		return true;
	}
}
	
removeDependencyIn = function(element, stereotypeName){
	var child = getDependencyIn(element, stereotypeName)
	if(child!=null){
		deleteElement(child);
	}
}

unapplyStereotypeFromDependencyChild = function(element, stereotypeName){
	var children = getDependenciesIn(element, stereotypeName)
	logger.fine(element.getFullPathName()+" has "+children.length+" "+stereotypeName);
	if(children.length>0){
		var stCount = children.length;
		for (var i = 0; i < stCount; i++) {
			unapplyStereotype(children[i], stereotypeName);
		}
		return true;
	}
	return false;
}

moveAnnotationsFromTo = function(from, to){
	var annotations = from.getAnnotations();
    var stCount = annotations.getCount();
    for (var i = 1; i <= stCount; i++) {
		try{
			var annotation = annotations.getItem(i);
			annotation.setOwner(to);
			logger.fine("move "+annotation.getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
		}
		catch(e){
			logger.fine("error, impossible to move "+annotation.getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
		}
	}
	return null;
}

moveNestedElementsFromTo = function(from, to){
	logger.fine("move nestedElements from "+from.getFullPathName()+" to "+to.getFullPathName()); 
	var elementsToMove = [];
	elementsToMove = returnNestedElements(from, elementsToMove, 0);
	var arrayLength = elementsToMove.length;
	for (var i = 0; i < arrayLength; i++) {
		try{
			if(elementsToMove[i]!=null){
				logger.fine("try to move "+elementsToMove[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
				elementsToMove[i].setOwner(to);
				logger.fine("move "+elementsToMove[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName()+" on place"+i);
			}
			else{
				logger.fine("impossible to move element because null element on place"+i);
			}
		}
		catch(e){
			logger.severe("error, impossible to move  from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
		}
	}
	return null;
}


returnNestedElements = function(element, elementsToMove, place) {
    var nestedElements = element.getNestedElements();
    var count = nestedElements.getCount();
    var otherClass=null;
    if(iRPInstance.class.isInstance(element)){
        otherClass=(element.getObjectAsObjectType());
    }
    for (var i = 1; i <= count; i++) {
        var nested = nestedElements.getItem(i);
        if(!nested.equals(otherClass)){
            logger.fine("nestedElement found "+nested.getClass().getSimpleName() + ": "+nested.getFullPathName()+" place : "+place);
			elementsToMove[place]=nested;
			place++;
        }
    }
    if(otherClass!=null)
        return returnNestedElements(otherClass, elementsToMove, place++);
	return elementsToMove;
}

returnReferencesElements = function(element) {
	try{
		var references = element.getReferences();
		var elementsToMove = [];
		var count = references.getCount();
		for (var i = 1; i <= count; i++) {
			try{
				var reference = references.getItem(i);
				logger.fine("reference found "+reference.getName()+ ": "+reference.getFullPathName()+" place : "+i-1);
				elementsToMove[i-1]=reference;
			}
			catch(e){
				logger.severe("error during getting refs on"+element.getName()+" : "+e);
			}
		}
	}
	catch(e){
		logger.severe("error, impossible to get references from "+element.getFullPathName()+" : "+e);
	}
	return elementsToMove;
}

updateReferencesElementsFromTo = function(from, to){
	logger.fine("updating references from "+from.getFullPathName()+" to "+to.getFullPathName());
	try{
		var elementsToUpdate = [];
		elementsToUpdate = returnReferencesElements(from);
		var arrayLength = elementsToUpdate.length;
		for (var i = 0; i < arrayLength; i++) {
			if(elementsToUpdate[i]!=null){
				logger.fine("try to update ref "+elementsToUpdate[i].getName()+" from "+from.getFullPathName()+" in "+to.getFullPathName());
				if(iRPDependency.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setDependsOn(to);
				}
				else if(iRPGeneralization.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setBaseClass(to);
				}
				else if(iRPHyperLink.class.isInstance(elementsToUpdate[i])){
					elementsToUpdate[i].setDependsOn(to);
				}
				else if(iRPObjectModelDiagram.class.isInstance(elementsToUpdate[i])){
					updateGraphicalElements(elementsToUpdate[i], from, to);
				}
			}
			else{
				logger.fine("impossible to update ref  because null element on place"+i);
			}
		}
	}
	catch(e){
		logger.severe("error, impossible to update ref from "+from.getFullPathName()+" in "+to.getFullPathName()+" : "+e);
	}
	logger.fine("end to update ref from "+from.getFullPathName()+" in "+to.getFullPathName());
	return null;
}


deleteElement = function(element){
	var fullName = element.getFullPathName();
	logger.fine("try to delete "+element.getFullPathName());
	element.deleteFromProject();
	logger.fine("delete sucess of "+fullName);
}

create = function(owner, metaType, name){
	logger.fine("create "+metaType+" in "+owner.getFullPathName()+" named "+name);
	try{
		var newElement = owner.addNewAggr(metaType, name);
		return newElement;
	}
	catch(e){
		logger.severe("error during creating "+metaType+" in "+owner.getFullPathName()+" "+e);
	}
	return null;
}

createWithStereotype = function(owner, metaType, stereotypeName, name){
	logger.fine("create "+metaType+" in "+owner.getFullPathName()+" stereotyped "+stereotypeName+"and named "+name);
	var element = create(owner, metaType, name);
	applyStereotype(element, stereotypeName);
	return element;
}

changeMetaClassAndStereotype = function(element, newStereotypeName){
	var stereotypeF = findStereotype(newStereotypeName);
	if(stereotypeF!=null){
		var metaClass = stereotypeF.getOfMetaClass();
		var name = element.getName();
		logger.fine("Change metaClass and stereotype on "+name+", metaClass : "+metaClass+" and stereotype : "+newStereotypeName);
		var owner = element.getOwner();
		var newElement;
		try{
			if(metaClass.equals("Object") && iRPPackage.class.isInstance(owner)){
				newElement = owner.addImplicitObject(element.getName());
			}
			else
				newElement = create(owner, metaClass, element.getName());
				
			if(newElement==null){
				try{
					newElement = owner.addNewAggr(newStereotypeName, element.getName());	
				}
				catch(e1){
					logger.severe("impossible to create "+newStereotypeName+" in "+owner.getFullPathName()+"  : "+e1);
				}
			}
		}
		catch(e){
			try{
				newElement = owner.addNewAggr(newStereotypeName, element.getName());	
			}
			catch(e2){
				logger.severe("impossible to create "+newStereotypeName+" in "+owner.getFullPathName()+"  : "+e2);
			}
		}
		if(newElement!=null){
			moveNestedElementsFromTo(element, newElement);
			updateReferencesElementsFromTo(element, newElement);
			applyStereotype(newElement, newStereotypeName);
			deleteElement(element);
			logger.fine(name+" has been replaced by "+newElement.getFullPathName()+" with success");
			return newElement;
		}
		else{
			logger.severe(newStereotypeName+" hasn't been create in "+owner.getFullPathName());
		}
	}
	else{
		logger.severe("Stereotype not found "+newStereotypeName);
	}
	return null;
}

updateGraphicalElements = function(diagram, from, to){
	logger.warning("Deleting element "+from.getName()+" will cause GraphNode or GraphEdge removing in diagram "+diagram.getFullPathName());
}

moveTagInNewFlexrayClusterContent = function(element, tagName){
	if(hasStereotypeApplied(element, "FlexrayCluster")){
	
		var objectAsObjectType;
		if(iRPInstance.class.isInstance(element)){
			objectAsObjectType=(element.getObjectAsObjectType());
		}
		var flexRayClusterContent;
		var oldTag;
		if(objectAsObjectType!=null){
			oldTag = getLocalTag(objectAsObjectType, tagName);
			flexRayClusterContent = getOrCreateElement(objectAsObjectType, "FlexrayClusterVariant", null);
		}
		else{
			oldTag = getLocalTag(element, tagName);
			flexRayClusterContent = getOrCreateElement(element, "FlexrayClusterVariant", null);
		}
		
		if(flexRayClusterContent!=null && oldTag!=null){
			var tagValues = getTagValues(oldTag);
			if(tagValues!=null){
				setTagValue(flexRayClusterContent, tagName, tagValues);
				deleteElement(oldTag);
			}
		}
	}
}

getOrCreateElement = function(owner, stereotypeName, name){
	var stereotype = findStereotype(stereotypeName);
	if(stereotype!=null){
		var rhptype = stereotype.getOfMetaClass();
		if(rhptype!=null && !"".equals(rhptype)){
			logger.fine("search child "+rhptype+"<"+stereotype.getName()+">");
			var elements = null;
			if(iRPPackage.class.isInstance(owner)){
				elements = owner.getAllNestedElements();
			}
			else if(iRPInstance.class.isInstance(owner)){
				elements = owner.getObjectAsObjectType().getNestedElements();
			}
			else{
				elements = owner.getNestedElements();
			}
			var stCount = elements.getCount();
			var newElement;
			for (var i = 1; i <= stCount; i++) {
				var element = elements.getItem(i);
				if(element.getMetaClass().equals(rhptype) && hasStereotypeApplied(element, stereotypeName)){
					newElement = elements.getItem(i);
					logger.fine("child found "+newElement.getName()+" "+rhptype+"<"+stereotype.getName()+">");
					return newElement;
				}
			}
			if(newElement==null){
				return createElementOn(owner, stereotype.getName(), null);
			}
		}
		else{
			logger.severe("MetaClass is empty for Stereotype "+stereotypeName);
		}
	}
	else{
		logger.fine("Stereotype '"+stereotypeName+"' not found");
	}
}

fillInDataFilterTypeTag = function(element, filterTypeName){
	setTagValue(element, "dataFilterType", filterTypeName);
}

handleChangeReferenceToContainment = function(element, refStereotypeName){
	var dependency = getDependencyIn(element, refStereotypeName);
	if(dependency!=null){	
		var stereotype = dependency.getDependsOn().getStereotypes().getItem(0);
		if(stereotype!=null){
			var stereotypeName = stereotype.getName();
			var rhpType = stereotype.getMetaClass();
			var newElement = element.addNewAggr(rhpType, null);
			applyStereotype(newElement, stereotypeName);
			moveNestedElementsFromTo(dependency.getDependsOn(), newElement);
			deleteElement(dependency);
		}
	}
}

hasThisTagInStereotype = function(stereotype, tagName){
	var elements = stereotype.getAllTags();
	var stCount = elements.getCount();
	for (var i = 1; i <= stCount; i++) {
		var tag = elements.getItem(i);
		if(tag.getName().equals(tagName)){
			return true;
		}
	}
	return false;
}

moveTagToOwner = function(element, tagName){
	var tag = getTag(element, tagName);
	if(tag!=null){
		var values = getTagValues(tag);
		setTagValue(element.getOwner(), values);
		deleteElement(tag);
	}
}
moveTagInSwAddrmethod = function(element, tagName, dependencyStereotypeName){
	var tag = getLocalTag(element, tagName);
	if(tag!=null){
		var values = getTagValues(tag);
		var dependency = getDependencyIn(element, dependencyStereotypeName);
		if(dependency!=null){
			var dependsOn = dependency.getDependsOn();
			setTagValue(dependsOn, tagName, values);
			deleteElement(tag);
		}
		else{
			transformTagToLocalTag(element, tagName);
		}
	}
}

renameParamConfContainerDefRecursively = function(element){
	if(element!=null){
		try{
			logger.fine("rename of ParamConfContainerDef in "+element.getFullPathName());
			var underchildren = getAnnotationsIn(element, "ParamConfContainerDef");
			var stCountUC = underchildren.length;
			logger.fine(element.getFullPathName()+" has "+stCountUC+" ParamConfContainerDef(s)");
			for (var k = 0; k < stCountUC; k++) {
				var uchild = underchildren[k];
				logger.fine("rename of "+uchild.getFullPathName()+" ParamConfContainerDef in EcucParamConfContainerDef");
				uchild = handleRenameStereotype(uchild, "ParamConfContainerDef", "EcucParamConfContainerDef");
				if(uchild!=null){
					renameAllContainerDef(uchild);
					renameParamConfContainerDefRecursively(uchild);
				}
			}
		}
		catch(e){
			logger.severe("error when renameParamConfContainerDefRecursively : "+e);
		}
	}
}

renameAllContainerDef = function(child){
	var underchildren = getAnnotationsIn(child, "ChoiceContainerDef");
	var stCountUC = underchildren.length;
	 for (var k = 0; k < stCountUC; k++) {
	  var uchild = underchildren[k];
	  handleRenameStereotype(uchild, "ChoiceContainerDef", "EcucChoiceContainerDef");
	 }
	 
	  var underchildren = getAnnotationsIn(child, "BooleanParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "BooleanParamDef", "EcucBooleanParamDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "ChoiceReferenceParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "ChoiceReferenceParamDef", "EcucChoiceReferenceDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "EnumerationParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "EnumerationParamDef", "EcucEnumerationParamDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "FloatParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "FloatParamDef", "EcucFloatParamDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "ForeignReferenceParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "ForeignReferenceParamDef", "EcucForeignReferenceDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "FunctionNameDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "FunctionNameDef", "EcucFunctionNameDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "InstanceReferenceParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "InstanceReferenceParamDef", "EcucInstanceReferenceDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "IntegerParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "IntegerParamDef", "EcucIntegerParamDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "LinkerSymbolDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "LinkerSymbolDef", "EcucLinkerSymbolDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "ReferenceParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "ReferenceParamDef", "EcucReferenceDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "StringParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "StringParamDef", "EcucStringParamDef");
	  }
	  
	  var underchildren = getAnnotationsIn(child, "SymbolicNameReferenceParamDef");
	  var stCountUC = underchildren.length;
	  for (var k = 0; k < stCountUC; k++) {
	   var uchild = underchildren[k];
	   handleRenameStereotype(uchild, "SymbolicNameReferenceParamDef", "EcucSymbolicNameReferenceDef");
	  }

}
